
####################################################################
# (C) Copyright 2012-2013 Hewlett-Packard Development Company, L.P.
# @(#) Serviceguard generic resource monitoring script for disk.
# @(#) Product Name                :  HP Serviceguard
# @(#) Product Version             :  A.12.10.00
# @(#) Patch Name                  :  
#
#
# **********************************************************************
# *  This is an example script that demonstrates how to monitor cpu    *
# *  utilization using extended generic resources.                     *
# *  Please modify this to suit your own needs.                        *
# *                                                                    *
# *  This script is only a sample to demonstrate the use of generic    *
# *  resources. This has to be modified to suit your requirements      *
# *  after copying it to the appropriate package directory.            *
# *  This is an example and HP does not support this script.           *
# *                                                                    *
# *  You may want to include additional error handling which has been  *
# *  omitted.                                                          *
# *                                                                    *
# *  This sample script monitors the average load among all the CPU's  *
# *  present in the node by using the dstat tool.                      *
# *  dstat tool can be used to monitor individual CPU's too. For more  *
# *  information on this one can check the man page for dstat.         *
# *                                                                    *
# *  Script monitors the CPU utilization using dstat tool and sets the *
# *  utilization percentage used as a value to the corresponding       *
# *  extended generic resource configured in package using             *
# *  "cmsetresource" command.                                          *
# *                                                                    *
# *  According to the generic_resource_up_criteria specificed for      *
# *  extended generic resource, status will be changed.                *
# *  For example in package generic_resource_up_criteria is "<=30",    *
# *  when the specified CPU utilization usage becomes 31%, the value   *
# *  of extended generic resource will be 31 and status will be down.  *
# *                                                                    *
# *  To use this script, copy it to the package specific directory     *
# *  and use it.                                                       * 
# *                            ---------                               *
# *                            I N P U T                               *
# *                            ---------                               *
# *  The script expects the following parameters when configured as a  *
# *  service.                                                          *
# *                                                                    *
# *      $1 : Generic resource name : The name of the generic resource * 
# *           configured in this package that corresponds to this      *
# *           monitoring script.                                       *
# *      $2 : Monitor interval      : The time interval between        *
# *           successive monitors.                                     *
# *                                                                    *
# *  NOTE: By default the dstat tool might not be installed            *
# *                                                                    *
# *       - For Redhat 5.x and 6.x, install the dstat-xxxxxxx.rpm      *
# *         from the DVD. For example in Red Hat 6.2 it is             *
# *         dstat-0.7-1.el6.noarch.rpm                                 *
# *                                                                    *
# *       - For SLES 11-SP.x the dstat tool is not shipped in          *
# *         the DVD.                                                   *
# *         One must download the appropriate dstat rpm and install    *
# *         before using this script.                                  *
# **********************************************************************
#
###################################################################

###########################
# Source utility functions.
###########################

if [[ -z $SG_UTILS ]]
then
    . /etc/cmcluster.conf
    SG_UTILS=$SGCONF/scripts/mscripts/utils.sh
fi

if [[ -f ${SG_UTILS} ]]
then
    . ${SG_UTILS}
    if (( $? != 0 ))
    then
        echo "ERROR: Unable to source package utility functions file: ${SG_UTILS}"
        exit 1
    fi
else
    echo "ERROR: Unable to find package utility functions file: ${SG_UTILS}"
    exit 1
fi

###########################################
# Initialize the variables & command paths
###########################################
WHICH=/usr/bin/which
RM=`$WHICH rm`
CAT=`$WHICH cat`
GREP=`$WHICH grep`
CUT=`$WHICH cut`
AWK=`$WHICH awk`
TAIL=`$WHICH tail`
DSTAT=`$WHICH dstat`
CMGETRESOURCE=$SGSBIN/cmgetresource
CMSETRESOURCE=$SGSBIN/cmsetresource

RES_NAME=$1
MONITOR_INTERVAL=$2
DSTAT_CPU_LOG=/tmp/cpu.log

###########################################
# Source the package environment variables.
###########################################

typeset postfix=$(date +"%H.%M.%S")
SG_ENV_FILE=/var/tmp/${SG_PACKAGE}.$postfix.$$.tmp
$SGSBIN/cmgetpkgenv $SG_PACKAGE > $SG_ENV_FILE
if (( $? != 0 ))
then
    echo "ERROR: Unable to retrieve package attributes."
    exit 1
fi
. $SG_ENV_FILE
$RM -f $SG_ENV_FILE

usage()
{
    $CAT >&2 <<EOM
    Usage:
      sample_generic_resource_cpu_utilization.sh <resource_name> <monitor_interval>
EOM
    exit 1
}

function print_usage_if_help_or_no_arg
{
    if [ $# -ne 2 ]; then
        usage
    fi
    if [ $# -eq 1 ] && \
       [ "$1" = "-h" ]; then
        usage
    fi
    return 0
}


print_usage_if_help_or_no_arg $*

# Set the Resource value to requested value  
function set_genres_value
{
    new_value=$1
    old_value=`$CMGETRESOURCE -r $RES_NAME | $GREP $RES_NAME | $AWK '{print $2}'`
    if [[ "$old_value" != "$new_value" ]]; then
        sg_log 0 "Setting the value of resource $RES_NAME to $new_value"
        $CMSETRESOURCE -r $RES_NAME $new_value
    fi
}

function halt_script
{
    sg_log 0 "Stopping the CPU utilization monitor"
    exit 1
}

# cmhaltpkg will issue SIGTERM signal to all the services configured in package
# As part of SIGTERM signal handler the script will exits.

trap "halt_script" SIGTERM

sg_log 0 "Starting the CPU utilization monitor for generic resource $RES_NAME"

while [ 1 ]
do

    $DSTAT -c --output $DSTAT_CPU_LOG 1 5 > /dev/null 2> /dev/null
    value=`$TAIL -1 $DSTAT_CPU_LOG | $CUT -d',' -f2 | $CUT -d'.' -f1`
    sg_log 2 "The dstat output is :"`$TAIL -1 $DSTAT_CPU_LOG`
    if [ $value -eq 0 ]
    then
        set_genres_value 1
    elif [ -z "$value" ]
    then
        sg_log 0 "Unable to get CPU utilization information."
    else
        set_genres_value $value
    fi

    $RM -f $DSTAT_CPU_LOG

    # Sleep for given interval of time before checking the size of the mount point for the next time
    sleep $MONITOR_INTERVAL
done
