####################################################################
# (C) Copyright 2016-2017 Hewlett Packard Enterprise Development LP.
# (C) Copyright 2006-2015 Hewlett-Packard Development LP.
# @(#) Serviceguard UPCC Package Configuration Script.
# @(#) Product Name                :  HP Serviceguard  
# @(#) Product Version             :  A.12.10.00 
# @(#) Patch Name                  :  
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HP Serviceguard.
#
# Changing this file may lead to unrecoverable package startup
# and/or shutdown problems.
#
################################################################
# sg_log logs a message to the package log file or stdout 
#
# Arguments:  $1 : log level  
#             $* : message  
#  
# Globals referenced :    
#             SG_LOG_LEVEL :  package attribute; default it to 0.  
#             SG_SCRIPT_LOG_FILE : package attribute  
#             SG_DATE_FORMAT : format of the date and time in each  
#                              log message. This variable can be   
#                              configured to set a different format.  
#             op_code :  set in the master_control_script.sh   
#                
################################################################

# File Handling
date_prefix=$(date +"%H_%M_%S_%N")
str_prefix="Log_"$$_
log_prefix=$str_prefix$date_prefix

typeset -i cmviewcl_retry_interval=2
typeset tmp_dir="/var/tmp"
typeset cmviewcl_output=$tmp_dir/$log_prefix"_cmviewcl.$$"
typeset cmviewcl_fline_pkg_op=""
typeset _input_file=$tmp_dir/$log_prefix"_input_file_$$.txt"

local_host_name=${SG_HOST_NAME%%.*}
typeset -i local_node_id=0
typeset -i SG_RUNNING_NODE_DURING_VALIDATE
typeset CMVIEWCL="$SGSBIN/cmviewcl"


function sg_log
{
    typeset -i level=$1 
    typeset script_name=${0##*/}

    #
    # variables SG_DATE_FORMAT & SG_LOG_PREFIX are defined
    # in master_control_script and here. Any changes to these
    # variables should be made in both the places.
    #
    if [[ -z $SG_DATE_FORMAT ]] 
    then
        typeset SG_DATE_FORMAT=${SG_DATE_FORMAT:-"+%b %e %H:%M:%S"}
    fi
    if [[ -z $SG_LOG_PREFIX ]]
    then
        typeset SG_LOG_PREFIX="$(whoami)@$(hostname)"
    fi

    #
    # SG_LOG_LEVEL controls the amount of log messages printed
    # to either stdout or to a script log file.
    #

    SG_LOG_LEVEL=${SG_LOG_LEVEL:-0}

    #
    # Check for log level 999 and check if it is reconfiguration.
    # The contents of this file will have details of SUCCESS/ERRORS/
    # WARNING(for the changes that were not attempted).
    # The contents of this file will be flushed to package log in
    # case of an error. Otherwise it will be deleted.
    #
    if (( level == 999 ))
    then
        if sg_check_if_it_is_reconfiguration
        then
            # Log messages for online reconfig failure scenarios
            # Before logging shift the shift the argument to remove the log level
            shift
            echo "$(date "$SG_DATE_FORMAT") $SG_LOG_PREFIX $script_name[$$]: $*" >> $SG_SCRIPT_ONLINE_DETAILS_LOG_FILE
        fi
    fi

    if (( SG_LOG_LEVEL >= level ))
    then
        shift
        if [[ "${op_code}" != "validate" ]]
        then
            # log messages goes to $SG_SCRIPT_LOG_FILE
            echo "$(date "$SG_DATE_FORMAT") $SG_LOG_PREFIX $script_name[$$]: $*" >> $SG_SCRIPT_LOG_FILE
            # Logging for DRR package
            if sg_check_if_pkg_is_drr_pkg; then
                if (( level == 0 )); then
                    DRR_LOG_FILE="$SGRUN/DRxR/$SG_REHEARSAL_ID/.$SG_PACKAGE_NAME.pkglog"
                    # Remove the new line character if any in the line
                    str=`echo "$*" | tr '\r' ' ' | tr '\n' ' '`
                    echo "$SG_PACKAGE_NAME|pkg$op_code|$str" >> $DRR_LOG_FILE
                fi
            fi
        else
            # log messages goes to stdout without any LOG_PREFIX
            echo "$*"
        fi
    fi
}

################################################################
# Function is called to set the flag to identify if it is DRR  #
# package or not                                               #
################################################################
function sg_set_if_drr_package_true
{
    postfix=$(date +"%H.%M.%S")
    tmp_file="/var/tmp/exp_var_$postfix.$$.drr_pkg_true"
    if echo "$SG_PACKAGE_NAME" | grep -q "DRxR_"; then
        echo "export drr_pkg_flag=\"1\"" > $tmp_file
        . ${tmp_file}
        rm -f $tmp_file
    fi
}

################################################################
# Function is called to check if the current package is DRR pkg#
# or not                                                       #
################################################################
function sg_check_if_pkg_is_drr_pkg
{
    # this value is set in function sg_set_if_drr_package_true
    if [[ "X$drr_pkg_flag" != "X" ]] && [ $drr_pkg_flag -eq 1 ]
    then
        return 0
    else
        return 1
    fi
}

################################################################
# Function is called to check if failure has occured during an #
# online reconfiguration                                       #
################################################################
function sg_check_if_reconfig_has_failed
{
    if sg_check_if_it_is_reconfiguration && [ -f $SG_RECONFIG_FAIL_FLAG ]
    then
        return 0
    else
        return 1
    fi
}

################################################################
# Function is called to check if the current OP is reconfigure #
################################################################
function sg_check_if_it_is_reconfiguration
{
    if [[ "X$reconfig_flag" != "X" ]] && [ $reconfig_flag -eq 1 ]
    then
        return 0
    else
        return 1
    fi
}

################################################################
# Writes the required variables during online reconfiguration  #
# to a file and exports.                                       #
# The varialbles exported are                                  #
#   1. Log file to capture the SUCCESS/ERROR/NOT ATTEMPTED     #
#      details                                                 #
#   2. Sets the reconfig flag                                  #
#   3. File name used to distinguish the failure during online #
#      reconfiguration.                                        #
################################################################
function sg_set_reconfigure
{
    postfix=$(date +"%H.%M.%S")
    tmp_file="/var/tmp/exp_var_$postfix.$$.tmpreconfig"
    SG_SCRIPT_ONLINE_DETAILS_LOG_FILE=/var/tmp/.online_details.$postfix.$$_tmpreconfig
    SG_RECONFIG_FAIL_FLAG=/var/tmp/.online_failure.$postfix.$$_file
    echo "export reconfig_flag=\"1\"" > $tmp_file
    echo "export SG_SCRIPT_ONLINE_DETAILS_LOG_FILE=$SG_SCRIPT_ONLINE_DETAILS_LOG_FILE" >> $tmp_file
    echo "export SG_RECONFIG_FAIL_FLAG=$SG_RECONFIG_FAIL_FLAG" >> $tmp_file
    . ${tmp_file}
    rm -f $tmp_file
}
################################################################
# Call this function set the failure, once set other modules   #
# can use to check if there is a failure already or not        #
################################################################
function sg_set_reconfig_failure
{
    if sg_check_if_it_is_reconfiguration
    then
        if [ ! -f $SG_RECONFIG_FAIL_FLAG ]
        then
            touch $SG_RECONFIG_FAIL_FLAG
            sg_log 999 "ERROR: Will not attempt to apply remaining changes if any due to the error encountered"
        fi
    fi
}

# Set the flag for skipping modules; 
function sg_set_storage_module_skip
{
    postfix=$(date +"%H.%M.%S")
    tmp_file="/var/tmp/exp_var_$postfix.$$.skip_pr_fs_vg_module"
    echo "export skip_storage_module_skip_flag=\"1\"" > $tmp_file
    . ${tmp_file}
    rm -f $tmp_file
}

# Other modules use this function to decide to skip or not
function sg_check_if_storage_module_skip
{
    if [[ "X$skip_storage_module_skip_flag" != "X" ]] && [ $skip_storage_module_skip_flag -eq 1 ]
    then
        return 0
    else
        return 1
    fi
}

# Set to skip PR module in the start/stop block
function sg_set_skip_pr_modules
{
    postfix=$(date +"%H.%M.%S")
    tmp_file="/var/tmp/exp_var_$postfix.$$.skip_pr_module"
    echo "export skip_module_pr_flag=\"1\"" > $tmp_file
    . ${tmp_file}
    rm -f $tmp_file
}

# PR modules decides to skip or not in start/stop block
function sg_check_if_skip_pr_modules
{
    if [[ "X$skip_module_pr_flag" != "X" ]] && [ $skip_module_pr_flag -eq 1 ]
    then
        return 0
    else
        return 1
    fi
}

# VMFS module decides to call PR module and sets the flag
function sg_set_pr_module_required
{
    postfix=$(date +"%H.%M.%S")
    tmp_file="/var/tmp/exp_var_$postfix.$$.pr_module_required"
    echo "export pr_module_required_flag=\"1\"" > $tmp_file
    . ${tmp_file}
    rm -f $tmp_file
}

function sg_check_if_pr_module_required
{
    if [[ "X$pr_module_required_flag" != "X" ]] && [ $pr_module_required_flag -eq 1 ]
    then
        return 0
    else
        return 1
    fi
}

################################################################
#
# INTERNAL USE ONLY
#
# sg_exit
#   exit function for master control script
#   It will perform cleanup steps before exiting
#
# Arguments:  $1 : exit code - if none specified, default is 0
#             $2 : action code - start/stop/validate
#             $3 : input package environment file
#
################################################################
function  sg_exit
{
    typeset -i exit_code=${1:-0}

    if [[ -n $SG_ENV_FILE ]]
    then
        rm $SG_ENV_FILE
    elif (( $# == 3 ))
    then
        rm $3
    fi
    exit $exit_code
} 

##################
#
# INTERNAL USE ONLY
#
# Based on the exit_code, print the 
# package start completion marking message.
#
# Arguments:  None. 
#
# It assumes that caller has set the aggregate_exit_val
#
##################
function sg_log_pkg_start_completion_msg
{
   if (( aggregate_exit_val == 0 ))
   then
      sg_log 0 "###### Package start completed for $SG_PACKAGE ######"
   else
      sg_log 0 "###### Failed to start package for $SG_PACKAGE ######"
   fi
}

##################
#
# INTERNAL USE ONLY
#
# Based on the exit_code, print the 
# package halt completion marking message.
#
# Arguments:  None. 
#
# It assumes that caller has set the aggregate_exit_val 
#
##################
function sg_log_pkg_stop_completion_msg
{
   if (( aggregate_exit_val == 0 ))
   then
      sg_log 0 "###### Package halt completed for $SG_PACKAGE ######"
   else
      sg_log 0 "###### Failed to halt package for $SG_PACKAGE ######"
   fi
}

function sg_log_pkg_reconfigure_completion_msg
{
    if sg_check_if_reconfig_has_failed
    then
        sg_log 0 "#########################################################################"
        sg_log 0 "###### Package reconfigure completed with failures for $SG_PACKAGE ######"
        sg_log 0 "###### Below is the summary of changes for $SG_PACKAGE ######"
        cat $SG_SCRIPT_ONLINE_DETAILS_LOG_FILE >> $SG_SCRIPT_LOG_FILE
        sg_log 0 "#########################################################################"
        rm -f $SG_RECONFIG_FAIL_FLAG
        rm -f $SG_SCRIPT_ONLINE_DETAILS_LOG_FILE
    else
        sg_log 0 "###### Package reconfigure completed for $SG_PACKAGE ######"
    fi
}


#############################
#
# INTERNAL USE ONLY
#
# prints the steps of the script
#############################
function sg_print_sequence_array
{
    typeset -i i=0

    sg_log 5  "SG_Operation_Sequence array:"
    while (( i < SG_OPERATION_SEQUENCE_SIZE ))
    do
        sg_log 3 "[$i] ${SG_OPERATION_SEQUENCE[$i]}"
        (( i = i + 1 ))
    done
}

###############################################################################
#
# INTERNAL USE ONLY
#
# This function sets the aggregte exit value based on the old_exit_val.
# It looks at the current exit value, compares it to the old exit value and
# returns a new exit value based on the following matrix.  
# 
#                      old exit value
#                      0    1    2   3
#  current exit val 0  0    1    2   0
#                   1  1    1    1   1
#                   2  2    1    2   2
#                   3  16   16   16  -
#
# Arguments:  $1 : retval of the previous operation 
#             $2 : old exit value
#
# Returns:    new aggregate exit value.
#
###############################################################################
function set_exit_val
{
    if (( $1 == 0 )) &&  (( $2 == 0 ))
    then
        return 0
    elif (( $1 == 1 )) || (($2 == 1))
    then
        return 1
    elif (( $1 == 2 )) || (($2 == 2))
    then
        return 2        
# for a package to be moved to abort state, we expect the modules to return value 3
# on getting a value of 3, we indicate to the package manager with a return of 16, that the
# package has to be moved to aborted state
    else
        return 2
    fi
}


###############################################################################
#
# INTERNAL USE ONLY
#
# Executes each of the defined steps.
# 
# Input   : 
#
#       op_code :  start|stop|validate
#
# Returns : aggregate_exit_val : Cumulative exit code from all the module scripts
#                                defined in an SG_OPERATION_SEQUENCE array.
#           The possible return values are:
#
#           0  : When the execution of all the action scripts was successful.
#           1  : When one or more action scripts exited with 1
#                This indicates that the package should not be 
#                restarted as a result of the error.
#           2  : When one or more action scripts exited with 2, but there
#                were some that were successful. This indicates that the 
#                package can be restarted from another node.
#
# Globals referenced :
#           SG_OPERATION_SEQUENCE_SIZE :  set in the master_control_script.sh; 
#                                         size of the an array of the 
#                                         operation sequence
#           SG_OPERATION_SEQUENCE      :  operation sequence array which 
#                                         defines the module action scripts.
#
###############################################################################
function sg_execute_steps
{
    typeset operation
    typeset -i count=0
    typeset -i retval=0
    typeset -i aggregate_exit_val=0
    typeset -i start_success_count=0

    sg_log 5 "sg_execute_steps"  $op_code 

    # set the the flag to true if this is a DRR package
    sg_set_if_drr_package_true
    
    #
    # Log the package startup/shutdown sequence startup marker in the log file
    #
    if [[ "${op_code}" = "start" ]] 
    then
        if [[ "$SG_MODULE_SCRIPT" != "" ]]
        then
            if [[ "$SG_EXCLUDE_MODULE_SCRIPT" != "" ]]
            then
                sg_log 0 "###### Starting package $SG_PACKAGE up to $SG_MODULE_SCRIPT and excluding $SG_EXCLUDE_MODULE_SCRIPT ######"
            else
                sg_log 0 "###### Starting package $SG_PACKAGE up to $SG_MODULE_SCRIPT ######"
            fi
        else
            if [[ "$SG_EXCLUDE_MODULE_SCRIPT" != "" ]]
            then
                sg_log 0 "###### Starting package $SG_PACKAGE excluding $SG_EXCLUDE_MODULE_SCRIPT ######"
            else 
                sg_log 0 "###### Starting package $SG_PACKAGE ######"
            fi
        fi

    elif [[ "${op_code}" = "stop" ]] 
    then
        if [[ "$SG_SYMMETRIC_HALT" != "" ]]
        then
            if [[ "$SG_MODULE_SCRIPT" != "" ]]
            then
                if [[ "$SG_EXCLUDE_MODULE_SCRIPT" != "" ]]
                then
                    sg_log 0 "###### Halting package $SG_PACKAGE from $SG_MODULE_SCRIPT and excluding $SG_EXCLUDE_MODULE_SCRIPT ######"
                fi
            else 
                if [[ "$SG_EXCLUDE_MODULE_SCRIPT" != "" ]]
                then
                    sg_log 0 "###### Halting package $SG_PACKAGE excluding $SG_EXCLUDE_MODULE_SCRIPT ######"
                else 
                    sg_log 0 "###### Halting package $SG_PACKAGE from $SG_MODULE_SCRIPT ######"
                fi
            fi
        else
            sg_log 0 "###### Halting package $SG_PACKAGE ######"
        fi
    elif [[ "${op_code}" = "reconfigure" ]]
    then
        sg_log 0 "###### reconfiguring package $SG_PACKAGE ######"
    fi
    
    #
    # Prints the sequence array at SG_LOG_LEVEL 3
    #
    sg_print_sequence_array

    # stop package
    if [[ "${op_code}" = "stop" ]] 
    then 
        if [[ "$SG_PACKAGE_DETACHED" = "yes" ]] 
        then
            sg_log 0 "Begin halting detached package $SG_PACKAGE"
        fi

        if [[ "$SG_SYMMETRIC_HALT" != "" ]] 
        then 
            if [[ "$SG_MODULE_SCRIPT" != "" ]]
            then
                sg_get_module_script_index ${SG_MODULE_SCRIPT}
                count=$?
            else 
                (( count = SG_OPERATION_SEQUENCE_SIZE - 1 ))
            fi
        else 
            (( count = SG_OPERATION_SEQUENCE_SIZE - 1 ))
        fi 

        while (( count >= 0 )) 
        do
            operation=${SG_OPERATION_SEQUENCE[$count]}

            if [[ -n "${operation}" ]]
            then
                if [[ "${operation}" != "$SG_EXCLUDE_MODULE_SCRIPT" ]]
                then
                $operation $op_code >> $SG_SCRIPT_LOG_FILE
                retval=$?

                    if [[ $retval = 3  && "$SG_PACKAGE_TYPE" = "failover" ]]
                    then
                        if [[ "$SG_ABORT_HALT_ON_FAILURE" = "yes" ]]
                        then
                            sg_log 0 "Package halt aborted in module ${SG_OPERATION_SEQUENCE[$count]}"
                            (( count = count - 1 ))
                            sg_log 2 "The following modules will not be halted :"
                            while (( count >= 0 ))
                            do
                                sg_log 0 " ${SG_OPERATION_SEQUENCE[$count]}"
                                (( count = count - 1 ))
                            done
                            count=0
                            #
                            # we will be setting the value to 16, this is for PM to put the package in
                            # aborted state. If we are here it means we have to abort the package.
                            #
                            aggregate_exit_val=16
                            break
                        else
                            #
                            # we will be returning 0 here for -f options issued with cmhaltcl or cmhaltnode
                            # this return code will make sure the node or cluster will be halted even if we
                            # have a aborted pacakge
                            #
                            retval=0
                        fi
                    fi
                    if [[ $retval = 98 ]]
                    then
                        sg_set_skip_pr_modules

                        # reset the retval
                        retval=0
                    fi

                    # Aggregate the exit value
                    set_exit_val $retval $aggregate_exit_val 
                    aggregate_exit_val=$?
                fi
            fi

            #
            # Halt should move on inspite of an error condition
            # generated by the operation sequence.
            #
            (( count = count - 1 ))
        done
    elif [[ "${op_code}" = "reconfigure" ]]
    then
        process_reconfigure
        aggregate_exit_val=$?
    else  
         
        (( count = 0 ))               
        (( aggregate_exit_val = 0 ))

        #
        # File SG_SRC_FRM_CHILD used to communicate between modules
        # This file will be maintained and cleanud up in the master control
        # context
        #

        postfix=$(date +"%H.%M.%S")
        SG_SRC_FRM_CHILD="/var/tmp/child_to_parent_$postfix.$$.comm_content"
        export SG_SRC_FRM_CHILD

        #
        # This code is shared by "validate" and "start" actions.
        # Execute each action from the operation sequence. For
        # "start" operation we exit out of the loop as soon as we 
        # hit any error condition. 
        #
        while (( count < SG_OPERATION_SEQUENCE_SIZE ))
        do
            operation=${SG_OPERATION_SEQUENCE[$count]}
            if [[ -n "${operation}" ]]
            then
                start_success_count=$count
                # if SG_MODULE_SCRIPT exists, then we can only
                # start up to and including the SG_MODULE_SCRIPT.
                if [[ "${operation}" = "$SG_MODULE_SCRIPT" ]]
                then
                    count=$SG_OPERATION_SEQUENCE_SIZE
                fi
            
                if [[ "${operation}" != "$SG_EXCLUDE_MODULE_SCRIPT" ]]
                then
                    if [[ "${op_code}" = "start" ]] 
                    then
                        # For start operation, direct the output
                        # to the package log file.
                        $operation $op_code >> $SG_SCRIPT_LOG_FILE
                        retval=$?
                    else
                        # For validation operation, such as cmapplyconf, 
                        # cmcheckconf, do not direct the output to 
                        # package log file.
                        $operation $op_code
                        retval=$?
                    fi
                    if [ -f $SG_SRC_FRM_CHILD ]; then
                        # if the file exists source & delete the file
                        . ${SG_SRC_FRM_CHILD}
                        rm -f $SG_SRC_FRM_CHILD
                    fi

                    if (( $retval != 0 ))
                    then
                        if [[ "${op_code}" = "validate" ]] 
                        then
                            case $retval in
                            99)
                                # Case where VMFS module is setting for skip 
                                # of PR/VG/FS modules.
                                # Better if we check if it is VMFS modules??
                                sg_set_storage_module_skip
                            ;;
                            98)
                                # Case where this node is the eligible node to 
                                # run cmapplyconf on this node
                                sg_set_skip_pr_modules
                            ;;
                            100)
                                # Case where vmfs module failed, we will skip PR module and  
                                # continue with others and also set aggregate_value
                                sg_set_skip_pr_modules
                                sg_log 0 "ERROR: Failed to validate $operation"
                                aggregate_exit_val=1
                            ;;
                            *)
                                #
                                # Display the name of the script that
                                # failed in validation.
                                #
                                sg_log 0 "ERROR: Failed to validate $operation"
                                aggregate_exit_val=1
                            ;;
                            esac
                        else
                            #
                            # Start action script: 
                            # exit with the last failed return value
                            #
                            case $retval in
                            97)
                                # Virtual node with VMFS params are configured.
                                # But since the PREV node is PHY and failed we 
                                # need to clear PR keys from the disk
                                sg_set_pr_module_required
                            ;;
                            98)
                                # No need to run the previous node would have 
                                # cleared the keys or it was a virtual node
                                sg_set_skip_pr_modules
                            ;;
                            *)
                                aggregate_exit_val=retval
                                count=$start_success_count
                                break
                            ;;
                            esac
                        fi
                    fi
                fi
            fi
               (( count = count + 1 ))
        done

        #
        # In case of start action, in case of an error 
        # rollback operation sequence
        #
        if (( aggregate_exit_val != 0 )) 
        then
            if [[ "${op_code}" != "validate" ]] 
            then
                sg_log 0 "##### Failed to start package $SG_PACKAGE, rollback steps #####"
                #
                # Rollback the modules for which the start functions
                # were executed. Assuming that the stop functions
                # are idempotent. 
                #
                while (( count >= 0 )) 
                do
                    operation=${SG_OPERATION_SEQUENCE[$count]}
                    sg_log 5 $operation "stop"
                    $operation "stop"
                    (( count = count - 1 ))
                done
            fi
        fi
    fi

    #
    # Log the package startup/shutdown end marker in the log file
    #
    if [[ "${op_code}" = "start" ]]
    then
        sg_log_pkg_start_completion_msg
    elif [[ "${op_code}" = "stop" ]]
    then
        sg_log_pkg_stop_completion_msg
    elif [[ "${op_code}" = "reconfigure" ]]
    then
        sg_log_pkg_reconfigure_completion_msg
        rm $PREVIOUS_ENV_PATH/$SG_PACKAGE.modenv
        rm $PREVIOUS_ENV_PATH/$SG_PACKAGE.env
        rm -f $SG_SCRIPT_ONLINE_DETAILS_LOG_FILE
    fi
    
    return $aggregate_exit_val
}

######################################################################
# Source the package environment variables
#
# All configured package attributes (SG provided attributes as well
# as customer defined ones) of a UPCC package can be 
# retrieved in the form of POSIX shell variable via cmgetpkgenv 
# followed by the package name. 
#
# Input    arg1 : start|stop|validate
#          arg2 : optional argument specifies the package environment file name
#                 it is specified in case of validate. 
#
# For all operations, if a SG_ENV_FILE environment variable is already
# defined, then that's the file it source to retrieve the package attributes.
# For validation operation read pkg attributes from the file <arg2>
# For start/stop operations, for the first call get pkg attributes from
# cmgetpkgenv, store this into a tmp file for subsequent calls. 
#
# The SG_ENV_FILE variable is exported to the environment for all operations 
# to be used by sub layers of scripts. The file only exists for the duration
# of the script run. It is removed when the master control script exits.
#
# Returns : 0  (SUCCESS)
#           1  (FAILED)
#
# Globals referenced :
#             SG_PACKAGE:  Set by package manager shell while invoking
#                          the start/halt sequence
#
######################################################################
function sg_source_pkg_env
{
    typeset postfix
    typeset -i timeout

    PREVIOUS_ENV_PATH=${SGRUN}/pkgenv
    if [[ "${op_code}" = "reconfigure" ]]
    then

        #Give sometime for the package env file to be written out 
        #by Serviceguard. The value of 75 seconds is chosen as we found
        #when a large number of packages are configured it took a while
        # >60 seconds, for the file to written out

        (( timeout = 75 ))
        while (( timeout > 0 ))
        do
            [[ -f $PREVIOUS_ENV_PATH/$SG_PACKAGE.env ]] && break
            (( timeout = timeout - 1 ))
            sleep 1
        done

        if [[ -f $PREVIOUS_ENV_PATH/$SG_PACKAGE.env ]]
        then
            sed 's/^/PREV_/' $PREVIOUS_ENV_PATH/$SG_PACKAGE.env > $PREVIOUS_ENV_PATH/$SG_PACKAGE.modenv
            if [[ -f $PREVIOUS_ENV_PATH/$SG_PACKAGE.modenv ]]
            then
                # Source the $SG_PACKAGE.env file first to get the SG_<env> variables
                . $PREVIOUS_ENV_PATH/$SG_PACKAGE.env
                # Source the previous env file
                . $PREVIOUS_ENV_PATH/$SG_PACKAGE.modenv
                # Unset the SG_<env> variables
                sed -e s/^/unset\ / -e s/=.*// $PREVIOUS_ENV_PATH/$SG_PACKAGE.env > \
                $PREVIOUS_ENV_PATH/$SG_PACKAGE.unsetenv
                # entries in the file will be of the following form : unset SG_PACKAGE_NAME.
                # Now source the file for the changes
                if [[ -f $PREVIOUS_ENV_PATH/$SG_PACKAGE.unsetenv ]]
                then
                    . $PREVIOUS_ENV_PATH/$SG_PACKAGE.unsetenv
                    rm -f $PREVIOUS_ENV_PATH/$SG_PACKAGE.unsetenv
                fi 
            else
                sg_log 0 "ERROR: Modified environment file for $SG_PACKAGE \
                                   missing at $PREVIOUS_ENV_PATH/$SG_PACKAGE.modenv"
                sg_exit 1
            fi
        else
            sg_log 0 "ERROR: Environment file for $SG_PACKAGE missing at \
                                    $PREVIOUS_ENV_PATH/$SG_PACKAGE.env"
            sg_exit 1
        fi
    fi

    # if SG_ENV_FILE variable is defined, source it
    if [[ -n $SG_ENV_FILE ]]
    then
        . ${SG_ENV_FILE}
        if [[ -z $SG_PACKAGE ]]
        then
            SG_PACKAGE=$SG_PACKAGE_NAME
            export SG_PACKAGE
        fi
    
        # The following piece of code is added to obtain the PREV_ env variables
        # as part of validate code block in our module scripts.
        if [[ "${op_code}" = "validate" ]]
        then
            postfix=$(date +"%H.%M.%S")
            PREV_SG_ENV_FILE=/var/tmp/${SG_PACKAGE}.$postfix.$$.tmpvalidate
            cmgetpkgenv $SG_PACKAGE > $PREV_SG_ENV_FILE 2>&1 
            if (( $? == 0 ))
            then
                sed 's/^/PREV_/' $PREV_SG_ENV_FILE > $PREVIOUS_ENV_PATH/$SG_PACKAGE.prevenv
                . $PREVIOUS_ENV_PATH/$SG_PACKAGE.prevenv
                rm -rf $PREVIOUS_ENV_PATH/$SG_PACKAGE.prevenv
            else
                cat $PREV_SG_ENV_FILE | grep -e "package $SG_PACKAGE is not configured" \
                                        -e "$SG_PACKAGE has no environment variables defined" >/dev/null
                if (( $? != 0 ))
                then 
                    sg_log 0 "ERROR: Function sg_source_pkg_env"
                    sg_log 0 "       Unable to retrieve package attributes."
                    sg_exit 1
                fi 
            fi
            rm -rf $PREV_SG_ENV_FILE 
        fi
        if [[ -z $SG_PACKAGE_DETACHED ]]
        then
            SG_PACKAGE_DETACHED="no"
        fi
        export SG_PACKAGE_DETACHED
        return 0
    fi

    # the remainder of the function should only be exercised by
    # the master control script. Or else the sg_exit will
    # remove SG_ENV_FILE prematurely

    if (( $# != 1 )) &&  (( $# != 2 ))
    then
        sg_log 0 "ERROR: Function sg_source_pkg_env"
        sg_log 0 "       Invalid number of arguments: $#"
        sg_exit 1
    fi

    # if there is an environment file being passed in,
    # use that instead of cmgetpkgenv
    if (( $# == 2 ))
    then
       SG_ENV_FILE=$2
    else
        postfix=$(date +"%H.%M.%S")
        SG_ENV_FILE=/var/tmp/${SG_PACKAGE}.$postfix.$$.tmp

        # It is observed that during online modification under 
        # rare conditions and mostly on physical systems, 
        # cmgetpkgenv fails to list the changes that were made.
        # This is because cmproxyd is not yet updated with the
        # latest change. Fix for QXCR1001314874
        if [[ "${op_code}" = "reconfigure" ]]
        then
            sleep 3
        fi
        cmgetpkgenv $SG_PACKAGE > $SG_ENV_FILE
        if (( $? != 0 ))
        then 
            sg_log 0 "ERROR: Function sg_source_pkg_env"
            sg_log 0 "       Unable to retrieve package attributes."
            sg_exit 1
        fi 
    fi

   export SG_ENV_FILE
   . ${SG_ENV_FILE}
   if [[ -z $SG_PACKAGE ]]
   then
       SG_PACKAGE=$SG_PACKAGE_NAME
       export SG_PACKAGE
   fi
   return 0
}

###############################################################################
# This function checks the type of package. 
# If no pkg name is passed as the argument, then SG_PACKAGE is taken
# as the argument. 
# 
#
# Arguments:  $1 : package name (optional)
#
# Returns:    0 - if package is UPCC
#             1 - if package is NonUPCC
#             exits if the pkg does not exist or if the pkg is not configured.
#
###############################################################################
function sg_check_pkg_type
{
    typeset -i ret
    typeset pkg_name

    if (($# == 1))
    then
    pkg_name=$1
    else
    pkg_name=$SG_PACKAGE
    fi

    cmgetconf -p $pkg_name > /dev/null
    ret=$?
    if (( $ret == 1))
    then
    sg_log 0 "ERROR: Function sg_check_pkg_type"
    sg_log 0 "       Cannot find the package $pkg_name"
    sg_exit 1
    fi

    cmgetconf -p $pkg_name | grep -i -q "^module_name"
    ret=$?
    return $ret
}
###############################################################################
# This function checks the existence of a specific module in 
# a package ascii file. 
# If no module_name is passed as the argument, then the utility exits with an
# error.
# Assumption: $SG_PACKAGE represents the package that is checked.
# 
#
# Arguments:  $1 : module_name
#
# Returns:    0 - if module_name exists
#             1 - if module_name does not exist.
#             exits if the module_name is not passed as an argument.
#
###############################################################################
function sg_check_module_in_pkg
{
    typeset -i ret
    typeset module_name=$1

    if (($# != 1))
    then
        sg_log 0 "ERROR: Function sg_check_module_in_pkg"
        sg_log 0 "       module_name is not passed as a parameter"
        sg_exit 1
    fi

    for module in ${SG_MODULE_NAME[@]}
    do
        if [ "$module" = "$module_name" ]; then
            return 0
        fi
    done

    return 1
}

function cleanup_before_exit
{
    typeset -i _rc=$1

    # Cleanup all temporary files
    # For this process all the files will have the prefix similar to below
    # Log_30564_10_39_02_579991896
    # ^^^^^^^^  ^^^^^^^^^^^^^^^^^^
    # Log_$$    $(date +"%H_%M_%S_%N")
    rm -f $tmp_dir/$log_prefix* > /dev/null 2>&1

    #
    # Exit to indicate success/failure to the master_control_script.sh
    #
    exit $_rc
}

function cache_cmviewcl_output_lx
{
    typeset -i _retry
    typeset _cmviewcl_stderr=$tmp_dir/$log_prefix_"cmviewcl.err.$$"
    typeset _cmviewcl_command

    if [ -s $cmviewcl_output ]; then
        return 0
    fi

    _cmviewcl_command="$SGSBIN/cmviewcl -vf line -s config"

    rm -f $cmviewcl_output 2>/dev/null
    $_cmviewcl_command > $cmviewcl_output 2>/dev/null
    if [ $? -ne 0 ]; then
        # cmviewcl will retry when it encountered transient error
        _retry=$cmviewcl_retry_interval
        while [ $_retry -gt 0 ]; do
            sleep $cmviewcl_retry_interval
            rm -f $cmviewcl_output 2>/dev/null
            rm -f $_cmviewcl_stderr 2>/dev/null
            $_cmviewcl_command > $cmviewcl_output 2>$_cmviewcl_stderr
            cmviewcl_rc=$?
            if [ $cmviewcl_rc -eq 0 ]; then
                rm -f $_cmviewcl_stderr 2>/dev/null
                return 0
            elif [ $cmviewcl_rc -eq 2 ]; then
                # Return of 2 from cmviewcl says there is no cluster configured 
                rm -f $cmviewcl_output 2>/dev/null
                rm -f $_cmviewcl_stderr 2>/dev/null
                return 1 
            fi
            (( _retry = _retry - 1 ))
        done
        # Abort if retry failed
        sg_log 0 "Error: Failed to collect cluster configuration information"
        sg_log 0 "$(cat $cmviewcl_output $_cmviewcl_stderr 2>/dev/null)"
        rm -f $cmviewcl_output 2>/dev/null
        rm -f $_cmviewcl_stderr 2>/dev/null
        cleanup_before_exit 1
    fi
    return 0
}

function get_node_state_from_node_name
{
    _node_name=$1
    echo `$CMVIEWCL -f line -n $_node_name | awk -F= '/^state=/ { print $2 }'`
}

function get_node_type_from_node_name
{
    typeset _node_name=$1
    typeset prev_node_type=""

    prev_node_type=`$CMVIEWCL -vf line -s config | grep -w  $_node_name | awk -F= '/^node:[^|]*\|virt_tech=/ {print $2}'`

    echo $prev_node_type
}

function get_package_nodes
{
    typeset _package_node_name=""

    if [ "${SG_NODE_NAME[0]}" == "*" ]; then
        # All the cluster nodes
        _package_node_name=`cat "$cmviewcl_output" | awk '/^node:[^|]*\|name=/' | cut -d "=" -f 2`
    else
        # Whatever is configured
        _package_node_name=`echo "${SG_NODE_NAME[@]}"`
    fi
    echo $_package_node_name
}

function get_cmviewcl_fline_pkg_output
{
    typeset -i ret=0

    if [ -z "$cmviewcl_fline_pkg_op" ] ; then
        # First check if the package is configured:
        cat $cmviewcl_output | grep -q  ^package:$SG_PACKAGE_NAME'|'name=$SG_PACKAGE_NAME$
        if (( $? != 0 )); then
            return 1
        fi
        cmviewcl_fline_pkg_op=`$CMVIEWCL -f line -p $SG_PACKAGE_NAME`
        if [ -z "$cmviewcl_fline_pkg_op" ] ; then
            #try again
            cmviewcl_fline_pkg_op=`$CMVIEWCL -f line -p $SG_PACKAGE_NAME`
            ret=$?
            if (( $ret != 0 ))
            then
                sg_log 0 "ERROR: get_cmviewcl_fline_pkg_output: Execution of command cmviewcl failed."
                sg_log 0 "ERROR: cmviewcl output:"
                $CMVIEWCL -vf line -p $SG_PACKAGE_NAME
                cleanup_before_exit $ret
            fi
        fi
    fi
    return $ret
}

function get_running_package_nodeid
{
    typeset package_status=""
    typeset package_running_node=""
    typeset -i _node_id=0

    get_cmviewcl_fline_pkg_output
    if (( $? == 1 )); then
        SG_RUNNING_NODE_DURING_VALIDATE=0
    else
        package_status=`echo "$cmviewcl_fline_pkg_op" | awk -F= '/^status=/ { print $2 }'`
        package_running_node=`echo "$cmviewcl_fline_pkg_op" | awk -F= '/^owner=/ { print $2 }'` 

        case "$package_status" in
        'up')
            SG_RUNNING_NODE_DURING_VALIDATE=`cat $cmviewcl_output | grep -w $package_running_node | awk -F= \
                     '/^node:[^|]*\|id=/ { print $2 }'`
        ;;
        *)
            SG_RUNNING_NODE_DURING_VALIDATE=0
        ;;
        esac
    fi
}

###############################################################################
# This function takes a module script name and returns either the index
# to the operations sequence array or -1 to the caller. 
#
# This function can be used to compare if a module script is after or before
# another in the start up sequence. 
# Arguments:  $1 : module_script_name
#
# Returns:    0 or > - if module_script_name exists
#             -1 - if module_script_name does not exist.
###############################################################################
function sg_get_module_script_index
{
    typeset -i i=0
    typeset -i idx=-1
    typeset module_script_name=$1

    sg_log 5  "sg_get_module_script_index"

    if (($# != 1))
    then
    sg_log 0 "ERROR: Function sg_get_module_script_index"
    sg_log 0 "ERROR: module_script_name is not passed as a parameter"
    sg_exit 1
    fi


    (( SG_OPERATION_SEQUENCE_SIZE = ${#SG_OPERATION_SEQUENCE[*]} ))
    while (( i < SG_OPERATION_SEQUENCE_SIZE ))
    do
        if [[ "${module_script_name}" = ${SG_OPERATION_SEQUENCE[$i]} ]]
        then
             sg_log 3 "${SG_OPERATION_SEQUENCE[$i]} is index [$i]"
             idx=$i
             break
        fi
        (( i = i + 1 ))
    done

    return $idx
}

##################################################################
# This function is called during online package reconfiguration.
# For all the modules in the previous configuration it calls 
# reconfigure_stop on all the modules in the reverse order. 
# After all the modules have been processed for reconfigure_stop,
# function goes through all the modules from the current 
# configuration and checks if the module itself is added
# new. If so, this function calls the modules's 'start' if the
# module is already existing then this function calls the 
# modules's reconfigure_start. 
#
# If there is an error encountered during reconfigure_stop or
# start/reconfigure_start, the individual modules set reconfigure
# failure flag(set_reconfig_failure()).
# Once this flag is set none of the changes in the successive 
# modules will be attempted. This function will still call the
# remaining modules to log the changes that were not attempted.
#
# A new value 18(P_RECONFIG_FAIL) will be returned to PM, which
# keeps the package running.
##################################################################

function process_reconfigure
{
    typeset -i count=0
    typeset -i i=0
    typeset -i j=0
    typeset -i k=0
    typeset -i found=0
    typeset -i retval=0
    typeset -i aggregate_exit_val=0
    # P_RECONFIG_FAIL is the return value in case error is
    # encountered during reconfiguration.
    typeset -i P_RECONFIG_FAIL=18

    #Set the reconfiguration flag
    sg_set_reconfigure

    aggregate_exit_val=0
    (( i=${#PREV_SG_OPERATION_SEQUENCE[*]} - 1 ))
    while ((i >= 0))
    do
        operation=${PREV_SG_OPERATION_SEQUENCE[$i]}
        # check if the module has the changes to handle the new online
        # reconfiguration behavior in case of a failure.
        if sg_check_if_reconfig_has_failed
        then
            grep -q -e sg_check_if_reconfig_has_failed \
                 -e sg_check_if_it_is_reconfiguration $operation
            if (( $? != 0 ))
            then
                ((i=i-1))
                sg_log 999 "WARNING: The changes that are not attempted" \
                           "in the module $operation, cannot be listed."
                continue
            fi
        fi
        if [[ -n "${operation}" ]]
        then
            $operation "reconfigure_stop"
            retval=$?

            if (( $retval != 0 ))
            then
                if [[ $retval = 98 ]]
                then
                    sg_set_skip_pr_modules
                    # reset the retval
                    retval=0
                else
                    aggregate_exit_val=1
                    sg_set_reconfig_failure
                fi
            fi
        fi
        ((i=i-1))
    done
    
    i=0
    while (($i < ${#SG_OPERATION_SEQUENCE[*]}))
    do
        operation=${SG_OPERATION_SEQUENCE[$i]}
        # check if the module has the changes to handle the new online
        # reconfiguration behavior in case of a failure.
        if sg_check_if_reconfig_has_failed
        then
            grep -q -e sg_check_if_reconfig_has_failed \
                 -e sg_check_if_it_is_reconfiguration $operation
            if (( $? != 0 ))
            then
                ((i=i+1))
                sg_log 999 "WARNING: The changes that are not attempted" \
                           "in the module $operation, cannot be listed."
                continue
            fi
        fi
        if [[ -n "${operation}" ]]
        then
            #looking for module $operation
            found=0
            j=0
            while [[ $j -lt ${#PREV_SG_OPERATION_SEQUENCE[*]} ]]
            do
                if [[ $operation = ${PREV_SG_OPERATION_SEQUENCE[$j]} ]]
                then
                    found=1
                    break
                else
                    ((j=j+1))
                fi
            done
            if ((found == 0))
            then
                $operation "start"
                retval=$?
            
                if (( $retval != 0 ))
                then
                    if [[ $retval = 98 ]]
                    then
                        sg_set_skip_pr_modules
                        # reset the retval
                        retval=0
                    else
                        aggregate_exit_val=1
                        sg_set_reconfig_failure
                    fi
                fi
            else
                $operation "reconfigure_start"
                retval=$?
            
                if (( $retval != 0 ))
                then
                    if [[ $retval = 98 ]]
                    then
                        sg_set_skip_pr_modules
                        # reset the retval
                        retval=0
                    else
                        aggregate_exit_val=1
                        sg_set_reconfig_failure
                    fi
                fi
            fi
        fi
        ((i=i+1))
    done

    if sg_check_if_reconfig_has_failed
    then
        return $P_RECONFIG_FAIL
    fi
    return $aggregate_exit_val
}
######################################################################
#
# This function is used by toolkit pacakges to send alert notification 
# emails. Recipients of the mail will be picked up from the email_id
# attribute of the package ascii file. If the value for email_id 
# attribute is not specified in package ascii file, email will not be 
# sent.
#
# Arguments:  $1 :  toolkit defined subject line (optional) 
#             $2 :  File with toolkit defined mail contents(optional) 
#
# If toolkit defined message needs to be appended to subject line, use
#         sg_send_mail $SUBJECT
#                where $SUBJECT is not more than 210 chars. 
#                 
# If only default subject line is to be sent but the mail contents
# need to be defined, use
#         sg_send_mail "" $FILE_NAME
#     
# Globals referenced : ${SG_E_MAIL[*]} - The array containing the
#                          email address(es) mentioned in package
#                          ascii file attribute email_id
#
# Returns : none
#
######################################################################

function sg_send_mail 
{
    # Initialize the required parameters 
    
    typeset ALERT_SUBJECT="Serviceguard Alert: ${1}"
    typeset ALERT_MAIL_IDS=${SG_EMAIL_ID[*]}
    typeset MAILFILE=${2}
    typeset HOST=`hostname`
    typeset SUBJ_LEN=${#ALERT_SUBJECT}
    typeset CLUSTER_NAME=`$SGSBIN/cmviewcl -s config -f line | grep name | head -n 1 | cut -f 2 -d =`
    typeset -i MIN_CH_LIMIT=1
    typeset -i MAX_CH_LIMIT=$((2044-$SUBJ_LEN))
    
    typeset TMP="`mktemp`"
    trap "rm $TMP 2> /dev/null;exit" 0 2 3
    
    # If there is no one on the "ALERT_MAIL_IDS" list then do nothing
    if [ -z "$ALERT_MAIL_IDS" ];then
       return
    fi

    if [ $MAX_CH_LIMIT -lt ${#ALERT_MAIL_IDS} ]; then
        sg_log 0 "WARNING: Lenth of all email_id values exceeds the maximum "
        sg_log 0 "number of allowed characters which is $MAX_CH_LIMIT. Email "
        sg_log 0 "notification may not be received by all recipients. "
        sg_log 0 "Use a alias/distribution list to correct the issue. "
        ALERT_MAIL_IDS=`echo $ALERT_MAIL_IDS | cut -c$MIN_CH_LIMIT-$MAX_CH_LIMIT`
        # Remove, if any, stray characters of last email ID in the ALERT_MAIL_IDS
        ALERT_MAIL_IDS=${ALERT_MAIL_IDS% *}
    fi      
    
    if [ -f "$MAILFILE" ];then 
        STATUS_CMD="cat $MAILFILE"
    else 
        STATUS_CMD="" 
    fi 
    cat <<=EOD= > $TMP

Hi,

There seems to be an issue in the package $SG_PACKAGE in your Serviceguard cluster. 
For more information, check the package and system logs of $HOST. 

   Time of failure        : `date`
   Cluster Name           : $CLUSTER_NAME
   Node name              : $HOST
   Location of package log: ${SG_SCRIPT_LOG_FILE}
   
   `$STATUS_CMD`

Thanks
=EOD=

    cat $TMP | mailx -s "$ALERT_SUBJECT" $ALERT_MAIL_IDS
    sg_log 0 "Serviceguard Alert message sent to: $ALERT_MAIL_IDS"   
}

###############################################################################
# Function sg_validate_atx does validation for ATX.
#
# It checks for -
#                1. Existence of ATX software
#                2. Valid ATX license
#
# Arguments: no argument
#
# Returns:    0 - if all checks succeeds
#             1 - if any of the check fails
###############################################################################

function sg_validate_atx
{
    sg_log 5 "sg_validate_atx"

    #Check whether ATX is installed or not
    if [ ! -f /usr/bin/hpe-atx ];then
        sg_log 0 "ERROR: ATX is not Installed."
        return 1;
    else
        # ATX is installed, now check for valid license
        # ATX license command "/usr/bin/hpe-atx -s" returns:
        #             0 - for valid license
        #             1 - for invalid/expired license

        /usr/bin/hpe-atx -s  > /dev/null 2>&1
        if [ $? == 0 ];then
            sg_log 5 "ATX Validation is Successful."
            return 0;
        else
            sg_log 0 "ERROR: ATX License is Invalid."
            sg_log 0 "ERROR: ATX Validation Failed."
            return 1;
        fi
    fi
}

