####################################################################
# (C) Copyright 2013 Hewlett-Packard Development Company, L.P.
# @(#) Serviceguard cluster verification script
# @(#) Product Name              :  HP Serviceguard
# @(#) Product Version           :  A.12.10.00
# @(#) Patch Name                :  
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HP Serviceguard.
#
# Changing this file may lead to failures during Cluster verification
#
#####################################################################
. ${SGCONFFILE:=/etc/cmcluster.conf}
. ${SGUTILS:=$SGCONF/scripts/cluster_master_script_utils.sh}

typeset env_file="/var/tmp/master_cluster_script.env"
typeset -i consolidate_call

#################################################################
# This function checks if NTP server is running on the local node
#################################################################

function is_ntpd_running
{
     val=`ps -f --ppid 1 | grep -w ntpd`
     if [[ $? -eq 0 ]]; then
         return 0
     fi
     return 1
}

#########################################################
# This function checks whether NTP server is configured
# on the local node
#########################################################

function sg_validate_ntp_server
{
    found=0
    is_ntpd_running
    if [[ $? -eq 0 ]];then
        val=`/usr/sbin/ntpq -p -n| sed '1,2d' | cut -d "" -f1 | awk -F" " '{print $1}'`
        if [ -n "$val" ];then
            found=1
        fi 
        sg_log_cv 0 "NTP_SERVER=$found"
        sg_log_cv 1 "NTP_SERVER=$found"
    else
        sg_log_cv 1 "NTP Daemon is not running"
        sg_log_cv 0 "NTP_SERVER=0" 
    fi
}


###################################################
# This function validates whether all the nodes
# in the cluster have NTP server configured
###################################################

function sg_consolidate_ntp
{
   . $env_file
   typeset -i num_nodes=${#SG_NODES[*]}
   typeset -i ind=0
   typeset -i index=0
   declare -a unavail_nodes

   while (( $index < $num_nodes ))
   do
       is_ntp_running="SG_${SG_NODES[$index]}_NTP_SERVER"
       if (( $is_ntp_running == 0 )); then
           unavail_nodes[$ind]=${SG_NODES[$index]}
           (( ind = ind + 1 ))
       fi
       (( index = index + 1 ))
   done
   if (( $num_nodes > 0 )); then
       if [ ${#unavail_nodes[@]} -ne 0 ]; then
           sg_log_cv 0 "WARNING: NTP Server does not seem to be configured on node(s): ${unavail_nodes[@]}"
           sg_log_cv 0 "Applications that require time synchronization among nodes may not work properly" 
       fi
   fi
}

#############################
#     MAIN ROUTINE
#############################

typeset logfile=""
typeset -i debug=0

HOST=$(uname -n | cut -d . -f 1)
typeset exit_val=0

while getopts "o:O:D" opt;do
case "$opt" in
    o)
        if [[ $OPTARG == "consolidate" ]]; then
            consolidate_call=1
        fi
        ;;
    D)
        debug=1
        export "SG_LOG_DEBUG=1"
        ;;
    O)
        logfile=$OPTARG
        export "SG_LOG_FILE=$logfile"
        ;;
    *)
        echo "INFO: Unknown operation - $1"
        exit_val=1
        ;;
esac
done

if (( consolidate_call == 1 )); then
    sg_consolidate_ntp
else
    sg_validate_ntp_server

fi
exit $exit_val

