####################################################################
# (C) Copyright 2017 Hewlett-Packard Development Company, L.P.
# @(#) Serviceguard UPCC Package Configuration Script.
# @(#) Product Name                :  HP Serviceguard
# @(#) Product Version             :  A.12.10.00
# @(#) Patch Name                  :  
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HP Serviceguard.
#
# Changing this file may lead to unrecoverable package startup
# and/or shutdown problems.
#
# **********************************************************************
# *                                                                    *
# *  This script contains the functions for handling optional package  * 
# *  external script execution during package start and halt time.     *
# *  It allows user to add one or more external programs before        *
# *  step VOLUME_GROUPS.                                               *
# *  For more details about the external scripts and the parameters,   *    
# *  please look at the module listing external_pre.1.                 *
# *                                                                    *
# **********************************************************************
#
###################################################################

###########################
# Source utility functions. 
###########################

if [[ -f ${SG_UTILS} ]]; then
    . ${SG_UTILS}
    if (( $? != 0 ))
    then
        echo "ERROR: Unable to source package utility functions file: ${SG_UTILS}"
        exit 1
    fi
else
    echo "ERROR: Unable to find package utility functions file: ${SG_UTILS}"
    exit 1
fi

###################################################################
#
# Get the environment for this package through utility function
# sg_source_pkg_env().
#
###################################################################

sg_source_pkg_env $*

#############################################################################
#
# unconfigure_storage() is called while halting the drr package
#
#############################################################################

function unconfigure_storage
{
    _pri_pkg_name=$1
    CMD="/usr/bin/python /opt/drenabler/bin/cmdrrprepstorage.py"

    $CMD STOP "$_pri_pkg_name" "$SG_REHEARSAL_ID" 0 $SGRUN/DRxR/$SG_REHEARSAL_ID/.unconf_error_file_$SG_PACKAGE
    retval=$?
    if (( $retval != 0 ))
    then
        sg_log 0 "ERROR: Failed to remove the storage configured."
        while IFS= read -r line
        do
            sg_log 0 "`echo $line | awk -F'|' '{print $3}'`"
        done < "$SGRUN/DRxR/$SG_REHEARSAL_ID/.unconf_error_file_$SG_PACKAGE"
        rm -f $SGRUN/DRxR/$SG_REHEARSAL_ID/.unconf_error_file_$SG_PACKAGE
        return 1
    fi
}


###################################################################
#
# This function will add or remove tags for vgs.
#
###################################################################
function delete_hosttag
{
    typeset vg=$1
    typeset tag=$2
    typeset result

    sg_log 5 "delete_hosttag"
  
    result=$(vgchange --deltag $tag $vg 2>&1)
    case $result in
        *Volume*group*successfully*changed*)
        ;;
        *Volume*group*does*not*support*tags*)
            sg_log 0 "ERROR: Function delete_hosttag:"
            sg_log 0 "VG $vg does not support hosttags."
            ret=1
        ;;
        *)
            sg_log 0 "ERROR: Function delete_hosttag:"
            sg_log 0 "Failed to remove hosttag \"$tag\" on VG $vg"
            sg_log 0 $result
            ret=1
        ;;
    esac
    return $ret
}

#############################################################################
#
# sg_start_drr_module() is called while starting a package (cmrunpkg)
#
# Globals Referenced defined in main() :
#
#############################################################################

function sg_start_drr_module
{
    typeset -i fs=0
    typeset -i num_fs=0
    sg_log 5 "Deactivate LV's, Delete hosttags if any, Deactivate VGS "\
             "and set to skip PR module since this is a rehearsal package"

    num_fs=${#SG_FS_NAME[*]}

    # assign mount points into temp array
    while (( $fs < $num_fs ))
    do
        lvchange -a n ${SG_FS_NAME[$fs]} > /dev/null 2>&1
        (( fs = fs + 1 ))
    done
    
    for I in ${SG_VG[@]}
    do
        result=$(vgdisplay $I 2>&1)

        # always return 0 status, so check for output of command
        case $result in
            *Volume*group*not*found*)
                sg_log 0 "ERROR: Function sg_start_drr_module:"
                sg_log 0 "The volume group \"$I\" is missing."
                ret=1
                continue
                ;;
        esac

        hosttags=`vgs --noheadings -o +tags $I | awk  '{print $8}'`
        for tag in $hosttags
        do
            delete_hosttag $I $tag
        done

        vgchange -a n $I 2>&1
    done

    if (( $ret != 0 ))
    then
        cleanup_before_exit $ret
    fi

    # skip the PR module by exiting with 98 
    cleanup_before_exit $set_pr_module_skip
}

#############################################################################
#
# sg_stop_drr_module() is called while stopping a package (cmhaltpkg) 
# or during rollback operation to recover from the package startup.
#
# Globals Referenced defined in main() :
#
#############################################################################

function sg_stop_drr_module
{
    CMVIEWCL="cmviewcl -vf line -s config"
    CMVIEWCL_PKG="cmviewcl -f line -p "
    typeset -i retval=0
    # For a rehearsal which is in ABORTED/ABORTING state, node switching has to 
    # be enabled. These values are defined in h/cmrehearsald_yo.h
    # RHL_STATUS_ABORTING=0x40, RHL_STATUS_ABORTED=0x80
    # The combined value for these two statuses is 192. 
    typeset -i abrt_status=192
    typeset nodes_list=""
    typeset resource_name="genres_rehearsal_$SG_REHEARSAL_ID"

    # get the primary package ID from DRR package. [DRxR_15361]
    pri_pkg_id=`echo "$SG_PACKAGE_NAME" | cut -d '_' -f  2 | tr -d '[:space:]'`

    # get the primary package name using the package ID. [package:pkg1|id=15361]
    pri_pkg_name=`$CMVIEWCL | awk '/^package:[^|]*\|id='$pri_pkg_id'/' | sed 's/|id='${pri_pkg_id}'//;s/package://'`

    sg_log 5 "Remove the storage configured for the rehearsal package $SG_PACKAGE"
    unconfigure_storage $pri_pkg_name
    retval=$?
    if (( $retval != 0 ))
    then
        sg_log 0 "ERROR: Unable to remove the storage configured for rehearsal package $SG_PACKAGE."
        ret=1
        return
    else
        sg_log 5 "Enable the node switching for the primary package on target nodes"
        _R=0
        while (($_R < ${#SG_ENABLED_TARGET_NODES[*]}))
        do
            nodes_list="$nodes_list${SG_ENABLED_TARGET_NODES[$_R]} "
            (( _R = _R + 1 ))
        done

        # we are checking the global switching of the primary package. This is case
        # when the user has done a halt of the primary package. 
        # when the global switching is disabled we do not want to enable the 
        # node switching of the primary package. Typically for cmhaltcl kind of scenario.

        read global_switching_flag _pkg_state <<< `$CMVIEWCL_PKG $pri_pkg_name | awk -F= '
            /^autorun=/             { autorun = $2 }
            /^state=/               { state = $2 }
            END {
                if (length(autorun) == 0)
                    autorun = "NA"
                if (length(state) == 0)
                    state = "NA"

            printf("%s %s\n", autorun, state);
        }'`

        overall_rh_status=`$SGSBIN/cmviewrehearsal -n | grep rh_overall_status |cut -d : -f 2`
        if [[ $overall_rh_status != "" ]]
        then
            cur_rh_status=`/usr/bin/perl -e "print ($overall_rh_status & $abrt_status)"`
        fi
        if [[ $global_switching_flag != "NA" ]]
        then
            if [[ "$global_switching_flag" == "enabled" ]] && \
                            [[ $_pkg_state != "halted"  || $cur_rh_status != 0 ]]
            then
                nodes_list=`echo $nodes_list | sed 's/ / -n /g'`
                cmmodpkg -r -e -n $nodes_list $pri_pkg_name 
                retval=$?
                if (( $retval != 0 ))
                then
                    sg_log 0 "ERROR: Failed to enable node switching for $pri_pkg_name on ${SG_ENABLED_TARGET_NODES[$_R]}"
                    ret=1
                fi
            fi
        fi
        $SGSBIN/cmsetresource -r $resource_name -s down
    fi
}


################
# main routine
################
typeset -i ret=0
#-----------------------------------------------------------------------------------------------#
# return value              | used in op code   | used to                                       #
#---------------------------|-------------------|-----------------------------------------------#
# set_storage_module_skip   | validate          | Set to skip PR/VG/FS(partially) by the nodes  #
#                           |                   | cmapplyconf/cmcheckconf                       #
#---------------------------|-------------------|-----------------------------------------------#
# set_pr_module_skip        | start             | Set to skip PR module by the eligible node    #
#-----------------------------------------------------------------------------------------------#
typeset -i set_pr_module_skip=98
typeset -i set_storage_module_skip=99

sg_log 5 "drr_module"

#########################################################################
#
# External pre_script must be specified with three required entry points:
# start, stop, and validate.
#
#########################################################################

case ${1} in
    start)
          sg_start_drr_module
          ;;
    stop)
          sg_stop_drr_module
          ;;
     validate)
           # Let us skip PR module if it is a DRR package
           cleanup_before_exit $set_storage_module_skip
           ;;
    reconfigure_stop)
           # No Op
           ;;
    reconfigure_start)
           # No Op
           ;;
    *)
          sg_log 0 "INFO: Unknown operation: $1"
          ;;
esac

#
# Exit to indicate success/failure to the master_control_script.sh
#
exit $ret
