####################################################################
# (C) Copyright 2006-2013 Hewlett-Packard Development Company, L.P.
# @(#) Serviceguard UPCC Package Configuration Script.
# @(#) Product Name                :  HP Serviceguard
# @(#) Product Version             :  A.12.10.00
# @(#) Patch Name                  :  
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HP Serviceguard.
#
# Changing this file may lead to unrecoverable package startup
# and/or shutdown problems.
#
# **********************************************************************
# *                                                                    *
# *  This script contains the functions for handling optional package  *
# *  external script execution during package start and halt time.     *
# *  It allows user to add one or more external programs between       *
# *  step IP_ADDRESSES and SERVICES. It replaces the existing          *
# *  "customer_defined_run_cmds()" and "customer_defined_halt_cmds()"  *
# *  functions in the existing control script.                         *
# *  For more details about the external scripts and the parameters,   *
# *  please look at the module listing external.1.                     *
# *                                                                    *
# **********************************************************************
#
###################################################################

###########################
# Source utility functions. 
###########################

if [[ -f ${SG_UTILS} ]]; then
    . ${SG_UTILS}
    if (( $? != 0 ))
    then
        echo "ERROR: Unable to source package utility functions file: ${SG_UTILS}"
        exit 1
    fi
else
    echo "ERROR: Unable to find package utility functions file: ${SG_UTILS}"
    exit 1
fi

###################################################################
#
# Get the environment for this package through utility function
# sg_source_pkg_env().
#
###################################################################

sg_source_pkg_env $*

##########################################################################
# 
# sg_validate_external_script() is called while validating the package 
# configuration (through cmcheckconf/cmapplyconf)
#
# Globals Referenced defined in main() :  
#                       len     :  Size of an SG_EXTERNAL_SCRIPT array
#                       ext_aggregate_exit_val :  
#                                  Aggregate exit code to indicate success 
#                                  or failure to the master_control_script.sh
#
##########################################################################

function sg_validate_external_script
{

    typeset -i count=0
    typeset entry

    sg_log 5 "sg_validate_external_script"

    while (( count < len ))
    do
        entry=${SG_EXTERNAL_SCRIPT[$count]}
        
        $entry  "validate"  $SG_ENV_FILE
        
        if (( $? != 0 ))
        then
            sg_log 0 "ERROR: Failed to validate $entry"
            ext_aggregate_exit_val=1
        fi
        (( count = count + 1 ))
    done
}

# This function is called during online package
# reconfiguration. 
# 
#  This function attempts to stop all the removed
#  external scripts from the previous configuration.
#  It will loop through all the external scripts for 
#  each of the external script from the previous 
#  configuration. If the script is not found in the
#  current configuration this function runs stop
#  operation on the script
# 
 
function sg_reconfigure_stop_external_script
{
    #Index to scan through all the external scripts from
    # previous configuration
    typeset -i count=0  

    typeset -i retval=0

    # Number of external scripts from the previous configuration
    typeset -i len=0

    #Index to scan through all the external scripts from
    # current configuration
    typeset -i j=0
    typeset -i found=0

    sg_log 5 "Reconfigure: stopping removed external scripts"

    ((len=${#PREV_SG_EXTERNAL_SCRIPT[@]}))

    while (( count < len ))
    do
        found=0
        j=0
        while [[ j -lt ${#SG_EXTERNAL_SCRIPT[@]} ]]
        do
            if [[ ${SG_EXTERNAL_SCRIPT[$j]} = \
                  ${PREV_SG_EXTERNAL_SCRIPT[$count]} ]]
            then
                sg_log 5 "Found ${PREV_SG_EXTERNAL_SCRIPT[$count]}"
                found=1
                break
            else
                ((j = j+1))
            fi
        done

        if ((found==0))
        then
            if sg_check_if_reconfig_has_failed
            then
                sg_log 999 "WARNING: Not attempting to stop external script "${PREV_SG_EXTERNAL_SCRIPT[$count]}
                ext_aggregate_exit_val=1
            else
                sg_log 0 "Stopping ${PREV_SG_EXTERNAL_SCRIPT[$count]}"
                ${PREV_SG_EXTERNAL_SCRIPT[$count]} "stop"
                retval=$?

                if (( $retval != 0 ))
                then
                    sg_log 0 "ERROR: Failed to stop external script ${PREV_SG_EXTERNAL_SCRIPT[$count]}"
                    sg_log 999 "ERROR: Failed to stop external script ${PREV_SG_EXTERNAL_SCRIPT[$count]}"
                    ext_aggregate_exit_val=1
                    sg_set_reconfig_failure
                else
                    sg_log 999 "SUCCESS: Successfully stopped external script ${PREV_SG_EXTERNAL_SCRIPT[$count]}"
                fi
            fi
        fi
        (( count = count + 1 ))
    done
}

# This function is called during online package
#  reconfiguration. 
# 
# This function attempts to start all the newly added 
#  external scripts from the previous configuration.
#  It will loop through all the external scripts from the
#  previous configuratio for 
#  each of the external script from the current 
#  configuration. If the script is not found in the
#  previous configuration this function runs start
#  operation on the script

function sg_reconfigure_start_external_script
{
    typeset -i count=0
    typeset -i retval=0
    typeset -i j=0
    typeset -i found=0
    typeset -i len=0

    sg_log 5 "Reconfigure: Starting newly added external scripts"

    ((len=${#SG_EXTERNAL_SCRIPT[@]}))
    while (( count < len ))
    do
        found=0
        j=0
        while [[ j -lt ${#PREV_SG_EXTERNAL_SCRIPT[@]} ]]
        do
            if [[ ${SG_EXTERNAL_SCRIPT[$count]} = \
                  ${PREV_SG_EXTERNAL_SCRIPT[$j]} ]]
            then
                sg_log 5 "Found ${SG_EXTERNAL_SCRIPT[$count]}"
                found=1
                break
            else
                ((j=j+1))
            fi
        done

        if ((found==0))
        then
            if sg_check_if_reconfig_has_failed
            then
                sg_log 999 "WARNING: Not attempting to start external script "${SG_EXTERNAL_SCRIPT[$count]}
                ext_aggregate_exit_val=1
            else
                sg_log 0 "Starting ${SG_EXTERNAL_SCRIPT[$count]}"
                ${SG_EXTERNAL_SCRIPT[$count]} "start"
                retval=$?

                if (( $retval != 0 ))
                then
                    sg_log 0 "ERROR: Failed to start external script ${SG_EXTERNAL_SCRIPT[$count]}"
                    sg_log 999 "ERROR: Failed to start external script ${SG_EXTERNAL_SCRIPT[$count]}"
                    ext_aggregate_exit_val=1
                    sg_set_reconfig_failure
                else
                    sg_log 999 "SUCCESS: Successfully started external script ${SG_EXTERNAL_SCRIPT[$count]}"
                fi
            fi
        fi
        (( count = count + 1 ))
    done
}
##########################################################################
# 
# sg_start_external_script() is called while starting a package (cmrunpkg)
#
# Globals Referenced defined in main() :  
#                       len     :  Size of an SG_EXTERNAL_SCRIPT array
#                       ext_aggregate_exit_val :  
#                                  Aggregate exit code to indicate success 
#                                  or failure to the master_control_script.sh
#
##########################################################################

function sg_start_external_script
{
    typeset -i count=0
    typeset -i retval=0
    typeset entry

    sg_log 5 "Starting External Scripts"

    while (( count < len ))
    do
        entry=${SG_EXTERNAL_SCRIPT[$count]}
        if sg_check_if_reconfig_has_failed
        then
            sg_log 999 "WARNING: Not attempting to start external script "${SG_EXTERNAL_SCRIPT[$count]}
            ext_aggregate_exit_val=1
        else
            $entry  "start"
            retval=$?
            
            if (( $retval != 0 ))
            then
                sg_log 0 "ERROR: Failed to start $entry"
                if sg_check_if_it_is_reconfiguration
                then
                    sg_log 999 "ERROR: Failed to start external script $entry"
                    ext_aggregate_exit_val=1
                    sg_set_reconfig_failure
                else
                    #
                    # Aggregate the exit value
                    #
                    set_exit_val $retval $ext_aggregate_exit_val
                    ext_aggregate_exit_val=$?
                fi
            else
                sg_log 999 "SUCCESS: Successfully started external script ${SG_EXTERNAL_SCRIPT[$count]}"
            fi
        fi
        (( count = count + 1 ))
    done
}

##########################################################################
# 
# sg_stop_external_script() is called while stopping a package (cmhaltpkg)
# or during rollback operation to recover from the package startup.
#
# Globals Referenced defined in main() :  
#                       len     :  Size of an SG_EXTERNAL_SCRIPT array
#                       ext_aggregate_exit_val :  
#                                  Aggregate exit code to indicate success 
#                                  or failure to the master_control_script.sh
#
##########################################################################

function sg_stop_external_script
{
    typeset -i count
    typeset -i retval=0
    typeset entry

    sg_log 5 "Stopping External Scripts"


    (( count = len - 1 )) 

    while (( count > -1  ))
    do
        entry=${SG_EXTERNAL_SCRIPT[$count]}
        $entry "stop"
        retval=$?
        
        # external scripts must be idempotent, that is if the halt action 
        # has already been performed successfully before, 
        # it should not generate an error, but exit successfully.
        
        if (( $retval != 0 ))
        then
            sg_log 0 "ERROR: Failed to stop $entry"
            #
            # Aggregate the exit value
            #
            set_exit_val $retval $ext_aggregate_exit_val
            ext_aggregate_exit_val=$?
        fi
        (( count = count - 1 ))
    done
}

################
# main routine
################



# Set len to the number of scripts defined in an array 
#
typeset -i len
len=${#SG_EXTERNAL_SCRIPT[*]}


#
# The variable ext_aggregate_exit_val indicates a cumulative success or failure
# of the all the scripts defined in an SG_EXTERNAL_SCRIPT array when executed
# as a part of an entry point.
#
typeset -i ext_aggregate_exit_val=0

sg_log 5 "sg_external_script"

#########################################################################
#
# External script must be specified with three required entry points: 
# start, stop, and validate. 
#
#########################################################################

case ${1} in
     start)
           sg_start_external_script
           ;;
     stop)
           sg_stop_external_script
           ;;

     validate)
           sg_validate_external_script
           ;;

     reconfigure_start)
	   sg_reconfigure_start_external_script
	   ;;

     reconfigure_stop)
	   sg_reconfigure_stop_external_script
	   ;;

     *)    
           sg_log 0 "INFO: Unknown operation: $1"
           ;;
esac

#
# Exit to indicate success/failure to the master_control_script.sh
#
exit $ext_aggregate_exit_val
