####################################################################
# (C) Copyright 2006-2015 Hewlett-Packard Development Company, L.P.
# @(#) Serviceguard UPCC Package Configuration Script.
# @(#) Product Name                :  HP Serviceguard
# @(#) Product Version             :  A.12.10.00
# @(#) Patch Name                  :  
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HP Serviceguard.
#
# Changing this file may lead to unrecoverable package startup
# and/or shutdown problems.
#
###################################################################

###########################
# Source utility functions.
###########################

if [[ -f ${SG_UTILS} ]]; then
    . ${SG_UTILS}
    if (( $? != 0 ))
    then
        echo "ERROR: Unable to source package utility functions file: ${SG_UTILS}"
        exit 1
    fi
else
    echo "ERROR: Unable to find package utility functions file: ${SG_UTILS}"
    exit 1
fi

###################################################################
#
# Get the environment for this package through utility function
# sg_source_pkg_env().
#
###################################################################

sg_source_pkg_env $*

##########################################################################
#
# create_ip_subnet_list() is called by sg_add_ip_address(),
# and sg_remove_ip_address(). It creates
# LOCAL_IP_SUBNET[] that consists of SG_IP_SUBNET available on the local
# host and are defined in package configuration file along with
# SG_IP_SUBNET_NODE.
#
##########################################################################
function create_ip_subnet_list
{

    typeset -i J=0
    typeset -i K=0
    typeset -i L=0
    typeset -i i=0
    typeset -i found=0

    sg_log 5 "create_ip_subnet_list"

    host=$1

    if [[ $2 = "new" ]]
    then
        while (( $i < ${#SG_IP_SUBNET[@]} ))
        do
            TMP_SG_IP_SUBNET[$i]=${SG_IP_SUBNET[$i]}
            TMP_SG_IP_SUBNET_NODE[$i]=${SG_IP_SUBNET_NODE[$i]}
            (( i = i + 1 ))
        done
    else
        while (( $i < ${#PREV_SG_IP_SUBNET[@]} ))
        do
            TMP_SG_IP_SUBNET[$i]=${PREV_SG_IP_SUBNET[$i]}
            TMP_SG_IP_SUBNET_NODE[$i]=${PREV_SG_IP_SUBNET_NODE[$i]}
            (( i = i + 1 ))
        done
    fi

    # Create a new LOCAL_IP_SUBNET ARRAY that consists of all SUBNETS that
    # this node has configured.

    for I in ${TMP_SG_IP_SUBNET[@]}
    do
        found=0
        if [[ -n "${TMP_SG_IP_SUBNET_NODE[$J]}" ]]; then

	    # TMP_SUBNET_NODE[] array has all SUBNETS that has
	    # TMP_SG_IP_SUBNET_NODE associated with it.

	    TMP_SUBNET_NODE[$L]=$I
	    (( L = L + 1 ))

            if [[ "${TMP_SG_IP_SUBNET_NODE[$J]}" = $host ]]
            then
                #Check if TMP_LOCAL_IP_SUBNET has already
                #the subnet
                for J1 in ${TMP_LOCAL_IP_SUBNET[@]}
                do
                    if [[ $I = $J1 ]]; then
                        found=1
                        break
                    fi
                done
                if (( found == 0 ))
                then
                    TMP_LOCAL_IP_SUBNET[$K]=$I
                    (( K = K + 1))
                fi
            fi
        fi
        (( J = J + 1))
    done
}

##########################################################################
#
# add_to_ip_subnet_array(): this function adds all those subnets to
# LOCAL_IP_SUBNET[] that do not have subnet_node defined.
#
##########################################################################
function add_to_ip_subnet_array
{

    typeset -i K=0
    typeset -i found=0
    typeset -i i=0

    sg_log 5 "add_to_ip_subnet_array"

    if [[ $1 = "new" ]]
    then
        while (( $i < ${#SG_IP_SUBNET[@]} ))
        do
            TMP_SG_IP_SUBNET[$i]=${SG_IP_SUBNET[$i]}
            TMP_SG_IP_SUBNET_NODE[$i]=${SG_IP_SUBNET_NODE[$i]}
            (( i = i + 1 ))
        done
    else
        while (( $i < ${#PREV_SG_IP_SUBNET[@]} ))
        do
            TMP_SG_IP_SUBNET[$i]=${PREV_SG_IP_SUBNET[$i]}
            TMP_SG_IP_SUBNET_NODE[$i]=${PREV_SG_IP_SUBNET_NODE[$i]}
            (( i = i + 1 ))
        done
    fi

    K=${#TMP_LOCAL_IP_SUBNET[*]}

    # Now add those subnets that have no subnet_node defined

    for I in ${TMP_SG_IP_SUBNET[@]}
    do
        found=0
        for M in ${TMP_SUBNET_NODE[@]}
        do
            if [[ $I = $M ]]; then
                found=1
                break
            fi
        done   
        if (( found != 1 )); then
            for J in ${TMP_LOCAL_IP_SUBNET[@]}
            do
                if [[ $I = $J ]]; then
                    found=1
                    break
                fi
            done
        fi
        if (( found != 1 )); then
            TMP_LOCAL_IP_SUBNET[$K]=$I
            (( K = K + 1 ))
        fi
    done   
}

##########################################################################
#
# sg_validate_ip_subnet() is called while validating the package
# configuration (through cmcheckconf/cmapplyconf)
#
##########################################################################

function sg_validate_ip_subnet
{
    typeset -i S=0
    typeset -i retval=0
    typeset host
    typeset -i no_of_ips=${#SG_IP_ADDRESS[*]}

    sg_log 5 "sg_validate_ip_subnet"
    if [[ $no_of_ips != 0 ]]
    then 
        sg_log 0 "Verifying if the IP addresses could be added to corresponding subnets"
    fi
    S=0
    host=${SG_HOST_NAME%%.*}
    
    create_ip_subnet_list $host "new"
    add_to_ip_subnet_array "new"

    declare -a LOCAL_IP_SUBNET=(${TMP_LOCAL_IP_SUBNET[@]})

    unset TMP_SUBNET_NODE
    unset TMP_LOCAL_IP_SUBNET
    
    for I in ${SG_IP_SUBNET[@]}
    do
        if [[ -n "${SG_IP_ADDRESS[$S]}" ]]
        then
            for J in ${LOCAL_IP_SUBNET[@]}
            do
                if [[ $J = $I ]]
                then

                    sg_log 1 "Checking IP address ${SG_IP_ADDRESS[$S]} to subnet $I"
                    XX=$(cmmodnet -t -i ${SG_IP_ADDRESS[$S]} $I 2>&1)
                    retval=$?

                    if (( retval != 0 ))
                    then
                        #If a package is applied in offline case, cmnetd wouldn't be
                        # running, then we cannot do the validation
                        if [[ $(echo $XX | grep "start the daemon on this node") != "" ]]
                        then
                            sg_log 0 "NOTE: Validation of adding an IP address to a subnet" \
                                     "cannot be done when the cluster is offline."
                            sg_log 0 "$XX"
                        else
                            sg_log 1 "$XX"
                            if [[ $(echo $XX | grep "is already configured") = "" ]]
                            then
                                sg_log 0 "ERROR: ${SG_IP_ADDRESS[$S]} cannot be added to subnet $I"
                                to_exit=1
                            else
                                sg_log 1 "${SG_IP_ADDRESS[$S]} is already configured"
                            fi
                        fi
                    fi
                    break
                fi
            done #All the current local subnets
        fi #if -n ${SG_IP_ADDRESS[$S]}
        ((S=S+1))
    done #All the subnets
    return
}

function reconfigure_start_ip_addresses
{
    typeset -i S=0
    typeset -i local_found=0
    typeset -i old_conf_found=0
    typeset -i retval=0

    sg_log 5 "reconfigure_start_ip_addresses"

    host=${SG_HOST_NAME%%.*}
    unset TMP_SUBNET_NODE
    unset TMP_LOCAL_IP_SUBNET

    create_ip_subnet_list $host "new"
    add_to_ip_subnet_array "new"

    declare -a CURR_LOCAL_IP_SUBNET=(${TMP_LOCAL_IP_SUBNET[@]})

    unset TMP_SUBNET_NODE
    unset TMP_LOCAL_IP_SUBNET

    create_ip_subnet_list $host "old"
    add_to_ip_subnet_array "old"

    declare -a PREV_LOCAL_IP_SUBNET=(${TMP_LOCAL_IP_SUBNET[@]})

    unset TMP_SUBNET_NODE
    unset TMP_LOCAL_IP_SUBNET

    for I in ${SG_IP_SUBNET[@]}
    do
        if [[ -n "${SG_IP_ADDRESS[$S]}" ]]
        then
            local_found=0
            for J in ${CURR_LOCAL_IP_SUBNET[@]}
            do
                if [[ $J = $I ]]
                then
                    local_found=1
                    old_conf_found=0

                    #Check if subnet exists in the previous configuration
                    for K in ${PREV_LOCAL_IP_SUBNET[@]}
                    do
                        if [[ $J = $K ]]
                        then
                            P=0
                            for P in ${PREV_SG_IP_ADDRESS[@]}
                            do
                                if [[ $P = ${SG_IP_ADDRESS[$S]} ]]
                                then
                                    sg_log 5 "found IP Address ${SG_IP_ADDRESS[$S]}" \
                                             " in the old configuration"
                                    #The ip address for the local node
                                    #is present in the old config also
                                    old_conf_found=1
                                    break 3;
                                fi #match for Previous IP Address
                            done #Previous IP addreses
                        fi #if subnet exist in the previous configuration 
                    done #For all the previous local ip subnets
                fi #current Local subnets
            done #All the current local subnets

            #If not found in the old configuration
            #it is a new ip address, so add it
            if ((old_conf_found==0 && local_found==1))
            then
                if sg_check_if_reconfig_has_failed
                then
                    sg_log 999 "WARNING: Not attempting to add IP ${SG_IP_ADDRESS[$S]} to subnet $I"
                    to_exit=1
                else
                    sg_add_ip_address ${SG_IP_ADDRESS[$S]} $I
                    if sg_check_if_reconfig_has_failed
                    then
                        sg_log 999 "ERROR: Failed to add IP ${SG_IP_ADDRESS[$S]} to subnet $I"
                    else
                        sg_log 999 "SUCCESS: Successfully added IP ${SG_IP_ADDRESS[$S]} to subnet $I"
                    fi
                fi
            fi
        fi #if -n ${SG_IP_ADDRESS[$S]}
        ((S=S+1))
    done #All the subnets
}

##########################################################################
#
# sg_add_ip_addresses() is called while starting a package (cmrunpkg)
#
# For each {IP address/subnet} pair, add the IP address to the subnet
# using cmmodnet(1m).
#
##########################################################################

function sg_add_ip_addresses
{
    typeset -i S=0
    typeset -i retval=0
    typeset -i found=0

    sg_log 5 "sg_add_ip_addresses"

    # Create a new LOCAL_IP_SUBNET[] that consists of all SUBNETS that are
    # defined in pkg configuration file along with IP_SUBNET_NODE set
    # as local host.
    host=${SG_HOST_NAME%%.*}

    create_ip_subnet_list $host "new"

    # Now add all those subnets that are defined on all nodes of the
    # cluster, which are specified as LOCAL_IP_SUBNET without any
    # IP_SUBNET_NODE.

    add_to_ip_subnet_array "new"
   
    #declare -a SUBNET_NODE=(${TMP_SUBNET_NODE[@]})
    declare -a LOCAL_IP_SUBNET=(${TMP_LOCAL_IP_SUBNET[@]})

    unset TMP_SUBNET_NODE
    unset TMP_LOCAL_IP_SUBNET

    # Now add only the ip_address for the subnet available on this node.

    for I in ${SG_IP_SUBNET[@]}
    do
        if [[ -n "${SG_IP_ADDRESS[$S]}" ]]; then

	    # Add the IP_ADDRESS only if its SUBNET is in LOCAL_IP_SUBNET array
            found=0
            for J in ${LOCAL_IP_SUBNET[@]}
            do
                if [[ $J = $I ]]; then
		    found=1
                fi
            done

	    if (( found != 0 ))
	    then
            if sg_check_if_reconfig_has_failed
            then
                sg_log 999 "WARNING: Not attempting to add IP ${SG_IP_ADDRESS[$S]} to subnet $I"
                to_exit=1
            else
                sg_add_ip_address ${SG_IP_ADDRESS[$S]} $I
                if sg_check_if_reconfig_has_failed
                then
                    sg_log 999 "ERROR: Failed to add IP ${SG_IP_ADDRESS[$S]} to subnet $I"
                elif sg_check_if_it_is_reconfiguration
                then
                    sg_log 999 "SUCCESS: Successfully added IP ${SG_IP_ADDRESS[$S]} to subnet $I"
                fi
            fi
	    fi
	fi
        (( S = S + 1 ))
    done
}

function sg_add_ip_address
{
    
    sg_log 0 "Adding IP address $1 to subnet $2"

    XX=$(cmmodnet -a -i $1 $2 >> $SG_SCRIPT_LOG_FILE 2>&1)
    retval=$?

    if (( retval != 0 ))
    then
        if [[ -n "$(echo $XX | grep "heartbeat IP")" ]]
        then
            # IP has been configured as a heartbeat IP address.
            sg_log 0 "$XX" 
            to_exit=1
        else
            IPADDR=$1
            # checking the existence of IPv4 ip address
            YY=$( ip -o -4 addr show | awk -F '[ /]+' '$4 == "'$IPADDR'"')
            if [[ -z "${YY}" ]]
            then
                # checking the existence of IPv6 ip address
                YY=$( ip -o -6 addr show | awk -F '[ /]+' '$4 == "'$IPADDR'"')
                if [[ -z "${YY}" ]]
                then
                    if sg_check_if_it_is_reconfiguration
                    then
                        sg_set_reconfig_failure
                    fi
                    sg_log 0 "$XX" 
                    sg_log 0 "ERROR: Function sg_add_ip_address"
                    sg_log 0 "ERROR: Failed to add IP $1 to subnet $2"
                    to_exit=1
                fi
            else
                sg_log 0 "WARNING: Function sg_add_ip_address"
                sg_log 0 "         Package_ip - IP $1 is already configured on the subnet $2"
            fi
        fi
    fi
}

function sg_remove_ip_address
{
    typeset -i S=0

    sg_log 0 "Removing IP address $1 from subnet $2"
    XX=$(cmmodnet -r -i $1 $2 >> $3 2>&1)
    retval=$?

    #
    # NOTE: sg_remove_ip_address action is NOT idempotent.
    # We will be able to be idempotent when cmmodnet command 
    # returns a specific exit value that the halt script can 
    # use to recognize the error that the specified IP does
    # not exist on the system. 
    #

    if (( retval != 0 ))
    then
        if [[ $(echo $XX | 
        grep "is not configured on the subnet") != "" ]]
        then
            if sg_check_if_it_is_reconfiguration
            then
                sg_set_reconfig_failure
            fi
            sg_log 0 "$XX" 
            sg_log 0 "ERROR: Function sg_remove_ip_address"
            sg_log 0 "ERROR: Failed to remove IP address ${SG_IP_ADDRESS[$S]} from subnet $I"
            to_exit=1
        fi
    fi


}


#######################################################################
#
# sg_remove_ip_addresses() is called while stopping a package (cmhaltpkg)
# or during rollback operation to recover from the package startup.
#
# For each IP address/subnet pair, remove the IP address from the subnet
# using cmmodnet(1m).
#
#######################################################################

function sg_remove_ip_addresses
{
    typeset -i retval=0

    sg_log 5 "sg_remove_ip_address"

    # Create a new LOCAL_IP_SUBNET[] that consists of all SUBNETS that are
    # defined in pkg configuration file along with IP_SUBNET_NODE set
    # as local host.

    host=${SG_HOST_NAME%%.*}
    create_ip_subnet_list $host "new"

    # Now add all those subnets that are defined on all nodes of the
    # cluster, which are specified as LOCAL_IP_SUBNET without any
    # IP_SUBNET_NODE.

    add_to_ip_subnet_array "new"
   
    #declare -a SUBNET_NODE=(${TMP_SUBNET_NODE[@]})
    declare -a LOCAL_IP_SUBNET=(${TMP_LOCAL_IP_SUBNET[@]})

    unset TMP_SUBNET_NODE
    unset TMP_LOCAL_IP_SUBNET

    for I in ${SG_IP_SUBNET[@]}
    do
        if [[ -n "${SG_IP_ADDRESS[$S]}" ]]; then

	    # Add the IP_ADDRESS only if its SUBNET is in LOCAL_IP_SUBNET array
            found=0
            for J in ${LOCAL_IP_SUBNET[@]}
            do
                if [[ $J = $I ]]; then
		    found=1
                fi
            done

	    if (( found != 0 ))
	    then
                sg_remove_ip_address ${SG_IP_ADDRESS[$S]} $I $SG_SCRIPT_LOG_FILE
            fi
        fi
        (( S = S + 1 ))
    done
}

function reconfigure_stop_ip_addresses
{
    typeset -i retval=0
    typeset -i S=0
    typeset -i found=0
    typeset -i local_found=0
    typeset -i new_conf_found=0

    sg_log 5 "reconfigure_stop_ip_addresses"

    host=${SG_HOST_NAME%%.*}
    unset TMP_SUBNET_NODE
    unset TMP_LOCAL_IP_SUBNET

    create_ip_subnet_list $host "old"
    add_to_ip_subnet_array "old"

    declare -a PREV_LOCAL_IP_SUBNET=(${TMP_LOCAL_IP_SUBNET[@]})

    unset TMP_SUBNET_NODE
    unset TMP_LOCAL_IP_SUBNET

    create_ip_subnet_list $host "new"
    add_to_ip_subnet_array "new"

    declare -a CURR_LOCAL_IP_SUBNET=(${TMP_LOCAL_IP_SUBNET[@]})

    unset TMP_SUBNET_NODE
    unset TMP_LOCAL_IP_SUBNET

    for I in ${PREV_SG_IP_SUBNET[@]}
    do
        if [[ -n "${PREV_SG_IP_ADDRESS[$S]}" ]]
        then
            local_found=0
            for J in ${PREV_LOCAL_IP_SUBNET[@]}
            do
                if [[ $J = $I ]]
                then
                    local_found=1
                    new_conf_found=0
                    #Check if subnet exists in the current configuration
                    for K in ${CURR_LOCAL_IP_SUBNET[@]}
                    do
                        if [[ $K = $J ]]
                        then
                            P=0
                            for P in ${SG_IP_ADDRESS[@]}
                            do
                                if [[ $P = ${PREV_SG_IP_ADDRESS[$S]} ]]
                                then
                                    sg_log 5 "found IP Address ${SG_IP_ADDRESS[$S]}" \
                                             " in the new configuration"
                                    #The ip address for the local node
                                    #is present in the new config also
                                    new_conf_found=1
                                    break 3;
                                fi #match for Current IP Address
                            done #Current IP addreses
                        fi #If subnet exists in current configuration
                    done #Current local subnets
                fi #Local prev subnet match
            done #For all the previous local subnets

            #If not found in the old configuration
            #it is a new ip address, so add it
            if ((new_conf_found==0 && local_found==1))
            then
                sg_log 0 "Removing IP address ${PREV_SG_IP_ADDRESS[$S]} from subnet $I"
                if sg_check_if_reconfig_has_failed
                then
                    sg_log 999 "WARNING: Not attempting to remove IP ${PREV_SG_IP_ADDRESS[$S]} from subnet $I"
                    to_exit=1
                else
                    sg_remove_ip_address ${PREV_SG_IP_ADDRESS[$S]} $I $PREV_SG_SCRIPT_LOG_FILE
                    if sg_check_if_reconfig_has_failed
                    then
                        sg_log 999 "ERROR: Failed to remove IP ${PREV_SG_IP_ADDRESS[$S]} from subnet $I"
                        to_exit=1
                    else
                        sg_log 999 "SUCCESS: Successfully removed IP ${PREV_SG_IP_ADDRESS[$S]} from subnet $I"
                    fi
                fi
            fi
        fi #if -n ${PREV_SG_IP_ADDRESS[$S]}
        ((S=S+1))
    done
}

################
# main routine
################

#
# Module script must be specified with three required entry points:
# start, stop, and validate.
#
# The variable to_exit indicates the success or failure of the entry 
# point.
#

typeset -i to_exit=0

sg_log 5 "sg_ip_addresses"

case ${1} in
     start)      
             sg_add_ip_addresses
             ;;

     stop)       
           if [[ "$SG_PACKAGE_DETACHED" = "yes" ]]
           then
              sg_log 5 "Calling cmnetd_remove_detached_ips -D $SG_LOG_LEVEL" \
                       "-O $SG_SCRIPT_LOG_FILE $SG_PACKAGE 2>>$SG_SCRIPT_LOG_FILE"
              cmnetd_remove_detached_ips -D $SG_LOG_LEVEL  \
                  -O $SG_SCRIPT_LOG_FILE $SG_PACKAGE 2>>$SG_SCRIPT_LOG_FILE
              retval=$?
              if (( retval != 0 ))
              then
                 sg_log 0 "ERROR: Function cmnetd_remove_detached_ips: Failed"
                 to_exit=1
              fi
           else
              sg_remove_ip_addresses
           fi
           ;;
     validate)   
             sg_validate_ip_subnet
	     ;;

     reconfigure_stop)
           reconfigure_stop_ip_addresses
           ;;

     reconfigure_start)
           reconfigure_start_ip_addresses
           ;;

     *)
             sg_log 0 "INFO: Unknown operation - $1"
             ;;
esac

#
# Exit to indicate success/failure to the master_control_script.sh
#
exit $to_exit
