####################################################################
# (C) Copyright 2006-2014 Hewlett-Packard Development Company, L.P.
# @(#) Serviceguard UPCC Package Configuration Script.
# @(#) Product Name                :  HP Serviceguard
# @(#) Product Version            :  A.12.10.00
# @(#) Patch Name                  :  
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HP Serviceguard.
#
# Changing this file may lead to unrecoverable package startup
# and/or shutdown problems.
#

# Source in /etc/cmcluster.conf to get the Serviceguard path environment
# variables.

. ${SGCONFFILE:=/etc/cmcluster.conf}


###########################
# Source utility functions.
###########################

if [[ -f ${SG_UTILS} ]]; then
    . ${SG_UTILS}
    if (( $? != 0 ))
    then
        echo "ERROR: Unable to source package utility functions file: ${SG_UTILS}"
        exit 1
    fi
else
    echo "ERROR: Unable to find package utility functions file: ${SG_UTILS}"
    exit 1
fi

###################################################################
#
# Get the environment for this package through utility function
# sg_source_pkg_env().
#
###################################################################

sg_source_pkg_env $*

###########################################################################
# This function put persistent reservation and release reservation from luns
# which are being used by the package.
#
# This function has one parameter passed to it, depending on that this will 
# either put reservation or release reservation from the lun. 
#
# - start - to indicate the package is starting up
# - stop  - to indicate the package is shutting down
#
###########################################################################
function persistent_reservation
{
  typeset -i retval=0
  typeset act=$1
  typeset RESERVATION_SET="0"
  typeset VM_NODE="0"
  typeset OS_TYPE=""
  typeset lun=""
  typeset vol=""

  sg_log 5 "persistent_reservation"

  
  if [[ -f $SGCONFFILE ]]; then
     if [[ "$SG_VIRTUAL_NODE_TYPE" == "vmware" ]]; then 
        VM_NODE="1"
     fi
  fi

  for vol in ${SG_VG[@]}
  do
     for lun in ` vgdisplay -v $vol  2> /dev/null| grep 'PV Name' | awk '{ print $NF }' `
     do
	#Check if reservation exists on LUN
	sg_persist -r $lun | grep 'NO reservation' >> /dev/null 
        if (( $? != 0 )); then
	   RESERVATION_SET="1"
        fi		
       

	# Check if node is VM or if a scsi reservation is already present
	if [ "$VM_NODE" -eq "1" -o  "$RESERVATION_SET" -eq "1" ]; then
	   
	   if [[ "$act" == "start" ]]; then
           	# Ignore previous registration key if exists
		# and put new key.
	   	sg_persist --out --register-ignore --param-sark=1000 $lun >> /dev/null 
           	if (( $? == 0 )); then
		   # Clear registration and reservation
              	   sg_persist --out --clear --param-rk=1000 $lun  >> /dev/null
                   if (( $? == 0 )); then
		      # Register the LUN
                      sg_persist --out --register --param-sark=1000 $lun  >> /dev/null
                      if (( $? == 0 )); then
			 # reserve the LUN
                         sg_persist --out --reserve --param-rk=1000 --prout-type=1 $lun >> /dev/null
                         if (( $? != 0 )); then
                              sg_log 0 "Failed to reserve LUN: $lun"
                              retval=1
                         else
                              sg_log 0 "LUN: $lun Reservation Successful"
                         fi
                      else 
                         sg_log 0 "Failed to register LUN: $lun"
                         retval=1
                      fi
                   else 
                      sg_log 0 "Failed to clear reservation on LUN: $lun"
                      retval=1
                   fi
                else
                    sg_log 0 "Failed to register after ignoring existing reservation on LUN: $lun"
                    retval=1
                fi

            elif [[ "$act" == "stop" ]]; then
	       # Clear registration and reservation
               sg_persist --out --clear --param-rk=1000 $lun >> /dev/null
               if (( $? != 0 )); then
                  sg_log 0 "Failed to release LUN: $lun"
                  retval=1
               fi
            fi
        fi

	#Reset Reservation flag 
	RESERVATION_SET="0"
      done
   done

return $retval
}

##########################################################################
#
# sg_validate_command() is called while validating the package 
# configuration (through cmcheckconf/cmapplyconf)
#
##########################################################################

function sg_validate_command
{
    sg_log 5 "sg_validate_command"

    # no op
    return
}

##########################################################################
#
# sg_start_command() is called while starting a package (cmrunpkg)
#
##########################################################################

function sg_start_command
{
   sg_log 5 "sg_start_command"

   # check if vmware guest support is activated 
   if [[ $SG_SGLX_VM == "on" ]]
   then

      #check if sg_persist command exists 
      if [[ -x /usr/bin/sg_persist ]]
      then 
         #check the file system . if it is not gfs then only persistent reservation will be 
         # called
         if [[  "$SG_FS_TYPE" != "gfs" ]];then
            persistent_reservation $1
            if (( $? != 0 ));then
              to_exit=1
            fi
          fi
       else 
         sg_log 0 "ERROR: Function persist_reservation"
         sg_log 0 "       /usr/bin/sg_persist command not found"
         to_exit=1
      fi
   fi

 
}

##########################################################################
#
# sg_stop_command() is called while stopping a package (cmhaltpkg)
# or during rollback operation to recover from the package startup.
#
##########################################################################

function sg_stop_command
{
    sg_log 5 "sg_stop_command"

   # check if vmware guest support is activated 
   if [[ $SG_SGLX_VM == "on" ]]
   then

      # check if sg_persist command exists  
      if [[ -x /usr/bin/sg_persist ]]
      then
        #check the file system . if it is not gfs then only persistent reservation will be
        #called
        if [[  "$SG_FS_TYPE" != "gfs" ]];then
          persistent_reservation $1
          if (( $? != 0 ));then
            to_exit=1
          fi
        fi
      else 
         sg_log 0  "ERROR: Function persist_reservation"
         sg_log 0  "       /usr/bin/sg_persist command not found"    
         to_exit=1
      fi
   fi
}

################
# main routine
################

#
# Module script must be specified with three required entry points:
# start, stop, and validate.
#
# The variable to_exit indicates the success or failure of the entry 
# point.
#

typeset -i to_exit=0

sg_log 5 "sg_ha_applications"

case ${1} in
     start) 
          sg_start_command $1
          ;;
     stop)
          sg_stop_command $1
          ;;

     validate)
          sg_validate_command $1 $2
          ;;

     *)
          sg_log 0 "Unknown entry point $1"
          ;;
esac

#
# Exit to indicate success/failure to the master_control_script.sh
#
exit $to_exit
