###########################################################################
# (C) Copyright 2016-2017 Hewlett Packard Enterprise Development, LP.
# (C) Copyright 2008-2016 Hewlett Packard Enterprise Development LP.
# @(#) Serviceguard UPCC Package Configuration Script.
# @(#) Product Name                :  HP Serviceguard
# @(#) Product Version             :  A.12.10.00
# @(#) Patch Name                  :  
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HP Serviceguard.
#
# Changing this file may lead to unrecoverable package startup
# and/or shutdown problems.
#
############################################################################

############################################################################
# NAME:    pr_util
#
# SYNOPSIS:
#    pr_util [ start | stop | validate | reconfig | failed ] 
#            -k <NodePrKey> -p <package_name>
#
#    start       Execute package startup and perform Persistent reservations
#    stop        Execute package stop or failed and delete the PR key
#                from the underlying storage devices of the package.
#    validate    Execute a validate check to validate that the storage 
#                devices support Persistent Reservations.
#    reconfig    Execute a reconfig to see if any new VGs have been 
#                added or underlying LUNs added to existing VGs
#    failed      Called from MNP daemon when an instance of a package fails
#                on another node.
#    -k <NodePrKey> The node PR key to use in the persistent reservations. 
#                   For -start and -halt this would be the PR key of the
#                   the current node.  For failed it would be the PR key
#                   of the failed node.
#    -p <pkg_name>  The name of the package.  The script will use this
#                   name to extract the packages volume groups
#
# DESCRIPTION:
# Called by the master control script (indirectly via pr_cntl.sh script)
# during start or stop to set or revoke the persistent reservations 
# of the shared Storage devices accessed by the package applications.
# It is also called by the MNP Daemon to set or revoke the persistent 
# reservations of the shared Storage devices accessed by the package 
# applications. 
#
# Following 5 action (cmd) are specified. 
#
# start - Indicating that the package is starting up. The node PR key
#         and volume groups will be extracted from package environment.
#         A PR reservation type of "Write Exclusive Registrants Only" 
#         will be made to each underlying storage device.
# validate - The validation check is being done.  The volume groups
#            will be extracted from the package environment and the
#            underlying storage devices validated.
# stop     - The package is being halted.
# failed   _ An instance of the package has failed on a remote node
#            and the MNP daemon called the script to revoke the stale
#            reservations. 
#
# Validate usage:
# Called by cmapplyconf to validate the storage devices
#
#    pr_util validate 
#
# Called by MCS script during package start up
#
#    pr_util start
#
# Called by UPCC script during package shutdown
#
#    pr_util stop  
#
# Called by MNP Daemon to remove failed node PR registrations. Note
# that both the package name and stale key are passed in.  More than
# one stale key value can be passed in.  Each must be prefixed by -k.
#
#    pr_util failed -p <package_name> -k <remove_pr_key>
#
############################################################################# 
CUT="/usr/bin/cut"
PGREP="/usr/bin/pgrep"
if [ -f /etc/redhat-release ]
then  
    LOGGER="/usr/bin/logger" 
    GREP="/bin/grep"
    FGREP="/bin/fgrep"
else   
   LOGGER="/bin/logger"
   GREP="/usr/bin/grep" 
   FGREP="/usr/bin/fgrep"
fi

RH7_VERSION3="7.3"

if [ -f /etc/redhat-release ]
then
   rh7_version=`cat /etc/redhat-release | awk '{print $7}'`
   var=$(awk 'BEGIN{ print "'$rh7_version'"<"'$RH7_VERSION3'" }')
   if [ "$var" -eq 1 ];then
      MATCHING_STRING="unknown device"
   else
      MATCHING_STRING="unknown"
   fi
else
   MATCHING_STRING="unknown device"
fi

########################################################################
# node_pr_keys for all nodes
########################################################################

NODE_KEYS=`cmviewcl -l node -f line -s config 2> /dev/null | grep node_pr_key | \
           cut -d = -f 2` > /dev/null 2>&1
########################################################################
# Function to check if each registered key is one of the node_pr keys.
########################################################################
function validate_reg_keys
{
    KEYS=$*
    FOUND=""
    
    for KEY in $KEYS
    do
        KEY=`echo $KEY | sed -e 's/^0x//'`
        FOUND=`echo $NODE_KEYS | grep -w $KEY` > /dev/null 2>&1
        if [ "$FOUND" = "" ]
        then
            return 1
        fi
    done
    return 0
}

########################################################################
# Function that reads and returns the registration keys for a PV.
########################################################################
function read_registration
{
    PV=$1
    KEYS=""
    TMP=`$SG_PERSIST_RDKEYS $PV 2>/dev/null`
    if [ $? -ne 0 ]
    then
        echo "ERROR"
        return
    else 
        if [[ $TMP = *"NO registered"* ]]
        then
            KEYS="NONE"
        return
        else
            TMP=${TMP##*follow*:}
            KEYS=$TMP
        fi
    fi
    KEYS=`echo $KEYS | tr ' ' '\n' | sort | uniq | tr '\n' ' '`
    echo $KEYS
}

OP="-N"
GET_KEY=0
STALE_KEYS=""
MY_KEY=""
GET_PKG_NAME=0
PKG_NAME=""
IO_DISK_ERROR=0

##
## Aliases for sg_persist command
##
PR_TYPE_WERO="--prout-type=5"
ABORT_KEY="--param-sark"

SG_PERSIST_RDKEYS="sg_persist -k"
SG_PERSIST_RDRESV="sg_persist -r"
SG_PERSIST_REG="sg_persist --out -G --param-sark"
SG_PERSIST_REG_IGN="sg_persist --out -I --param-sark"
SG_PERSIST_UNREG="sg_persist --out -G --param-rk"
SG_PERSIST_RESV="sg_persist --out -R --param-rk"
SG_PERSIST_PREEMPT="sg_persist --out -A --param-rk"
SG_PERSIST_CLEAR="sg_persist --out -C --param-rk"
DM_DETAILS="dmsetup info"
MPATH_LIST="multipath -ll"
CM_CHKDISK="${SGSBIN}/cmcheckdisk"
UDEV_INFO="udevadm info -q name -n"

test -f /etc/redhat-release && \
if /bin/grep "Tikanga" /etc/redhat-release >/dev/null 2>&1		
then		
     UDEV_INFO="udevinfo -q name -n"
elif /bin/grep "Maipo" /etc/redhat-release >/dev/null 2>&1
then
    OS_Version="RH7"
fi

DEVICE_MAPPER_PATTERN="-e s/_part[0-9]+$// -e s/-part[0-9]+$// -e s/p[0-9]+$//"

########################################################################
# Source utility functions.
# If we are running within the context of the MSC then SG_UTILS will be
# set otherwise if we are being called from the MNP Daemon we will need
# to set it up. This will be done in the sg_node_env_setup() function 
# that will be called after the parameters have been proceesed. 
########################################################################

if [[ -f ${SG_UTILS} ]]; 
then
    . ${SG_UTILS}
    if (( $? != 0 ))
    then
        echo "ERROR: Unable to source package utility functions file: ${SG_UTILS}"
        exit 1
    fi
###################################################################
#
# Get the environment for this package through utility function
# sg_source_pkg_env().
#
###################################################################

    sg_source_pkg_env $*
fi
#####################################################################
# Failed node environment setup
# In the event that we are called by an MNP because of membership
# change we will not have the environment established that we would
# if we were called by the MCS. In this case we have setup our
# own.
#####################################################################
function sg_node_env_setup
{
    # Must have been called from the context of the MNP Daemon
    # lets get serviceguard configuration parameters
    . ${SGCONFFILE:=/etc/cmcluster.conf}

    PATH=$SGSBIN:/usr/bin:/usr/sbin:/etc:/bin:/sbin
    export PATH

    # and then try to pick up the utility functions
    SG_UTILS=${SGCONF}/scripts/mscripts/utils.sh
    . ${SG_UTILS}
    if [ $? -ne 0 ]    
    then
        echo "ERROR: Unable to source package utility functions file: ${SG_UTILS}"
        exit 1
    fi
}
#####################################################################
# Package Start Function
# Lets register the node PR KEY with each device in the list and if
# successful make the reservation. For failover package we just go
# ahead and make the reservation, but for MNP we first check to see
# if there already is a reservation before attempting to make it.
# Note the type is a WriteExclusiveRegistrantsOnly (5) since
# not all devices support the preferred WriteExclusiveAllRegistrants.
#####################################################################
function sg_activate_pr
{
    typeset -i ONE_PATH
    typeset -i fail=0
    typeset -i already_reserved=0
    typeset -i to_exit=0
    for DEV in $DEV_LIST 
    do

        if sg_check_if_reconfig_has_failed
        then
            sg_log 999 "WARNING: Not attempting to activate PR on device "$I
            fail=1
            continue
        fi

        # Let's check to see if this is a failover package and if so are
        # there any stale reservations.  If there are we need to do a 
        # register and clear of these before we proceed.
        
        if [ "$SG_PACKAGE_TYPE" = "failover" ]
        then
            REG_KEYS=$(read_registration $DEV)
            
            if [ "$REG_KEYS" != "NONE" ] 
            then
                #validate all the registered keys are the node_pr_keys, 
                #else it is a stale VG
                if [ "$REG_KEYS" != "ERROR" ] 
                then
                    validate_reg_keys $REG_KEYS 
                    if [ $? -ne 0 ] 
                    then
                        sg_log 0 "ERROR: sg_activate_pr : Physical Volume" $DEV "used in the "
                        sg_log 0 "Volume Group has registration which is not from any of the cluster nodes."
                        sg_log 0 "Use pr_cleanup utility to clear the keys."
                        if sg_check_if_it_is_reconfiguration
                        then
                            sg_log 999 "ERROR: Physical Volume $DEV used in the Volume group has foreign key registration"
                            sg_set_reconfig_failure
                            continue
                        else
                            exit 1
                        fi
                    fi
                fi
                TMP_DEV=$DEV
                check_if_multipath $DEV
                if [ $? -eq 0 ]
                then
                    get_dm_paths $DEV single
                    DEV=$DM_DEV_LIST
                fi 
                $SG_PERSIST_REG=$MY_KEY $DEV > /dev/null 2>&1
                RET=$?
                if [ $RET -ne 0 ]
                then
                    $SG_PERSIST_REG_IGN=$MY_KEY $DEV > /dev/null 2>&1
                    RET=$?
                fi
                if [ $RET -eq 0 ]
                then
                    $SG_PERSIST_CLEAR=$MY_KEY $DEV > /dev/null 2>&1
                    RET=$?
                fi
                DEV=$TMP_DEV
            fi    
        fi

        if sg_check_if_pr_module_required; then
            if [ $RET -ne 0 ]
            then
                sg_log 0 "ERROR: The package was running on a node which failed to clear the PR keys"
                sg_log 0 "ERROR: Since the package is configured with VMFS disks, cannot proceed further. "\
                         "Ensure to clear the keys on the disk before restarting the package"
                to_exit=1
            fi
            continue
        else
            # Checks whether we are working with multipath device.
            PATH_TO_RESERVE=""
            already_reserved=0
            check_if_multipath $DEV
            if [ $? -eq 0 ]
            then
                sg_log 0 "sg_activate_pr: Starting PR operation on multipath device "$DEV
                ONE_PATH=0
                get_dm_paths $DEV all 
                # Create registrations for each of the path.
                for DM_DEV in $DM_DEV_LIST
                do
                    $SG_PERSIST_REG=$MY_KEY $DM_DEV > /dev/null 
                    RET=$?
                    if [ $RET -ne 0 ]
                    then
                        sg_log 0 "WARNING: sg_activate_pr: registration failed on disk $DM_DEV, retrying"
                        $SG_PERSIST_REG_IGN=$MY_KEY $DM_DEV > /dev/null 
                        RET=$?
                    fi
                    # Go ahead and reserve it, if registration is successful on at least one path
                    if [ $RET -eq 0 ]
                    then
                        ONE_PATH=1
                        # Create a list of paths, for which registration is successful.
                        PATH_TO_RESERVE=$PATH_TO_RESERVE" "$DM_DEV
                    else
                        sg_log 0 "ERROR: sg_activate_pr: registration failed again on disk $DM_DEV"
                    fi
                done
            else 
                $SG_PERSIST_REG=$MY_KEY $DEV > /dev/null 
                RET=$?
                if [ $RET -ne 0 ]
                then
                    sg_log 0 "WARNING: sg_activate_pr: registration failed on disk $DEV, retrying"
                    $SG_PERSIST_REG_IGN=$MY_KEY $DEV > /dev/null 
                    RET=$?
                    if [ $RET -ne 0 ]
                    then
                        sg_log 0 "ERROR: sg_activate_pr: registration failed again on disk $DEV"
                    else
                        PATH_TO_RESERVE=$PATH_TO_RESERVE" "$DEV
                    fi
                else
                    PATH_TO_RESERVE=$PATH_TO_RESERVE" "$DEV 
                fi
            fi

            if [[ $RET -eq 0 || $ONE_PATH -eq 1 ]]
            then
                for DEV in $PATH_TO_RESERVE
                do
                    if [ $already_reserved -eq 0 ]; then
                        if [ "$SG_PACKAGE_TYPE" = "failover" ]
                        then
                            $SG_PERSIST_RESV=$MY_KEY $PR_TYPE_WERO $DEV > /dev/null 
                            if [ $? -ne 0 ]
                            then
                                sg_log 0 "WARNING: sg_activate_pr : Unable to make reservation on the disk $DEV"
                                if sg_check_if_it_is_reconfiguration
                                then
                                    sg_log 999 "ERROR: Failed to make reservation on the disk $DEV"
                                    sg_set_reconfig_failure
                                    continue
                                fi
                            else
                                already_reserved=1
                            fi
                        else
                            $SG_PERSIST_RDRESV $DEV | fgrep "NO reservation" > /dev/null 2>&1
                            if [ $? -eq 0 ]
                            then
                                $SG_PERSIST_RESV=$MY_KEY $PR_TYPE_WERO $DEV > /dev/null 
                                if [ $? -ne 0 ]; then
                                    sg_log 0 "WARNING: sg_activate_pr : Unable to make reservation on the disk $DEV"
                                else
                                    already_reserved=1
                                fi
                            else
                                already_reserved=1
                            fi
                        fi
                        if [ $already_reserved -eq 1 ]; then
                            sg_log 0 "sg_activate_pr: activating PR on "$DEV
                            if sg_check_if_it_is_reconfiguration
                            then
                                sg_log 999 "SUCCESS: Successfully activated PR on "$DEV
                            fi
                        fi
                    fi
                done
                if [ $already_reserved -eq 0 ] ; then
                    if ! sg_check_if_it_is_reconfiguration
                    then
                        sg_log 0 "ERROR: Unable to make reservation on any of the paths"
                        exit 1
                    fi
                fi
            else
                sg_log 0 "sg_activate_pr: activating PR FAILED on "$DEV
                if sg_check_if_it_is_reconfiguration
                then
                    sg_log 999 "ERROR: activating PR FAILED on "$DEV
                    sg_set_reconfig_failure
                    continue
                else
                    exit 1
                fi
            fi
        fi
    done

    if [ $to_exit -eq 1 ]; then 
        if sg_check_if_pr_module_required; then
            exit 1
        fi
    fi
    if [ $fail -eq 1 ] && sg_check_if_reconfig_has_failed
    then
        exit 1
    else
        exit 0
    fi
}
#####################################################
# Package Stop 
# Lets unregister the node PR KEY with each device in
# the list. If this is a failover package lets do a
# clear to ensure that the device is cleared of any
# PR registrations and reservation .
# Otherwise lets just unregister.
#####################################################
function sg_deactivate_pr
{
    typeset -i RET
    typeset -i maxTries=3
    typeset -i trys=0
    typeset -i fail=0
    for DEV in $DEV_LIST
    do
        if sg_check_if_reconfig_has_failed
        then
            sg_log 999 "WARNING: Not attempting to deactivate PR on device "$I
            fail=1
            continue
        fi

        (( RET = 1 ))
        (( trys = 0 ))
        sg_log 0 "sg_deactivate_pr: deactivating PR on "$DEV
        while (( RET != 0  &&  trys < maxTries )) ; do
            if [ "$SG_PACKAGE_TYPE" = "failover" ]
            then
                $SG_PERSIST_RDRESV $DEV | fgrep "NO reservation" > /dev/null 2>&1
                RET=$?
                if [ $RET -ne 0 ]
                then
                    $SG_PERSIST_CLEAR=$MY_KEY $DEV > /dev/null 2>&1
                    RET=$?
                fi
            else 
                $SG_PERSIST_RDKEYS $DEV | fgrep $MY_KEY > /dev/null 2>&1
                if [ $? -eq 0 ]
                then
                    check_if_multipath $DEV
                    if [ $? -eq 0 ]
                    then 
                        get_dm_paths $DEV all
                        for DM_DEV in $DM_DEV_LIST
                        do
                            $SG_PERSIST_UNREG=$MY_KEY $DM_DEV > /dev/null  2>&1
                            RET=$?
                        done
                    else
                        $SG_PERSIST_UNREG=$MY_KEY $DEV > /dev/null  2>&1
                        RET=$?
                    fi
                else
                    RET=0
                fi
            fi
            if [ $RET -ne 0 ]
            then
                (( trys = trys + 1 ))
                sg_log 0 "sg_deactivate_pr : retry number : $trys"
            fi
        done
   
        if [ $RET -ne 0 ]
        then
            sg_log 0 "sg_deactivate_pr: deactivating PR FAILED on "$DEV
            if sg_check_if_it_is_reconfiguration
            then
                sg_log 999 "ERROR: Deactivating PR FAILED on "$DEV
                sg_set_reconfig_failure
            fi
            fail=1
        else
            if sg_check_if_it_is_reconfiguration
            then
                sg_log 999 "SUCCESS: Successfully deactivated PR on "$DEV
            fi
        fi
    done
    if [ $fail -eq 1 ]
    then
        if ! sg_check_if_reconfig_has_failed
        then
           sg_log 0 "ERROR: sg_deactivate_pr: Deactivating PR failed for atleast one disk "
        fi
        exit 1
    fi
    exit 0
}
##########################################################
# Package Failure Function
# Check if the stale key is registered with the devices
# and if so let's do a preempt and abort. This will both
# remove the stale registration and if the key also holds
# the reservation reassign it to this node.
##########################################################
function sg_failed_pr
{

  typeset -i fail=0
    cmprd_pid=`$PGREP cmprd`
    NODE_KEY="0x$MY_KEY"

    for DEV in $DEV_LIST
    do
        for KEY in $STALE_KEYS
        do
            $LOGGER -t cmprd[$cmprd_pid] \
            "sg_failed_pr: deactivating PR key $KEY on $DEV in package $PKG_NAME"
            $SG_PERSIST_RDKEYS $DEV | $FGREP $KEY > /dev/null 2>&1
            if [ $? -eq 0 ]
            then
                $SG_PERSIST_PREEMPT=$MY_KEY $ABORT_KEY=$KEY $PR_TYPE_WERO $DEV > /dev/null 2>&1
                if [ $? -ne 0 ]
                then
                    $LOGGER -t cmprd[$cmprd_pid] \
                    "ERROR: Revoking key $KEY from $DEV for package $PKG_NAME failed."
                    "Ignore this message if $PKG_NAME is in starting or failing state in $HOST."
                    fail=1
                    break
                fi
            fi
            #############################################################
            # The node whose key is being revoked need not reserve again.
            #############################################################

            $SG_PERSIST_RDRESV $DEV | $FGREP "NO reservation" > /dev/null 2>&1
            if [ $? -eq 0 ]
            then
                $SG_PERSIST_RESV=$MY_KEY $PR_TYPE_WERO $DEV > /dev/null 2>&1
            fi
        done

        key_on_disk=`$SG_PERSIST_RDRESV $DEV 2> /dev/null | $GREP "Key" | $CUT -d = -f 2`
        if [ "$key_on_disk" != "" ]
        then
            for KEY in $STALE_KEYS
            do
                if [ "$key_on_disk" = "$KEY" ]
                then
                    $LOGGER -t cmprd[$cmprd_pid] \
                    "ERROR: Disk $DEV has the stale entry $KEY in it."
                    "Ignore this message if $PKG_NAME is in starting or failing state in $HOST."
                    fail=1
                fi
            done
        fi
    done
    if [ $fail -eq 1 ]
    then
        exit 1
    fi
    exit 0

}

#####################################################
# Package Reconfigure Function
# Lets walk the list of devices and check if there are
# any that we need to register with.  If found then
# go ahead and  register and then check to see if we
# also need to make the reservation.
#####################################################
function sg_reconfig_pr
{
    typeset -i ONE_PATH
    typeset -i already_reserved=0
    for DEV in $DEV_LIST
    do
        $SG_PERSIST_RDKEYS $DEV | fgrep $MY_KEY > /dev/null 2>&1
        if [ $? -ne 0 ]
        then
            # Checks whether we are working with multipath device.
            PATH_TO_RESERVE=""
            already_reserved=0
            check_if_multipath $DEV
            if [ $? -eq 0 ]
            then
                sg_log 0 "sg_activate_pr: Starting PR operation on multipath device "$DEV
                ONE_PATH=0 
                get_dm_paths $DEV all
                # Create registrations for each of the path.
                for DM_DEV in $DM_DEV_LIST
                do
                    $SG_PERSIST_REG=$MY_KEY $DM_DEV > /dev/null 
                    RET=$?
                    if [ $RET -ne 0 ]
                    then
                        sg_log 0 "WARNING: sg_reconfig_pr: registration failed on disk $DM_DEV, retrying"
                        $SG_PERSIST_REG_IGN=$MY_KEY $DM_DEV > /dev/null
                        RET=$?
                    fi
                    # Go ahead and reserve it, if registration is successful on at least one path
                    if [ $RET -eq 0 ]
                    then
                        ONE_PATH=1
                        # Create a list of paths, for which registration is successful.
                        PATH_TO_RESERVE=$PATH_TO_RESERVE" "$DM_DEV
                    else
                        sg_log 0 "ERROR: sg_reconfig_pr: registration failed again on disk $DM_DEV"
                    fi
                done
            else      
                $SG_PERSIST_REG=$MY_KEY $DEV > /dev/null 
                RET=$?
                if [ $RET -ne 0 ]
                then
                    sg_log 0 "WARNING: sg_reconfig_pr: registration failed on disk $DEV, retrying"
                    $SG_PERSIST_REG_IGN=$MY_KEY $DEV > /dev/null 
                    RET=$?
                    if [ $RET -ne 0 ]
                    then
                        sg_log 0 "ERROR: sg_reconfig_pr: registration failed again on disk $DEV"
                    else
                        PATH_TO_RESERVE=$PATH_TO_RESERVE" "$DEV
                    fi
                else
                    PATH_TO_RESERVE=$PATH_TO_RESERVE" "$DEV
                fi
            fi

            if [[ $RET -eq 0 || $ONE_PATH -eq 1 ]]
            then
                for DEV in $PATH_TO_RESERVE
                do
                    if [ $already_reserved -eq 0 ]; then
                        $SG_PERSIST_RDRESV $DEV | fgrep "NO reservation" > /dev/null 2>&1
                        if [ $? -eq 0 ]
                        then
                            $SG_PERSIST_RESV=$MY_KEY $PR_TYPE_WERO $DEV > /dev/null
                            if [ $? -ne 0 ]
                            then
                                sg_log 0 "ERROR: sg_reconfig_pr: Failed to make reservation on the disk "$DEV
                            else
                                already_reserved=1
                            fi
                        fi
                    fi
                    if [ $already_reserved -eq 1 ]; then
                        sg_log 0 "sg_activate_pr: activating PR on "$DEV
                    fi
                done
            else
                sg_log 0 "sg_reconfig_failed to register with "$DEV
            fi
        fi
    done

    exit 0
}

####################################################################
# If we encounter one disk that does not support PR then the package
# cannot use PR so we exit the for loop and return 1.
# To test a device we do the default sg_persist read-keys operation.
# If the device supports PR it returns 0 otherwise a different value
# depending on whether the device responded or another error was
# encountered. Just to make sure well also try a read reservation.
####################################################################
function sg_validate_pr
{
    typeset -i is_local_node_eligible=0
    typeset -i ret=0
    ######################################################
    # If cluster pr mode is disabled then do not perform
    # any validation.
    #####################################################
    cluster_pr_mode=`cmviewcl -v -f line -s config 2> /dev/null| \
                     grep cluster_pr_mode | cut -d = -f 2`
    if [ "$cluster_pr_mode" = "pr_forced_disabled" ]
    then
        exit 3 
    fi
    #######################################################
    # Lets first make sure we have the sg_persist command.
    #######################################################

    if [ ! -x /usr/bin/sg_persist ]
    then
        sg_log 0 "pkg_pr_mode: DISABLED sg_persist not installed"
        exit 1
    fi

    cache_cmviewcl_output_lx 
    if [ $? -eq 0 ]
    then
        local_node_id=`cat "$cmviewcl_output" | grep -w $local_host_name | awk -F= '/^node:[^|]*\|id=/ {print $2}'`
        check_if_local_node_is_eligible_to_run_validate
        is_local_node_eligible=$?
        # if the function returns 1 then it is not eligible return success.
        if [ $is_local_node_eligible -eq 1 ]
        then
            cleanup_before_exit 0
        fi    
    else
        # Check if the current node is the node with least node ID
        # SG_P_LEAST_NODE is sourced as part of environment variable 
        if [[ $local_host_name != $SG_P_LEAST_NODE ]]
        then
            cleanup_before_exit 0
        fi
    fi 

    typeset -i foreign_registration=0
    for DEV in $DEV_LIST
    do     
        $SG_PERSIST_RDKEYS $DEV >> /dev/null  2>&1
        if [ $? -ne 0 ]
        then
            if [ ! -x /opt/drenabler/utils/mccheckfeature ]
            then
                sg_log 0 "ERROR: In package $SG_PACKAGE_NAME :$DEV does not support Persistent Reservations"
                sg_log 0 "       Persistent Reservations disabled for package $SG_PACKAGE_NAME"
                sg_log 0 "       In some configurations this may result in data"
                sg_log 0 "       corruption under certain conditions.  Please"
                sg_log 0 "       check the documentation for more details."
                cleanup_before_exit 1
            else
                $(/opt/drenabler/utils/mccheckfeature "PR" "validate" ${DEV})
                RET_VAL=$?
                if [ $RET_VAL -eq 2 ]
                then
                    sg_log 0 "ERROR: In package $SG_PACKAGE_NAME :$DEV does not support Persistent Reservations"
                    sg_log 0 "       Persistent Reservations disabled for package $SG_PACKAGE_NAME"
                    sg_log 0 "       In some configurations this may result in data"
                    sg_log 0 "       corruption under certain conditions.  Please"
                    sg_log 0 "       check the documentation for more details."
                    cleanup_before_exit 1
                else
                    if [ $RET_VAL -eq 0 ]
                    then
                        cleanup_before_exit 0
                    else
                        cleanup_before_exit 1
                    fi

                fi
            fi
        else
            REG_KEYS=$(read_registration $DEV)
            if [ "$REG_KEYS" != "NONE" ] 
            then
                #validate all the registered keys are the node_pr_keys, 
                #else it is a stale VG
                if [ "$REG_KEYS" != "ERROR" ] 
                then
                    if [ "$NODE_KEYS" = "" ] 
                    then 
                        cleanup_before_exit 0
                    fi
                    validate_reg_keys $REG_KEYS 
                    if [ $? -ne 0 ] 
                    then
                        foreign_registration=1
                        sg_log 0 "ERROR: sg_validate_pr : Physical Volume" $DEV "used in the "                        
                        sg_log 0 "Volume Group has registration which is not from any of the cluster nodes."
                        sg_log 0 "Use pr_cleanup utility to clear the keys."
                    fi             
                fi
            fi
        fi
    done
    if [ $foreign_registration -eq 1 ]
    then
        sg_log 0 "Atleast one of the devices had a foreign registration."
        cleanup_before_exit 1
    fi
    cleanup_before_exit 0
}


#####################################################################
# Build the list of physical volumes. These are devices files such as
# /dev/sda or /dev/sdb1.
# If no package name was passed in then we were called by the pr_cntl
# script for either a start, stop, validate, reconfigure_start or
# reconfigure_stop action.  In the latter two cases we are looking
# for a difference between the current and previous set of VGs.
# If the package name was passed in then we were called by the MNP
# daemon to with the failed option where a node has gone down.
####################################################################
function build_pv_list
{
    typeset postfix

    if [ "$PKG_NAME" = "" ]
    then
        PKG_NAME=$SG_PACKAGE_NAME
        if [ "$OP" = "reconfigure_start" ]
        then
            for I in ${SG_PV[*]}
            do
                found=0
                for J in ${PREV_SG_PV[*]}
                do
                    if [[ $I = $J ]]
                    then
                        found=1
                        break;
                    fi
                done
                if ((found == 0 ))
                then
                    if ! sg_check_if_reconfig_has_failed
                    then
                        sg_log 0 "New PV $I"
                    fi
                    DEV_LIST=$DEV_LIST" "$I
                fi
            done
        else
            if [ "$OP" = "reconfigure_stop" ]
            then
                for I in ${PREV_SG_PV[*]}
                do
                    found=0
                    for J in ${SG_PV[*]}
                    do
                        if [[ $I = $J ]]
                        then
                            found=1
                            break
                        fi
                    done
                    if ((found == 0))
                    then
                        if ! sg_check_if_reconfig_has_failed
                        then
                            sg_log 0 "PV removed $I"
                        fi
                        DEV_LIST=$DEV_LIST" "$I
                    fi
                done
            else
                for PV in ${SG_PV[@]}
                do
                    DEV_LIST=$DEV_LIST" "$PV
                done
            fi
        fi
    else
        if [ "$OP" = "failed" ]
        then
            postfix=$(date +"%H.%M.%S")
            SG_ENV_DEV_FILE=/var/tmp/${PKG_NAME}.${OP}.${postfix}.tmp
            cmgetpkgenv $PKG_NAME > $SG_ENV_DEV_FILE
            if (( $? != 0 ))
            then
                sg_log 0 "ERROR: Function sg_source_pkg_env"
                sg_log 0 "       Unable to retrieve package attributes."
                rm -f ${SG_ENV_DEV_FILE}
                sg_exit 1
            fi
            export SG_ENV_DEV_FILE
            . ${SG_ENV_DEV_FILE}
            rm -f ${SG_ENV_DEV_FILE}
        fi


        for PV in ${SG_PV[@]}
        do
            DEV_LIST=$DEV_LIST" "$PV
        done
    fi
}

#####################################################################
# Build the list of volume groups.  If no package name was passed in
# we were called by the pr_cntl scripts for either a start, stop
# validate, reconfigure_start or reconfigure_stop.  In the
# latter two cases we are looking for a difference between the
# current and previous set of VGs.
####################################################################
function build_vg_list
{
    VG_LIST=""

    if [ "$PKG_NAME" = "" ]
    then
        PKG_NAME=$SG_PACKAGE_NAME
        if [ "$OP" = "reconfigure_start" ]
        then
            for I in ${SG_VG[*]}
            do
                found=0
                for J in ${PREV_SG_VG[*]}
                do
                    if [[ $I = $J ]]
                    then
                        found=1
                        break;
                    fi
                done
                if [ $found -eq 0 ] 
                then
                    if ! sg_check_if_reconfig_has_failed
                    then
                        sg_log 0 "New VG $I"
                    fi
                    VG_LIST=$VG_LIST" "$I
                fi
            done
        else
            if [ "$OP" = "reconfigure_stop" ]
            then
                for I in ${PREV_SG_VG[*]}
                do
                    found=0
                    for J in ${SG_VG[*]}
                    do
                        if [[ $I = $J ]]
                        then
                            found=1
                            break
                        fi
                    done
                    if [ $found -eq 0 ]
                    then
                        if ! sg_check_if_reconfig_has_failed
                        then
                            sg_log 0 "VG removed $I"
                        fi
                        VG_LIST=$VG_LIST" "$I
                    fi
                done
            else       
                for VG in ${SG_VG[@]} 
                do
                    VG_LIST=$VG_LIST" "$VG
                done
            fi
        fi
        PKG_NAME=""
    else
        for VG in $(cmgetpkgenv $PKG_NAME | fgrep "SG_VG[" | cut -d \" -f 2) 
        do
            VG_LIST=$VG_LIST" "$VG
        done
    fi
   
    if [ "$VG_LIST" != "" ]
    then
        for VG in $VG_LIST
        do
            results=$(vgdisplay -v $VG 2>&1)
            case $results in
                *Input/output*error*)
                    IO_DISK_ERROR=1;
                    ;;
                *Volume*group*not*found*)
                    IO_DISK_ERROR=1;
                    ;;
            esac
            for DEV in `vgdisplay -v $VG 2> /dev/null | fgrep "PV Name" | \
                        grep -v "$MATCHING_STRING" | awk '{print $NF}'`
            do
                grep -w "$(basename ${DEV})" /proc/mdstat >/dev/null 2>&1
                if [ $? -ne 0 ]; 
                then
                    check_if_multipath $DEV
                    if [ $? -eq 0 ]
                    then
                        echo $DEV | grep -q -e "/dev/mapper/" -e "/dev/mpath/"
                        if [ $? -eq 0 ]
                        then
                           if [ "$OS_Version" = "RH7" ]
                           then
                               DEV=`echo $DEV`
                           else 
                               DEV=`echo $DEV | sed -r $DEVICE_MAPPER_PATTERN` 
                           fi
                        fi
                        DEV_LIST=$DEV_LIST" "$DEV
                        continue
                    fi
                    DEV_tmp=`readlink $DEV`
                    if [ $? -eq 0 ]
                    then
                        DEV="/dev/"$DEV_tmp
                    fi
                    DEV=`echo $DEV | sed 's/[0-9]*//g'`

                    DEV_LIST=$DEV_LIST" "$DEV
                else
                    md_found=0
                    if [ "$OP" = "reconfigure_stop" ]
                    then
                        for ((  I = 0;  I < ${#PREV_XDC_RAID_DEVICE[@]};  I++  ))
                        do
                            if [[ "${DEV}" = "${PREV_XDC_RAID_DEVICE[$I]}" ]]; then
                               md_found=1
                               index=$I
                               break
                            fi
                        done
                        if [ $md_found -ne 0 ]; then
                           $UDEV_INFO ${PREV_XDC_DEVICE_0[$index]} >/dev/null 2>&1
                           if [ $? -eq 0 ]; then
                              # Checking if XDC_DEVICE_0 is available or not.
                              # If available, add to DEV_LIST
                              $CM_CHKDISK /dev/$($UDEV_INFO ${PREV_XDC_DEVICE_0[$index]} 2>/dev/null) >/dev/null 2>&1
                              if [ $? -eq 0 ]; then
                                 DEVICE="/dev/"$($UDEV_INFO ${PREV_XDC_DEVICE_0[$index]})
                                 # For MD device, multipath device comes as /dev/dm-n
                                 # We have to get corresponding /dev/mapper/mpathn name
                                 echo $DEVICE | grep -q "/dev/dm-"
                                 if [ $? -eq 0 ]
                                 then
                                    if [ "$OS_Version" = "RH7" ]
                                    then
                                        MD_DEV="/dev/mapper/"$($DM_DETAILS $DEVICE | grep Name | awk '{print $2}') > /dev/null 2>&1
                                    else	 		
                                        MD_DEV="/dev/mapper/"$($DM_DETAILS $DEVICE | grep Name | awk '{print $2}' \
                                                       | sed -r $DEVICE_MAPPER_PATTERN) > /dev/null 2>&1
                                    fi	
                                    DEV_LIST=$DEV_LIST" "$MD_DEV
                                 else
                                     DEV_tmp=`readlink $DEVICE`
                                     if [ $? -eq 0 ]
                                     then
                                         DEVICE="/dev/"$DEV_tmp
                                     fi
                                     DEVICE=`echo $DEVICE | sed 's/[0-9]*//g'`
                                     DEV_LIST=$DEV_LIST" "$DEVICE
                                 fi
                              fi
                           fi
                           $UDEV_INFO ${PREV_XDC_DEVICE_1[$index]} > /dev/null 2>&1
                           if [ $? -eq 0 ]; then
                              # Checking if XDC_DEVICE_1 is available or not.
                              # If available, add to DEV_LIST
                              $CM_CHKDISK /dev/$($UDEV_INFO ${PREV_XDC_DEVICE_1[$index]} 2>/dev/null) >/dev/null 2>&1
                              if [ $? -eq 0 ]; then
                                 DEVICE="/dev/"$($UDEV_INFO ${PREV_XDC_DEVICE_1[$index]})
                                 # For MD device, multipath device comes as /dev/dm-n
                                 # We have to get corresponding /dev/mapper/mpathn name
                                 echo $DEVICE | grep -q "/dev/dm-"
                                 if [ $? -eq 0 ]
                                 then
                                    if [ "$OS_Version" = "RH7" ]
                                    then
                                        MD_DEV="/dev/mapper/"$($DM_DETAILS $DEVICE | grep Name | awk '{print $2}') > /dev/null 2>&1
                                    else	
                                        MD_DEV="/dev/mapper/"$($DM_DETAILS $DEVICE | grep Name | awk '{print $2}' \
                                                           | sed -r $DEVICE_MAPPER_PATTERN) > /dev/null 2>&1
                                     fi	
                                     DEV_LIST=$DEV_LIST" "$MD_DEV
                                 else
                                     DEV_tmp=`readlink $DEVICE`
                                     if [ $? -eq 0 ]
                                     then
                                         DEVICE="/dev/"$DEV_tmp
                                     fi
                                     DEVICE=`echo $DEVICE | sed 's/[0-9]*//g'`
                                     DEV_LIST=$DEV_LIST" "$DEVICE
                                 fi
                              fi
                           fi
                        fi
                    else
                        for ((  I = 0;  I < ${#XDC_RAID_DEVICE[@]};  I++  ))
                        do
                            if [[ "${DEV}" = "${XDC_RAID_DEVICE[$I]}" ]]; then
                               md_found=1
                               index=$I
                               break
                            fi
                        done
                        if [ $md_found -ne 0 ]; then
                           $UDEV_INFO ${XDC_DEVICE_0[$index]} >/dev/null 2>&1
                           if [ $? -eq 0 ]; then
                              # Checking if XDC_DEVICE_0 is available or not.
                              # If available, add to DEV_LIST
                              $CM_CHKDISK /dev/$($UDEV_INFO ${XDC_DEVICE_0[$index]} 2>/dev/null) >/dev/null 2>&1
                              if [ $? -eq 0 ]; then
                                 DEVICE="/dev/"$($UDEV_INFO ${XDC_DEVICE_0[$index]})
                                 # For MD device, multipath device comes as /dev/dm-n
                                 # We have to get corresponding /dev/mapper/mpathn name
                                 echo $DEVICE | grep -q "/dev/dm-"
                                 if [ $? -eq 0 ]
                                 then
                                    if [ "$OS_Version" = "RH7" ]
                                    then
                                        MD_DEV="/dev/mapper/"$($DM_DETAILS $DEVICE | grep Name | awk '{print $2}') \
                                               > /dev/null 2>&1
                                    else	
                                        MD_DEV="/dev/mapper/"$($DM_DETAILS $DEVICE | grep Name | awk '{print $2}' \
                                                    | sed -r $DEVICE_MAPPER_PATTERN) > /dev/null 2>&1
                                    fi
                                    DEV_LIST=$DEV_LIST" "$MD_DEV
                                 else
                                     DEV_tmp=`readlink $DEVICE`
                                     if [ $? -eq 0 ]
                                     then
                                         DEVICE="/dev/"$DEV_tmp
                                     fi
                                     DEVICE=`echo $DEVICE | sed 's/[0-9]*//g'`
                                     DEV_LIST=$DEV_LIST" "$DEVICE
                                 fi
                              fi
                           fi
                           $UDEV_INFO ${XDC_DEVICE_1[$index]} > /dev/null 2>&1
                           if [ $? -eq 0 ]; then
                              # Checking if XDC_DEVICE_1 is available or not.
                              # If available, add to DEV_LIST
                              $CM_CHKDISK /dev/$($UDEV_INFO ${XDC_DEVICE_1[$index]} 2>/dev/null) >/dev/null 2>&1
                              if [ $? -eq 0 ]; then
                                 DEVICE="/dev/"$($UDEV_INFO ${XDC_DEVICE_1[$index]})
                                 # For MD device, multipath device comes as /dev/dm-n
                                 # We have to get corresponding /dev/mapper/mpathn name
                                 echo $DEVICE | grep -q "/dev/dm-"
                                 if [ $? -eq 0 ]
                                 then
                                     if [ "$OS_Version" = "RH7" ]
                                     then
                                         MD_DEV="/dev/mapper/"$($DM_DETAILS $DEVICE | grep Name | awk '{print $2}') \
                                                > /dev/null 2>&1
                                     else	
                                         MD_DEV="/dev/mapper/"$($DM_DETAILS $DEVICE | grep Name | awk '{print $2}' \
                                                | sed -r $DEVICE_MAPPER_PATTERN) > /dev/null 2>&1
                                     fi
                                     DEV_LIST=$DEV_LIST" "$MD_DEV
                                 else
                                     DEV_tmp=`readlink $DEVICE`
                                     if [ $? -eq 0 ]
                                     then
                                         DEVICE="/dev/"$DEV_tmp
                                     fi
                                     DEVICE=`echo $DEVICE | sed 's/[0-9]*//g'`
                                     DEV_LIST=$DEV_LIST" "$DEVICE
                                 fi
                              fi
                           fi
                        fi
                    fi
                fi
            done
        done
    fi
}

####################################################################
# Checks whether a device is a DM multipath device.
# Any device in format /dev/mapper/XXXX is a multipath device.
# On RHEL 5.x, device can also be in format /dev/mpath/mpathX
# For udev, we do a readlink and if its in dm-x format, it is a
# multipath device. 
####################################################################

function check_if_multipath
{
    typeset device=$1

    echo $device | grep -q -e "/dev/mapper/" -e "/dev/mpath/"
    if [ $? -eq 0 ]
    then
        return 0
    else
        udev_tmp=`readlink $device`
        if [ $? -eq 0 ]
        then
            echo $udev_tmp | grep -q "dm-"
            if [ $? -eq 0 ]
            then 
                return 0
            fi
        fi
        return 1            
    fi
}

####################################################################
# Populates the active running paths for a specified DM multipath 
# device. It takes two parameters - 
# 1. Multipath Device Name
# 2. "all" or "single" : whether to return a single path or all paths 
####################################################################

function get_dm_paths
{
    typeset device=$1
    typeset path=$2
    DM_LIST=""
    DM_DEV_LIST=""

    echo $device | grep -q -e "/dev/mapper/" -e "/dev/mpath/"
    if [ $? -eq 0 ] 
    then
        dm_dev=`echo $device | sed -e 's/\/dev\/mapper\///' -e 's/\/dev\/mpath\///'`
    else
        dm_dev=$($DM_DETAILS "/dev/"$(basename `readlink $device`) | grep Name | \
                 awk '{print $2}'| sed -r $DEVICE_MAPPER_PATTERN) > /dev/null 2>&1  
    fi

    if [ "$path" = "all" ]
    then
        ## For Redhat 7, run MPATH_LIST over UUID of parent device ##
        if [ "$OS_Version" = "RH7" ]
        then
            dm_dev_uuid=`$DM_DETAILS $dm_dev | grep UUID | awk -F- '{print $NF}'`
            DM_LIST=$DM_LIST" "$($MPATH_LIST $dm_dev_uuid | grep sd | grep active | sed 's/|//g' \
                    | awk '{print $3}') > /dev/null 2>&1
        else 
            DM_LIST=$DM_LIST" "$($MPATH_LIST $dm_dev | grep sd | grep active | sed 's/|//g' \
                    | awk '{print $3}') > /dev/null 2>&1
        fi
        for DEV_PATH in $DM_LIST
        do
            DEV_PATH="/dev/"`echo $DEV_PATH | sed 's/[0-9]*//g'`
            DM_DEV_LIST=$DM_DEV_LIST" "$DEV_PATH
        done
    else
        dm_dev_uuid=`$DM_DETAILS $dm_dev | grep UUID | awk -F- '{print $NF}'`
        DM_DEV_LIST="/dev/"$($MPATH_LIST $dm_dev_uuid | grep sd | grep active | sed 's/|//g' \
                         | awk '{print $3}' | awk 'NR==1 {print}' | sed 's/[0-9]*//g')
    fi
}

function find_if_local_node_has_least_node_id
{
    typeset node_ids=""
    typeset least_node=""
    typeset -i node_id=0
    typeset -i least_node_id=0
    typeset package_node_names=""

    node_ids_line=`cat "$cmviewcl_output" | awk '/^node:[^|]*\|id=/'`

    least_node_id=$local_node_id
    least_node=$local_host_name

    # Function get_package_nodes is defined in SG_UTILS
    package_node_names=`get_package_nodes`
    for node in $package_node_names
    do
        node_id=`echo "$node_ids_line" | grep -w $node | cut -d "=" -f 2`
        if [[ $? == 0 ]]; then
            if (( $node_id <= $least_node_id  ));then
                least_node_id=$node_id
                least_node=$node
            fi
        fi
    done

    if [[ "$least_node" == "$local_host_name" ]] ;then
        # True
        return 0
    else
        # False
        return 1
    fi
}

function check_if_local_node_is_eligible_to_run_validate
{
    typeset -i ret=0
    # Check for below
    # 1. Package is running and it is on the current node
    #   a. if package is running check if it is the current node
    #       i.If not skip this module on the current node.
    #                       ELSE
    # 2. Check if the node has the least node id
    #       i.If not skip this module on the current node.

    # Function get_running_package_nodeid is defined in SG_UTILS
    get_running_package_nodeid

    if [ $SG_RUNNING_NODE_DURING_VALIDATE != 0 ] ; then
        if [[ $local_node_id == $SG_RUNNING_NODE_DURING_VALIDATE ]];then
            # This is the eligible node. Return TRUE
            return 0
        else
            # The package is not running on this node
            return 1
        fi
    else
        find_if_local_node_has_least_node_id
        ret=$?
        if [[ $ret != 0 ]]; then
            return 1
        else
            # This is the eligible node. Return TRUE
            return 0
        fi
    fi
}

########################################
# Main routine
########################################

##################################################################
# Get the PR Key for this node
##################################################################
HOST=$(uname -n | cut -d . -f 1)

########################################
# Process the command line arguments.
########################################
while (($#>0))
do
    case $1 in
        start | stop | validate | reconfigure_start | \
        reconfigure_stop | reconfig | failed )
            OP=$1
            shift 1
            ;;
        -k)
            STALE_KEYS=$STALE_KEYS" "$2
            shift 2
            ;;
        -p)
            PKG_NAME=$2
            shift 2
            ;;
        /dev/*)
            DEV_LIST=$1
            action=$2
            sg_node_env_setup
            MY_KEY=$(cmviewcl -f line -l node -s config -n $HOST 2> /dev/null | \
                     fgrep node_pr_key | cut -d = -f 2)
            if [[ "${action}" = "activate" ]]; then
                sg_activate_pr
            else
                sg_deactivate_pr
            fi
            exit $?
            ;;
        *)
            sg_log 5 "pr_util: Invalid command option"
            exit 1
    esac
done

if [ "$OP" = "failed" ]; then
    sg_node_env_setup
fi
MY_KEY=$(cmviewcl -f line -l node -s config -n $HOST 2> /dev/null | \
         fgrep node_pr_key | cut -d = -f 2)

#################################
# Initialize the DEV_LIST to NULL
#################################

DEV_LIST=""

####################################
# Initialize the DM_DEV_LIST to NULL
####################################

DM_DEV_LIST=""

################################################################
# If there are any logical volumes specified in the package
# configuration file lets get the physical LUN devices for them.
################################################################
build_vg_list

#####################################################
# Now lets see if any physical volumes were specified
# and add them to the list of storeage devices.
#####################################################
build_pv_list

#######################################################
# If this package isn't using any storage devices just
# return with the exit value of 1 indicating this
# package is not using PR.
#######################################################
if [ "$DEV_LIST" = "" ]
then
    case $OP in
        start)
            # In case of I/O disk or VG not found error, we
            # should show error related disk accessibility and
            # return an error code 2, that will cause package
            # to failover to an adaptive node.
            if [[ $IO_DISK_ERROR == 1 ]]
            then
                sg_log 0 "ERROR: Some disks are not accessible"
                exit 2
            # In case of package startup because of any issue 
            # other than VG or Disk inaccessable. We should 
            # return an error code 3 so that package fails to 
            # faiover to an adoptive node.
            else 
                exit 3
            fi
            ;;
        stop)
            # In case of stop i.e halt if the disk is not accessible 
            # or VG not found then we should return an error code 2
            # that will package to faiover to an adoptive node.
            if [[ $IO_DISK_ERROR == 1 ]]
            then
                sg_log 0 "ERROR: Some disks are not accessible"
                exit 2
            # In case of stop i.e halt if any issue other than VG or 
            # Disk inaccessable. We should return an error code 3 
            # so that package fails to faiover to an adoptive node.
            else
                exit 3
            fi
            ;;
        *)
            exit 3
            ;;
    esac
fi

###############################
# Process the command request
###############################
RET=0
case $OP in 
    start)
        sg_activate_pr
        ;;
    stop)
        sg_deactivate_pr
        ;;
    validate)
        sg_validate_pr
        ;;
    reconfigure_start)
        sg_activate_pr 
        ;;
    reconfigure_stop)
        sg_deactivate_pr
        ;;
    reconfig)
        sg_reconfig_pr
        ;;
    failed)
        sg_failed_pr
        ;;
    *)
        sg_log 0 "Unknown entry point $1"
        exit 2
        ;;
esac

