####################################################################
# (C) Copyright 2014 Hewlett-Packard Development Company, L.P.
# @(#) Serviceguard UPCC Package Configuration Script.
# @(#) Product Name                :  HP Serviceguard
# @(#) Product Version             :  A.12.10.00
# @(#) Patch Name                  :  
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HP Serviceguard.
#
# Changing this file may lead to unrecoverable package startup
# and/or shutdown problems.
#
###################################################################

###########################
# Source Site Controller utility functions.
###########################
. /etc/cmcluster.conf
SC_UTILS=${SGCONF}/scripts/sg/sc_util.sh

# Set PATH to reference the appropriate directories.
PATH=${SGSBIN}:/bin:/sbin:/usr/bin:/usr/sbin

if [[ -f ${SC_UTILS} ]]; then
    . ${SC_UTILS}
    if (( $? != 0 ))
    then
        echo "ERROR: Unable to source package utility functions file: ${SC_UTILS}"
        exit 1
    fi
else
    echo "ERROR: Unable to find package utility functions file: ${SC_UTILS}"
    exit 1
fi

###################################################################
#
# Get the environment for this package through utility function
# sg_source_pkg_env().
#
###################################################################

sg_source_pkg_env $*


# Global Variables
typeset pkgs_to_monitor    
typeset monitor_logic
typeset no_of_pkgs

###################################################################
#
# calculate_packages_to_monitor() function calculates the
# packages to be monitored
#
###################################################################

function calculate_packages_to_monitor
{
    typeset siteid
    
    pkgs_to_monitor=""
    siteid=`get_site_id_by_name $local_site`
    
    #get the critical package to monitor
    pkgs_to_monitor=`echo "${critical_pkg_list[$siteid]}"`
    
    #if critical package is not configured get all managed pkgs
    if [[ -z "$pkgs_to_monitor" ]]
    then
        pkgs_to_monitor=`echo "${managed_pkg_list[$siteid]}"`
        # Set monitoring logic to managed
        monitor_logic="managed"
    else 
        # Set monitoring logic to critical
        monitor_logic="critical"
    fi

    no_of_pkgs=`echo $pkgs_to_monitor | wc -w`

    if [[ $no_of_pkgs -gt 0 ]]
    then
        sg_log 5 "$pkgs_to_monitor package(s) will be monitored."
    fi
}

###################################################################
#
# start_monitoring_service() function monitors critical or managed packages
# 
#
###################################################################


function start_monitoring_service
{
    typeset sc=${SG_PACKAGE_NAME}
    typeset local_site
    typeset failed_pkg_list
    typeset -i failed_pkgs_count=0
    typeset -i num_of_pkgs_cannot_failover_within_site=0
    typeset pkgs_cannot_failover_within_site
    typeset -i ret=1
    typeset -i failed=0
    typeset -i down=0
    typeset -i siteid=0
    typeset -i sc_state=0
    typeset -i role_reverse_validate=0
    
    sg_log 5 "Function start_monitoring_service"
    # Initialize cache
    init_cache $sc
    
    sg_check_module_in_pkg $SGESAP_MODULE
    if [ $? -eq 0 ] 
    then
        while :; do
            has_genres_ever_set $gen_res_sc
            if [ $? -eq 1 ] 
            then 
                sleep ${SG_SC_SITE_MONITOR_INTERVAL}
            else
                break
            fi 
        done
    fi
    
    #Set up the local and remote site name 
    #start the monitoring service only if active site is configured.
    local_site=""
    while :; do
        for sitename in ${site_array[@]}
        do
            site_mode=`sc_get_site_mode $sc $sitename`
            #See if any site is yet active
            if [ "$site_mode" = "${ACTIVE_STATE}" ]
            then
                local_site=`echo $sitename`         
                break
            fi
            
            #See if any site is intermediate
            #if so, get the site packages and see if they come up
            if [ "$site_mode" = "${INTERMEDIATE_STATE}" ]
            then
                siteid=`get_site_id_by_name $sitename`           
                site_pkgs=`echo "${critical_pkg_list[$siteid]} ${managed_pkg_list[$siteid]}"`

                pkgs_wait_status up ${site_pkgs[@]}
                if [[ $? -eq 1 ]]
                then
                    sg_log 0 "One or more package(s) failed to start. Exiting..."
                    to_exit=1
                    return     
                fi
                #loop till the site mode becomes active
                break
            fi
        done
        
        #If none of the site is active, see if the workload packages are up or 
        #in the process of coming up. If so wait for monitoring interval 
        if [ -n "$local_site" ]; then
            break
        else         
            sleep ${SG_SC_SITE_MONITOR_INTERVAL}
        fi    
    done
    
    populate_already_identified_local_and_remote_site
    if [ -z "$local_site" ]; then
        sg_log 0 "Failed to get local site name. Exiting ..."
        to_exit=1
        return 
    fi

    sg_log 0 "Site Controller is monitoring workload on site $local_site ..."

    calculate_packages_to_monitor

    #start the monitoring loop
    while :; do

        #Even if a node goes down on active site (local_site)
        #once the node is up, this call will set the node to 
        #ACTIVE_STATE            
        sc_set_site_mode $sc $local_site ${ACTIVE_STATE}
        if [[ $? -eq 1 ]]; then
            sg_log 0 "Warning: Not able to open the site safety latch at site"
            sg_log 0 "$local_site in monitoring service"
        fi

        #Even if a node goes down on passive site (remote_site)
        #once the node is up, this call will set the node to
        #REMOTE_PASSIVE_STATE
        sc_set_site_mode $sc $remote_site ${REMOTE_PASSIVE_STATE}
        if [[ $? -eq 1 ]]; then
            sg_log 0 "Warning: Not able to set the site safety latch value at site"
            sg_log 0 "$remote_site in monitoring service"
        fi
    
        if [[ $no_of_pkgs -gt 0 ]]
        then
        
            cache_clview
            
            sc_state=`is_pkg_in_maintenance $sc`
            if [[ $sc_state -ne 0 ]]
            then
                #check whether package(s) are down and failed based on monitoring logic
                for pkg in ${pkgs_to_monitor[@]}
                do
                    down=0
                    failed=0
                    
                    down=`is_package_down $pkg`

                    if [[ $down -eq 0 ]] #package is down
                    then    
                        # This package is down, Refresh cmviewcl cache
                        cache_clview       
                        #check if it has failed
                        package_failed $pkg
                        failed=$?

                        if [[ $failed -eq 0 ]] # package is down-failed
                        then
                            # This package has failed
                            sys_log "$pkg package is down having failed"
                            failed_pkgs_count=`expr $failed_pkgs_count + 1`
                            if [ -z "${failed_pkg_list}" ]
                            then
                                failed_pkg_list=`echo $pkg`
                            else
                                failed_pkg_list=${failed_pkg_list}" "$pkg
                            fi   
                            
                            #Skip checking other packages when monitoring logic is "critical"
                            [[ "$monitor_logic" = "critical" ]] &&  break
                            
                        # package is down-halted
                        elif [[ $failed -eq 2 && "$monitor_logic" = "managed" ]] 
                        then
                            # This package is halted
                            # Skip checking other packages when monitoring logic is "managed"  
                            break
                        fi
                    elif [[ "$monitor_logic" = "managed" ]] #package is not down
                    then
                        #This package is not down 
                        #Skip checking other packages when monitoring logic is "managed"
                        break;
                    fi
                done            

                #Check if site failover should be initiated
                #If monitoring logic is managed, then all packages must be failed
                #If monitoring logic is critical, then at least one package must have
                #failed
                if [[ "$monitor_logic" = "managed" && "$failed_pkgs_count" = "$no_of_pkgs" ]] \
                      || [[ "$monitor_logic" = "critical" && "$failed_pkgs_count" -gt "0"  ]]
                then
                    num_of_pkgs_cannot_failover_within_site=0
                    for mypkg in ${failed_pkg_list}
                    do
                        #if global_switching is disabled, package cannot failover within site.
                        #This check has to be done for failover packages only
                        is_failover_pkg ${mypkg}
                        if [[ $? -eq 0 ]]
                        then
                             is_global_switching_disabled ${mypkg}
                             if [[ $? -eq 0 ]]
                             then
                                 num_of_pkgs_cannot_failover_within_site=`expr ${num_of_pkgs_cannot_failover_within_site} + 1`
                                 if [ -z "${pkgs_cannot_failover_within_site}" ]
                                 then
                                     pkgs_cannot_failover_within_site=`echo $mypkg`
                                 else
                                     pkgs_cannot_failover_within_site=${pkgs_cannot_failover_within_site}" "$mypkg
                                 fi                                
                                 continue
                             fi
                        fi 
                    
                        has_package_halt_failed_on_all_nodes ${mypkg}
                        if [[ $? -eq 0 ]]
                        then
                            num_of_pkgs_cannot_failover_within_site=`expr ${num_of_pkgs_cannot_failover_within_site} + 1`
                            if [ -z "${pkgs_cannot_failover_within_site}" ]
                            then
                                pkgs_cannot_failover_within_site=`echo $mypkg`
                            else
                                pkgs_cannot_failover_within_site=${pkgs_cannot_failover_within_site}" "$mypkg
                            fi       
                        fi
                    done
                    
                    if [[ "${num_of_pkgs_cannot_failover_within_site}" -gt "0" ]]
                    then
                        sg_log 0 "${pkgs_cannot_failover_within_site[@]} package(s) have failed"
                        # Suppose Monitor_logic is managed and at least one of the
                        # packages can failover to other node within the same site,
                        # Don't reverse the site roles.
                        # The following check handles the afore-mentioned scenario.
                        if [[ "$failed_pkgs_count" -gt "${num_of_pkgs_cannot_failover_within_site}" ]] \
                        &&  [[ "$monitor_logic" = "managed" ]]
                        then
                            sg_log 0 "Not reversing site roles as one or more managed package(s)"
                            sg_log 0 "can failover to other node within the active site" 
                        else
                            role_reverse_validate=`sg_can_role_reversal_happen`
                            if [[ $role_reverse_validate -eq 1 ]]
                            then
                                sc_initiate_graceful_exit $sc
                                to_exit=1
                                return
                            fi
                            
                            # Package cannot run on any node. Should initiate site role reversal.                            
                            sg_log 5 "reversing site roles"
                            reverse_site_mode
                            if [ $? -eq 1 ] 
                            then
                                sg_log 0 "Failed to reverse site roles"
                                to_exit=1
                                return
                            fi
                            #After reversing site, recalculate packages to be monitored
                            calculate_packages_to_monitor
                        fi
                    fi
                fi            
            fi
        fi 

        failed_pkgs_count=0
        pkgs_cannot_failover_within_site=""
        failed_pkg_list=""
        #sleep for monitoring interval
        sg_log 5 "Sleeping for ${SG_SC_SITE_MONITOR_INTERVAL} seconds ..."
        
        sleep ${SG_SC_SITE_MONITOR_INTERVAL}
        
    done   

}


################
# main routine
################

#
# Site controller monitoring service script entry point(s):
# monitor
#
# The variable to_exit indicates the success or failure of the entry
# point.
#

typeset -i to_exit=0

sg_log 5 "Site Controller monitoring service"

case ${1} in

     monitor)
           start_monitoring_service
           ;;
     
     *)
           sg_log 0 "ERROR: Illegal entry point specification $1."
           to_exit=1
           ;;
esac

#
# Exit to indicate success/failure to the master_control_script.sh
#
exit $to_exit
