####################################################################
# (C) Copyright 2015-2017 Hewlett Packard Enterprise Development LP.
# (C) Copyright 2014 Hewlett-Packard Development Company, L.P.
# @(#) Serviceguard UPCC Package Configuration Script.
# @(#) Product Name                :  HP Serviceguard
# @(#) Product Version             :  A.12.10.00
# @(#) Patch Name                  :  
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HP Serviceguard.
#
# Changing this file may lead to unrecoverable package startup
# and/or shutdown problems.
#
###################################################################

#
# Utility functions file used by the site controller utility script
#
SG_UTILS=$SGCONF/scripts/mscripts/utils.sh
export SG_UTILS

# Set PATH to reference the appropriate directories.
PATH=${SGSBIN}:/bin:/sbin:/usr/bin:/usr/sbin

#
# Source in the utility functions used by the module scripts
#
if [[ -f ${SG_UTILS} ]]; then
    . ${SG_UTILS}
    if (( $? != 0 ))
    then
        echo "ERROR: Unable to source package utility functions file: ${SG_UTILS}"
        exit 1
    fi
else
    echo "ERROR: Unable to find package utility functions file: ${SG_UTILS}"
    exit 1
fi

# OS command constants
cmexec=${SGSBIN}/cmexec
cmmodpkg=${SGSBIN}/cmmodpkg
cmrunpkg=${SGSBIN}/cmrunpkg
cmviewcl=${SGSBIN}/cmviewcl 

# Global variable to cache the output of "cmviewcl -v -f line"
typeset VCLVIEW=""

# Global variable to cache the output of "cmviewcl -v -f line -s config"
typeset VCLVIEW_SCONFIG=""

# Global variable describing different states
ACTIVE_STATE="active"
PASSIVE_STATE="passive"
REMOTE_PASSIVE_STATE="remote_passive"
INTERMEDIATE_STATE="intermediate"
PSEUDO_PASSIVE_STATE="pseudo_passive"

UNKNOWN_STATE_VALUE=0
PASSIVE_STATE_VALUE=1
PSEUDO_PASSIVE_STATE_VALUE=2
REMOTE_PASSIVE_STATE_VALUE=3
ACTIVE_STATE_VALUE=4
INTERMEDIATE_STATE_VALUE=5

NEVER_SET=1
EVER_SET=0

# Global variable to hold the output of cmexec
typeset CMEXEC=""

typeset -i clview_retry_interval
typeset -i clview_retry_times
typeset -i cluster_reform_interval
typeset -i sgsadta_cmexec_err=255
typeset -i cmexec_retry_interval
typeset site_array
typeset site_array_ids
typeset site_nodelist
typeset total_sitenodes
typeset local_site
typeset remote_site
typeset gen_res_sc
typeset -a critical_pkg_list
typeset -a managed_pkg_list
typeset -a remote_managed_pkg_list
typeset -a prev_critical_pkg_list
typeset -a prev_managed_pkg_list
typeset -a prev_remote_managed_pkg_list
typeset SGESAP_MODULE="sgesap/hdbsico"
typeset sgsadta_tmp_dir="/var/tmp" 

typeset -i add_flag=0
typeset -i remove_flag=0
typeset -a added_pkg_list=""
typeset -a removed_pkg_list=""
typeset -a added_pkg_list_on_local_site=""
typeset local_node_name

typeset monitoring_service_name
typeset monitoring_service_cmd

###############################################################################
# This function caches cmviewcl -f line output in the variable VCLVIEW.
# Internal Use only
#
#
# Arguments:  None
#
#
# Returns:    none
#
###############################################################################

function cache_clview
{
   typeset -i counter=$clview_retry_times
   sg_log 5 "Function cache_clview"
   sg_log 5 "Refreshing cmviewcl -f line cache"
   VCLVIEW=`$cmviewcl -f line 2>&1`
   if [[ $? -ne 0 ]]; then
      while [[ $counter -gt 0 ]]
      do
         sg_log 0 "Execution of cmviewcl -f line failed"
         sg_log 0 "$VCLVIEW"
         sg_log 0 "Retrying"
         #sleep for retry interval
         sleep $clview_retry_interval
         VCLVIEW=`$cmviewcl -f line 2>&1`
         if [[ $? -eq 0 ]]; then
            break
         fi
         counter=`expr $counter - 1`
      done
      if [[ $counter -eq 0 ]]; then
         sg_log 0 "Execution of cmviewcl -f line failed"
         sg_log 0 "$VCLVIEW"
         sg_log 0 "Exiting"
         exit 2
      fi
   fi
}

###############################################################################
# This function caches cmviewcl -vf line -s configoutput in the variable 
# VCLVIEW_SCONFIG.
# Internal Use only
#
#
# Arguments:  None
#
#
# Returns:    none
#
###############################################################################

function cache_clview_sconfig
{
   typeset -i counter=$clview_retry_times
   sg_log 5 "Function cache_clview_sconfig"
   sg_log 5 "Refreshing cmviewcl -v -f line -s config cache"
   VCLVIEW_SCONFIG=`$cmviewcl -v -f line -s config 2>&1`
   if [[ $? -ne 0 ]]; then
      while [[ $counter -gt 0 ]]
      do
         sg_log 0 "Execution of cmviewcl -v -f line -s config failed"
         sg_log 0 "$VCLVIEW_SCONFIG"
         sg_log 0 "Retrying"
         #sleep for retry interval
         sleep $clview_retry_interval
         VCLVIEW_SCONFIG=`$cmviewcl -v -f line -s config 2>&1`
         if [[ $? -eq 0 ]]; then
            break
         fi
         counter=`expr $counter - 1`
      done
      if [[ $counter -eq 0 ]]; then
         sg_log 0 "Execution of cmviewcl -v -f line -s config failed"
         sg_log 0 "$VCLVIEW_SCONFIG"
         sg_log 0 "Exiting"
         exit 2
      fi
   fi
}


###############################################################################
# This function executes a command on the specified node using cmexec command
# and stores its output in the variable CMEXEC.
#
# Internal Use only
#
#
# Arguments:  $1   : node name
#             $2   : command
#             $3.. : arguments to the command
#
#
# Returns:    none
#
###############################################################################
function do_cmexec()
{
  typeset _node_name=$1
  typeset _cmd=$2
  typeset _args=
  typeset _output=$sgsadta_tmp_dir/cmexec.$$
  typeset _stderr=$sgsadta_tmp_dir/cmexec.err.$$
  typeset _out
  typeset -i _rc=0
  typeset _line
  typeset _node_status
  typeset -i counter=$clview_retry_times
  
  shift 2
  sg_log 5 "Function do_cmexec -t $cmexec_retry_interval $_node_name $_cmd ..."
  while [ $# -gt 0 ]; do
    if [ -z "$1" ]; then
      _args="$_args \"\""
    else
      _args="$_args $1"
    fi
    shift
  done
  
  sg_log 5 "Executing $SGSBIN/cmexec $_node_name $_cmd $_args 2>&1"
  $cmexec $_node_name -o $_output -t $cmexec_retry_interval $_cmd $_args >/dev/null 2>$_stderr
  grep -q ^cmexec: $_stderr
  if [ $? -eq 0 ]; then
     while [[ $counter -gt 0 ]]
     do
        # Refresh cmviewcl cache
        cache_clview
        _node_status=`echo "$VCLVIEW" | awk -F "[:|=]" '(($1 == "node") && \
                      ($2 == "'$_node_name'") && ($3 == "status")){print $4}'`
        
        if (( "$_node_status" = "up" )); then
           _out="$(cat $_output)"
           sg_log 0 "$_out"
           sg_log 0 "Site Controller operation on node $_node_name failed"
           sg_log 0 "Retrying Site Controller operation on node $_node_name"
           #sleep for retry interval
           sleep $clview_retry_interval
           $cmexec $_node_name -o $_output -t $cmexec_retry_interval $_cmd $_args >/dev/null 2>$_stderr
           grep -q ^cmexec: $_stderr
           if [[ $? -ne 0 ]]; then #if cmexec not failed
              sg_log 0 "Site Controller operation on node $_node_name succeeded"
              break
           fi
           counter=`expr $counter - 1`
        else
             sg_log 0 "Node $_node_name is no longer part of the cluster." 
             sg_log 0 "Suspending Site Controller operation on node $_node_name till it rejoins the cluster."
             break
        fi
      done
    
      if [[ $counter -eq 0 ]]; then
         _out="$(cat $_output)"
         sg_log 0 "_out"
         sg_log 0 "Site Controller operation on node $_node_name failed"
         rm -f $_output 2>/dev/null
         rm -f $_stderr 2>/dev/null
         return $sgsadta_cmexec_err
      fi
  fi
  
  _out="$(cat $_output)"
  sg_log 5 "[OUT] $_out"
  rm -f $_output 2>/dev/null
  rm -f $_stderr 2>/dev/null
  echo "$_out" | \
  {
  read _rc
  while read -r _line; do
    if [ $_rc -eq 0 ]; then
      echo "$_line"
    else
      echo "$_line" >&2
    fi
  done
  }
  return $_rc
}

###############################################################################
# This function initializes all the Package related global variables used to
# hold the cached data. This function will be called during validate, 
# reconfigure start and reconfigure stop only.
#
# Internal Use only
#
# Arguments:  $1 : Site Controller Package Name
#
# Returns:    none
#
###############################################################################

function cache_prev_pkg_info
{
    typeset sitename
    typeset sc=$1
    typeset -i i=0
    typeset -i i1=0
    typeset -i i2=0
    typeset -i j=0
    typeset prev_critical_pkgs
    typeset prev_managed_pkgs
    typeset prev_remote_managed_pkgs
    typeset -i siteid=0

    sg_log 5 "Function cache_prev_pkg_info for $sc . Site Array: ${site_array[@]}"

    for sitename in ${site_array[@]}
    do
        siteid=`get_site_id_by_name $sitename`
        [[ "$siteid" -eq 0 ]] && return
        sg_log 5 "site : $sitename siteid: ${site_array_ids[$siteid]}"
        j=0
        prev_critical_pkgs=(`echo ""`)
        prev_managed_pkgs=(`echo ""`)
        prev_remote_managed_pkgs=(`echo ""`)

        while [ -n  "${PREV_SG_SC_SITE[$j]}" ]; do
            if [ "${PREV_SG_SC_SITE[$j]}" == "$sitename" ]; then
                if [ -n "${PREV_SG_CRITICAL_PACKAGE[$j]}" ]; then
                    prev_critical_pkgs[$i]=${PREV_SG_CRITICAL_PACKAGE[$j]}
                    i=`expr $i + 1`
                fi

                if [ -n "${PREV_SG_MANAGED_PACKAGE[$j]}" ]; then
                    prev_managed_pkgs[$i1]=${PREV_SG_MANAGED_PACKAGE[$j]}
                    i1=`expr $i1 + 1`
                fi

                if [ -n "${PREV_SG_REMOTE_MANAGED_PACKAGE[$j]}" ]; then
                    prev_remote_managed_pkgs[$i2]=${PREV_SG_REMOTE_MANAGED_PACKAGE[$j]}
                    i2=`expr $i2 + 1`
                fi
            fi
            j=`expr $j + 1`
        done

        prev_critical_pkg_list[$siteid]=${prev_critical_pkgs[@]}
        prev_managed_pkg_list[$siteid]=${prev_managed_pkgs[@]}
        prev_remote_managed_pkg_list[$siteid]=${prev_remote_managed_pkgs[@]}

    done

    sg_log 5 "Already applied critical Pkg list Total: ${prev_critical_pkg_list[@]}"
    sg_log 5 "Already applied managed Pkg list Total: ${prev_managed_pkg_list[@]}"
    sg_log 5 "Already applied remote managed Pkg list Total: ${prev_remote_managed_pkg_list[@]}"

    sg_log 5 "Function cache_prev_pkg_info Exit"

 }

###############################################################################
# This function initializes all the Package related global variables used to 
# hold the cached data. This function should be called in the beginning of 
# the main scripts that use cached data.
#
# Internal Use only
#
# Arguments:  $1 : Site Controller Package Name
#
# Returns:    none
#
###############################################################################

function cache_pkg_info
{
    typeset sitename
    typeset sc=$1
    typeset -i i=0
    typeset -i i1=0   
    typeset -i i2=0    
    typeset -i j=0
    typeset critical_pkgs
    typeset managed_pkgs
    typeset remote_managed_pkgs
    typeset -i siteid=0
    
    
    sg_log 5 "Function cache_pkg_info for $sc . Site Array: ${site_array[@]}"
    
    gen_res_sc="${SG_GENERIC_RESOURCE_NAME[0]}"
    sg_log 5 "Function cache_pkg_info for $sc . Generic Resource: $gen_res_sc"
    
    for sitename in ${site_array[@]} 
    do  
        siteid=`get_site_id_by_name $sitename`
        [[ "$siteid" -eq 0 ]] && return
        
        sg_log 5 "site : $sitename siteid: ${site_array_ids[$siteid]}"
        j=0
        critical_pkgs=(`echo ""`)
        managed_pkgs=(`echo ""`)
        remote_managed_pkgs=(`echo ""`)
    
        while [ -n  "${SG_SC_SITE[$j]}" ]; do
            if [ "${SG_SC_SITE[$j]}" == "$sitename" ]; then
                if [ -n "${SG_CRITICAL_PACKAGE[$j]}" ]; then                    
                    critical_pkgs[$i]=${SG_CRITICAL_PACKAGE[$j]}
                    i=`expr $i + 1`
                fi
            
                if [ -n "${SG_MANAGED_PACKAGE[$j]}" ]; then                    
                    managed_pkgs[$i1]=${SG_MANAGED_PACKAGE[$j]}
                    i1=`expr $i1 + 1`
                fi
                
                if [ -n "${SG_REMOTE_MANAGED_PACKAGE[$j]}" ]; then                    
                    remote_managed_pkgs[$i2]=${SG_REMOTE_MANAGED_PACKAGE[$j]}
                    i2=`expr $i2 + 1`
                fi
            fi
            j=`expr $j + 1`
        done
        
        critical_pkg_list[$siteid]=${critical_pkgs[@]}
        managed_pkg_list[$siteid]=${managed_pkgs[@]}
        remote_managed_pkg_list[$siteid]=${remote_managed_pkgs[@]}
        
    done
    
    sg_log 5 "critical Pkg list Total: ${critical_pkg_list[@]}"
    sg_log 5 "Managed Pkg list Total: ${managed_pkg_list[@]}"
    sg_log 5 "Remote site Pkg list Total: ${remote_managed_pkg_list[@]}"
   
    sg_log 5 "Function cache_pkg_info Exit"
    
 }

###############################################################################
# This function returns an index value of 1 or 2 for the given sitename by 
# referencing site_array. This does not represent the actual siteid in cluster.
# The actual siteid can be obtained from site_array_ids. 
# This function can be called only after cache_site_info has setup site array.
#
# Internal Use only
#
# Arguments:  $1 : Site Name
#
# Returns:    none
#
###############################################################################

function get_site_id_by_name
{
  typeset site_name=$1
  typeset -i index=1
  typeset -i count=0

  count=${#site_array[@]}

  while [ $index -le $count ]; do
    if [ "$site_name" == "${site_array[$index]}" ]; then
        echo $index
        return
    fi
    index=`expr $index + 1`
  done
  index=0
  sg_log 0 "Site $site_name is not configured in the cluster"
  echo $index
  return
}

###############################################################################
# This function sets up local_site and remote_site as global variables. This 
# function is called in the beginning of the main scripts that  use cached data. 
# If site_arg is passed, the site_arg is the local_site, else the site where 
# the script is currently executing is considered the local site.
#
# Internal Use only
#
# Arguments:  $1 : Site Name
#
# Returns:    none
#
###############################################################################

function setup_local_and_remote_site
{
  typeset site_arg=$1 
  typeset -i index=1
  typeset sitename
  typeset node
  typeset -i count=0
  
  if [ -n "$site_arg" ]; then
      local_site=`echo "$site_arg"`
  else  
      if [ -z "$local_node_name" ]; then
          local_node_name=${SG_HOST_NAME%%.*}
      fi  
      sg_log 5 "Function setup_local_and_remote_site Local Node : $local_node_name"
 
      count=${#site_nodelist[@]}
  
      while [ $index -le $count ]; do
          for node in ${site_nodelist[$index]}
          do
             if [ "$node" == "$local_node_name" ]; then
                 local_site=${site_array[$index]}
                 break
             fi
          done
          index=`expr $index + 1`
      done
  fi

  for sitename in ${site_array[@]}
  do
      if [ "$sitename" != "$local_site" ]; then
          remote_site=$sitename
      fi
  done
  
  sg_log 5 "Function setup_local_and_remote_site Local Site: $local_site"
  sg_log 5 "Function setup_local_and_remote_site Remote Site: $remote_site"
    
}

###############################################################################
# This function identifies active/passive site based on the site state which is
# already identified. This function accordingly sets up local_site and  
# remote_site as global variables. This function is called in the beginning of  
# the main scripts that use cached data.
#
# Internal Use only
#
# Arguments:  none
#
# Returns:    1 : failure
#             0 : success
#
###############################################################################

function populate_already_identified_local_and_remote_site
{
  typeset sitename
  typeset site_arg=""
  typeset site_mode
 
  sg_log 5 "Function populate_already_identified_local_and_remote_site"
  
  local_site=""
  remote_site=""
  
  for sitename in ${site_array[@]}
  do
     site_mode=`sc_get_site_mode $sc $sitename`                 
     if [ "$site_mode" = "${ACTIVE_STATE}" ]
     then
         local_site=`echo $sitename`         
     else
         remote_site=`echo $sitename`         
     fi
  done
  
  if [ -z "$local_site" ]; then
      sg_log 5 "No Active site configured yet."
      local_site=""
      remote_site=""
      return
  fi
     
  sg_log 5 "Function populate_already_identified_local_and_remote_site" 
  sg_log 5 "Local Site: $local_site"
  sg_log 5 "Function populate_already_identified_local_and_remote_site" 
  sg_log 5 "Remote Site: $remote_site"
    
  return
}

###############################################################################
# This function initializes all the Site related global variables used to hold 
# the cached data. This function should be called in the beginning of the main 
# scripts that use cached data
#
# The term 'siteid' in this function is just an array index of value 1 or 2. 
# The actual siteid vaule -'cluster_siteid' is stored in site_array_ids 
# indexed by the 'siteid'.
# So for the site in 'site_array' at position 'x', its corresponding siteid is
# present in 'site_array_ids' in position 'x'.'site_array_ids' is used in other 
# functions for printing siteid values in log messages. 
#
# Internal Use only
#
# Arguments:  $1 : Site Controller Package Name
#
# Returns:    none
#
###############################################################################

function cache_site_info
{
    typeset sites
    typeset sc=$1
    typeset -i count=0
    typeset -i siteid=1
    typeset -i cluster_siteid=0 
    typeset sitename
    typeset site_nodes
    
    set site_array

    sg_log 5 "Function cache_site_info $sc"
       
    sites=`echo "$VCLVIEW_SCONFIG" | awk -F "[:|=]" '(($1 == "package") && \
                ($2 == "'$sc'") && ($3 ~ ".*site$") && ($5 ~ ".*site$")) \
                {print $6}'`
    
    sg_log 5 "Sites: $sites"
    
    for i in $sites
    do
        #get the siteid for the given site
        cluster_siteid=`echo "$VCLVIEW" | awk -F "[:|=]" '(($1 == "site") && ($2 == "'$i'") && \
                ($3 == "id")){print $4}'`
        [[ "$cluster_siteid" -eq 0 ]] && return

        site_array[$siteid]="$i"
        site_array_ids[$siteid]=$cluster_siteid
 
        #get the site nodes
        site_nodes=`echo "$VCLVIEW" | awk -F "[:|=]" '(($1 == "node") && \
                    ($3 == "siteid") && ($4 == "'$cluster_siteid'")){print $2}'`
        site_nodelist[$siteid]=$site_nodes     
        for sitename in ${site_nodelist[$siteid]}
        do
            count=`expr $count + 1`
        done
        total_sitenodes[$siteid]=$count
        count=0

        siteid=`expr $siteid + 1`
    done
   
    sg_log 5 "Function cache_site_info Site-List: ${site_array[@]}"
    sg_log 5 "Function cache_site_info Site-Ids: ${site_array_ids[@]}"
    sg_log 5 "Function cache_site_info Site-nodeList: ${site_nodelist[@]}"
    sg_log 5 "Function cache_site_info total_sitenodes: ${total_sitenodes[@]}"
}

###############################################################################
# This function initializes all the global variables used to hold the cached
# data. This function should be called in the beginning of the main scripts
# that use cached data
#
# Internal Use only
#
# Arguments:  $1 : Site Controller Package Name
#
# Returns:    none
#
###############################################################################

function init_cache
{
   typeset sc=$1
    
   sg_log 5 "Function init_cache $sc"
   
   clview_retry_interval=50 #twice the recommended average cluster member timeout
   clview_retry_times=3
   cmexec_retry_interval=5 

   # Cache cmviewcl -v fline -s config output
   cache_clview_sconfig

   # Cache cmviewcl -f line output
   cache_clview
   
   # Cache Site information
   cache_site_info $sc 
   
   cache_pkg_info $sc
}

###############################################################################
# Given a site name this function returns the nodes which are UP (status=up) in 
# the cluster
# Internal Use only
#
#
# Arguments:  $1 : Site name
#             
#
# Returns:    site up nodes - success
#             (null) - no node is up in the given site
#
###############################################################################

function get_site_up_nodes
{
    typeset site=$1
    typeset -i siteid=0
    typeset site_nodes
    typeset site_up_nodes
    typeset status
    typeset -i j=0

    sg_log 5 "Function get_site_up_nodes $site"
    
    siteid=`get_site_id_by_name $site`
    [[ "$siteid" -eq 0 ]] && return

    site_nodes=`echo "${site_nodelist[$siteid]}"`
    
    sg_log 5 "Site Nodes: $site_nodes"

    # Refresh cmviewcl cache
    cache_clview

    #get the site nodes which are up in the cluster
    for i in ${site_nodes[@]}
    do
        status=`echo "$VCLVIEW" | awk -F "[:|=]" '(($1 == "node") && \
                ($2 == "'$i'") && ($3 == "status")){print $4}'`
        if [[ "$status" = "up" ]]
        then
           j=`expr $j + 1`  
           site_up_nodes[$j]=$i
        fi
    done

    sg_log 5 "site up node : ${site_up_nodes[@]}"
    echo ${site_up_nodes[@]}
}

########################################################################
# This function checks the failed state of an MNP or failover package, 
# This function is called when a pacakge is down. 
# package_halted flag is checked to understand failed state of the package. 
#
# Internal Use only
#
#
# Arguments:  $1: multi node or failover package name
#
# Returns:    0 - if the last instance of MNP has failed or failover
#                 package has failed. 
#             2 - last instance of MNP or failover has not failed 
#             1 - In case of error
#
########################################################################
function package_failed
{
    typeset -i ret
    typeset pkg
    typeset halted="error"
    typeset status
    typeset state
    pkg=$1

    #$pkg status should be down
    status=`echo "$VCLVIEW" | awk -F "[:|=]" '(($1 == "package") && \
                ($2 == "'$pkg'") && ($3 == "status")) {print $4}'`

    state=`echo "$VCLVIEW" | awk -F "[:|=]" '(($1 == "package") && \
              ($2 == "'$pkg'") && ($3 == "state")) {print $4}'`

    if [[ "$state" = "maintenance" || "$state" = "detached" ]]
    then
        sg_log 0 "Package $pkg is in $state."
        ret=0
        return $ret
    fi
                
    if [[ "$status" != "down" ]]
    then        
        sg_log 0 "ERROR: Package $pkg status not down"
        ret=1
        return $ret
    fi

    #check for package halted
    halted=`echo "$VCLVIEW" | awk -F "[:|=]" '(($1 == "package") && \
                ($2 == "'$pkg'") && ($3 == "package_halted")) {print $4}'`
    sg_log 5 "package_halted value for $pkg is $halted"

    if [ "$halted" = "yes" ]
    then
        ret=2
    elif [ "$halted" = "no" ]
    then
        ret=0
    else
    #If package has not run even once, then package_halted flag is set to ""
        ret=1
    fi
    return $ret
}

########################################################################
# This function checks the halt status of an MNP/failover package, This 
# function should be called when the MNP/failover package is down.
#
# Internal Use only
#
#
# Arguments:  $1 : multi node  or failover package name
#
# Returns:    0 - All instances are in clean state
#             1 - at least one instance has last_halt_failed=yes and switching=disabled
#
########################################################################
function package_halt_cleaned
{   
    typeset state    
    typeset status
    typeset pkg=$1
    typeset halt_failed_nodes
    typeset switching

    sg_log 5 "package_halt_cleaned $pkg"

    #$pkg status should be down
    status=`echo "$VCLVIEW" | awk -F "[:|=]" '(($1 == "package") && \
                ($2 == "'$pkg'") && ($3 == "status")) {print $4}'`

    state=`echo "$VCLVIEW" | awk -F "[:|=]" '(($1 == "package") && \
              ($2 == "'$pkg'") && ($3 == "state")) {print $4}'`

    if [[ "$state" = "maintenance" || "$state" = "detached" ]]
    then
        sg_log 5 "$pkg is in $state."
        return 0
    fi

    if [[ "$status" != "down" ]]
    then        
        sg_log 0 "ERROR: Package $pkg status not down"
        return 1
    fi

    halt_failed_nodes=`echo "$VCLVIEW" | awk -F "[:|=]" '(($1 == "package") && \
                ($2 == "'$pkg'") && ($3 == "node") && ($5 == "last_halt_failed") && \
                ($6 == "yes")) {print $4}'`
 
   for i in $halt_failed_nodes
   do
        switching=`echo "$VCLVIEW" | awk -F "[:|=]" '(($1 == "package") &&\
                    ($2 == "'$pkg'") && ($3 == "node") && ($4 == "'$i'") &&\
                    ($5 == "switching")) {print $6}'`
        if [[ "$switching" = "disabled" ]]; then
                sg_log 0 "Package $pkg has not halted cleanly on node $i"
                return 1
            fi
   done

   sg_log 5 "Package $pkg halt status is clean"

   return 0
}

###############################################################################
# This function gets a site's mode. Site mode can be active or passive
#
# Internal Use only
#
#
# Arguments:  $1 : site controller
#             $2 : site
#
# Returns:    4 active - if the given site is an active site
#             1 and 2  passive - if the given site is a passive site
#             5 intermediate - if the given site is in intermediate state
#             3 remote_passive - if the given site is in remote_passive state
#             -1 error - in case of failure
#
###############################################################################

function sc_get_site_mode
{
    typeset sc=$1
    typeset site=$2
    typeset -i value="1"

    typeset site_nodes
    typeset ret="error"
    typeset node_status

    typeset -i is_down=0
    typeset -i is_remote_passive=0
    typeset -i is_pseudo_passive=0
    typeset -i is_up=0
    typeset -i is_intermediate=0
    typeset -i no_of_up_nodes=0
    typeset -i no_of_down_nodes=0
    typeset -i siteid=0
    typeset -i total_site_nodes=0
    typeset _output=$sgsadta_tmp_dir/cmexec_getres_out.$$
    typeset _out
    typeset element

    sg_log 5 "Function sc_get_site_mode $1 $2"
    
    # There is only 1 generic resource allowed in an SC package. Thus we need 
    # to consider only the first value.
    sg_log 5 "Generic Resource of $sc is $gen_res_sc"
        
    #get the siteid for the given site
    siteid=`get_site_id_by_name $site`
    [[ "$siteid" -eq 0 ]] && return

    #get the site nodes
    total_site_nodes=total_sitenodes[$siteid]    
 
    sg_log 5 "sc_get_site_mode total_site_nodes: $total_site_nodes"
    
    #get the nodes which are up in the current site
    site_nodes=`get_site_up_nodes $site`

    if [[ $? -eq 1 ]]; then
        echo $ret
        return
    fi

    if [[ -z "$site_nodes" ]]; then
        sg_log 0 "No node in site $site is up in the cluster"
        sg_log 0 "Site safety latch at site $site is assumed to be closed"
        ret="${PASSIVE_STATE}"
        echo $ret
        return
    fi

    #get generic resource value from all nodes in site_nodes
    for i in ${site_nodes[@]}
    do
        sg_log 5 "Executing: cmexec $i cmgetresource -r $gen_res_sc"
        do_cmexec $i ${SGSBIN}/cmgetresource -r $gen_res_sc >$_output 2>&1
        if [ $? -eq 0 ]; then
            value=`cat "$_output" \
                    | grep $gen_res_sc \
                    | awk '{print $2}'`
        fi
       
        if [[ -z "$value" ]];then
            #check whether the node is up in the cluster
            # Refresh cmviewcl cache
            cache_clview
            node_status=`echo "$VCLVIEW" | awk -F "[:|=]" '(($1 == "node") && \
                         ($2 == "'$i'") && ($3 == "status")){print $4}'`
            # In case of unavailability to get generic resource, we suspect that
            # cmviewcl doesn't have the correct output as cluster might be in 
            # transient state. Let's set the value to -1 
            if [[ "$node_status" = "up" ]];then
                _out="$(cat $_output)"
                sg_log 0 "$_out"
                sg_log 0 "Error: Failed to get Site Controller Generic resource"
                sg_log 0 " value from $i"
                value=-1
            fi
        fi
        
        rm -f $_output 2>/dev/null
  
        case ${value} in
            # DOWN
            "0") is_down=`expr $is_down + 1` ;;
            "1") is_down=`expr $is_down + 1` ;;
            "2") is_pseudo_passive=`expr $is_pseudo_passive + 1` ;;
            "3") is_remote_passive=`expr $is_remote_passive + 1` ;;
            # UP
            "4") is_up=`expr $is_up + 1` ;;
            # INTERMEDIATE
            "5") is_intermediate=`expr $is_intermediate + 1` ;;
            "-1")no_of_down_nodes=`expr $no_of_down_nodes + 1` ;;
            *)
            # Refresh cmviewcl cache
            cache_clview
            node_status=`echo "$VCLVIEW" | awk -F "[:|=]" '(($1 == "node") && \
                        ($2 == "'$i'") && ($3 == "status")){print $4}'`
            if [[ "$node_status" = "up" ]];then
                sg_log 0 "Error: Failed to get Site Controller Generic resource"
                sg_log 0 " value from $i"
            fi
            ;;
        esac
    done 


    no_of_up_nodes=`get_site_up_nodes $site | wc -w`

    if (( "$is_up" >= 1 )); then
        ret="${ACTIVE_STATE}"
        sg_log 5 "Site safety latch at site $site is open"
    elif (( "$is_intermediate" >= 1 )); then
        ret="${INTERMEDIATE_STATE}"
        sg_log 5 "Site safety latch at site $site is in transient state"
    elif (( "$is_remote_passive" >= 1 )); then 
        ret="${REMOTE_PASSIVE_STATE}"
        sg_log 5 "Site safety latch at site $site is remote passive"         
    elif (( "$is_pseudo_passive" >= 1 )); then 
        ret="${PSEUDO_PASSIVE_STATE}"
        sg_log 5 "Site safety latch at site $site is pseudo passive"         
    elif (( "$is_down" >= "$no_of_up_nodes" )); then
        ret="${PASSIVE_STATE}"   
        sg_log 5 "Site safety latch at site $site is close"
    elif (( "$no_of_down_nodes" == "${#total_site_nodes[@]}" )); then
        # Unless all the nodes are down, we are fine.
        ret="${PASSIVE_STATE}"
        sg_log 5 "Site safety latch at site $site is close"
    else
        ret="error"
        sg_log 0 "Site safety latch state at site $site can not be determined"
    fi

    echo $ret
    return
}

###############################################################################
# This function sets a site's mode. Site mode can be active, passive or 
# intermediate. It accepts site name, Site Controller package name and the 
# mode to which the site should be set to.
#
# Internal Use only
#
#
# Arguments:  $1 : site controller
#             $2 : site
#             $3 : mode (mode can be active, passive , remote_passive 
#                  and intermediate)
#
# Returns:    0 - success
#             1 - failure
#
###############################################################################

function sc_set_site_mode
{

    typeset sc=$1
    typeset site=$2
    typeset mode=$3
    typeset -i value
    typeset site_nodes
    typeset node_status
    typeset -i retry_flag=0
    typeset _output=$sgsadta_tmp_dir/cmexec_setres_out.$$
    
    sg_log 5 "Function sc_set_site_mode $1 $2 $3"

    sg_log 5 "[Generic Resource Value]: $gen_res_sc"
    
    #get the nodes which are up in the current site
    site_nodes=`get_site_up_nodes $site`

    [ $? -eq 1 ] && return 1

    if [[ -z "$site_nodes" ]]; then
        sg_log 0 "No node in site $site is up in the cluster"
        if [[ "${mode}" = "${PASSIVE_STATE}" ]]; then
            sg_log 0 "Site safety latch at site $site is closed"
            return 0
        fi
        sg_log 0 "Cannot set site safety latch at site $site to $mode"
        sg_log 0 "as the node(s) in the cluster are not reachable."
        return 0
    fi

    if [[ "${mode}" = "${ACTIVE_STATE}" ]]
    then
        value=`echo ${ACTIVE_STATE_VALUE}`
    elif [[ "${mode}" = "${PASSIVE_STATE}" ]]
    then
        value=`echo ${PASSIVE_STATE_VALUE}`
    elif [[ "${mode}" = "${PSEUDO_PASSIVE_STATE}" ]]
    then
        value=`echo ${PSEUDO_PASSIVE_STATE_VALUE}`
    elif [[ "${mode}" = "${REMOTE_PASSIVE_STATE}" ]]
    then
        value=`echo ${REMOTE_PASSIVE_STATE_VALUE}`
    elif [[ "${mode}" = "${INTERMEDIATE_STATE}" ]]
    then
        value=`echo ${INTERMEDIATE_STATE_VALUE}`
    else
        sg_log 0 "Error: illegal site safety latch state: $mode"
        return 1
    fi

    #set generic resource value to all nodes in site_nodes
    for i in ${site_nodes[@]}
    do
        sg_log 5 "Executing: cmexec $i $SGSBIN/cmsetresource -r $gen_res_sc $value"
        do_cmexec $i ${SGSBIN}/cmsetresource -r $gen_res_sc $value >$_output 2>&1
        
        if [[ $? -ne 0 ]]
        then
            # check whether the node is up in the cluster
            # Refresh cmviewcl cache
            cache_clview
            node_status=`echo "$VCLVIEW" | awk -F "[:|=]" '(($1 == "node") && \
                         ($2 == "'$i'") && ($3 == "status")){print $4}'`
            if [[ "$node_status" = "up" ]];then
                _out="$(cat $_output)"
                sg_log 0 "$_out"
                sg_log 0 "Error: Failed to set site safety latch value on $i"
                sys_log "Warning: Site Controller $sc not able to set site"
                sys_log "safety latch value to $mode on node $i. Check the"
                sys_log "Site Controller package $sc log for more details"
                retry_flag=1
                # We used to return from here. But now we continue to check the 
                # status on the next node. And Set retry flag.
            fi
        fi
        rm -f $_output 2>/dev/null
    done
    if (( retry_flag == 1 )); then
         return 1
    fi    

   return 0
}



###############################################################################
# This function deactivates a site. 
#
# Internal Use only
#
#
# Arguments:  $1 : Site controller package name
#             $2 : Site name
#             
#
# Returns:    0 - success
#             1 - failure
#
###############################################################################

function sc_deactivate_site
{
    typeset sc=$1
    typeset site=$2
    typeset state=$3
    typeset retry_time
    typeset -i siteid=0

    sg_log 5 "Function sc_deactivate_site $sc $site"

    sg_log 0 "Closing site safety latch at site $site" 
    
    sc_set_site_mode $sc $site $state

    if [[ $? -eq 1 ]]
    then
        # Especially "cmhaltnode -f" operation, we hit a condition where cluster
        # is in transient state. So let us wait for MAX_REFORMATION_DURATION and 
        # retry 
        retry_time=`echo "$VCLVIEW_SCONFIG" | grep "max_reformation_duration" | cut -d = -f 2`
        sg_log 0 "Waiting for stable cluster..."
        sleep $retry_time
        sg_log 0 "Retrying again..."
        sc_set_site_mode $sc $site $state
        if [[ $? -eq 1 ]]
        then
            sg_log 0 "Failed to close site safety latch at site $site."
            return 1
        fi  
    fi

    #wait for the SCCP to halt
    #get Site Controller controlled packages

    typeset site_pkgs
    
    siteid=`get_site_id_by_name $site`           
    site_pkgs=`echo "${critical_pkg_list[$siteid]} ${managed_pkg_list[$siteid]} ${remote_managed_pkg_list[$siteid]}"`
    
    sg_log 0 "Checking all packages managed by Site Controller"

    #wait for down status for all packages
    pkgs_wait_status down ${site_pkgs[@]}

    if [[ $? -eq 1 ]]
    then
        sg_log 0 "All packages managed by Site Controller at site $site"
        sg_log 0 "have not halted"
        return 1
    fi    

    sg_log 0 "At Site $site: $site_pkgs have halted."

    # Refresh cmviewcl -v cache
    cache_clview
    
    #packages halt scripts should also return success
    for pkg in ${site_pkgs[@]}
    do
        package_halt_cleaned $pkg
        if [[ $? -eq 1 ]]
        then
            sg_log 5 "Package $pkg has not halted cleanly at site $site"
            return 1
        fi
    done
    sg_log 0 "All packages managed by Site Controller at site $site"
    sg_log 0 "have halted cleanly."
   
    return 0
}

###############################################################################
# This function activates a site. 
#
# Internal Use only
#
#
# Arguments:  $1 : Site controller package name
#             $2 : Site name
#             
#
# Returns:    0 - success
#             1 - failure
#
###############################################################################

function sc_activate_site
{
    typeset sc=$1
    typeset site=$2
    typeset temp_file
    typeset -i ret=0
    typeset pkg
    typeset -i siteid=0
    
    postfix=$(date +"%H.%M.%S")
    temp_file=$sgsadta_tmp_dir/${SG_PACKAGE_NAME}_cmrunpkg_${postfix}_$$.tmpvalidate
    
    sg_log 5 "Function sc_activate_site $sc $site"

    #get SCCP to start
    typeset site_pkgs

    siteid=`get_site_id_by_name $site`           
    site_pkgs=`echo "${critical_pkg_list[$siteid]} ${managed_pkg_list[$siteid]}"`

    sg_log 5 "sc_activate_site site_pkgs : ${site_pkgs[@]}"  
    
    sg_log 0 "Opening site safety latch at site $site"  
    #set site mode as intermediate (3)
    sc_set_site_mode $sc $site ${INTERMEDIATE_STATE}

    if [[ $? -eq 1 ]]
    then
        sg_log 0 "Failed to open site safety latch at site $site"       
        return 1
    fi
        
    #the moment site will be set as intermediate, packages
    #having global switching turned on, will automatically be
    #started. 

    #If there are no critical and managed packages site_pkgs variable
    #will contain a space. We need to remove that space before checking
    #for not NULL
    if [[ -z "${site_pkgs% }" ]]; then
        sg_log 0 "No Critical/Managed packages managed by Site Controller"
        sg_log 0 "($sc) at site $site"
    else
    
        #cmrunpkg returns 2 as the exit value if the command fails 
        #due to a package being in a transient state (i.e. starting)
      
        ret=2
        sg_log 0 "Starting packages managed by Site Controller ($sc) at site $site"
        sg_log 0 "Executing: cmrunpkg -a ${site_pkgs[@]}"
        while [ "$ret" -eq 2 ] 
        do
            sg_log 5 "cmrunpkg -a ${site_pkgs[@]}"      

            # adding "-a" option to cmrunpkg. This will choose the most eligible 
            # node for the package to start. In case the package is a MULTI_NODE
            # package, it will run on all eligible nodes in the package's
            # configured node list.
            $cmrunpkg -a ${site_pkgs[@]} >$temp_file
            ret=$?            

            if [[ "$ret" -eq 1 ]]; then
                sg_log 0 "`cat $temp_file`"
                sg_log 5 "One or more packages managed by Site Controller ($sc)"
                sg_log 5 "failed to start"
                display_pkgs_status 5 ${site_pkgs[@]}
                sg_log 0 "Check the log files of the packages managed by Site"
                sg_log 0 "Controller for more details."
                /bin/rm -f $temp_file 2>/dev/null
                return 1
            elif [[ "$ret" -eq 2 ]]
            then
                sg_log 0 "Waiting for packages to start ..."
                display_pkgs_status 5 ${site_pkgs[@]}
                #sleep for 10 seconds
                sleep 10
            fi
        
        done
        /bin/rm -f $temp_file 2>/dev/null

        pkgs_wait_status up ${site_pkgs[@]}

        if [[ $? -eq 1 ]]
        then
            sg_log 0 "One or more package(s) failed to start"
            return 1
        fi

        sg_log 0 "Successfully started all packages managed by Site Controller"
        sg_log 0 "($sc): ${site_pkgs[@]} at site $site"

        typeset failover_pkgs
        failover_pkgs=`get_list_of_failover_pkgs`
        if [[ ! -z "${failover_pkgs[*]}" ]]; then
            sg_log 0 "Enabling global package switching for the failover packages"
            sg_log 0 "managed by Site Controller ($sc) at site $site"
            for pkg in ${failover_pkgs[@]} 
            do 
                sg_log 0 "Executing: cmmodpkg -e $pkg"
                $cmmodpkg -e "$pkg" > $temp_file
                ret=$?
                /bin/chmod 644 $temp_file 2>/dev/null

                if [[ "$ret" -eq 1 ]]; then
                    sg_log 0 "Warning: Could not enable global package switching"
                    sg_log 0 "for the failover packages managed by Site Controller"
                    sg_log 0 "`cat $temp_file`"
                    sg_log 0 "Check the log files of the packages managed by Site"
                    sg_log 0 "Controller for more details."
                else
                    sg_log 0 "Successfully enabled global package switching for the"
                    sg_log 0 "failover packages managed by Site Controller ($sc) at"
                    sg_log 0 "site $site"
                fi
                /bin/rm -f $temp_file 2>/dev/null
            done 
        fi
    fi

    #set site mode as active (4)
    sc_set_site_mode $sc $site ${ACTIVE_STATE}
    
    if [[ $? -eq 1 ]]
    then
        sg_log 0 "Failed to open site safety latch at site $site"
        return 1
    fi

    sg_log 5 "Opened site safety latch at site $site"

    return 0

}

###############################################################################
# This function sets a site to remote passive state. 
#
# Internal Use only
#
#
# Arguments:  $1 : Site controller package name
#             $2 : Site name
#             
#
# Returns:    0 - success
#             1 - failure
#
###############################################################################

function sc_remote_managed_pkg_start
{
    typeset sc=$1
    typeset remote_site=$2
    typeset temp_file
    typeset -i ret=0
    typeset pkg
    typeset -i siteid=0
    typeset remote_pkgs
    
    postfix=$(date +"%H.%M.%S")
    temp_file=$sgsadta_tmp_dir/${SG_PACKAGE}_remote_cmrunpkg_${postfix}_$$.tmpvalidate
    
    sg_log 5 "Function sc_remote_managed_pkg_start $sc $remote_site"

    siteid=`get_site_id_by_name $remote_site`           
    remote_pkgs=`echo "${remote_managed_pkg_list[$siteid]}"` 
    
    sg_log 5 "sc_remote_managed_pkg_start site_pkgs: ${remote_pkgs[@]}"  
    
    sg_log 0 "Setting site safety latch to remote passive state at site $remote_site"  
    #set site mode as remote passive 
    sc_set_site_mode $sc $remote_site ${REMOTE_PASSIVE_STATE}

    if [[ $? -eq 1 ]]
    then
        sg_log 0 "Failed to set site safety latch to remote passive state"
        sg_log 0 "at site $remote_site"       
        return 1
    fi
        
    #If there are no remote managed packages remote_pkgs variable
    #will contain a space. We need to remove that space before checking
    #for not NULL
    if [[ -z "${remote_pkgs% }" ]]; then
        sg_log 0 "No Remote Managed packages managed by Site Controller"
        sg_log 0 "($sc) at site $remote_site"        
    else
    
        #cmrunpkg returns 2 as the exit value if the command fails 
        #due to a package being in a transient state (i.e. starting)
      
        ret=2
        sg_log 0 "Starting remote managed packages managed by Site Controller"
        sg_log 0 "($sc) at site $remote_site"
        sg_log 0 "Executing: cmrunpkg -a ${remote_pkgs[@]}"
        while [ "$ret" -eq 2 ] 
        do
            sg_log 5 "cmrunpkg -a ${remote_pkgs[@]}"      

            # adding "-a" option to cmrunpkg. This will choose the most eligible 
            # node for the package to start. In case the package is a MULTI_NODE
            # package, it will run on all eligible nodes in the package's
            # configured node list.
            $cmrunpkg -a ${remote_pkgs[@]} >$temp_file
            ret=$?            

            if [[ "$ret" -eq 1 ]]; then
                sg_log 0 "`cat $temp_file`"
                sg_log 5 "One or more remote managed packages failed to start"
                display_pkgs_status 5 ${remote_pkgs[@]}
                sg_log 0 "Check the log files of the remote managed packages"
                sg_log 0 "for more details."
                /bin/rm -f $temp_file 2>/dev/null
                return 1
            elif [[ "$ret" -eq 2 ]]
            then
                sg_log 0 "Waiting for packages to start ..."
                display_pkgs_status 5 ${remote_pkgs[@]}
                #sleep for 10 seconds
                sleep 10
            fi
        
        done
        /bin/rm -f $temp_file 2>/dev/null

        pkgs_wait_status up ${remote_pkgs[@]}

        if [[ $? -eq 1 ]]
        then
            sg_log 0 "One or more package(s) failed to start"
            return 1
        fi

        sg_log 0 "Successfully started all the remote managed packages:"
        sg_log 0 "${remote_pkgs[@]} at site $remote_site"

        for pkg in ${remote_pkgs[@]} 
        do 
            packagetype=`echo "$VCLVIEW" | awk -F "[:|=]" '(($1 == "package") && \
                        ($2 == "'$pkg'") && ($3 == "type")) {print $4}'`
            if [[ "$packagetype" = "failover" ]]
            then
                sg_log 0 "Executing: cmmodpkg -e $pkg"
                $cmmodpkg -e "$pkg" >$temp_file
                ret=$?
                /bin/chmod 644 $temp_file 2>/dev/null

                if [[ "$ret" -eq 1 ]]; then
                    sg_log 0 "Warning: Could not enable global package switching for"
                    sg_log 0 "the remote site failover packages"
                    sg_log 0 "`cat $temp_file`"
                    sg_log 0 "Check the log files of the packages for more details."
                else
                    sg_log 0 "Successfully enabled global package switching for the"
                    sg_log 0 "remote site failover packages at site $remote_site"
                fi
                /bin/rm -f $temp_file 2>/dev/null
            fi            
        done 
    fi
    
    return 0

}

###############################################################################
# This function waits for a package status to come. If multiple packages are 
# passed the function waits until all packages come to the given status.
#
# Internal Use only
#
#
# Arguments:  $1 : status to wait
#             $2 : list of packages
#             
#
# Returns:    0 - success
#             1 - failure
#
###############################################################################

function pkgs_wait_status
{
    typeset wait_status=$1

    shift

    typeset pkgs=$*
    typeset -i status_reached=0
    typeset -i no_of_pkgs=`echo $pkgs | wc -w`
    typeset -i i=0

    typeset status
    typeset state
    typeset next_pkgs


    while [ "$status_reached" -lt "$no_of_pkgs" ]
    do
        #sleep for five seconds
        sleep 5
        
        # Refresh cmviewcl cache
        cache_clview
        for pkg in ${pkgs[@]}
        do
            status=`echo "$VCLVIEW" | awk -F "[:|=]" '(($1 == "package") && \
                        ($2 == "'$pkg'") && ($3 == "status")) {print $4}'`

            state=`echo "$VCLVIEW" | awk -F "[:|=]" '(($1 == "package") && \
                      ($2 == "'$pkg'") && ($3 == "state")) {print $4}'`
                        
            if [ "$status" = "$wait_status" ]
            then
                status_reached=`expr $status_reached + 1`


            #if the wait status is up and actual status is down, it will never be up
            elif [ "$status" = "down" -a "$wait_status" = "up" ]
            then
                sg_log 0 "Package $pkg is down when expected to be starting/up"
                sg_log 0 "Check the package log file for $pkg for more details"
                return 1
            #if the wait status is down and state is maintenance, ignore this package
            elif [ "$state" = "maintenance" -a "$wait_status" = "down" ]
            then
                sg_log 0 "Package $pkg is in Maintenance, hence not attempting"
                sg_log 0 "to bring down the package."
                status_reached=`expr $status_reached + 1`
            #if the wait status is down and state is detached, ignore this package
            elif [ "$state" = "detached" -a "$wait_status" = "down" ]
            then
                sg_log 0 "Package $pkg is detached, hence not attempting"
                sg_log 0 "to bring down the package."
                status_reached=`expr $status_reached + 1`                
            else
                #If wait_status is not reached the $pkg will be checked again 
                #in the next iteration. If wait_status is reached $pkg is dropped
                #from the list
                i=`expr $i + 1`
                next_pkgs[$i]=$pkg
            fi
        done

        #Next iteration will run only for the packages for which 
        #wait_status have not been reached
        pkgs=`echo "${next_pkgs[@]}"`
        unset next_pkgs

    done

    return 0

}

###############################################################################
# This function displays the status of the package at specified log level. If
# multiple packages are passed then it displays the status of all the packages.
#
# Internal Use only
#
#
# Arguments:  $1 : log level
#             $2 : list of packages
#
#
# Returns:  none
#
###############################################################################

function display_pkgs_status
{
    typeset log_level=$1
    shift
    typeset pkgs=$*
    typeset status

    # Refresh cmviewcl cache
    cache_clview 
    for pkg in ${pkgs[@]}
    do
        status=`echo "$VCLVIEW" | awk -F "[:|=]" '(($1 == "package") && \
                ($2 == "'$pkg'") && ($3 == "status")) {print $4}'`
        #Spaces are entered for formatting purpose
        sg_log ${log_level} "       Package $pkg is $status"
    done
}

##############################################################################
# This function checks whether a package is safe to failover or not
#
# Internal Use only
#
#
# Arguments:  $1 : package name
#             $2 : site mode
#             
#
# Returns:    0 - safe (package is down and clean)
#             1 - not safe (package is down and not clean) 
#             2 - package is up ( upto the caller to decide)

#             3 - not safe (package is down and in maintenance mode)
#
###############################################################################

function package_failover_safe
{
    typeset pkg=$1
    typeset site_mode=$2
    
    typeset status
    typeset state
    typeset critical_pkg 

    sg_log 5 "Function package_failover_safe $1 $2"
    
    status=`echo "$VCLVIEW" | awk -F "[:|=]" '(($1 == "package") && \
                ($2 == "'$pkg'") && ($3 == "status")) {print $4}'`

    state=`echo "$VCLVIEW" | awk -F "[:|=]" '(($1 == "package") && \
                ($2 == "'$pkg'") && ($3 == "state")) {print $4}'`

    case ${status} in
    up)
          sg_log 5 "$pkg is up"
          return 2
          ;;
    down)
          sg_log 5 "$pkg is down"
          if [ "$state" = "halted" -o "$state" = "failed" ]
          then
                if [ "$site_mode" = "${ACTIVE_STATE}" ]
                then

                    #check whether package has failed or not
                    package_failed $pkg
                
                    if [[ $? -ne 0 ]]
                    then
                        sg_log 5 "Failing over of $pkg is not safe"
                        sg_log 0 "Error: Site failover conditions are not met. Unexpected Site Controller startup"
                        return 3
                    fi
                fi
                
                #check whether the package is clean or not
                package_halt_cleaned $pkg
                
                if [[ $? -eq 1 ]]
                then
                    sg_log 5 "Package $pkg has not halted cleanly"
                    sg_log 5 "Failing over of $pkg is not safe"
                    return 1
                fi
          else
                #state other than "halted"
                sg_log 0 "$state state of package $pkg is not considered to be safe for failover"
                return 1
          fi
                
          ;;

    *)
          sg_log 0 "$status status of package $pkg is not considered to be safe for failover"
          return 1
          ;;
    
    esac

    return 0
    
}

###############################################################################
# In case of remote failover or initial start up, Site Controller checks whether 
# it is safe to start monitored and managed packages in the local site. For this 
# it has to check the condition of remote managed packages. As an example Site 
# Controller should not initiate a failover if any monitored or managed package 
# has not been halted cleanly or in maintenance mode.
#
# Internal Use only
#
#
# Arguments:  $1 : site controller name
#             $2 : remote site name
#             
#
# Returns:    0 - safe 
#             1 - not safe 
#             
#
###############################################################################

function site_failover_safe
{

    typeset sc=$1
    typeset remote_site=$2
    
    typeset remote_up_node
    typeset remote_site_mode
    typeset remote_critical_pkgs
    typeset remote_managed_pkgs
    typeset -i failed_pkgs_count=0

    sg_log 5 "Function site_failover_safe $1 $2"
    # Refresh cmviewcl -v cache
    #cache_clview

    siteid=`get_site_id_by_name $remote_site`           
    #get remote side critical package.
    remote_critical_pkgs=`echo "${critical_pkg_list[$siteid]}"`
    sg_log 5 " remote_site: $remote_site ---- remote_critical_pkgs: ${remote_critical_pkgs[@]}"
    
    sg_log 0 "Checking if site failover conditions are met"

    if [ -n "$remote_critical_pkgs" ] #critical pkg has been configured
    then

        for pkg in ${remote_critical_pkgs[@]}
        do 
                package_failover_safe $pkg ${ACTIVE_STATE}

                ret=$?

                if [[ $ret -eq 0 ]]  #if remote critical pkg is down and clean
                then
                    sg_log 0 "Critical package $pkg has failed at site $remote_site and has halted cleanly"
                    failed_pkgs_count=`expr $failed_pkgs_count + 1`
                elif [[ $ret -eq 2 ]]
                then
                    sg_log 0 "Critical package $pkg at site $remote_site is up"
                elif [[ $ret -eq 3 ]]
                then
                    sg_log 0 "Critical package $pkg is manually halted by administrator"
                    sg_log 0 "Unable to initiate site failover at site $local_site"
                    return 1
                else
                    sg_log 0 "Critical package $pkg failed to halt succcessfully"
                    sg_log 0 "Unable to initiate site failover at site $local_site"
                    return 1
                fi
        done

        if [[ $failed_pkgs_count -gt 0 ]]
        then 
                sg_log 0 "Atleast one critical package has failed at site $remote_site and has halted cleanly"
                return 0
        else
                # All critical packages are up
                sg_log 0 "All critical  packages are up in the site $remote_site"
                sg_log 0 "Error: Site failover conditions are not met. Unexpected Site Controller startup"
                sg_log 0 "Unable to initiate site failover at site $local_site"
                return 1
        fi
    fi

    #critical packages has not been configured

    #get all remote managed packages
    remote_managed_pkgs=`echo "${managed_pkg_list[$siteid]}"`
    sg_log 5 " remote_site: $remote_site ---- remote_managed_pkgs: ${remote_managed_pkgs[@]}"
    sg_log 5 "Checking status of managed packages: ${remote_managed_pkgs[@]} at site $remote_site"
    #if any managed package is up or not safe return "not safe"
    for pkg in ${remote_managed_pkgs[@]}
    do
        package_failover_safe $pkg ${ACTIVE_STATE}
		ret=$?

        if [[ $ret -eq 0 ]]  #if remote managed package is down and clean
        then
            sg_log 5 "Managed package $pkg at site $site has failed at site $remote_site and has halted clean"
            continue
        elif [[ $ret -eq 2 ]]
        then
            sg_log 0 "Managed package $pkg is up in the site $remote_site"
            sg_log 0 "Error: Site failover conditions are not met. Unexpected Site Controller startup"
            sg_log 0 "Unable to initiate site failover at site $local_site"
            return 1
         elif [[ $ret -eq 3 ]]
         then
            sg_log 0 "Managed package $pkg is manually halted by administrator"
            sg_log 0 "Unable to initiate site failover at site $local_site"
            return 1
        else
            sg_log 0 "Managed package $pkg failed to halt succcessfully"
            sg_log 0 "Unable to initiate site failover at site $local_site"
            return 1

        fi

    done

    sg_log 0 "All managed packages: ${remote_managed_pkgs[@]} have failed and halted cleanly at site $remote_site"
    sg_log 0 "Site safety latch at site $remote_site is open"
    return 0


}


function sys_log
{
    SC_TAG=" Site Controller ($SG_PACKAGE_NAME):"

    logger -t $SC_TAG "$*"
}

###############################################################################
# This function scans the list of the packages controlled by a site
# and returns the list of failover packages managed by the site controller 
# packages.
#
# Internal Use only
#
#
# Arguments: None
#
#
# Returns: None 
#
###############################################################################

function get_list_of_failover_pkgs
{
    sg_log 5 "Function get_list_of_failover_pkgs"

    typeset sc=${SG_PACKAGE_NAME}
    typeset site_pkgs
    typeset failover_pkgs
    typeset i=0
    typeset site
    
    if [[ -z "$local_site" ]]
    then
        sg_log 0 "Failed to get local site name, cluster might be in transient state. Exiting."
        to_exit=1
        return
    fi
    sg_log 5 "Function get_list_of_failover_pkgs local_site: $local_site"
    
    siteid=`get_site_id_by_name $local_site`           
    site_pkgs=`echo "${critical_pkg_list[$siteid]} ${managed_pkg_list[$siteid]}"`
    
    sg_log 5 "get_list_of_failover_pkgs site_pkgs: ${site_pkgs[@]}"
    
    # There will not be any packages when an empty site controller package is configured
    # This function  will not proceed further
    if [[ -z "${site_pkgs[*]}" ]]
    then
        to_exit=0
        return
    fi

    for pkg in ${site_pkgs[@]}
    do
        packagetype=`echo "$VCLVIEW" | awk -F "[:|=]" '(($1 == "package") && \
                    ($2 == "'$pkg'") && ($3 == "type")) {print $4}'`
        if [[ "$packagetype" = "failover" ]]
        then
            i=`expr $i + 1`
            failover_pkgs[$i]=$pkg
        fi
    done
    echo ${failover_pkgs[@]}
}

#######################################################################
#
# Function has_genres_ever_set is called to check if the site safety 
# latch value was ever set. 
#
# Arguments:  $1   : Generic Resource Name
#     
#
# Returns:     0   : EVER_SET 
#              1   : NEVER_SET (generic_resource value is 0 on all nodes)
#
#######################################################################

function has_genres_ever_set
{
    typeset genres=$1
    typeset _output
    typeset nodename
    typeset site_nodes
    typeset site_node_list
    typeset sitename

    sg_log 5 "Function has_genres_ever_set"

    postfix=$(date +"%H.%M.%S")
    _output=$sgsadta_tmp_dir/genres_${postfix}_$$
    
    for sitename in ${site_array[@]}
    do
        site_nodes=`get_site_up_nodes $sitename`
        if [ -z "${site_node_list}" ]; then
            site_node_list=`echo ${site_nodes}`
        else
            site_node_list=${site_node_list}" "${site_nodes}
        fi
    done
    
    sg_log 5 "Site Nodes which are up: ${site_node_list[@]}"
    
    for nodename in ${site_node_list[@]}
    do
        do_cmexec $nodename ${SGSBIN}/cmgetresource -r $genres >$_output 2>&1
        if [ $? -eq 0 ]; then
            value=`cat "$_output" \
                    | grep $genres \
                    | awk '{print $2}'`
        fi
        rm -f $_output 2>/dev/null

        if [ -z "${value}" ]; then
            value=0
        fi
        
        if [ ${value} -gt ${UNKNOWN_STATE_VALUE} ]; then
            return ${EVER_SET}
        fi
    done
    return ${NEVER_SET}
}

#######################################################################
#
# Function reverse_site_mode is called to make the current active site 
# to passive site and vice versa.
#
# Arguments:  None
#     
#
# Returns:    1 : Failure
#             0 : Success
#
#######################################################################

function reverse_site_mode
{
    typeset prev_active_site=$local_site
    typeset prev_passive_site=$remote_site
    typeset -i ret=0
    
    sg_log 5 "Function reverse_site_mode"

    sg_log 5 "Before Reversing the definition Local site: $local_site"
    sg_log 5 "Remote Site: $remote_site"
    
    #Reverse the Local/Remote site definition
    local_site=`echo $prev_passive_site`
    remote_site=`echo $prev_active_site`
    sg_log 5 "After Reversing the definition Local site: $local_site"
    sg_log 5 "Remote Site: $remote_site" 
   
    #Reset the both site modes to Pseudo passive state because the Remote Managed
    #package and Critical and Managed Package(s) have to be halted 
    sc_deactivate_site $sc $prev_passive_site ${PSEUDO_PASSIVE_STATE}
    if [[ $? -eq 1 ]]
    then
        sg_log 0 "Site Controller failed to deactivate $prev_passive_site site "       
        ret=1
        return $ret
    fi
    
    sc_deactivate_site $sc $prev_active_site ${PSEUDO_PASSIVE_STATE}
    if [[ $? -eq 1 ]]
    then
        sg_log 0 "Site Controller failed to deactivate $prev_active_site site "       
        ret=1
        return $ret
    fi
   
    #Check if it is safe to failover
    sg_log 5 "Check if it is safe to start at site $local_site "

    site_failover_safe $sc $remote_site
    if [[ $? -eq 1 ]]
    then
        sg_log 0 "Site Controller startup on $local_site is not safe"
        ret=1
        return $ret
    fi
    sg_log 5 "Safe to start at site $local_site"
 
    #Reverse the site modes: Promote the passive site 
    sg_log 0 "Initiating site failover procedure at site $local_site"
    sc_activate_site $sc $prev_passive_site    
    if [[ $? -eq 1 ]]
    then
        sg_log 0 "Check for any error messages in the package log file on all"
        sg_log 0 "nodes in the site \"$local_site\" for the packages managed"
        sg_log 0 "by Site Controller ( $sc )"
        sg_log 0 "Fix any issue reported in the package log files and "
        sg_log 0 "enable node switching for the packages on nodes that have failed."
        sg_log 0 "Reset the site \"$local_site\" using \"$SGSBIN/cmresetsc\""
        sg_log 0 "command and start $sc again"
        sg_log 0 "Site Controller failed to bring up workload package(s)"
        ret=1
        return $ret
    fi

    sc_remote_managed_pkg_start $sc $remote_site
    if [[ $? -eq 1 ]]
    then
        sg_log 0 "Check for any error messages in the remote managed package log"
        sg_log 0 "file on all nodes in the site \"$remote_site\" for the packages."
        sg_log 0 "Site Controller could not start remote managed packages"
        sg_log 0 "Fix any issue reported in the package log files,"
        sg_log 0 "enable node switching for the packages on nodes that have failed"
        sg_log 0 "and manually start the packages"
    fi
    
    sg_log 0 "Site Modes have been reversed."
    sg_log 0 "$prev_active_site is no longer the active Site"
    sg_log 0 "$prev_passive_site is the active site"
    sg_log 5 "Function reverse_site_mode exit"
    
    return $ret
}

########################################################################
# This function checks the if MNP or failover package, package_halted  
# flag is not set on any node. 
#
# Internal Use only
#
#
# Arguments:  $1: multi node or failover package name
#
# Returns:    0 - pkg_halted flag is set on all nodes
#             1 - otherwise
#
########################################################################
function has_package_halt_failed_on_all_nodes
{
    typeset status
    typeset pkg=$1
    typeset eligible_nodes
    typeset switching

    sg_log 5 "has_package_halt_failed_on_all_nodes $pkg"

    cache_clview
    
    #$pkg status should be down
    status=`echo "$VCLVIEW" | awk -F "[:|=]" '(($1 == "package") && \
                ($2 == "'$pkg'") && ($3 == "status")) {print $4}'`
    if [[ "$status" != "down" ]]
    then
        #sg_log 0 "ERROR: $pkg status not down"
        return 1
    fi
    
    #Eligible nodes are those nodes on which package halt_failed is not set
    eligible_nodes=`echo "$VCLVIEW" | awk -F "[:|=]" '(($1 == "package") && \
                ($2 == "'$pkg'") && ($3 == "node") && ($5 == "last_halt_failed") && \
                ($6 == "no")) {print $4}'`

   if [ -z "${eligible_nodes[@]}" ]; then
       sg_log 5 "In the Package $pkg, the following package node(s) do not have"
       sg_log 5 "pkg_halted flag set ${eligible_nodes[@]}"
       return 0
   fi

   for i in $eligible_nodes
   do
       switching=`echo "$VCLVIEW" | awk -F "[:|=]" '(($1 == "package") &&\
                   ($2 == "'$pkg'") && ($3 == "node") && ($4 == "'$i'") &&\
                   ($5 == "switching")) {print $6}'`
       if [[ "$switching" = "enabled" ]]; then
           sg_log 0 "Package $pkg can run on eligible node $i"
           return 1
       fi
   done
 
   sg_log 5 "Package $pkg halted on all nodes"
   return 0
}


########################################################################
# This function prints error message and subsequent actions to be taken 
# post SC startup failure.
#
# Internal Use only
#
#
# Arguments:  $1: local site name  
#             $2: remote site name
#             $3: site controller name 
#
# Return   : void function 
#
########################################################################
function sc_print_reset_message
{
    typeset local_site_mode=$1
    typeset remote_site_mode=$2
    typeset sc=$3
    
    if [[ "$local_site_mode" = "${INTERMEDIATE_STATE}" ]] \
    || [[ "$local_site_mode" = "${PSEUDO_PASSIVE_STATE}" ]]
    then          
        sg_log 0 "Site Controller start up on the site \"$local_site\" has failed."
        sg_log 0 "Clean up the site and start $sc again"
        sg_log 0 "Check for any error messages in the package log file on all"
        sg_log 0 "nodes in the site \"$local_site\" for the packages managed"
        sg_log 0 "by Site Controller ( $sc )"
        sg_log 0 "Fix any issue reported in the package log files and enable"
        sg_log 0 "node switching for the packages on nodes that have failed."
        sg_log 0 "Reset the site \"$local_site\" using \"$SGSBIN/cmresetsc\""
        sg_log 0 "command and start $sc again"
        sg_log 0 "Site Controller startup failed"
        sys_log "Site Safety Latch on site \"$local_site\" is in INTERMEDIATE state."
        sys_log "Site Controller ($sc) package can be started successfully only "
        sys_log "after cleaning up the site ($local_site) using cmresetsc command."
        sys_log "Site Controller ($sc) startup failed."
    fi
   
    if [[ "$remote_site_mode" = "${INTERMEDIATE_STATE}" ]] \
    || [[ "$remote_site_mode" = "${PSEUDO_PASSIVE_STATE}" ]]
    then          
        sg_log 0 "Site Controller start up on the site \"$remote_site\" has failed."
        sg_log 0 "Clean up the site \"$remote_site\" and start $sc again"
        sg_log 0 "Check for any error messages in the package log file on all"
        sg_log 0 "nodes in the site \"$remote_site\" for the packages managed"
        sg_log 0 "by Site Controller ( $sc )"
        sg_log 0 "Fix any issue reported in the package log files and enable"
        sg_log 0 "node switching for the packages on nodes that have failed."
        sg_log 0 "Reset the site \"$remote_site\" using \"$SGSBIN/cmresetsc\""
        sg_log 0 "command and start $sc again"
        sg_log 0 "Site Controller startup failed"
        sys_log "Site Safety Latch on site \"$remote_site\" is in INTERMEDIATE state."
        sys_log "Site Controller ($sc) package can be started successfully only "
        sys_log "after cleaning up the site ($remote_site) using cmresetsc command."
        sys_log "Site Controller ($sc) startup failed."
    fi
 
    return
}

########################################################################
# This function checks if the package is of failover type 
#
# Internal Use only
#
# Arguments:  $1: Package name
#
# Return   : 0 for failover package
#            1 otherwise 
#
########################################################################
function is_failover_pkg
{
    typeset pkg=$1
    typeset packagetype

    sg_log 5 "is_failover_pkg $pkg"

    packagetype=`echo "$VCLVIEW" | awk -F "[:|=]" '(($1 == "package") && \
                    ($2 == "'$pkg'") && ($3 == "type")) {print $4}'`

    if [[ "$packagetype" = "failover" ]]
    then
        return 0
    else
        return 1
    fi
}

########################################################################
# This function checks if global switching is disabled 
#
# Internal Use only
#
# Arguments:  $1: Package name
#
# Return   : 0 if global switching is disabled 
#            1 otherwise
#
########################################################################
function is_global_switching_disabled
{
    typeset pkg=$1
    typeset global_switching

    sg_log 5 "is_global_switching_disabled $pkg"

    global_switching=`echo "$VCLVIEW" | awk -F "[:|=]" '(($1 == "package") && \
                           ($2 == "'$pkg'") && ($3 == "autorun")) {print $4}'` 


    if [[ "$global_switching" = "disabled" ]]
    then
         return 0
    else
         return 1
    fi
}

########################################################################
# This function checks if package is in the maintenance mode
#
# Internal Use only
#
# Arguments:  $1: Package name
#
# Return   : 0 if package is in maintenance mode
#            1 otherwise
#
########################################################################
function is_pkg_in_maintenance
{
    typeset pkg=$1
    typeset pkg_state
    typeset ret

    pkg_state=`echo "$VCLVIEW" | awk -F "[:|=]" '(($1 == "package") && \
                      ($2 == "'$pkg'") && ($3 == "state")) {print $4}'`

    if [[ "$pkg_state" = "maintenance" ]]
    then
        ret=0
    else
        ret=1
    fi
    
    echo $ret
    return
}

########################################################################
# This function returns the list of all the packages in maintenance mode
#
# Internal Use only
#
# Arguments:  $1: List of all the packages on a given site
#
# Return   : empty list or list of all the packages in maintenance mode
#
########################################################################
function get_list_of_pkgs_in_maintenance
{
    typeset site_pkgs=$1
    typeset pkg_state
    typeset -a maintenance_pkg_list
    
    for pkg in ${site_pkgs[@]}
    do
        pkg_state=`is_pkg_in_maintenance $pkg`
        if [[ $pkg_state -eq 0 ]]
        then
            if [ -z "$maintenance_pkg_list" ]
            then
                maintenance_pkg_list="$pkg"
            else
                maintenance_pkg_list=`echo "$maintenance_pkg_list $pkg"`
            fi
        fi
    done
 
    echo "${maintenance_pkg_list[@]}"
    return
}

########################################################################
# This function checks if any of the workload packages are in maintenance
# mode.
#
# Internal Use only
#
# Arguments: None 
#
# Return   : 0 if any workload package is in maintenance
#            1 otherwise
#
########################################################################

function sg_check_any_workload_pkg_in_maintenance
{
   typeset sitename
   typeset -i siteid=0
   typeset site_pkgs
   typeset all_pkgs_list
   typeset maintenance_pkg_list

   for sitename in ${site_array[@]}
   do
        siteid=`get_site_id_by_name $sitename`
        [[ "$siteid" -eq 0 ]] && return

        site_pkgs=`echo "${critical_pkg_list[$siteid]} ${managed_pkg_list[$siteid]} ${remote_managed_pkg_list[$siteid]}"`
        
        if [ -z "${all_pkgs_list[@]}" ]; then
            all_pkgs_list="$site_pkgs"
        else
           all_pkgs_list=`echo "$all_pkgs_list $site_pkgs"`
        fi
        site_pkgs=""
    done
    
    sg_log 5 "List of all workload packages on all sites: ${all_pkgs_list[@]}"
    
    maintenance_pkg_list=`get_list_of_pkgs_in_maintenance "${all_pkgs_list[@]}"`
    
    if [ -n "${maintenance_pkg_list[@]}" ]
    then
        sg_log 0 "The following package(s) ${maintenance_pkg_list[@]}"
        sg_log 0 "is/are in maintenance mode."
        return 0
    fi   
    
    return 1
}

#######################################################################
#
# function is_package_up checks if the package is up or not
#
#######################################################################

function is_package_up
{
    typeset pkgname=$1
    typeset status

    sg_log 5 "Function is_package_up $pkgname"

    status=`echo "$VCLVIEW" | awk -F "[:|=]" '(($1 == "package") && \
                ($2 == "'$pkgname'") && ($3 == "status")) {print $4}'`

    if [[ "$status" = "up" ]]
    then
        return 0
    else
        return 1
    fi
}

#######################################################################
#
# function is_package_down checks if the package is down or not
#
#######################################################################

function is_package_down
{
    typeset ret
    typeset pkgname=$1
    typeset status

    sg_log 5 "Function is_package_down $pkgname"

    status=`echo "$VCLVIEW" | awk -F "[:|=]" '(($1 == "package") && \
                ($2 == "'$pkgname'") && ($3 == "status")) {print $4}'`

    if [[ "$status" = "down" ]]
    then
        ret=0
    else
        ret=1
    fi

    echo $ret
    return
}

#######################################################################
#
# function sc_get_monitor_service() gets the name of site controller
# monitoring service.
#
#######################################################################

function sc_get_monitor_service
{
   if [[ ${#SG_SERVICE_NAME[@]} -ne 0 ]]; then
       for (( i=0; i<${#SG_SERVICE_NAME[@]}; i++ ))
       do
           FOUND=`echo "${SG_SERVICE_CMD[$i]}" | grep -w 'sc_mon.sh'` > /dev/null 2>&1
           if [[ "$FOUND" != "" ]]
           then
               monitoring_service_name=${SG_SERVICE_NAME[$i]}
               monitoring_service_cmd=${SG_SERVICE_CMD[$i]}
               sg_log 5 "Found monitoring service ${monitoring_service_name}" 
               return 
           fi 
       done
   fi
}

#######################################################################
#
# function sc_get_removed_pkg_list populates the packages to be removed 
# during online reconfiguration.
#
#######################################################################

function sc_get_removed_pkg_list
{
   typeset sitename
   typeset -i siteid=0
   typeset prevpkg
   typeset newpkg 
   typeset -i found=0 
  
   sg_log 5 "Function sg_get_removed_pkg_list"
 
   for sitename in ${site_array[@]}
   do
       siteid=`get_site_id_by_name $sitename`
       [[ "$siteid" -eq 0 ]] && return
       sg_log 5 "site : $sitename siteid: ${site_array_ids[$siteid]}"
        
       for prevpkg in ${prev_critical_pkg_list[$siteid]}
       do
           found=0
           for newpkg in  ${critical_pkg_list[$siteid]}
           do
               if [[ "${prevpkg}" = "${newpkg}" ]]
               then
                   found=1
                   break
               fi
           done
           if (( found == 0 ))
           then
               remove_flag=1
               if [ -z "${removed_pkg_list[@]}" ]
               then
                   removed_pkg_list=${prevpkg}
               else
                   removed_pkg_list=${removed_pkg_list[@]}" "${prevpkg}
               fi
           fi
       done

       for prevpkg in ${prev_managed_pkg_list[$siteid]}
       do
           found=0
           for newpkg in  ${managed_pkg_list[$siteid]}
           do
               if [[ "${prevpkg}" = "${newpkg}" ]]
               then
                   found=1
                   break
               fi
           done
           if (( found == 0 ))
           then
               remove_flag=1
               if [ -z "${removed_pkg_list[@]}" ]
               then
                   removed_pkg_list=${prevpkg}
               else
                   removed_pkg_list=${removed_pkg_list[@]}" "${prevpkg}
               fi
           fi
       done

       for prevpkg in ${prev_remote_managed_pkg_list[$siteid]}
       do
           found=0
           for newpkg in  ${remote_managed_pkg_list[$siteid]}
           do
               if [[ "${prevpkg}" = "${newpkg}" ]]
               then
                   found=1
                   break
               fi
           done
           if (( found == 0 ))
           then
               remove_flag=1
               if [ -z "${removed_pkg_list[@]}" ]
               then
                   removed_pkg_list=${prevpkg}
               else
                   removed_pkg_list=${removed_pkg_list[@]}" "${prevpkg}
               fi
           fi
       done
   done
}  

#######################################################################
#
# function sc_get_added_pkg_list populates the packages to be added
# during online reconfiguration.
#
#######################################################################

function sc_get_added_pkg_list
{
   typeset sitename
   typeset -i siteid=0
   typeset prevpkg
   typeset newpkg
   typeset -i found=0

   sg_log 5 "Function sg_get_added_pkg_list"
 
   for sitename in ${site_array[@]}
   do
       siteid=`get_site_id_by_name $sitename`
       [[ "$siteid" -eq 0 ]] && return
       sg_log 5 "site : $sitename siteid: ${site_array_ids[$siteid]}"

       for newpkg in ${critical_pkg_list[$siteid]}
       do
           found=0
           for prevpkg in  ${prev_critical_pkg_list[$siteid]}
           do
               if [[ "${newpkg}" = "${prevpkg}" ]]
               then
                   found=1
                   break
               fi
           done
           if (( found == 0 ))
           then
               add_flag=1
               if [ -z "${added_pkg_list[@]}" ]
               then
                   added_pkg_list=${newpkg}
               else
                   added_pkg_list=${added_pkg_list[@]}" "${newpkg}
               fi
                
               if [[ "${sitename}" = "$local_site" ]]; then
                   if [ -z "${added_pkg_list_on_local_site[@]}" ]; then
                       added_pkg_list_on_local_site=${newpkg}
                   else
                       added_pkg_list_on_local_site=${added_pkg_list_on_local_site[@]}" "${newpkg}
                    fi
               fi
           fi
       done

       for newpkg in ${managed_pkg_list[$siteid]}
       do
           found=0
           for prevpkg in  ${prev_managed_pkg_list[$siteid]}
           do
               if [[ "${newpkg}" = "${prevpkg}" ]]
               then
                   found=1
                   break
               fi
           done
           if (( found == 0 ))
           then
               add_flag=1
               if [ -z "${added_pkg_list[@]}" ]
               then
                   added_pkg_list=${newpkg}
               else
                   added_pkg_list=${added_pkg_list[@]}" "${newpkg}
               fi
                
               if [[ "${sitename}" = "$local_site" ]]; then
                   if [ -z "${added_pkg_list_on_local_site[@]}" ]; then
                       added_pkg_list_on_local_site=${newpkg}
                   else
                       added_pkg_list_on_local_site=${added_pkg_list_on_local_site[@]}" "${newpkg}
                   fi
               fi
           fi
       done
        
       for newpkg in ${remote_managed_pkg_list[$siteid]}
       do
           found=0
           for prevpkg in  ${prev_remote_managed_pkg_list[$siteid]}
           do
               if [[ "${newpkg}" = "${prevpkg}" ]]
               then
                   found=1
                   break
               fi
           done
           if (( found == 0 ))
           then
               add_flag=1
               if [ -z "${added_pkg_list[@]}" ]
               then
                   added_pkg_list=${newpkg}
               else
                   added_pkg_list=${added_pkg_list[@]}" "${newpkg}
               fi
               
               if [[ "${sitename}" = "$remote_site" ]]; then
                   if [ -z "${added_pkg_list_on_local_site[@]}" ]; then
                       added_pkg_list_on_local_site=${newpkg}
                   else
                       added_pkg_list_on_local_site=${added_pkg_list_on_local_site[@]}" "${newpkg}
                   fi
               fi
           fi
       done
   done 
}

########################################################################
# This function checks if package is in detached mode
#
# Internal Use only
#
# Arguments:  $1: Package name
#
# Return   : 0 if package is in detached mode
#            1 otherwise
#
########################################################################
function is_pkg_in_detached
{
    typeset pkg=$1
    typeset pkg_state
    typeset ret

    pkg_state=`echo "$VCLVIEW" | awk -F "[:|=]" '(($1 == "package") && \
                      ($2 == "'$pkg'") && ($3 == "state")) {print $4}'`

    if [[ "$pkg_state" = "detached" ]]
    then
        ret=0
    else
        ret=1
    fi
    
    echo $ret
    return
}

########################################################################
# This function returns the list of all the packages in detached mode
#
# Internal Use only
#
# Arguments:  $1: List of all the packages on a given site
#
# Return   : empty list or list of all the packages in detached mode
#
########################################################################
function get_list_of_pkgs_in_detached
{
    typeset site_pkgs=$1
    typeset pkg_state
    typeset -a detached_pkg_list
    
    for pkg in ${site_pkgs[@]}
    do
        pkg_state=`is_pkg_in_detached $pkg`
        if [[ $pkg_state -eq 0 ]]
        then
            if [ -z "$detached_pkg_list" ]
            then
                detached_pkg_list="$pkg"
            else
                detached_pkg_list=`echo "$detached_pkg_list $pkg"`
            fi
        fi
    done
 
    echo "${detached_pkg_list[@]}"
    return
}

########################################################################
# This function checks if any of the workload packages are in Detached
# mode.
#
# Internal Use only
#
# Arguments: None 
#
# Return   : 0 if any workload package is detached
#            1 otherwise
#
########################################################################

function sg_check_any_workload_pkg_in_detached
{
   typeset sitename
   typeset -i siteid=0
   typeset site_pkgs
   typeset all_pkgs_list
   typeset detached_pkg_list

    for sitename in ${site_array[@]}
    do
        siteid=`get_site_id_by_name $sitename`
        [[ "$siteid" -eq 0 ]] && return

        site_pkgs=`echo "${critical_pkg_list[$siteid]} ${managed_pkg_list[$siteid]} ${remote_managed_pkg_list[$siteid]}"`
        
        if [ -z "${all_pkgs_list[@]}" ]; then
            all_pkgs_list="$site_pkgs"
        else
            all_pkgs_list=`echo "$all_pkgs_list $site_pkgs"`
        fi
        site_pkgs=""
    done
    
    sg_log 5 "List of all workload packages on all sites: ${all_pkgs_list[@]}"
    
    detached_pkg_list=`get_list_of_pkgs_in_detached "${all_pkgs_list[@]}"`
    
    if [ -n "${detached_pkg_list[@]}" ]
    then
        sg_log 0 "The following package(s) ${detached_pkg_list[@]}"
        sg_log 0 "is/are in detached mode."
        return 0
    fi   
    
    return 1
}

########################################################################
# This function checks if a role reversal can happen or not
#
# Internal Use only
#
# Arguments: None 
#
# Return   : 0 if role reversal can happen
#            1 otherwise
#
########################################################################

function sg_can_role_reversal_happen
{
    typeset ret=0
    typeset sc=${SG_PACKAGE_NAME}
    typeset sc_pkg_state

    if [[ "${SG_FAILOVER_POLICY}" = "site_preferred_manual" ]]
    then
        sg_log 0 "Cannot failover site controller ($sc) to site $remote_site as"                            
        sg_log 0 "site controller failover policy is site_preferred_manual."
        sg_log 0 "Hence, cannot bring up workload packages on site $remote_site"
        ret=1
    fi  
    
    # If there are any workload packages in maintenance then do not 
    # initiate a site role reversal.

    cache_clview
    
    sc_pkg_state=`is_pkg_in_detached $sc`
    if [[ $sc_pkg_state -eq 0 ]]
    then
        sg_log 0 "Site Controller Package is in Detached state."
        sg_log 0 "Hence unable to initiate site role reversal"
        sg_log 0 "Re-attach the node and then attempt to start ${SG_PACKAGE_NAME}."            
        ret=1
    fi    
        
    sg_check_any_workload_pkg_in_maintenance
    if [ $? -eq 0 ]
    then
        sg_log 0 "Hence unable to initiate site role reversal"
        sg_log 0 "Turn off the maintenance mode for these package(s)"            
        sg_log 0 "and then attempt to start ${SG_PACKAGE_NAME}."
        ret=1
    fi    

    # If there are any workload packages in maintenance then do not 
    # initiate a site role reversal.
    sg_check_any_workload_pkg_in_detached
    if [ $? -eq 0 ]
    then
        sg_log 0 "Hence unable to initiate site role reversal"
        sg_log 0 "Either halt these package(s) or re-attach the node" 
        sg_log 0 "and then attempt to start ${SG_PACKAGE_NAME}."
        ret=1
    fi    

    echo $ret
    return
}

###############################################################################
# This function returns the node where SC is up or detached
#
# Internal Use only
#
# Arguments:  $1 : Site controller
#
# Returns:    node where SC is up/detached - success
#             (null) - no node is running SC
#
###############################################################################

function get_node_where_sc_is_up
{
    typeset sc_pkg=$1
    typeset sc_node
    
    sg_log 5 "Function get_node_where_sc_is_up"

    # Refresh cmviewcl cache
    cache_clview

    sc_node=`echo "$VCLVIEW" | awk -F "[:|=]" '(($1 == "package") && \
             ($2 == "'$sc_pkg'") && ($3 == "node") && ($5 == "status")) && \
             (($6 == "up") || ($6 == "detached")) {print $4}'`

    sg_log 5 "$sc_node is a node where site controller is up"

    echo ${sc_node}
}

###############################################################################
# This function returns the list of nodes where SC is not up
#
# Internal Use only
#
# Arguments:  $1 : Site controller
#
# Returns:    node list where SC is not up - success
#             (null) - no node is running SC
#
###############################################################################

function get_list_of_nodes_where_sc_is_not_up
{
    typeset sc_node=$1
    typeset active_site_up_nodes
    typeset passive_site_up_nodes
    typeset cluster_up_nodes
    typeset non_sc_node_list
    
    sg_log 5 "Function get_list_of_nodes_where_sc_is_not_up"

    active_site_up_nodes=`get_site_up_nodes $local_site`
    passive_site_up_nodes=`get_site_up_nodes $remote_site`
    cluster_up_nodes=${active_site_up_nodes[@]}" "${passive_site_up_nodes[@]}

    for node in ${cluster_up_nodes[@]}
    do
        if [[ "$node" != "$sc_node" ]]
        then
            if [ -z "${non_sc_node_list}" ]
            then
                non_sc_node_list=`echo ${node}`
            else
                non_sc_node_list=${non_sc_node_list}" "${node}
            fi
        fi
    done

    sg_log 5 "${non_sc_node_list[@]} is a list of nodes where site controller"
    sg_log 5 "is not up"

    echo ${non_sc_node_list[@]}
}

###############################################################################
# This function sets the passed mode for all nodes in the nodelist ($2).
# Nodes can be set to active, passive, pseudo_passive remote_passive or 
# intermediate. 
#
# Internal Use only
#
# Arguments:  $1 : Site controller Package Name
#             $2 : Node List or a single node-name
#             $3 : Mode (mode can be active, passive, remote_passive 
#                  pseudo_passive and intermediate)
#
# Returns:    0 - success
#             1 - failure
#
###############################################################################

function sc_set_nodes_mode
{
    typeset sc=$1
    typeset node_list=$2
    typeset mode=$3
    typeset -i value
    typeset node_status
    typeset -i retry_flag=0
    typeset _output=$sgsadta_tmp_dir/cmexec_setnodes_out.$$
    
    sg_log 5 "Function sc_set_nodes_mode $1 $2 $3"

    sg_log 5 "[Generic Resource Value]: $gen_res_sc"
    
    if [[ "${mode}" = "${ACTIVE_STATE}" ]]
    then
        value=`echo ${ACTIVE_STATE_VALUE}`
    elif [[ "${mode}" = "${PASSIVE_STATE}" ]]
    then
        value=`echo ${PASSIVE_STATE_VALUE}`
    elif [[ "${mode}" = "${PSEUDO_PASSIVE_STATE}" ]]
    then
        value=`echo ${PSEUDO_PASSIVE_STATE_VALUE}`
    elif [[ "${mode}" = "${REMOTE_PASSIVE_STATE}" ]]
    then
        value=`echo ${REMOTE_PASSIVE_STATE_VALUE}`
    elif [[ "${mode}" = "${INTERMEDIATE_STATE}" ]]
    then
        value=`echo ${INTERMEDIATE_STATE_VALUE}`
    else
        sg_log 0 "Error: illegal site safety latch state: $mode"
        return 1
    fi

    #set generic resource value to all nodes in site_nodes
    for nodename in ${node_list[@]}
    do
        sg_log 5 "Executing: cmexec $nodename $SGSBIN/cmsetresource -r $gen_res_sc $value"
        
        do_cmexec $nodename ${SGSBIN}/cmsetresource -r $gen_res_sc $value >$_output 2>&1       
        if [[ $? -ne 0 ]]
        then
            # check whether the node is up in the cluster
            # Refresh cmviewcl cache
            cache_clview
            node_status=`echo "$VCLVIEW" | awk -F "[:|=]" '(($1 == "node") && \
                         ($2 == "'$nodename'") && ($3 == "status")){print $4}'`
            if [[ "$node_status" = "up" ]]; then
                _out="$(cat $_output)"
                sg_log 0 "$_out"  
                sg_log 0 "Error: Failed to set site safety latch value on $nodename"
                sys_log "Warning: Site Controller $sc not able to set site"
                sys_log "safety latch value to $mode on node $nodename. Check the"
                sys_log "Site Controller package $sc log for more details"
                retry_flag=1
                # We used to return from here. But now we continue to check the 
                # status on the next node. And Set retry flag.
            fi
        fi
        rm -f $_output 2>/dev/null
    done
    
    if (( retry_flag == 1 )); then
         return 1
    fi    

    return 0
}

###############################################################################
# This function attempts to exit gracefully when the halt is a non-user
# initiated halt is done.
#
# Internal Use only
#
# Arguments:  $1 : Site controller Package Name
#             $2 : Node List or a single node-name
#
# Returns:    None
#
#
###############################################################################

function sc_initiate_graceful_exit
{
    typeset sc=$1
    typeset non_sc_node_list
    typeset sc_pkg_state
    typeset sc_node
    typeset -i ret=0
    
    sg_log 5 "Function sc_initiate_graceful_exit $1"
    sg_log 0 "Initiating graceful exit procedure..."
    
    sc_deactivate_site $sc $remote_site ${PSEUDO_PASSIVE_STATE}
    if [[ $? -eq 1 ]]
    then
        sg_log 0 "Site Controller failed to deactivate $remote_site site "
    fi
    
    sc_deactivate_site $sc $local_site ${PSEUDO_PASSIVE_STATE}
    if [[ $? -eq 1 ]]
    then
        sg_log 0 "Site Controller failed to deactivate $local_site site"
    fi
   
    sc_node=`get_node_where_sc_is_up $sc`

    non_sc_node_list=`get_list_of_nodes_where_sc_is_not_up $sc_node`
 
    sg_log 0 "Setting mode to ${PASSIVE_STATE} on ${non_sc_node_list[@]}"
    sc_set_nodes_mode $sc "$non_sc_node_list" ${PASSIVE_STATE}
    ret=$?    
    if [[ $ret -eq 1 ]] 
    then
        sg_log 0 "Failed to set mode to ${PASSIVE_STATE} on ${non_sc_node_list}"
    fi
   
    sc_pkg_state=`is_pkg_in_detached $sc`
    if [[ $sc_pkg_state -eq 0 ]]
    then
        sg_log 0 "Node ${sc_node} is in detached mode. Cannot "
        sg_log 0 "set mode to ${PASSIVE_STATE} on ${sc_node}"
    else
        sg_log 0 "Setting mode to ${PASSIVE_STATE} on $sc_node"
        sc_set_nodes_mode $sc "$sc_node" ${PASSIVE_STATE}
        ret=$?    
        if [[ $ret -eq 1 ]] 
        then
            sg_log 0 "Failed to set mode to ${PASSIVE_STATE} on ${sc_node}"
        fi
    fi
    
    return   
}
