####################################################################
# (C) Copyright 2015-2017 Hewlett Packard Enterprise Development LP.
# @(#) Serviceguard UPCC Package Configuration Script.
# @(#) Product Name                :  HP Serviceguard
# @(#) Product Version             :  A.12.10.00
# @(#) Patch Name                  :  
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HP Serviceguard.
#
# Changing this file may lead to unrecoverable package startup
# and/or shutdown problems.
#
###################################################################

###########################
# Globals
###########################
typeset vmfs_output=""
typeset package_uuids=""
typeset package_disk_type=""
typeset package_node_names=""
typeset read_only_package_nodes=""
typeset package_uuids_read_only=""
typeset vmware_package_node_names=""
typeset local_node_type=""

typeset -i type_esx=1
typeset -i type_vcenter=2
typeset -i MC_CONFIGURED=0
typeset -i XDC_CONFIGURED=0
typeset -i VALID_XDC_FAILURE=0
typeset -i SG_RUNNING_NODE_DURING_VALIDATE

typeset -i VMWARE_ONLY_CLUSTER=1
typeset -i UNSUPPORTED_CLUSTER=3
typeset -i CLUSTER_WITHOUT_VMWARE=4
typeset -i MIX_VMWARE_AND_PHYSICAL_CLUSTER=2
typeset -i PACKAGE_HAS_PHYSICAL_NODE=1

#-----------------------------------------------------------------------------------------------#
# return value              | used in op code   | used to                                       #
#---------------------------|-------------------|-----------------------------------------------#
# set_pr_module_required    | start             | To remove PR in case previous failed          #
#                           |                   | node is physical                              #
#---------------------------|-------------------|-----------------------------------------------#
# set_pr_module_skip        | validate/start/   | Set to skip PR module by the eligible node    #
#                           | reconfigure-start |                                               #
#---------------------------|-------------------|-----------------------------------------------#
# set_storage_module_skip   | validate          | Set to skip PR/VG/FS(partially) by the nodes  #
#                           |                   | cmapplyconf/cmcheckconf                       #
#-----------------------------------------------------------------------------------------------#
# set_vmfs_validate_failed  | validate          | Set when VMFS module validation failed or     #
#                           |                   | attach failed during validation. When set     #
#                           |                   | PR module will be skipped, and continue with  #
#                           |                   | other modules.                                #
#-----------------------------------------------------------------------------------------------#
# Eligible Node: That which has the lowest node id in the configured package nodes or           #
#                a node on which the package is running                                         #
#-----------------------------------------------------------------------------------------------#
typeset -i set_pr_module_required=97
typeset -i set_pr_module_skip=98
typeset -i set_storage_module_skip=99
typeset -i set_vmfs_validate_failed=100

declare -a TMP_SG_VMDK_FILE_NAME
declare -a TMP_SG_DATASTORE_NAME
declare -a TMP_SG_SCSI_CONTROLLER
declare -a TMP_SG_DISK_TYPE

###########################
# Source utility functions.
###########################

if [[ -f ${SG_UTILS} ]]; then
    . ${SG_UTILS}
    if (( $? != 0 ))
    then
        echo "ERROR: Unable to source package utility functions file: ${SG_UTILS}"
        exit 1
    fi
else
    echo "ERROR: Unable to find package utility functions file: ${SG_UTILS}"
    exit 1
fi

####################################################################
##
## Get the environment for this package through utility function
## sg_source_pkg_env().
##
####################################################################
sg_source_pkg_env $*

####################################################################
## Source the VMFS utility functions:
####################################################################
VMFS_UTIL=${SGCONF}/scripts/sg/vmfs_util.sh
if [[ -f ${VMFS_UTIL} ]]; then
    . ${VMFS_UTIL}
    if (( $? != 0 ))
    then
        echo "ERROR: Unable to source package vmfs utility functions file: ${VMFS_UTIL}"
        exit 1
    fi
else
    echo "ERROR: Unable to find package vmfs utility functions file: ${VMFS_UTIL}"
    exit 1
fi
VMFS_COMMON_UTIL=${SGCONF}/scripts/sg/vmfs_common_util.sh
if [[ -f ${VMFS_COMMON_UTIL} ]]; then
    . ${VMFS_COMMON_UTIL}
    if (( $? != 0 ))
    then
        echo "ERROR: Unable to source package vmfs utility functions file: ${VMFS_COMMON_UTIL}"
        exit 1
    fi
else
    echo "ERROR: Unable to find package vmfs utility functions file: ${VMFS_COMMON_UTIL}"
    exit 1
fi

function sg_reconfigure_attach_vmfs_disk
{
    typeset -i attach_ret=0

    check_if_local_uuid_is_part_of_the_cluster

    create_tmp_array_validate_attach_block $my_op_code $SG_LAST_RUN_NODE

	if check_xdc_is_configured; then
        XDC_CONFIGURED=1
    fi

    if (( ${#TMP_SG_VMDK_FILE_NAME[@]} > 0 ))
    then
        if sg_check_if_reconfig_has_failed
        then
            for I in ${TMP_SG_VMDK_FILE_NAME[@]}
            do
                sg_log 999 "WARNING: Not attempting to attach vmdk_file_name $I"
            done
        else
            sg_log 0 "Attempting to attach VMDK files:"
            _R=0
            while (($_R < ${#TMP_SG_VMDK_FILE_NAME[*]}))
            do
                echo "[${TMP_SG_DATASTORE_NAME[$_R]}] ${TMP_SG_VMDK_FILE_NAME[$_R]}" | sed -e 's/^/   /'
                (( _R = _R + 1 ))
            done
            
            query_attach_detach_disk "attach" $local_uuid $local_host_name $my_op_code
            attach_ret=$?
            if [[ $attach_ret != 0 ]] && [[ $XDC_CONFIGURED == 1 ]]; then
                sg_set_reconfig_failure
                handle_errors_based_on_op $attach_ret $my_op_code "attach"
                cleanup_before_exit $set_pr_module_skip
            elif [[ $attach_ret == 0 ]]; then
                cleanup_before_exit $set_pr_module_skip
            else
                sg_set_reconfig_failure
                handle_errors_based_on_op $attach_ret $my_op_code "attach"
                cleanup_before_exit $attach_ret
            fi
        fi
    else
        # No VMFS attributes exit now
        cleanup_before_exit $attach_ret
    fi
}

function sg_detach_vmfs_disk
{
    typeset -i detach_ret=0
    
    declare -a TMP_SG_VMDK_FILE_NAME
    declare -a TMP_SG_DATASTORE_NAME
    declare -a TMP_SG_SCSI_CONTROLLER
    declare -a TMP_SG_DISK_TYPE
    typeset -i R=0

    check_if_local_uuid_is_part_of_the_cluster

    while (($R < ${#SG_VMDK_FILE_NAME[*]}))
    do
        TMP_SG_VMDK_FILE_NAME[$R]=${SG_VMDK_FILE_NAME[$R]}
        TMP_SG_DATASTORE_NAME[$R]=${SG_DATASTORE_NAME[$R]}
        TMP_SG_SCSI_CONTROLLER[$R]=${SG_SCSI_CONTROLLER[$R]}
        TMP_SG_DISK_TYPE[$R]=${SG_DISK_TYPE[$R]}

        (( R = R + 1 ))
    done

    if (( ${#TMP_SG_VMDK_FILE_NAME[*]} == 0 )); then
        # No VMFS attributes exit now
        cleanup_before_exit $detach_ret
    fi

    sg_log 0 "Attempting to detach VMDK files:"
    _R=0
    while (($_R < ${#TMP_SG_VMDK_FILE_NAME[*]}))
    do
        echo "[${TMP_SG_DATASTORE_NAME[$_R]}] ${TMP_SG_VMDK_FILE_NAME[$_R]}" | sed -e 's/^/   /'
        (( _R = _R + 1 ))
    done

    query_attach_detach_disk "detach" $local_uuid $local_host_name $my_op_code
    detach_ret=$?

    if [[ $detach_ret == 0 ]]; then
        cleanup_before_exit $detach_ret
    else
        handle_errors_based_on_op $detach_ret $my_op_code "detach"
        cleanup_before_exit 1
    fi

}

function sg_attach_vmfs_disk
{
    typeset -i attach_ret=0
    typeset -i detach_ret=0
    typeset -i type_value=0
    typeset -i pr_module_status=$set_pr_module_skip

    if check_xdc_is_configured; then
        XDC_CONFIGURED=1
    fi

    create_tmp_array_validate_attach_block $my_op_code $SG_LAST_RUN_NODE

    if (( ${#TMP_SG_VMDK_FILE_NAME[*]} == 0 )); then
        # No VMFS attributes exit now
        cleanup_before_exit $attach_ret
    fi

    check_if_local_uuid_is_part_of_the_cluster

    # Check if the previous node has failed
    if check_if_the_previous_run_node_failed
    then
        prev_failed_node_name=`get_node_name_from_nodeid $SG_LAST_RUN_NODE`
        prev_failed_node_type=`cat "$cmviewcl_output" | awk '/^node:[^|]*\|virt_tech=/' | grep -w $prev_failed_node_name | cut -d "=" -f 2`

        # This function will return the values based on the failed node type and local node type
        # for the value's definition check the function get_current_prev_node_value
        get_current_prev_node_value $prev_failed_node_type
        type_value=$?

        case $type_value in
        1|3)
            # This is case when failed node was VMWare
            # and the disks were not detached. Even if the current 
            # node is physical we still need to detach the disks
            uuid=`get_uuid_from_node_name $prev_failed_node_name`
            sg_log 0 "Attempting to detach disks since previous node has failed:"
            _R=0
            while (($_R < ${#TMP_SG_VMDK_FILE_NAME[*]}))
            do
                echo "[${TMP_SG_DATASTORE_NAME[$_R]}] ${TMP_SG_VMDK_FILE_NAME[$_R]}" | sed -e 's/^/   /'
                (( _R = _R + 1 ))
            done

            query_attach_detach_disk "detach" $uuid $prev_failed_node_name $my_op_code
            detach_ret=$?
            if [[ $detach_ret != 0 ]]; then
                if [[ $local_node_type == 'none' ]]; then
                    sg_log 0 "WARNING: Failed to detach the VMDK disks which were attached to failed"\
                             "node \"$prev_failed_node_name\"."
                    sg_log 0 "         Ensure that the disks are detached manually using VMware"\
                             "recommended methods."
                    # return 0 and continue with PR and other modules
                    cleanup_before_exit 0
                else
                    handle_errors_based_on_op $detach_ret $my_op_code "detach"
                fi
            else
                if [[ $local_node_type == 'none' ]]; then
                    cleanup_before_exit 0
                fi
            fi
        ;;
        2)
            # This is case when failed node was PHY
            # and PR would still be there.
            pr_module_status=$set_pr_module_required
        ;;
        4)
            # Do nothing exit now
            cleanup_before_exit 0
        ;;
        esac
    fi

    if [[ $local_node_type == 'none' ]]; then
        cleanup_before_exit 0
    fi

    sg_log 0 "Attempting to attach VMDK files:"
    _R=0
    while (($_R < ${#TMP_SG_VMDK_FILE_NAME[*]}))
    do
        echo "[${TMP_SG_DATASTORE_NAME[$_R]}] ${TMP_SG_VMDK_FILE_NAME[$_R]}" | sed -e 's/^/   /'
        (( _R = _R + 1 ))
    done

    query_attach_detach_disk "attach" $local_uuid $local_host_name $my_op_code
    attach_ret=$?

    if [[ $attach_ret != 0 ]] && [[ $XDC_CONFIGURED == 1 ]]; then
        handle_errors_based_on_op $attach_ret $my_op_code "attach"
        cleanup_before_exit $pr_module_status
    elif [[ $attach_ret == 0 ]]; then
        # pr_module_status can have two different values
        # set_pr_module_required; if the failed node is PHY
        # set_pr_module_skip; if the failed node is VM or in case of 
        # normal start/graceful failover.
        cleanup_before_exit $pr_module_status
    else
        handle_errors_based_on_op $attach_ret $my_op_code "attach"
        cleanup_before_exit 1
    fi
}

function sg_validate_vmfs
{

    typeset -i ret=0
    typeset -i query_ret=0
    typeset -i attach_ret=0
    typeset -i cluster_type=0
    typeset -i _node_exist_in_mc_list=0

    if (( ${#SG_VMDK_FILE_NAME[*]} > 0 )); then

        check_if_local_uuid_is_part_of_the_cluster

        check_and_create_package_node_list
        ret=$?
        if [[ $ret == $set_storage_module_skip ]]; then
            # Skip this module
            # call cleanup_before_exit with '99' so we will skip the other required modules
            sg_log 5 "INFO: The VMDK files configured on the node $local_host_name are from read-only array."
            sg_log 5 "INFO: Skipping the validation of VMFS module on this node."
            cleanup_before_exit $set_storage_module_skip
        fi

        check_if_current_node_is_eligible_to_run_applyconf
        ret=$?
        if [[ $ret == $set_storage_module_skip ]]; then
            # Skip this module
            # call cleanup_before_exit with '99' so we will skip the other required modules
            sg_log 5 "INFO: Package configured with VMFS storage. Validation of VMFS module will be"\
                     "done on node which has lowest node id or on the node where package is running."
            cleanup_before_exit $set_storage_module_skip
        fi

        # Create the package node list with VMWare guests alone
        # If MC is configured the vmware_package_node_names will have 
        # uuid's of nodes connected to RW luns
        vmware_package_node_names=`create_vmware_only_package_node_list "$package_node_names"`

        # Update the global variable package_uuids
        package_uuids=`get_node_uuids_from_node_name "$vmware_package_node_names"`

        # Check if MC is configured; if so we need to query for all the VMware nodes
        # on both RO & RW site. And the validation of slots availability alone will be
        # done on RO site.
        if [ $MC_CONFIGURED == 1 ]; then
            vmware_package_node_names_read_only=`create_vmware_only_package_node_list \
                                                "$read_only_package_nodes"`
            package_uuids_read_only=`get_node_uuids_from_node_name \
                                    "$vmware_package_node_names_read_only"`
        fi

        # create TMP_ List of arrays:
        create_tmp_array_validate_attach_block $my_op_code $SG_RUNNING_NODE_DURING_VALIDATE

        # This is the case when you do a cmapplyconf/cmcheckconf of a package 
        # which is running and without any changes wrt VMFS attributes
        # We will assume that all that is existing is fine.
        if (( ${#TMP_SG_VMDK_FILE_NAME[*]} == 0 )) \
           && [ $SG_RUNNING_NODE_DURING_VALIDATE != 0 ]; then
            cleanup_before_exit $set_pr_module_skip
        fi

        query_attach_detach_disk "query" $uuid $local_host_name $my_op_code
        query_ret=$?

        if [[ $query_ret == 0 ]]; then
            sg_log 5 "SUCCESS: The query was successful"
        else
            handle_errors_based_on_op $query_ret $my_op_code "query"
        fi

        validate_data_store $my_op_code "query"
        ret=$?
        if [[ $ret != 0 ]]; then
            echo "export SFC_VMFS_EXITED_BEFORE_ATTACH=\"1\"" >> $SG_SRC_FRM_CHILD
            cleanup_before_exit $set_vmfs_validate_failed
        fi

        # Since we need to to validate the VG and FS we will attach the
        # vmdk files before we go to those modules.

        query_attach_detach_disk "attach" $local_uuid $local_host_name $my_op_code
        attach_ret=$?

        if [[ $attach_ret == 0 ]]; then
            cleanup_before_exit $set_pr_module_skip
        else
            sg_log 0 "ERROR: Failed to attach disks to node $local_host_name during validation."
            handle_errors_based_on_op $attach_ret $my_op_code "attach"
            cleanup_before_exit $set_vmfs_validate_failed
        fi

    else
        # No VMFS attribs configured
        cleanup_before_exit $ret
    fi
}

# MAIN
my_op_code=${1}
cache_local_variables
local_host_name=${SG_HOST_NAME%%.*}
local_node_type=`$CMVIEWCL -vf line -s config | grep -w $local_host_name | awk -F= '/^node:[^|]*\|virt_tech=/ {print $2}'`

if [[ "$local_node_type" == "RHEV" ]] || [[ "$local_node_type" == "KVM" ]]; then
    # If it neither vmware nor physical we will exit
    # But config should ensure that VMFS attributes are not configured
    # when such nodes are configured.
    cleanup_before_exit 0
fi

sg_log 5 "sg_vmfs"

case ${1} in
     start)
           if [[ $local_node_type == "vmware" ]] || [[ $local_node_type == "none" ]]; then
               sg_attach_vmfs_disk
           fi
           ;;
     stop)
           if [[ $local_node_type == "vmware" ]]; then
               sg_detach_vmfs_disk
           fi
           ;;
     validate)
           if [[ $local_node_type == "vmware" ]]; then
               sg_validate_vmfs
           fi
           ;;
    reconfigure_stop)           
           # No Op
           :
           ;;
    reconfigure_start)           
           if [[ $local_node_type == "vmware" ]]; then
               sg_reconfigure_attach_vmfs_disk
           fi
           ;;
     *)
           sg_log 0 "INFO: Unknown operation - $1"
           ;;
esac
cleanup_before_exit 0
