####################################################################
# (C) Copyright 2015-2016 Hewlett Packard Enterprise Development LP.
# @(#) Serviceguard UPCC Package Configuration Script.
# @(#) Product Name                :  HP Serviceguard
# @(#) Product Version             :  A.12.10.00
# @(#) Patch Name                  :  
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HP Serviceguard.
#
# Changing this file may lead to unrecoverable package startup
# and/or shutdown problems.
#
###################################################################

###########################
# GLOBAL ERRORS
###########################
typeset -i USAGE_ERROR=1 
typeset -i PARSING_ERROR=2 
typeset -i NOT_EXIST_ERROR=3 
typeset -i CONFIGURATION_ISSUE=4 
typeset -i FAILED_TO_POPULATE_OBJECT=5
typeset -i WRONG_CREDENTIAL=6 
typeset -i HOST_UNREACHABLE=7 
typeset -i FAIL_TO_ATTACH_DISK_ERROR=8 
typeset -i FAIL_TO_DETACH_DISK_ERROR=9 
typeset -i INCORRECT_JAVA_VERSION=10
typeset -i FILE_CORRUPTED_ERROR=11
typeset -i NO_VCENTER_CONFIGURED=13
typeset -i TGT_HOST_NOT_ELIGIBLE=14
typeset -i UNDEFINED_ERROR=254

###########################
# Globals
###########################
typeset -i type_esx=1
typeset -i type_vcenter=2

typeset -i vmfs_cmviewcl_retry_count=3
typeset -i vmfs_cmviewcl_retry_interval=2

if [ -f /etc/redhat-release ]
then
    LOGGER="/usr/bin/logger"
else
    LOGGER="/bin/logger"
fi

##### Error Handling Functions #####

function check_if_usage_error
{
    _ret=$1
    case $_ret in
        $USAGE_ERROR|$PARSING_ERROR)
            return 0
        ;;
        *)
            return 1
        ;;
    esac
}

function check_if_config_or_missing_error
{
    _ret=$1
    case $_ret in
        $NOT_EXIST_ERROR|$CONFIGURATION_ISSUE)
            return 0
        ;;
        *)
            return 1 
        ;;
    esac
}

function check_if_vm_object_error
{
    _ret=$1
    if [[ $_ret == $FAILED_TO_POPULATE_OBJECT ]];then
        return 0
    fi
    return 1
}

function check_if_wrong_credential_error
{
    _ret=$1
    if [[ $_ret == $WRONG_CREDENTIAL ]];then
        return 0
    fi
    return 1
}

function check_if_host_unreachable_error
{
    _ret=$1
    if [[ $_ret == $HOST_UNREACHABLE ]];then
        return 0
    fi
    return 1
}

function check_if_fail_to_attach_disk_error
{
    _ret=$1
    if [[ $_ret == $FAIL_TO_ATTACH_DISK_ERROR ]];then
        return 0
    fi
    return 1
}

function check_if_fail_to_detach_disk_error
{
    _ret=$1
    typeset -i _ret_val=0
    _error_str=""
    if [[ $_ret == $FAIL_TO_DETACH_DISK_ERROR ]];then
        return 0
    fi
    return 1
}

function check_if_credential_xml_file_corrupted
{
    _ret=$1
    if [[ $_ret == $FILE_CORRUPTED_ERROR ]];then
        return 0
    fi
    return 1
}

function check_if_undefined_error
{
    # This function should be checked after all the other cases
    _ret=$1
    if [[ $_ret == $UNDEFINED_ERROR ]];then
        return 0
    fi
    return 1
}

function print_detach_fail_errors
{
    sg_log 0 "$vmfs_output"
}

##### Error Handling Functions #####

######## Common Functions ########

#### command wrappers utility functions ####

function check_if_cluster_is_configured_with_esx_or_vcenter
{
    if cat $cmviewcl_output | grep -q vcenter_name
    then
        return $type_vcenter
    elif cat $cmviewcl_output | grep -q esx_name
    then
        return $type_esx
    fi
}

function get_esx_host_name_from_uuid
{
    _uuid=$1
	nname=`cat $cmviewcl_output | awk '/^node:[^|]*\|uuid=/' | grep -w $_uuid | sed 's/|uuid='${_uuid}'//;s/node://'`
    esx_name=`cat $cmviewcl_output | grep -w $nname | awk -F= '/^node:[^|]*\|esx_name=/ {print $2}'`
    echo $esx_name
}

function get_vcenter_name
{
    echo `cat $cmviewcl_output | awk -F= '/^vcenter_name=/ {print $2}'`
}
