####################################################################
# (C) Copyright 2016-2017 Hewlett Packard Enterprise Development LP.
# (C) Copyright 2015 Hewlett Packard Enterprise Development LP.
# @(#) Serviceguard UPCC Package Configuration Script.
# @(#) Product Name                :  HP Serviceguard
# @(#) Product Version             :  A.12.10.00
# @(#) Patch Name                  :  
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HP Serviceguard.
#
# Changing this file may lead to unrecoverable package startup
# and/or shutdown problems.
#
###################################################################

###########################
# Globals
###########################
typeset vmfs_output=""
declare -a TMP_SG_VMDK_FILE_NAME
declare -a TMP_SG_DATASTORE_NAME
declare -a TMP_SG_SCSI_CONTROLLER
declare -a TMP_SG_DISK_TYPE
typeset -i SG_RUNNING_NODE_DURING_VALIDATE
typeset local_node_id=""
typeset local_node_type=""
typeset local_host_name=""
typeset local_node_type=""
#---------------------------|-------------------|-----------------------------------------------#
# set_pr_module_skip        | stop/             | Set to skip PR module by the eligible node    #
#                           | reconfigure-stop  |                                               #
#---------------------------|-------------------|-----------------------------------------------#
# Eligible Node: That which has the lowest node id in the configured package nodes or           #
#                a node on which the package is running                                         #
#-----------------------------------------------------------------------------------------------#
typeset -i set_pr_module_required=97
typeset -i set_pr_module_skip=98
typeset -i set_storage_module_skip=99

# File Handling
date_prefix=$(date +"%H_%M_%S_%N")
str_prefix="Log_"$$_
log_prefix=$str_prefix$date_prefix
typeset vmfs_tmp_dir="/var/tmp"
typeset cmviewcl_output=$vmfs_tmp_dir/$log_prefix"_cmviewcl.$$"
typeset _input_file=$vmfs_tmp_dir/$log_prefix"_input_file_$$.txt"
typeset CMVIEWCL="$SGSBIN/cmviewcl"

###########################
# Source utility functions.
###########################

if [[ -f ${SG_UTILS} ]]; then
    . ${SG_UTILS}
    if (( $? != 0 ))
    then
        echo "ERROR: Unable to source package utility functions file: ${SG_UTILS}"
        exit 1
    fi
else
    echo "ERROR: Unable to find package utility functions file: ${SG_UTILS}"
    exit 1
fi

####################################################################
##
## Get the environment for this package through utility function
## sg_source_pkg_env().
##
####################################################################
sg_source_pkg_env $*

####################################################################
## Source the VMFS utility functions:
####################################################################
VMFS_UTIL=${SGCONF}/scripts/sg/vmfs_util.sh
if [[ -f ${VMFS_UTIL} ]]; then
    . ${VMFS_UTIL}
    if (( $? != 0 ))
    then
        echo "ERROR: Unable to source package vmfs utility functions file: ${VMFS_UTIL}"
        exit 1
    fi
else
    echo "ERROR: Unable to find package vmfs utility functions file: ${VMFS_UTIL}"
    exit 1
fi
VMFS_COMMON_UTIL=${SGCONF}/scripts/sg/vmfs_common_util.sh
if [[ -f ${VMFS_COMMON_UTIL} ]]; then
    . ${VMFS_COMMON_UTIL}
    if (( $? != 0 ))
    then
        echo "ERROR: Unable to source package vmfs utility functions file: ${VMFS_COMMON_UTIL}"
        exit 1
    fi
else
    echo "ERROR: Unable to find package vmfs utility functions file: ${VMFS_COMMON_UTIL}"
    exit 1
fi

###################################################################
function check_and_set_pr_module_skip
{
    if (( ${#SG_VMDK_FILE_NAME[*]} > 0 )); then
        cleanup_before_exit $set_pr_module_skip
    fi
}

# The VG's are available as long as the VMFS disks are attached.
# If the VG's added as part of the package applyconf are still
# existing it should not be considered.
function check_if_vg_dg_exists_post_detach
{
    typeset -i ret=0
    typeset status="$1"

    for I in ${SG_VG[*]}
    do
        found=0
        for J in ${PREV_SG_VG[*]}
        do
            if [[ $I = $J ]]
            then
                found=1
                break;
            fi
        done
        if (( found == 0 ))
        then
            vgs 2>/dev/null | grep -q $I
            if (( $? == 0 )); then
                if sg_check_if_skip_pr_modules; then
                    sg_log 5 "WARNING: Function check_if_vg_dg_exists_post_detach"
                    sg_log 0 "WARNING: Volume group $I is still available after detaching the VMDK files"\
                             "which was configured in the package."
                    sg_log 0 "WARNING: Since the disk is of VMFS type, the Volume group should not be available after"\
                             "detach."
                    sg_log 0 "WARNING: Ensure that the Volume group and the VMDK files are properly configured."
                    sg_log 0 "WARNING: Ignore the warning if the Volume group is part of already"\
                             "attached VMFS disks in the same package."
                else
                    sg_log 5 "WARNING: Function check_if_vg_dg_exists_post_detach"
                    sg_log 0 "WARNING: The Volume group $I is available on the node $local_host_name"\
                             "which is configured in the package even without attaching the VMFS disks."
                    sg_log 0 "WARNING: Ensure that the Volume group and the VMDK files are properly configured."
                fi
            fi
        fi
    done
    
    for I in ${SG_VXVM_DG[@]}
    do
        found=0
        for J in ${PREV_SG_VXVM_DG[@]}
        do
            if [[ $I = $J ]]
            then
                found=1
                break
            fi
        done
        if (( found == 0 ))
        then
            # Logic to check if VxVM DG are still available.
            vxdisk list -o alldgs | grep -q $I
            if (( $? == 0 )); then
                if sg_check_if_skip_pr_modules; then
                    sg_log 5 "WARNING: Function check_if_vg_dg_exists_post_detach"
                    sg_log 0 "WARNING: Disk group $I is still available after detaching the VMDK files"\
                             "which was configured in the package."
                    sg_log 0 "WARNING: Since the disk is of VMFS type, the Disk group should not be available after"\
                             "detach."
                    sg_log 0 "WARNING: Ensure that the Disk group and the VMDK files are properly configured."
                    sg_log 0 "WARNING: Ignore the warning if the Disk group is part of already"\
                             "attached VMFS disks in the same package."
                else
                    sg_log 5 "WARNING: Function check_if_vg_dg_exists_post_detach"
                    sg_log 0 "WARNING: The Disk group $I is available on the node $local_host_name"\
                             "which is configured in the package even without attaching the VMFS disks."
                    sg_log 0 "WARNING: Ensure that the Disk group and the VMDK files are properly configured."
                    
                fi
            fi
        fi
    done
    return $ret
}

function do_vg_dg_validate_on_skipped_nodes
{
    if (( ${#SG_VMDK_FILE_NAME[*]} > 0 )); then
        typeset -i ret=0
        check_if_vg_dg_exists_post_detach
        cleanup_before_exit $ret
    fi
}

function vmfs_validate_detach
{
    if (( ${#SG_VMDK_FILE_NAME[*]} > 0 )); then
        typeset -i detach_ret=0
        sg_log 5 "vmfs_validate_detach"

        # The variable SFC_VMFS_ALREADY_ATTACHED is set in vmfs_util.sh if the disk is already attached
        if [[ "X$SFC_VMFS_ALREADY_ATTACHED" != "X" ]] && [ $SFC_VMFS_ALREADY_ATTACHED -eq 1 ]
        then
            sg_log 0 "ERROR: Some of the disks configured in the package are already attached."\
                     "Detach the disks before retrying the command."
            # Let us set exit as 0 since the error is already logged in vmfs.sh
            cleanup_before_exit 0
        fi

        # This variable SFC_VMFS_EXITED_BEFORE_ATTACH is set in vmfs.sh
        # if disks were not attached as part of validation
        if [[ "X$SFC_VMFS_EXITED_BEFORE_ATTACH" != "X" ]] && [ $SFC_VMFS_EXITED_BEFORE_ATTACH -eq 1 ]
        then
            # Let us set exit as 0 since the error is already logged in vmfs.sh
            cleanup_before_exit 0
        fi

        cache_local_variables

        # Set the value of SG_RUNNING_NODE_DURING_VALIDATE if package is running
        get_running_package_nodeid

        create_tmp_array_validate_attach_block $my_op_code $SG_RUNNING_NODE_DURING_VALIDATE

        if (( ${#TMP_SG_VMDK_FILE_NAME[*]} == 0 )) \
           && [ $SG_RUNNING_NODE_DURING_VALIDATE != 0 ]; then
            cleanup_before_exit $detach_ret
        fi
        query_attach_detach_disk "detach" $local_uuid $local_host_name "validate"
        detach_ret=$?

        if [[ $detach_ret != 0 ]]; then
            sg_log 0 "ERROR: The detach of some of the disks has failed"\
                     "due to an error which occurred earlier"
            handle_errors_based_on_op $detach_ret $my_op_code "detach"
            cleanup_before_exit 1
        fi

        check_if_vg_dg_exists_post_detach

        cleanup_before_exit $detach_ret
    fi
}

################
# main routine
################

#
# Module script must be specified with three required entry points:
# start, stop, and validate.
#
# NOTE DO NOT DELETE THESE COMMENTS
# Adding the string "sg_check_if_it_is_reconfiguration" and
# "sg_check_if_reconfig_has_failed". This is to support the
# new online reconfiguration behavior. These strings are added
# as comments. This ensures the check from master_control_script
# that this module can handle the new behavior during online
# configuration.

my_op_code=${1}
local_host_name=${SG_HOST_NAME%%.*}
local_node_type=`$CMVIEWCL -vf line -s config | grep -w $local_host_name | awk -F= '/^node:[^|]*\|virt_tech=/ {print $2}'`

if [[ "$local_node_type" == "RHEV" ]] || [[ "$local_node_type" == "KVM" ]]; then
    # If it neither vmware nor physical we will exit
    # But config should ensure that VMFS attributes are not configured
    # when such nodes are configured.
    exit 0
fi

case ${1} in
       start)
                # no op
                ;;
        stop)
                if [[ $local_node_type == "vmware" ]]; then
                    check_and_set_pr_module_skip
                fi
                ;;
    validate)
                if sg_check_if_skip_pr_modules; then
                    vmfs_validate_detach                    
                elif sg_check_if_storage_module_skip; then
                    do_vg_dg_validate_on_skipped_nodes
                fi
                ;;
    reconfigure_stop)
                if [[ $local_node_type == "vmware" ]]; then
                    check_and_set_pr_module_skip
                fi
                ;;
    reconfigure_start)
                #no op
                ;;
           *)
                sg_log 0 "WARNING: Illegal entry point specification $1."
                ;;
esac
