####################################################################
# (C) Copyright 2015-2017 Hewlett Packard Enterprise Development LP.
# @(#) Serviceguard UPCC Package Configuration Script.
# @(#) Product Name                :  HP Serviceguard
# @(#) Product Version             :  A.12.10.00
# @(#) Patch Name                  :  
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HP Serviceguard.
#
# Changing this file may lead to unrecoverable package startup
# and/or shutdown problems.
#
###################################################################

function swap_bytes 
{
    input="$1"
    reverse=""
    len=${#input}
    for (( i=$len-2; i>=0; i=i-2 ))
    do
        reverse="$reverse${input:$i:2}"
    done
    echo $reverse
}

function transpose_uuid
{
    typeset swapped_uuid=""
    index=1
    while [ $index -le 5 ]
    do
        var=`echo $1 | cut -d '-' -f $index`
        if [[ $index -le 3 ]];then
            var3=`swap_bytes $var`
            if [[ $index -eq 1 ]];then
                swapped_uuid=`echo $var3`
            else
                swapped_uuid=`echo $swapped_uuid-$var3`
            fi
        else
            swapped_uuid=`echo $swapped_uuid-$var`
        fi
        index=$(($index+1))
    done
    echo $swapped_uuid
}

function cache_local_variables
{
    # Function cache_cmviewcl_output_lx is defined in SG_UTILS
    cache_cmviewcl_output_lx
    if [ $? -ne 0 ]; then
        # cache_cmviewcl_output_lx has returned 1 
        if (( ${#SG_VMDK_FILE_NAME[*]} > 0 )); then
            sg_log 0 "ERROR: Cluster is currently not configured. Apply the cluster first"\
                     "and then apply the package configured with VMFS disks."
            rm -f $cmviewcl_output 2>/dev/null
            cleanup_before_exit 1
        else
            rm -f $cmviewcl_output 2>/dev/null
            cleanup_before_exit 0
        fi
    fi
 
    if /bin/grep "Tikanga" /etc/redhat-release >/dev/null 2>&1; then 
        _cur_uuid=`dmidecode | grep UUID | cut -d ':' -f 2 | tr -d '[:space:]'`
        smbios=`dmidecode | grep ^SMBIOS | awk '{print $2}' | sed 's/\\.//' | tr -d '[:space:]'`
        if [ $smbios -ge 26 ]; then 
            local_uuid=`transpose_uuid $_cur_uuid`
        else
            local_uuid=$_cur_uuid
        fi
    else
        local_uuid=`/bin/cat /sys/class/dmi/id/product_uuid`
    fi
    local_node_id=`cat "$cmviewcl_output" | grep -w $local_host_name | awk -F= '/^node:[^|]*\|id=/ {print $2}'`
    local_node_type=`cat "$cmviewcl_output" | grep -w $local_host_name | awk -F= '/^node:[^|]*\|virt_tech=/ {print $2}'`
}

function check_if_local_uuid_is_part_of_the_cluster
{
    if [[ $local_node_type == "none" ]]; then
        # Skip the check for physical nodes
        return
    fi
    cat "$cmviewcl_output" | grep -q $local_uuid
    if (( $? != 0 )); then
        sg_log 0 "FATAL: The UUID of the node "$local_host_name" name has been changed since the"\
                 "cluster was last created."
        sg_log 0 "       Reapply the cluster configuration to rectify the issue."
        cleanup_before_exit 1
    fi
}

function get_current_prev_node_value
{
    typeset _prev_failed_node_name=$1
    typeset -i type_value=0

    # The return values are based on the below.
    ##########################
    # local  | failed| value #
    # node   | node  |       #
    #--------|-------|-------#
    #   V    |   V   |   1   #
    #--------|-------|-------#
    #   V    |   P   |   2   #
    #--------|-------|-------#
    #   P    |   V   |   3   #
    #--------|-------|-------#
    #   P    |   P   |   4   #
    ##########################
    case "$local_node_type" in
    'vmware')
        case "$_prev_failed_node_name" in
        'vmware')
            # local node->VMWare;failed node->VMWare
            type_value=1
        ;;
        'none')
            # local node->VMWare;failed node->Physical
            type_value=2
        ;;
        esac
    ;;
    'none')
        case "$_prev_failed_node_name" in
        'vmware')
            # local node->Physical;failed node->VMWare
            type_value=3
        ;;
        'none')
            # local node->Physical;failed node->Physical
            type_value=4
        ;;
        esac
    ;;
    esac
    return $type_value
}

# The utility checks the existence of MC modules &
# set the package_node_names accordingly
# Returns
#         : 0
#         : 99(set_storage_module_skip)
function check_and_create_package_node_list
{
    typeset -i ret=0
    typeset -i _node_exist_in_mc_list=0
    if check_metro_cluster_module_is_configured; then
        # If metro cluster is configured and VMDK is configured, 
        # the MC module would have set the Read write(SFC_MC_READ_WRITE_NODELIST)
        # node list and Read only(SFC_MC_READ_ONLY_NODELIST).
        package_node_names=$SFC_MC_READ_WRITE_NODELIST
        read_only_package_nodes=$SFC_MC_READ_ONLY_NODELIST
        for node in $package_node_names
        do
            if [ "$local_host_name" = "$node" ]; then
                _node_exist_in_mc_list=1
                break
            fi
        done
        if (( $_node_exist_in_mc_list == 0 )); then
            # Metro Cluster is configured but this node belongs to RO array side
            ret=$set_storage_module_skip
        fi
    else
        # Function get_package_nodes is defined in SG_UTILS
        package_node_names=`get_package_nodes`
    fi
    return $ret
}

function check_metro_cluster_module_is_configured
{
    typeset -i ret=1
    typeset -i found=0

    _mc_modules="dts/mcxpca dts/mccaeva dts/mc3parrc dts/mcsrdf"
    for mod in $_mc_modules
    do
        sg_check_module_in_pkg $mod
        if (($? == 0))
        then
            ret=0
            MC_CONFIGURED=1
            break
        fi
    done
    return $ret
}

function check_xdc_is_configured
{
    typeset -i ret=1
    typeset -i found=0

    cat $cmviewcl_output | grep -w $SG_PACKAGE_NAME | grep -q "pkg_data_replication_method"
    if (( $? == 0 )); then
        ret=0
    fi
    return $ret
}

function create_vmware_only_package_node_list
{
    _node_list=$1
    _vmware_only_node_list=""
    for node in $_node_list
    do
        _node_type=`cat "$cmviewcl_output" | awk '/^node:[^|]*\|virt_tech=/' | grep -w $node | cut -d "=" -f 2`
        if [[ $_node_type == "none" ]]; then
            continue
        else
            tmp=$node
            _vmware_only_node_list=$_vmware_only_node_list" "$tmp
        fi
    done
    # Remove the leading and trailing space
    _vmware_only_node_list=`echo $_vmware_only_node_list | sed 's/^ *//g' | sed 's/ *$//g'`
    echo $_vmware_only_node_list
}

function get_node_uuids_from_node_name
{
    _node_list=$1
    _uuid_list=""
    for node in $_node_list
    do
        tmp=`cat "$cmviewcl_output" | awk '/^node:[^|]*\|uuid=/' | grep -w $node | cut -d "=" -f 2`
        _node_uuids=$_node_uuids" "$tmp
    done
    # Remove the leading and trailing space
    _node_uuids=`echo $_node_uuids | sed 's/^ *//g' | sed 's/ *$//g'`
    echo $_node_uuids
}

function check_if_current_node_is_eligible_to_run_applyconf
{
    # Check for below
    # 1. Package is running and it is on the current node
    #   a. if package is running check if it is the current node    
    #       i.If not skip this module on the current node. Return $set_storage_module_skip
    #                       ELSE
    # 2. Check if the node has the least node id
    #       i.If not skip this module on the current node. Return $set_storage_module_skip 

    # if package is running the value of the SG_RUNNING_NODE_DURING_VALIDATE 
    # will be one of the node_id's

    # first set the SG_RUNNING_NODE_DURING_VALIDATE
    # Function get_running_package_nodeid is defined in SG_UTILS
    get_running_package_nodeid

    if [ $SG_RUNNING_NODE_DURING_VALIDATE != 0 ] ; then
        if [[ $local_node_id == $SG_RUNNING_NODE_DURING_VALIDATE ]];then
            # This is the eligible node. Return TRUE
            return 0
        else
            # The package is not running on this node
            return $set_storage_module_skip
        fi
    else
        find_if_local_node_has_least_node_id
        ret=$?
        if [[ $ret != 0 ]]; then
            return $set_storage_module_skip
        else
            # This is the eligible node. Return TRUE
            return 0
        fi
    fi
}

function find_if_local_node_has_least_node_id
{
    typeset node_ids=""
    typeset least_node=""
    typeset -i node_id=0
    typeset -i least_node_id=0

    node_ids_line=`cat "$cmviewcl_output" | awk '/^node:[^|]*\|id=/'`

    least_node_id=$local_node_id
    least_node=$local_host_name

    for node in $package_node_names
    do
        # Discard the physical nodes if any
        node_type=`cat "$cmviewcl_output" | awk '/^node:[^|]*\|virt_tech=/' | grep -w $node | cut -d "=" -f 2`
        if [[ $node_type != "vmware" ]]; then
            continue
        fi

        node_id=`echo "$node_ids_line" | grep -w $node | cut -d "=" -f 2`
        if [[ $? == 0 ]]; then
            if (( $node_id <= $least_node_id  ));then
                least_node_id=$node_id
                least_node=$node
            fi
        fi
    done

    if [[ "$least_node" == "$local_host_name" ]] ;then
        # True
        return 0
    else
        # False
        return 1
    fi
}

function check_if_the_previous_run_node_failed
{
    # SG_LAST_RUN_NODE is sourced as part of the environment variable by the package manager
    if [[ "X$SG_LAST_RUN_NODE" == "X" ]]; then
        return 1
    else
        if [ $SG_LAST_RUN_NODE -eq -1 ] || [ $SG_LAST_RUN_NODE -eq 0 ] ; then
            return 1
        fi
    fi
    node_name=`get_node_name_from_nodeid $SG_LAST_RUN_NODE`
    # The function get_node_state_from_node_name is defined in utils.sh
    _state=`get_node_state_from_node_name $node_name`
    if [[ "$_state" = "failed" ]]; then
        return 0
    else
        return 1
    fi
}

function get_uuid_from_node_name
{
    uuid=""
    node_name=$1
    uuid=`cat "$cmviewcl_output" | grep -w $node_name | awk -F= '/^node:[^|]*\|uuid=/ {print $2}'`
    echo $uuid
}

function get_node_name_from_nodeid
{
    node_id=$1
    nname=`cat "$cmviewcl_output" |  awk '/^node:[^|]*\|id='$node_id'/' | sed 's/|id='${node_id}'//;s/node://'`
    echo $nname
}

function get_node_name_from_uuid
{
    uuid=$1
    nname=`cat "$cmviewcl_output" | grep -w $uuid | sed 's/|uuid='${uuid}'//;s/node://'`
    echo $nname
}

#VirtualMachine:UUID|Datastore:XXXX|name=XXXX
#VirtualMachine:UUID|Datastore:XXXX|url=/vmfs/volumes/5551e5a5-bfd412d1-f1b4-e83935a8eef2
#VirtualMachine:UUID|Datastore:XXXX|availability=<yes>/<no>
#VirtualMachine:UUID|Datastore:XXXX|error=<NOT_EXIST/CONFIGURATION_ISSUE>
#VirtualMachine:UUID|Datastore:XXXX|wwn=<wwn>
function get_datastore_values_from_output
{
    _ds=$1
    _uuid=$2

    echo "$vmfs_output" | grep $_ds | grep $_uuid | awk -F= '
    /^VirtualMachine:[^|]*\|Datastore:[^|]*\|name=/             { ds_name = $2 }
    /^VirtualMachine:[^|]*\|Datastore:[^|]*\|url=/              { url_id = $2 }
    /^VirtualMachine:[^|]*\|Datastore:[^|]*\|availability=/     { availability = $2 }
    /^VirtualMachine:[^|]*\|Datastore:[^|]*\|error=/            { error = $2 }
    /^VirtualMachine:[^|]*\|Datastore:[^|]*\|action=/           { action = $2 }

    END {
        if (length(ds_name) == 0)
            ds_name = "NA"
        if (length(url_id) == 0)
            url_id = "NA"
        if (length(availability) == 0)
            availability = "NA"
        if (length(error) == 0)
            error = "NA"
        gsub(" ","_", error)
        if (length(action) == 0)
            action = "NA"
        printf("%s %s %s %s %s\n", ds_name, url_id, availability, error, action);
    }'
}

#VirtualMachine:UUID|Datastore:XXXX|File:AA/BB.vmdk|name=AA/BB.vmdk
#VirtualMachine:UUID|Datastore:XXXX|File:AA/BB.vmdk|attached=<yes>/<no>
#VirtualMachine:UUID|Datastore:XXXX|File:AA/BB.vmdk|error=<NOT_EXIST/CONFIGURATION_ISSUE/Other VMware errors>
#VirtualMachine:UUID|Datastore:XXXX|File:AA/BB.vmdk|action=<NOT_ATTEMPTED/SUCCEED/FAILED>
function get_vmdk_availability_from_output
{
    _uuid=$1
    _vmdk=$2
    _ds=$3

    echo "$vmfs_output" | grep $_vmdk | grep $_uuid | grep $_ds | awk -F= '
    /^VirtualMachine:[^|]*\|Datastore:[^|]*\|File:[^|]*\|name=/     {vmdk_name = $2}
    /^VirtualMachine:[^|]*\|Datastore:[^|]*\|File:[^|]*\|attached=/ {attached = $2}
    /^VirtualMachine:[^|]*\|Datastore:[^|]*\|File:[^|]*\|error=/    {error = $2}
    /^VirtualMachine:[^|]*\|Datastore:[^|]*\|File:[^|]*\|action=/   {action = $2}
    
    END {
        if (length(vmdk_name) == 0)
            vmdk_name = "NA"
        if (length(attached) == 0)
            attached = "NA"
        if (length(error) == 0)
            error = "NA"
        gsub(" ","_", error)
        if (length(action) == 0)
            action = "NA"
        printf("%s %s %s %s\n", vmdk_name, attached, error, action);
    }'
}

#VirtualMachine:UUID|Datastore:XXXX|File:AA/BB.vmdk|type=<VMFS>
#VirtualMachine:UUID|Datastore:YYYY|File:AA/BB.vmdk|type=<RDM>
function get_vmdk_type_from_output
{
    _ds=$1
    _uuid=$2
    _vmdk=$3

    echo "$vmfs_output" | grep $_uuid | grep $_ds | grep $_vmdk | awk -F= '
    /^VirtualMachine:[^|]*\|Datastore:[^|]*\|File:[^|]*\|type=/  {type = $2}
    
    END {if (length(type) == 0)
            type = "NA"
        printf("%s\n", type);
    }'
}

#VirtualMachine:UUID|Slot:1:2|name=1:2
#VirtualMachine:UUID|Slot:1:2|availability=<yes>/<no>/<unknown>
#VirtualMachine:UUID|Slot:1:2|controller_sharing_mode=<NO_SHARING/PHYSICAL_SHARING/UNKNOWN>
function get_vmdk_slot_from_output
{
    _uuid=$1
    _slot_no=$2

    echo "$vmfs_output" | grep $_uuid | grep "$_slot_no" | awk -F= '
    /^VirtualMachine:[^|]*\|Slot:[^|]*\|availability=/   {slot_availability = $2}
    /^VirtualMachine:[^|]*\|Slot:[^|]*\|controller_sharing_mode=/  {sharing_mode = $2}
    
    END {if (length(slot_availability) == 0)
            slot_availability = "NA"
        if (length(sharing_mode) == 0)
            sharing_mode = "NA"

        printf("%s %s\n", slot_availability, sharing_mode);
    }'
}

function log_failures
{
    sg_log 0 "$*"
    if sg_check_if_reconfig_has_failed
    then
        sg_log 999 "$*"
    fi
}

function print_multi_line_error
{
    typeset err_str=$1
    for line in $(echo "$err_str" | tr "<>" "\n" | sed 's/^_*//g' | sed 's/_*$//g')
    do
        log_failures "ERROR: VCENTER_ESX: $line"
    done
}

function check_if_error_has_multiline
{
    typeset err_str=$1
    typeset ret=1
    echo "$err_str" | grep -q "<>"
    if (( $? == 0 )); then
        ret=0
    fi
    return $ret
}

############################################################################################
# Function validate_data_store                                                             #
#                                                                                          #
# Globals Referred:                                                                        #
#       1. TMP_SG_VMDK_FILE_NAME                                                           #
#       2. package_uuids                                                                   #
#       3. package_uuids_read_only                                                         #
# Return Value:                                                                            #
#       1 in case of failure                                                               #
# For RW site and NON MC deployments we do the below validation                            #
#       1. Datastore Validation                                                            #
#       2. VMDK Validation                                                                 #
#       3. VMDK type Validation                                                            #
#       4. SLOT Validation                                                                 #
# The RW site nodes are stored in global "package_uuids" as part of the function           #
# "check_and_create_package_node_list"                                                     #
#                                                                                          #
# For RO site we do the below validation                                                   #
#       1. SLOT Validation only                                                            #
# The RO site nodes are stored in global "package_uuids_read_only" as part of the function #
# "sg_validate_vmfs"                                                                       #
#                                                                                          #
############################################################################################
function validate_data_store
{
    declare _TMP_DATASTORE_NAME
    declare _TMP_DATASTORE_WWN
    declare _TMP_DATASTORE_URL
    declare _TMP_DATASTORE_AVAILABILITY
    typeset -i R=0
    typeset -i ds_ret=0
    typeset -i vmdk_ret=0
    typeset -i slot_ret=0
    typeset mcs_op=$1
    typeset vmfs_op=$2

    while (($R < ${#TMP_SG_VMDK_FILE_NAME[*]}))
    do
        _TMP_DATASTORE_NAME=""
        _TMP_DATASTORE_URL=""
        _TMP_DATASTORE_AVAILABILITY=""

        for uuid in $package_uuids
        do
            nname=`get_node_name_from_uuid $uuid`
            read _ds _url _availability _error _action <<< \
            `get_datastore_values_from_output ${TMP_SG_DATASTORE_NAME[$R]} $uuid`
            if [[ $_ds != ${TMP_SG_DATASTORE_NAME[$R]} ]] || [[  "$_url" == "NA" ]] \
               || [[  "$_availability" == "NA" ]] || [[ $_error == "NOT_EXIST" ]]
            then
                sg_log 0 "ERROR: Unable to find datastore ${TMP_SG_DATASTORE_NAME[$R]}."\
                         "The datastore ${TMP_SG_DATASTORE_NAME[$R]} configured in this package"
                sg_log 0 "       does not exist for the node \"$nname\". Check for valid datastore."
                ds_ret=1
                continue
            fi

            if [[  $_TMP_DATASTORE_NAME == "" ]]
            then
                # Copy only once
                _TMP_DATASTORE_NAME=$_ds
                _TMP_DATASTORE_URL=$_url
                _TMP_DATASTORE_AVAILABILITY=$_availability
                if [[ $_TMP_DATASTORE_AVAILABILITY != "yes" ]]
                then
                    sg_log 0 "ERROR: Unable to find datastore ${TMP_SG_DATASTORE_NAME[$R]}."\
                             "The datastore ${TMP_SG_DATASTORE_NAME[$R]} configured in this package"
                    sg_log 0 "       does not exist for the node \"$nname\". Check for valid datastore."
                    ds_ret=1
                    continue
                fi
            elif [[  $_TMP_DATASTORE_URL != $_url ]]
            then
                sg_log 0 "ERROR: Unable to find datastore ${TMP_SG_DATASTORE_NAME[$R]}."\
                         "The datastore ${TMP_SG_DATASTORE_NAME[$R]} configured in this package"
                sg_log 0 "       does not exist for the node \"$nname\". Check for valid datastore."
                ds_ret=1
                continue
            fi
            check_and_print_vmdk_errors $R $uuid $mcs_op $vmfs_op
            if (( $? == 1 )); then
                vmdk_ret=1
            fi
        done
        if [ $MC_CONFIGURED == 1 ]; then
            for ro_uuid in $package_uuids_read_only
            do
                nname=`get_node_name_from_uuid $ro_uuid`
                check_and_print_vmdk_slot_errors $R $ro_uuid $nname $mcs_op $vmfs_op
                if (( $? == 1 )); then
                    slot_ret=1
                fi
            done
        fi
        (( R = R + 1 ))
    done

    if (( $vmdk_ret == 1 )) || (( $ds_ret == 1 )) || (( $slot_ret == 1 )); then
        return 1
    fi
}

function check_and_print_datastore_errors
{
    typeset -i R=0
    typeset mcs_op=$1
    typeset vmfs_op=$2
    typeset -i ret=0
    typeset -i ds_ret=0
    typeset -i vmdk_ret=0
    nname=`get_node_name_from_uuid $local_uuid`

    while (($R < ${#TMP_SG_VMDK_FILE_NAME[*]}))
    do
        read _ds _url _availability _error _action <<< \
        `get_datastore_values_from_output ${TMP_SG_DATASTORE_NAME[$R]} $local_uuid`
        if [[ $_ds != ${TMP_SG_DATASTORE_NAME[$R]} ]] || [[  "$_url" == "NA" ]] \
           || [[  "$_availability" != "yes" ]] || [[ $_error == "NOT_EXIST" ]]
        then
            # We consider this failure as valid only in case of XDC module being configured in the 
            # package. This can be a scenario where one leg of the mirror is not available. Let the
            # XDC module decide whether to fail or not.
            if [[ $XDC_CONFIGURED == 1 ]]; then
                log_failures "WARNING: Unable to find datastore ${TMP_SG_DATASTORE_NAME[$R]}."\
                             "The datastore ${TMP_SG_DATASTORE_NAME[$R]} configured in this package"
                log_failures "         does not exist for the node \"$nname\". Check for valid datastore."
                VALID_XDC_FAILURE=1
            else
                log_failures "ERROR: Unable to find datastore ${TMP_SG_DATASTORE_NAME[$R]}."\
                             "The datastore ${TMP_SG_DATASTORE_NAME[$R]} configured in this package"
                log_failures "       does not exist for the node \"$nname\". Check for valid datastore."
            fi
            ret=1
            (( R = R + 1 ))
            continue
        fi
        if [[ $_action == "FAILED" ]]; then
            # This is run time error printing the exception as got from the JAVA Utility "vmwutil"
            if check_if_error_has_multiline "$_error"; then
                print_multi_line_error "$_error"
            else
                log_failures "ERROR: VCENTER_ESX: $_error"
            fi
            ret=1
            (( R = R + 1 ))
            continue
        fi
        check_and_print_vmdk_errors $R $local_uuid $mcs_op $vmfs_op
        vmdk_ret=$?
        if (( $vmdk_ret == 1 )); then
            ret=1
        fi
        (( R = R + 1 ))
    done

    return $ret
}

function check_and_print_vmdk_slot_errors
{
    typeset -i index=$1
    typeset _uuid=$2
    typeset nname=$3
    typeset mcs_op=$4
    typeset vmfs_op=$5

    read _vmdk_slot_availability _sharing_mode <<< `get_vmdk_slot_from_output $_uuid \
                                                    ${TMP_SG_SCSI_CONTROLLER[$index]}`
    case "$_vmdk_slot_availability" in
    'yes')
        case "$_sharing_mode" in
        'NO_SHARING')
            case "$mcs_op" in
            'start'|'reconfigure_start')
                # This is a case where action is NOT_ATTEMPTED for vmdk 
                # We need to ensure that slot sharing mode was fine before printing this message
                log_failures "WARNING: Attach of VMDK disk [${TMP_SG_DATASTORE_NAME[$index]}]"\
                             "${TMP_SG_VMDK_FILE_NAME[$index]} not attempted."
                return 1
            ;;
            'stop')
                # This is a case where action is NOT_ATTEMPTED for vmdk 
                # We need to ensure that slot sharing mode was fine before printing this message
                log_failures "WARNING: Detach of VMDK disk [${TMP_SG_DATASTORE_NAME[$index]}]"\
                             "${TMP_SG_VMDK_FILE_NAME[$index]} not attempted."
                return 1
            ;;
            esac
        ;;
        'UNKNOWN')
            log_failures "ERROR: Invalid scsi_controller. The scsi_controller"\
                         "\"${TMP_SG_SCSI_CONTROLLER[$index]}\" is either unavailable or already"
            log_failures "       in use on node \"$nname\". Check for valid scsi_controller."
            return 1
        ;;
        'PHYSICAL_SHARING')
            log_failures "ERROR: Unsupported SCSI BUS sharing type. The only supported SCSI bus"\
                         "sharing type in Serviceguard is \"None\"."
            log_failures "       Change SCSI bus sharing for VMDK disk"\
                         "\"[${TMP_SG_DATASTORE_NAME[$index]}] ${TMP_SG_VMDK_FILE_NAME[$index]}\""\
                         "to \"None\" on node \"$nname\"."
            return 1
        ;;
        esac
    ;;
    'no'|'unknown'|'NA')
        # Case when slot is available, but a disk is attached or SCSI bus itself is not 
        # yet configured.
            log_failures "ERROR: Invalid scsi_controller. The scsi_controller"\
                         "\"${TMP_SG_SCSI_CONTROLLER[$index]}\" is either unavailable or already"
            log_failures "       in use on node \"$nname\". Check for valid scsi_controller."
        return 1
    ;;
    esac
}

function check_and_print_vmdk_disk_type_errors
{
    typeset -i index=$1
    typeset _uuid=$2
    typeset nname=$3
    typeset mcs_op=$4
    typeset vmfs_op=$5

    _vmdk_type=`get_vmdk_type_from_output ${TMP_SG_DATASTORE_NAME[$index]} $_uuid ${TMP_SG_VMDK_FILE_NAME[$index]}`
    if [[  $_vmdk_type == "NA" ]]
    then
        log_failures "ERROR: Unable to determine disk_type. The disk type for"\
                     "\"[${TMP_SG_DATASTORE_NAME[$index]}] ${TMP_SG_VMDK_FILE_NAME[$index]}\""
        log_failures "       could not be retrieved."
        return 1
    elif [[  $_vmdk_type != "${TMP_SG_DISK_TYPE[$index]}" ]]
    then
        log_failures "ERROR: Mismatch in disk_type. The disk_type \"${TMP_SG_DISK_TYPE[$index]}\""\
                     "specified for the VMDK disk"
        log_failures "       \"[${TMP_SG_DATASTORE_NAME[$index]}] ${TMP_SG_VMDK_FILE_NAME[$index]}\""\
                     "in the package differs from the actually configured."
        log_failures "       Correct the disk_type mismatch."
        return 1
    fi
}

function check_and_print_vmdk_errors
{
    typeset -i ret=0
    typeset -i _type_ret=0
    typeset -i _slot_ret=0
    typeset -i index=$1
    typeset _uuid=$2
    typeset mcs_op=$3
    typeset vmfs_op=$4
    nname=`get_node_name_from_uuid $_uuid`

    read _vmdk_file _attached _error _action <<< \
    `get_vmdk_availability_from_output $_uuid ${TMP_SG_VMDK_FILE_NAME[$index]} ${TMP_SG_DATASTORE_NAME[$index]}`
    if [[ "$_attached" == "yes" ]]; then
        log_failures "ERROR: VMDK disk in use. The VMDK disk \"[${TMP_SG_DATASTORE_NAME[$index]}]"\
                     "${TMP_SG_VMDK_FILE_NAME[$index]}\" is already attached to node \"$nname\"."
        log_failures "       Remove the attached VMDK disk from node using VMware recommended methods and"\
                     "try again."
        case "$mcs_op" in
        'validate')
            if [[  $vmfs_op == "query" ]]; then
                # This variable "SFC_VMFS_ALREADY_ATTACHED" is used to skip detach in vmfs_end_val.sh.
                # In case the disks are already attached we do not attach disks as part of validation.
                # The user has to detach the disks and retry cmapplyconf.
                echo "export SFC_VMFS_ALREADY_ATTACHED=\"1\"" >> $SG_SRC_FRM_CHILD
            fi
        ;;
        esac
        return 1
    fi
    case "$_action" in
    'SUCCEED')
        case "$mcs_op" in
        'start'|'reconfigure_start')
            log_failures "Attached VMDK disk \"[${TMP_SG_DATASTORE_NAME[$index]}]"\
                         "${TMP_SG_VMDK_FILE_NAME[$index]}\"."
        ;;
        'stop')
            log_failures "Detached VMDK disk \"[${TMP_SG_DATASTORE_NAME[$index]}]"\
                         "${TMP_SG_VMDK_FILE_NAME[$index]}\"."
        ;;
        esac
    ;;
    'FAILED')
        # This is run time error printing the exception as got from the JAVA Utility "vmwutil"
        case "$mcs_op" in
        'start'|'reconfigure_start'|'validate')
            if check_if_error_has_multiline "$_error"; then
                print_multi_line_error "$_error"
            else
                log_failures "ERROR: VCENTER_ESX: $_error"
            fi
        ;;
        esac
    ;;
    'NOT_ATTEMPTED')
        case "$_error" in
        'Not_exist'|'NOT_EXIST')
            log_failures "ERROR: VMDK disk \"[${TMP_SG_DATASTORE_NAME[$index]}]"\
                         "${TMP_SG_VMDK_FILE_NAME[$index]}\" does not exist. Unable to find VMDK"\
                         "disk \"${TMP_SG_VMDK_FILE_NAME[$index]}\""
            log_failures "       in datastore \"[${TMP_SG_DATASTORE_NAME[$index]}]\" from node"\
                         "\"$nname\". Check for valid VMDK disk."
            return 1
        ;;
        *)
            # During detach we will need not validate the disk type and slot's
            if [[  $vmfs_op != "detach" ]]; then
                check_and_print_vmdk_disk_type_errors $index $_uuid $nname $mcs_op $vmfs_op
                _type_ret=$?
                if (( $_type_ret != 0 )); then
                    ret=$_type_ret
                fi
                check_and_print_vmdk_slot_errors $index $_uuid $nname $mcs_op $vmfs_op
                _slot_ret=$?
                if (( $_slot_ret != 0 )); then
                    ret=$_slot_ret
                fi
            fi
        ;;
        esac
    ;;
    esac

    return $ret
}

function create_input_file_query_attach_detach
{
    typeset __uuid=$1
    typeset -i R=0

    rm -f $_input_file

    if [[ "X$__uuid" != "X" ]]; then
        echo "UUID:$__uuid" > $_input_file
    else
        for uuid in $package_uuids
        do
            echo "UUID:$uuid" >> $_input_file
        done
        if [ $MC_CONFIGURED == 1 ]; then
            # Let us get the query for all the nodes both on RW and RO sites
            for uuid in $package_uuids_read_only
            do
                echo "UUID:$uuid" >> $_input_file
            done
        fi
    fi

    while (($R < ${#TMP_SG_VMDK_FILE_NAME[*]}))
    do
        echo "VMDK:${TMP_SG_VMDK_FILE_NAME[$R]}" >> $_input_file
        echo "DS:${TMP_SG_DATASTORE_NAME[$R]}" >> $_input_file
        echo "SLOT:${TMP_SG_SCSI_CONTROLLER[$R]}" >> $_input_file
        echo "TYPE:${TMP_SG_DISK_TYPE[$R]}" >> $_input_file
        (( R = R + 1 ))
    done
}

function verify_attach_detach
{
    typeset -i _cmd_ret=$1
    typeset _vmfs_op=$2
    typeset _node_name=$3
    typeset _mcs_op=$4
    typeset print_string=""

    if [[  $_vmfs_op == "attach" ]]; then
        if [[ $_cmd_ret == 0 ]]; then
            print_str="Attached the"
        else
            print_str="Failed to attach"
        fi
    else
        if [[ $_cmd_ret == 0 ]]; then
            print_str="Detached the"
        else
            print_str="Failed to detach"
        fi
    fi

    if [[ $_cmd_ret == 0 ]]; then
        if [[ $_mcs_op != "validate" ]]; then
            _R=0
            while (($_R < ${#TMP_SG_VMDK_FILE_NAME[*]}))
            do
                log_failures "$print_str VMDK file \"[${TMP_SG_DATASTORE_NAME[$_R]}] "\
                             "${TMP_SG_VMDK_FILE_NAME[$_R]}\" on node $_node_name."
                (( _R = _R + 1 ))
            done
        fi
    else
        if sg_check_if_it_is_reconfiguration
        then
            sg_set_reconfig_failure
        fi
        if [[ $XDC_CONFIGURED == 1 ]]; then
            log_failures "WARNING: $print_str some or all of the disks."
        else
            log_failures "ERROR: $print_str some or all of the disks."
        fi
        # The rest of the failure details will be handled by function
        # handle_errors_based_on_op
    fi
}

function query_attach_detach_disk
{
    typeset vmfs_op=$1
    typeset _uuid=$2
    typeset _node_name=$3
    typeset mcs_op=$4
    typeset -i _ret=0

    if [[  $vmfs_op == "query" ]]; then
        cmd=$SGSBIN/cmvmdiskquery
        create_input_file_query_attach_detach
    elif [[  $vmfs_op == "attach" ]]; then
        cmd="$SGSBIN/cmvmdiskop -A"
        if check_metro_cluster_module_is_configured; then
            if [[ $SFC_MC_RESCAN_REQUIRED == 1 ]]; then
                cmd="$SGSBIN/cmvmdiskop -r -A"
                sg_log 0 "Issuing rescan of datastore before attaching disks"
            fi
        elif [[ $XDC_CONFIGURED == 1 ]]; then
            cmd="$SGSBIN/cmvmdiskop -x -A"
        fi
        create_input_file_query_attach_detach $_uuid
    elif [[  $vmfs_op == "detach" ]]; then
        cmd="$SGSBIN/cmvmdiskop -D"
        create_input_file_query_attach_detach $_uuid
    fi
    vmfs_output=`$cmd -F $_input_file -c $cmviewcl_output`
    ret=$?
    if [[  $vmfs_op != "query" ]]; then
        verify_attach_detach $ret $vmfs_op $_node_name $mcs_op
    fi

    return $ret
}

function create_tmp_array_validate_attach_block
{
    typeset mcs_op_code=$1
    typeset -i _is_package_running=$2
    typeset -i _ix=0
    typeset -i _iy=0
    typeset -i _iz=0
    typeset -i found=0
    typeset -i check_with_prev=0

    case $mcs_op_code in
    'validate')
        # This will ensure that already attached disks are skipped from validation
        if ([[ "X$_is_package_running" != "X" ]] && [ $_is_package_running -gt 0 ]); then
            check_with_prev=1
        fi
    ;;
    'reconfigure_start')
        check_with_prev=1
    ;;
    esac
    if [[ $check_with_prev == 1 ]]; then
        while [ -n "${SG_VMDK_FILE_NAME[$_ix]}" ]; do
            found=0
            _iy=0
            while [ -n "${PREV_SG_VMDK_FILE_NAME[$_iy]}" ]; do

                if [[ ${SG_VMDK_FILE_NAME[$_ix]} = ${PREV_SG_VMDK_FILE_NAME[$_iy]} ]] && \
                   [[ ${SG_DATASTORE_NAME[$_ix]} = ${PREV_SG_DATASTORE_NAME[$_iy]} ]]; then
                    found=1
                    break
                fi
                (( _iy = _iy + 1 ))
            done
            # Copy the newly found VMDK attributes to a temporary array.
            if ((found == 0)); then
                TMP_SG_VMDK_FILE_NAME[$_iz]=${SG_VMDK_FILE_NAME[$_ix]}
                TMP_SG_DATASTORE_NAME[$_iz]=${SG_DATASTORE_NAME[$_ix]}
                TMP_SG_SCSI_CONTROLLER[$_iz]=${SG_SCSI_CONTROLLER[$_ix]}
                TMP_SG_DISK_TYPE[$_iz]=${SG_DISK_TYPE[$_ix]}
                (( _iz = _iz + 1 ))
            fi

            (( _ix = _ix + 1 ))
        done
    else
        while (($_ix < ${#SG_VMDK_FILE_NAME[*]}))
        do
            TMP_SG_VMDK_FILE_NAME[$_ix]=${SG_VMDK_FILE_NAME[$_ix]}
            TMP_SG_DATASTORE_NAME[$_ix]=${SG_DATASTORE_NAME[$_ix]}
            TMP_SG_SCSI_CONTROLLER[$_ix]=${SG_SCSI_CONTROLLER[$_ix]}
            TMP_SG_DISK_TYPE[$_ix]=${SG_DISK_TYPE[$_ix]}
            (( _ix = _ix + 1 ))
        done
    fi
}

function get_detach_error_string_from_output
{

    _vmdk=$1
    echo "$vmfs_output" | grep "$_vmdk" | awk -F= '
    /^VirtualMachine:[^|]*\|Datastore:[^|]*\|File:[^|]*\|action=/   {action = $2}
    /^VirtualMachine:[^|]*\|Datastore:[^|]*\|File:[^|]*\|error=/   {error = $2}

    END {
        if (length(action) == 0)
            action = "NA"
        if (length(error) == 0)
            error = "NA"
        gsub(" ","_", error)
        printf("%s %s\n", action, error);
    }'
}

function check_if_host_unreachable_from_output
{
    typeset mcs_op=$1
    typeset vmfs_op=$2
    typeset -i R=0
    typeset -i ret=0
    while (($R < ${#TMP_SG_VMDK_FILE_NAME[*]}))
    do
        read _action _error_str <<< `get_detach_error_string_from_output ${TMP_SG_VMDK_FILE_NAME[$R]}`
        case "$_error_str" in
        *Unable*to*communicate*with*the*remote*host*since*it*is*disconnected*)
            ret=$HOST_UNREACHABLE
        ;;
        *An*error*occurred*while*communicating*with*the*remote*host*)
            ret=$HOST_UNREACHABLE
        ;;
        *Connection*timed*out*)
            ret=$HOST_UNREACHABLE
        ;;
        esac
        (( R = R + 1 ))
    done

    return $ret
}

function check_and_print_vmdk_errors_during_detach
{
    typeset mcs_op=$1
    typeset vmfs_op=$2
    typeset -i R=0
    while (($R < ${#TMP_SG_VMDK_FILE_NAME[*]}))
    do
        check_and_print_vmdk_errors $R $local_uuid $mcs_op $vmfs_op
        (( R = R + 1 ))
    done
}

function handle_errors_based_on_op
{
    typeset -i error_value=$1
    typeset mcs_op=$2
    typeset vmfs_op=$3
    typeset -i error_ret=1
    typeset -i _cluster_type=0

    if check_if_usage_error $error_value; then

        # This piece of code is retained for debugging.
        sg_log 5 "Error: Usage Error"
        sg_log 5 "`cat $_input_file`"
        sg_log 5 "$vmfs_output"
        cleanup_before_exit $error_ret

    elif check_if_config_or_missing_error $error_value; then

        # We will not exit now we will let the other functions
        # parse the errors in the output and then exit
        case "$mcs_op" in
        'validate')
            case $vmfs_op in
            'query')
                # Continue the validate_ functions will list 
                # the error and exit appropriately
                :
            ;;
            'attach')
                check_and_print_datastore_errors $mcs_op $vmfs_op
                cleanup_before_exit $set_vmfs_validate_failed
            ;;
            'detach')
                check_and_print_vmdk_errors_during_detach $mcs_op $vmfs_op
                cleanup_before_exit $error_ret
            ;;
            esac
        ;;
        'start')
            case $vmfs_op in
            'attach')
                check_and_print_datastore_errors $mcs_op $vmfs_op
                # If XDC is configured we will allow to go to XDC module
                # which will decide to fail the package or not.
                # This is required because even one leg of the mirrored device is
                # sufficient for rhe package to be functional
                if [[ $VALID_XDC_FAILURE == 0 ]]; then
                    cleanup_before_exit $error_ret
                else
                    log_failures "WARNING: Some of the datastore(s) is(are) missing. The array on"\
                                 "which the datastore(s) is(are)"
                    log_failures "         configured might be not reachable."
                fi
            ;;
            'detach')
                check_and_print_vmdk_errors_during_detach $mcs_op $vmfs_op
                cleanup_before_exit $error_ret
            ;;
            esac
        ;;
        'stop')
            check_and_print_vmdk_errors_during_detach $mcs_op $vmfs_op
            cleanup_before_exit $error_ret            
        ;;
        'reconfigure_start')
            check_and_print_datastore_errors $mcs_op $vmfs_op
            # If XDC is configured we will allow to go to XDC module
            # which will decide to fail the package or not.
            # This is required because even one leg of the mirrored device is
            # sufficient for rhe package to be functional
            if [[ $VALID_XDC_FAILURE == 0 ]]; then
                cleanup_before_exit $error_ret
            else
                    log_failures "WARNING: Some of the datastore(s) is(are) missing. The array on"\
                                 "which the datastore(s) is(are)"
                    log_failures "         configured might not be reachable."
            fi
        ;;
        esac

    elif check_if_vm_object_error $error_value; then
        case $vmfs_op in
        'attach'|'detach')
            package_uuids=$local_uuid
        ;;
        esac
        for uuid in $package_uuids
        do
            echo "$vmfs_output" | awk '/^VirtualMachine:[^|]*\|error=CONFIGURATION_ISSUE/' | grep -q $uuid
            if [[ $? == 0 ]]; then
                nname=`get_node_name_from_uuid $uuid`
                log_failures "ERROR: Invalid VCENTER_SERVER/ESX_HOST for node \"$nname\". The node"\
                             "\"$nname\" is no longer associated"
                log_failures "       with the vCenter/ESX host specified in the cluster configuration."
                log_failures "       Check the VCENTER_SERVER/ESX_HOST in cluster configuration"
            fi
        done
        cleanup_before_exit $error_ret

    elif check_if_wrong_credential_error $error_value; then

        log_failures "ERROR: Invalid VMware vCenter/ESX host. Unable to login to vCenter/ESX host"\
                     "using the credential"
        log_failures "       configured in the Serviceguard Credential Store(SCS)."\
                     "Update SCS with valid user credentials"
        log_failures "       using cmvusermgmt utility."
        cleanup_before_exit $error_ret

    elif check_if_host_unreachable_error $error_value; then

        error_str=`echo "$vmfs_output" | awk -F= '/^msgtype:ERROR|message=/  {print $2}'`
        check_if_cluster_is_configured_with_esx_or_vcenter
        _cluster_type=$?
        if (( $_cluster_type == $type_esx )); then
            _print_str="ESX host"
        else
            _print_str="vCenter host"
        fi
        case "$mcs_op" in
        'validate')
            case $vmfs_op in
            'query'|'attach')
                # This is mostly a case of ESX deployment. Highly unlikely because applyconf
                # would have failed much earlier.
                log_failures "ERROR: $_print_str $error_str"
                log_failures "ERROR: Ensure that the configured $_print_str is valid and reachable."
                echo "export SFC_VMFS_EXITED_BEFORE_ATTACH=\"1\"" >> $SG_SRC_FRM_CHILD
                cleanup_before_exit $set_vmfs_validate_failed
            ;;
            esac
        ;;
        'start'|'reconfigure_start')
            case $vmfs_op in
            'detach')
                # This is a case when remote attach issued to the failed node.
                log_failures "WARNING: Unable to detach disk from the failed node"\
                             "\"$prev_failed_node_name\" as the host on which it is configured is"
                log_failures "         not reachable. Ensure to detach the VMDK disk(s) from the node once"\
                             "the ESX host"
                log_failures "         is reachable using VMware recommended methods."
                return
            ;;
            'attach')
                sg_log 0 "ERROR: Since $_print_str $error_str"
                cleanup_before_exit $error_ret
            ;;
            esac
        ;;
        'stop')
            case $vmfs_op in
            'detach')
                sg_log 0 "ERROR: Since $_print_str $error_str. Ensure that the disks"
                sg_log 0 "       are detached from node \"$local_host_name\" before package can be"\
                        "restarted using VMware recommended methods."
                cleanup_before_exit $error_ret
            ;;
            esac
        ;;
        esac

    elif check_if_fail_to_attach_disk_error $error_value; then

        case "$mcs_op" in
        'validate')
            check_and_print_datastore_errors $mcs_op $vmfs_op
            cleanup_before_exit $set_vmfs_validate_failed
        ;;
        *)
            check_and_print_datastore_errors $mcs_op $vmfs_op
            # If XDC is configured we will allow to go to XDC module
            # which will decide to fail the package or not.
            # This is required because even one leg of the mirrored device is
            # sufficient for rhe package to be functional
            if [[ $VALID_XDC_FAILURE == 0 ]]; then
                cleanup_before_exit $error_ret
            else
                    log_failures "WARNING: Some of the datastore(s) is(are) missing. The array on"\
                                 "which the datastore(s)"
                    log_failures "          is(are) configured is not reachable."
            fi
        ;;
        esac

    elif check_if_fail_to_detach_disk_error $error_value; then

        check_if_host_unreachable_from_output $mcs_op $vmfs_op
        ret=$?
        if [[ $ret == $HOST_UNREACHABLE ]]; then
            # This is a case when remote attach issued to the failed node.
            if [[  $mcs_op == "start" ]] && [[  $vmfs_op == "detach" ]]; then
                log_failures "WARNING: Unable to detach disk from the failed node"\
                             "\"$prev_failed_node_name\" as the host on which it is configured is"
                log_failures "         not reachable. Ensure to detach the VMDK disk(s) from the node once"\
                             "the ESX host"
                log_failures "         is reachable using VMware recommended methods."
                return
            fi
            sg_log 0 "ERROR: Failed to detach disk from node \"$local_host_name\" as the"\
                     "vCenter server is unable to reach the"
            sg_log 0 "       ESX host on which the node \"$local_host_name\" is running. Ensure"\
                     "that the disks are detached before"
            sg_log 0 "       restarting the package using VMware recommended methods."
            cleanup_before_exit $error_ret
        else
            check_and_print_vmdk_errors_during_detach $mcs_op $vmfs_op
            cleanup_before_exit $error_ret
        fi

    elif check_if_undefined_error $error_value; then

        log_failures "ERROR: Undefined Error"
        if [[  $mcs_op == "start" ]] && [[  $vmfs_op == "detach" ]]; then
            log_failures "ERROR: Failed to detach disks from the failed node $prev_failed_node_name."
            log_failures "ERROR: Ensure that the disks are detached once the node is reachable"
        fi
        log_failures "$vmfs_output"
        cleanup_before_exit $error_ret

    else

        log_failures "ERROR: Unknown Error"
        log_failures "$vmfs_output"
        cleanup_before_exit $error_ret

    fi
}

