####################################################################
# (C) Copyright 2016-2017 Hewlett Packard Enterprise Development LP.
# (C) Copyright 2006-2015 Hewlett Packard Enterprise Development LP.
# @(#) Serviceguard UPCC Package Configuration Script.
# @(#) Product Name                :  HP Serviceguard
# @(#) Product Version             :  A.12.10.00
# @(#) Patch Name                  :  
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HP Serviceguard.
#
# Changing this file may lead to unrecoverable package startup
# and/or shutdown problems.
#
###################################################################


###########################
# Source utility functions.
###########################

if [[ -f ${SG_UTILS} ]]; then
    . ${SG_UTILS}
    if (( $? != 0 ))
    then
        echo "ERROR: Unable to source package utility functions file: ${SG_UTILS}"
        exit 1
    fi
else
    echo "ERROR: Unable to find package utility functions file: ${SG_UTILS}"
    exit 1
fi

###################################################################
#
# Get the environment for this package through utility function
# sg_source_pkg_env().
#
###################################################################

sg_source_pkg_env $*

function sg_activate_vxvm_disk_group
{
   typeset -i retval=0
   typeset DiskGroup

   sg_log 5 "sg_activate_vxvm_disk_group"

   for I in ${VXVM_DGS[@]}
   do

        if sg_check_if_reconfig_has_failed
        then
            sg_log 999 "WARNING: Not attempting to import Disk Group "$I
            to_exit=1
            continue
        fi

        sg_log 0 "$SG_HOST_NAME\": Importing disk group $I."

        # If the disk group is already imported on this node,
        # check_dg will return 10. Then we can skip the vxdg import
        # since that would fail anyway.
        check_dg $I
        retval=$?
        typeset -i cnt=0
        typeset ret_str
        typeset -i ret_cmd_val
        typeset EN_VXVM_DG_RETRY_VALUE="no"
        typeset msg
        typeset vxdg_scan
        if (( retval > 1 )) && (( retval != 10 )); then
            sg_log 0 "ERROR: Function check_dg"
            sg_log 0 "       Disk Group check failed."
            to_exit=1
            if sg_check_if_it_is_reconfiguration
            then
                sg_log 999 "ERROR: Failed to import Disk Group "$I
                sg_set_reconfig_failure
                continue
            else
                break
            fi
        fi
        if (( retval != 10 )) && (( retval != 1 )); then
                msg=`vxdg -tfC import $I 2>&1`
                ret_cmd_val=$?
        fi

        if [ ! -z "$(echo "$msg" | grep -i 'WARNING')" ]
        then
                vxdg_scan="YES"
        else
                vxdg_scan="NO"
        fi

        if [[ ${SG_VXVM_DG_RETRY:=$EN_VXVM_DG_RETRY_VALUE} = "yes" && $retval -ne 10 ]]; then
            if [[ $ret_cmd_val -ne 0 || $vxdg_scan = "YES" ]]; then

                sg_log 0 "$SG_HOST_NAME\": Import of $I failed, trying again."

                SG_VXVM_DG_RETRY="no"
                ret_str=$(UNIX95=1 ps -C vxdisk -f | awk '
                         /vxdisk scandisks/ { if ("root" == $1) { print $1; exit } }')
                while [[ $ret_str = "root" ]]
                do
                    cnt=$cnt+1
                    if (( $cnt % 5 == 0 )); then
                       sg_log 0 "Waiting for the command vxdisk scandisks to be completed."
                    fi
                    sleep 2
                    ret_str=$(UNIX95=1 ps -C vxdisk -f | awk '
                           /vxdisk scandisks/ { if ("root" == $1) { print $1; exit } }')
                done

                if (( $cnt == 0 )); then
                    vxdisk scandisks
                fi

                check_dg $I
                retval=$?

                if (( retval != 0 )) && (( retval != 10 ))
                then
                    sg_log 0 "ERROR: Function sg_activate_disk_group"
                    sg_log 0 "ERROR: Failed to import Disk Group $I"
                    to_exit=1
                    if sg_check_if_it_is_reconfiguration
                    then
                        sg_log 999 "ERROR: Failed to import Disk Group "$I
                        sg_set_reconfig_failure
                        continue
                    else
                        break
                    fi
                fi
                
                if (( retval != 10 )); then
                    vxdg -tfC import $I
                    retval=$?
                    if (( retval != 0 ))
                    then
                        sg_log 0 "ERROR: Function sg_activate_disk_group"
                        sg_log 0 "ERROR: Failed to import Disk Group $I"
                        to_exit=1
                        if sg_check_if_it_is_reconfiguration
                        then
                            sg_log 999 "ERROR: Failed to import Disk Group "$I
                            sg_set_reconfig_failure
                            continue
                        else
                            break
                        fi
                    fi
                fi
            fi
        fi

        DiskGroup=$I
        eval $SG_VXVOL_CMD
        retval=$?
        # Check for the return value if its 25(NO_OP) means the volumes are already enabled
        # because of the autostartvolume is set to "on", continue with others
        if (( retval == 25 ))
        then
            continue
        fi

        if (( retval != 0 ))
        then
            sg_log 0 "Warning: Function sg_activate_disk_group"
            sg_log 0 "Warning: Failed to $SG_VXVOL_CMD, trying again"

            eval $SG_VXVOL_CMD
            retval=$?

            # Check for the return value if its 25(NO_OP) means the volumes are already enabled
            # because of the autostartvolume is set to "on", continue with others
            if (( retval == 25 ))
            then
                continue
            fi
            if (( retval != 0 ))
            then
                sg_log 0 "ERROR: Function sg_activate_disk_group"
                sg_log 0 "ERROR: Failed to $SG_VXVOL_CMD"
                to_exit=1
                if sg_check_if_it_is_reconfiguration
                then
                    sg_log 999 "ERROR: Failed to $SG_VXVOL_CMD"
                    sg_set_reconfig_failure
                    continue
                else
                    break
                fi
            fi
        fi
    done
}

#################################
# This function will exits with
# an error if the volume group
# is already activated by another
# node.
##################################
function check_dg
{
    typeset -i retval=0
    typeset vg=$1
    typeset hostid=""
    typeset disk=""

    sg_log 5 "check_dg"

    vxdisk -s list > /dev/null 2>&1
    retval=$?
    if (( retval != 0 )) ; then
       sg_log 0 "ERROR: Function check_dg"
       sg_log 0 "       Error in vxdisk"
       return 1
    fi

    #get the disk

    disk=$(vxdisk -s list | awk -v vg=$1 '
           /^Disk:/   { disk = $2 }
           /^dgname:/ { if (vg == $2) { print disk; exit } }')

    if [[ "$disk" = "" ]]
    then
        sg_log 0 "ERROR: Function check_dg"
        sg_log 0 "       Error $vg does not have any disk"
        return 1
    fi

    #get the hostids in the  vg

    hostid=$(vxdisk list $disk | awk  '/^hostid:/ { print $2; exit }')

    #check hostid
    if [ "$hostid" != "" ]
    then
        status=""

        if [[ "$SG_HOST_NAME" != "$hostid" ]]
        then

           cl_hostid=${hostid%%.*}
           #get status of node
           status=$(cmviewcl -f line -n $cl_hostid 2>/dev/null | grep state)
           if [ -z "$status" ] ; then
               #try again
               status=$(cmviewcl -f line -n $cl_hostid 2>/dev/null | grep state)

               if (( $? != 0 ))
               then
                   # cmviewcl failed, most likely node is being added
                   # this check cannot be performed
                   return 0
               fi
           fi

           status=${status#*=}
           case $status in
                *ailed*)
                   sg_log 0 "WARNING: Function check_dg failed"
                   sg_log 0 "         $hostid status failed..okay to activate $vg"
                   ;;
                *) sg_log 0 "ERROR: Function check_dg failed"
                   sg_log 0 "$vg may still be imported on $hostid"
                   sg_log 0 "To correct this situation, logon to \"$hostid\" and" 
                   sg_log 0 "execute the following command:"    
                   sg_log 0 "vxdg deport $vg"
                   sg_log 0 "Once \"$vg\" has been deported from \"$hostid\", "
                   if [ $is_vxvm_validate = 1 ]; then
                       sg_log 0 "cmcheckconf(1M)/cmapplyconf(1M) can be executed again."
                   else 
                       sg_log 0 "this package may be restarted via either cmmodpkg(1M)"
                       sg_log 0 "or cmrunpkg(1M)."
                   fi
                   sg_log 0 ""
                   sg_log 0 "In the event that \"$hostid\" is either powered off"
                   sg_log 0 "or unable to boot, then \"$vg\" must be force "
                   sg_log 0 "imported. "
                   sg_log 0 ""
                   sg_log 0 "******************* WARNING **************************"
                   sg_log 0 ""
                   sg_log 0 "The use of force import can lead to data corruption if"
                   sg_log 0 "\"$hostid\" is still running and has \"$vg\""
                   sg_log 0 "imported. It is imperative to positively determine that"
                   sg_log 0 "\"$hostid\" is not running prior to performing the force"
                   sg_log 0 "import. See \"-C\" option on vxdg(1M)."
                   sg_log 0 ""
                   sg_log 0 "*******************************************************"
                   sg_log 0 ""
                   sg_log 0 "To force import \"$vg\", execute the following"
                   sg_log 0 "commands on the local system:"
                   sg_log 0 "   vxdg -tfC import $vg"
                   sg_log 0 "   vxdg deport $vg"
                   sg_log 0 ""
                   return 2
                   ;;
            esac
        else
                retval=10
        fi
    fi
    return $retval
}

function sg_dg_fuser
{
    typeset opts=""
    while (( $# > 1 ))
    do
        opts="$opts $1" ; shift
    done
    typeset dg=$1

    sg_log 5 "sg_dg_fuser"

    for vol in /dev/vx/rdsk/$dg/*
    do
        [[ -n "$opts" ]] && fuser $opts $vol 2>&1 | awk -F: '$2 !~ "^ *$"'
    done
}

function sg_deactivate_dg
{
    typeset dg=$1
    typeset -i retval=0
    typeset -i cmd_status=1
    typeset out

    sg_log 5 "sg_deactivate_dg"

    out=$(vxdg deport $dg 2>&1)
    cmd_status=$?
    # show output if failure.  if retryable, run fuser and return 2.
    if (( cmd_status != 0 ))
    then
        sg_log 0 "vxdg failed with exit code $cmd_status"
        sg_log 0 "$out"
        retval=1
        echo "$out" | grep -qi -e 'in use' -e 'is open' && retval=2
        (( retval == 2 )) && sg_dg_fuser $dg
    fi
    return $retval
}

function sg_deactivate_dg_with_retries
{
    typeset dg=$1
    typeset -i num_retries=0
    typeset -i retval=0

    sg_log 5 "sg_deactivate_dg_with_retries"

    sg_deactivate_dg $dg
    retval=$?
    while (( retval == 2 && num_retries <  SG_DEACTIVATION_RETRY_COUNT ))
    do
        sg_log 0 "WARNING: Function sg_deactivate_disk_group"
        sg_log 0 "WARNING: failed to deport disk group $dg, trying again."

        if  [[ $SG_KILL_PROCESSES_ACCESSING_RAW_DEVICES = "yes" ]]
        then
            sg_log 0 "WARNING: Running fuser to remove anyone using the raw device directly."
            sg_dg_fuser -ku $dg
        fi
        # vxvm has background processes that can touch dgs/volumes, so
        # wait a bit to let those finish before retrying
        cmsleep 1
        sg_deactivate_dg $dg
        retval=$?
        (( retval == 0 )) && sg_log 0 "Retry of deport on disk group $dg succeeded"

        (( num_retries = num_retries + 1 ))
    done

    return $retval
}

function sg_deactivate_vxvm_disk_group
{
   typeset -i retval=0

   sg_log 5 "sg_deactivate_vxvm_disk_group"

    for I in ${VXVM_DGS[@]}
    do
        if sg_check_if_reconfig_has_failed
        then
            sg_log 999 "WARNING: Not attempting to deport Disk Group "$I
            to_exit=1
            continue
        fi
        #
        # if disk group is imported on this node then
        # check_dg() returns 10.  Proceed with deporting
        # the disk group only if it's imported on this node.
        #
        check_dg $I
        retval=$?
        (( retval == 10 )) || continue

        sg_log 0 "Deporting disk group $I"
        sg_deactivate_dg_with_retries $I
        retval=$?
        if (( retval != 0 ))
        then
            sg_log 0 "ERROR: Function sg_deactivate_disk_group"
            sg_log 0 "ERROR: Failed to deport Disk Group $I"
            to_exit=1
            if sg_check_if_it_is_reconfiguration
            then
                sg_log 999 "ERROR: Failed to deport Disk Group "$I
                sg_set_reconfig_failure
            fi
        fi
    done
}

function sg_reconfigure_start_disk_group
{
    typeset DGS
    typeset VXVM_DGS

    count=0
    sg_log 5 "sg_reconfigure_start_dg"

    for I in ${SG_VXVM_DG[@]}
    do
        found=0
        for J in ${PREV_SG_VXVM_DG[@]}
        do
            if [[ $I = $J ]]
            then
                found=1
                break
            fi
        done
        if ((found == 0))
        then
            #"new VXVM DG $I"
            VXVM_DGS[$count]=$I
            ((count=count+1))
        fi
    done

    if (( ${#VXVM_DGS[@]} > 0 ))
    then
        if sg_check_if_reconfig_has_failed
        then
            for I in ${VXVM_DGS[@]}
            do
                sg_log 999 "WARNING: Not attempting to activate Disk Group "$I
            done
            to_exit=1
        else
            #"new vxvm dgs $dgs"
            sg_activate_vxvm_disk_group
        fi
    fi
}

function sg_reconfigure_stop_disk_group
{
    typeset DGS
    typeset VXVM_DGS

    count=0
    for I in ${PREV_SG_VXVM_DG[@]}
    do
        found=0
        for J in ${SG_VXVM_DG[@]}
        do
            if [[ $I = $J ]]
            then
                #"Found VxVM DG $I"
                found=1
                break
            fi
        done
        if ((found == 0))
        then
            #"removed VXVM DG $I"
            VXVM_DGS[$count]=$I
            ((count=count+1))
        fi
    done


    if (( ${#VXVM_DGS[@]} > 0 ))
    then
        if sg_check_if_reconfig_has_failed
        then
            for I in ${VXVM_DGS[@]}
            do
                sg_log 999 "WARNING: Not attempting to deport Disk Group "$I
            done
            to_exit=1
        else
            #"removed vxvm dgs ${VXVM_DGS[@]}"
            sg_deactivate_vxvm_disk_group
        fi
    fi
}

###################################################################
#
# Sanity Check on LVM. Will check for LVM version and metadata type.
#
###################################################################

function lvm_sanity_check
{
    typeset -i ret=0

    sg_log 5 "lvm_sanity_check"

# If lvm is lvm1 then just return.
    if (( LVM_VER == 1 )); then
        (( ret = 1 ))
    fi

    return $ret
}

###################################################################
#
# This function will add or remove tags for vgs.
#
###################################################################
function vg_tag
{
    typeset op=$1
    typeset vg=$2
    typeset tag=$3
    typeset result

    sg_log 5 "vg_tag"

    if [[ x$op = xaddtag ]] || [[ x$op = xdeltag ]]; then
        sg_log 0 "Attempting to $op to vg $vg..."
        result=$(vgchange --$op $tag $vg 2>&1)
        case $result in
              *Volume*group*successfully*changed*)
                  sg_log 0 "$op was successful on vg $vg."
              ;;
              *Volume*group*does*not*support*tags*)
                  sg_log 0 "VG $vg does not support tags."
              ;;
              *)
                  sg_log 0 "vgchange --$op error:"
                  sg_log 0 $result
              ;;
        esac
    else
        sg_log 0 "vg_tag: illegal operation: $op."
    fi

}

####################################################################
#
# This function is used to check if there are missing pvs in a vg
#
####################################################################

function is_pv_missing_in_vg
{
    typeset vg=$1

    if ! sg_check_if_skip_pr_modules; then
        return 0
    fi

    output=$(vgdisplay -v $vg 2>&1)
    case "$output" in
    *There*are*physical*volumes*missing*)
        sg_log 0 "WARNING: Volume group /dev/$vg has an offline(unknown) physical volume."
    ;;
    esac

    return 0
}

##########################################################################
#
# This function is used to check if there are multiple tags on a vg
#
##########################################################################

function is_multiple_tag_on_vg
{
    typeset vg=$1
    typeset hostid=""
    
    hostid=$(vgs --noheadings -o +tags $vg 2>/dev/null\
           | awk  '{print $8}')  

    if [[ "$hostid" = *,* ]]; then
        sg_log 0 "ERROR: Volume Group $vg has multiple tags \"($hostid)\" defined."
        sg_log 0 "There cannot be more than one tag."
        sg_log 0 "Remove the tags using the command(s):"
        OLDIFS=$IFS
        IFS=","
        for i in ${hostid}; do
            sg_log 0 "\"vgchange --deltag $i $vg\"."
        done
        IFS=$OLDIFS
        return 1
    fi
}

###################################################################
#
# This function will check to see if a VG is activated on another node.
# It will do this by using LVM2 'tags'.  
#
###################################################################

function activation_check
{
    typeset -i ok_to_activate=0
    typeset -i retval=0
    typeset vg=$1
    typeset hostid=""
    typeset cl_hostid=""
    typeset disk=""
    typeset chkvol=""

    sg_log 5 "activation_check"

    # lvm_sanity_check will verify that we are using lvm2.

    lvm_sanity_check $vg
    retval=$?

    if (( retval == 1 )); then
        # using lvm1
        return 0
    fi

    #Check whether disk is accessible.

    disk=$(vgs $vg 2>&1)
    chkvol=$(echo "$disk" | sed -n '$p')
    case $chkvol in
        *Volume*group*not*found*)
        case $disk in
            *Input/output*error*)
                sg_log 0 "Warning: Volume group $vg not found"
                sg_log 0 "Warning: Unable to access one of the disks belonging to the"
                sg_log 0 "         volume group: $vg. The disk may not be accessible."
                return 2
            ;;
        esac
        ;;
    esac

    # get the hostids in the vg and make sure there is only one tag.
    # abort if multiple tags.

    hostid=$(vgs --noheadings -o +tags $vg 2>/dev/null\
             | awk  '{print $8}')  

    # Abort if multiple.
    # Note: This is an abnormal condition and cannot be determined
    #       why this condition exists. A person had to have added
    #       multiple tags for some reason and as a result Serviceguard
    #       cannot assume why and thus cannot bring the package up
    #       or remove the tags automatically. That has to be done by
    #       a real person.
    is_multiple_tag_on_vg $vg
    if (( $? == 1 ))
    then
        return 1
    fi
   
    # check hostid

    if [[ "$hostid" != "" ]] ; then 

        status=""
  
        if [[ $SG_HOST_NAME != $hostid ]] ; then

            cl_hostid=${hostid%%.*}
            #get status of node
            status=$(cmviewcl -f line -n $cl_hostid 2>/dev/null | grep state)
            if [ -z "$status" ] ; then
               #try again  
                status=$(cmviewcl -f line -n $cl_hostid 2>/dev/null | grep state)
                if [ -z "$status" ] ; then
                    #give up
                    sg_log 0 "ERROR: activation_check: Error in cmviewcl."
                    sg_log 0 "ERROR: cmviewcl output:"
                    cmviewcl -n $cl_hostid
                    return 1 
                fi
            fi


            status=${status#*=}
            case $status in
                *ailed*) 
                   sg_log 0 "Warning: Function activation_check:"
                   sg_log 0 "$hostid status failed..okay to activate $vg" 
                   # remove the old tag since that host is failed.
                   vg_tag deltag $vg $hostid
                   ok_to_activate=1
                   ;;
                *)
                   typeset prev_node_type=""
                   prev_node_type=`get_node_type_from_node_name $hostid`

                   if [[ ${#SG_VMDK_FILE_NAME[*]} > 0  && "$prev_node_type"="vmware" ]]; then
                       sg_log 0 "Warning: Function activation_check:"
                       sg_log 0 "$hostid status up.."
                       sg_log 0 "vmfs setup.. okay to activate $vg"
                       # remove the old tag since that host is failed.
                       vg_tag deltag $vg $hostid
                       ok_to_activate=1
                   else
                      sg_log 0 "ERROR: Function activation_check"
                      sg_log 0 "Error $vg may still be activated on $hostid"

                      sg_log 0 "To correct this situation, logon to \"$hostid\" and"
                      sg_log 0 "execute the following commands:"
                      sg_log 0 "   vgchange -a n $vg"
                      sg_log 0 "   vgchange --deltag $hostid $vg"
                      sg_log 0 "Once \"$vg\" has been deactivated from \"$hostid\","
                      sg_log 0 "this package may be restarted via either cmmodpkg(1M)"
                      sg_log 0 "or cmrunpkg(1M)."
                      sg_log 0 "In the event that \"$hostid\" is either powered off"
                      sg_log 0 "or unable to boot, then \"$vg\" must be forced"
                      sg_log 0 "to be activated on this node."
                      sg_log 0 "******************* WARNING ***************************"
                      sg_log 0 "Forcing activation can lead to data corruption if"
                      sg_log 0 "\"$hostid\" is still running and has \"$vg\""
                      sg_log 0 "active. It is imperative to positively determine that"
                      sg_log 0 "\"$hostid\" is not running prior to performing"
                      sg_log 0 "this operation."
                      sg_log 0 "*******************************************************"
                      sg_log 0 "To force activate \"$vg\", execute the following"
                      sg_log 0 "command on the local system:"
                      sg_log 0 "   vgchange --deltag $hostid $vg"
                      sg_log 0 "The package may then be restarted via either"
                      sg_log 0 "cmmodpkg(1M) or cmrunpkg(1M) commands."
                      return 2
                   fi
                   ;;
            esac

        else
            ok_to_activate=1
        fi
    else
        ok_to_activate=1
    fi 
    if (( ok_to_activate == 1 )); then
        vg_tag addtag ${vg} $SG_HOST_NAME
    fi
    return 0
}

##########################################################################
#
# sg_validate_volume_group() is called while validating the package
# configuration (through cmcheckconf/cmapplyconf)
#
##########################################################################

function sg_validate_volume_group
{
    typeset -i r=0
    typeset result

    sg_log 5 "sg_validate_volume_group"
    #
    # validate vg's
    #

    for I in ${SG_VG[@]}
    do
        result=$(vgdisplay $I 2>&1)

        # always return 0 status, so check for output of command
        case $result in
            *Volume*group*not*found*)
                sg_log 0 "ERROR: Function sg_validate_volume_groups"
                sg_log 0 "       Missing Volume group $I"
                to_exit=1
                continue
                ;;
        esac
        
        # get the hostids in the vg and make sure there is only one tag.
        # abort if multiple tags.
        is_multiple_tag_on_vg $I
        if (( $? == 1 ))
        then
            to_exit=1
        fi

        # Warn if PV's are missing in VMFS storage configuration.
        is_pv_missing_in_vg $I
    done

    #
    # validate dgs
    #
    #When package is not running or if self is owner node
    #then we can run validate
    if [[ -z $owner_node \
          || $owner_node = $(HOST=`hostname`; echo ${HOST%%.*}) ]]
    then
        for I in ${SG_VXVM_DG[@]}
        do
              is_vxvm_validate=1
              check_dg $I
              retval=$?
              if (( retval > 0 )) && (( retval != 10 )); then
                 sg_log 0 "ERROR: Function sg_validate_volume_groups"
                 sg_log 0 "       check_dg failed for $I"
                 to_exit=1
              fi
         done
    fi
}

#######################################################################
#
# sg_activate_volume_group() is called while starting a package (cmrunpkg)
#
#######################################################################

function sg_activate_volume_group
{
    typeset result
    typeset -i retval=0
    typeset -i ret=0

    sg_log 5 "sg_activate_volume_group"

    for I in ${VGS[@]}
    do
        if sg_check_if_reconfig_has_failed
        then
            sg_log 999 "WARNING: Not attempting to activate Volume Group "$I
            continue
        fi
        activation_check $I
        retval=$?
        if (( retval != 0 ))
        then
            sg_log 0 "ERROR: Failed activation_check for $I"
            to_exit=$retval
            if sg_check_if_it_is_reconfiguration
            then
                sg_log 999 "ERROR: Failed to activate Volume Group $I"
                sg_set_reconfig_failure
                continue
            else
                return
            fi
        fi

        sg_log 0 "Activating volume group $I ."

        result=$($SG_VGCHANGE_CMD $I 2>&1)
        ret=$?
        # In Linux vgchange command returns non-zero value
        # status when activation of VGs fail. It will handle
        # error like "Refusing activation of partial LV".
        case $result in
            *Volume*group*not*found*)
                if sg_check_if_it_is_reconfiguration
                then
                    sg_log 999 "Error: Failed to activate Volume Group $I"
                    sg_set_reconfig_failure
                fi
                sg_log 0 "$result"
                sg_log 0 "ERROR: Failed to activate Volume Group $I"
                to_exit=1
                ;;
            *)
                if ((ret != 0))
                then
                    if sg_check_if_it_is_reconfiguration
                    then
                        sg_log 999 "Error: Failed to activate Volume Group $I"
                        sg_set_reconfig_failure
                    fi
                    sg_log 0 "$result"
                    sg_log 0 "ERROR: Failed to activate Volume Group $I"
                    to_exit=1
                fi
                ;;
        esac
    done
}

##########################################################################
#
# sg_deactivate_volume_group() is called while stopping a package (cmhaltpkg)
# or during rollback operation to recover from the package startup.
#
# Perform multiple volume group deactivations at same time.
# Limit the number of concurrent deactivations to 
# SG_CONCURRENT_VGCHANGE_OPERATIONS
#
##########################################################################

function sg_deactivate_volume_group
{
    typeset result
    typeset results
    typeset -i repeat
    typeset -i retval=0
    typeset -i numvols=0

    sg_log 5 "sg_deactivate_volume_group"

    for I in ${VGS[@]}
    do
        if sg_check_if_reconfig_has_failed
        then
            sg_log 999 "WARNING: Not attempting to deactivate Volume Group "$I
            continue
        fi

        sg_log 0 "Deactivating volume group $I"
        # Set a retry in case the vgchange -a n fails. Will always do 
        # 1 for sure.
        typeset -i fscount=0
        for J in ${SG_FS_NAME[@]}
        do
            echo $J | grep -q $I
            if [ $? == 0 ]
            then
                (( fscount = fscount + 1 ))
            fi
        done
        (( repeat = ($fscount*2) + 1 ))
        while (( repeat > 0 )); do
            (( retval = 0 ))
            (( cmd_retval = 0 ))
            results=$(vgchange -a n $I 2>&1)
            cmd_retval=$?
            result=$(echo "$results" | sed -n '$p')

            # always return 0 status, so check for output of command
            case $result in
                *successfully*deactivated*)
                    (( repeat = 0 ))
                    ;;
                *logical*volume*in*volume*group*now*active*)
                    numlvols=${result%%logical*}
                    if (( numlvols == 0 )); then
                       (( repeat=0 ))
                    else
                       sg_log 0 "$result"
                       (( repeat = repeat - 1 ))
                       sg_log 0 "Will retry deactivation of $I $repeat more times ..."
                       (( retval = 1 ))
                    fi
                    ;;
                # With RHEL 6.5 the "vgchange -a n" command has been added with extra string 
                # "Skipping volume group". When a service monitoring the underlying VG fails 
                # the package would not fail over to the adoptive node because the case 
                # "Volume group not found" is skipped due to addition of new string in the command.
                *Skipping*volume*group*)
                    sg_log 0 "$result"
                    (( repeat = repeat - 1 ))
                    sg_log 0 "Will retry deactivation of $I $repeat more times ..."
                    case $results in
                        *Volume*group*not*found*)
                        case $results in
                            *Input/output*error*)
                                sg_log 0 "Warning: Volume group $I not found"
                                sg_log 0 "Warning: Unable to access one of the disks belonging to the"
                                sg_log 0 "         volume group: $I. The disk may not be accessible."
                                if [ $repeat -eq 0 ]
                                then
                                    (( retval = 2 ))
                                fi
                             ;;
                        esac
                         ;;
                    esac
                    ;;
                *Volume*group*not*found*)
                    sg_log 0 "$result"
                    (( repeat = repeat - 1 ))
                    sg_log 0 "Will retry deactivation of $I $repeat more times ..."
                    case $results in
                        *Input/output*error*)
                            sg_log 0 "Warning: Volume group $I not found"
                            sg_log 0 "Warning: Unable to access one of the disks belonging to the"
                            sg_log 0 "         volume group: $I. The disk may not be accessible."
                            if [ $repeat -eq 0 ]
                            then
                                (( retval = 2 ))
                            fi
                         ;;
                    esac
                    ;;
                *)
                    if [ $cmd_retval -eq 0 ]
                    then
                        sg_log 0 "Successfully deactivated volume group $I"
                        break
                    else 
                        (( repeat = repeat - 1 ))
                        case $result in
                            *Can*t*deactivate*volume*group*with*open*logical*volume*)
                                sg_log 0 "VG $I busy, will retry deactivation $repeat more times ..."
                                ;;
                            *)
                                sg_log 0 "$result"
                                sg_log 0 "Will retry deactivation of $I $repeat more times ..."
                                ;; 
                        esac 
                        sleep 1
                        (( retval = 1 ))
                    fi
                    ;;
            esac
        done

        if (( retval != 0 ))
        then
            if sg_check_if_it_is_reconfiguration
            then
                sg_log 999 "ERROR: Failed to deactivate Volume Group $I"
                sg_set_reconfig_failure
            fi
            sg_log 0 "ERROR: Failed to deactivate Volume Group $I"
            sg_log 0 "$results"
            to_exit=$retval
        else
            if sg_check_if_it_is_reconfiguration
            then
                sg_log 999 "SUCCESS: Successfully deactivated Volume Group $I"
            fi
        fi

        # The vg needs to be in lvm2 format, otherwise tags will not work.
        if ! sg_check_if_reconfig_has_failed
        then
            lvm_sanity_check $I
            if (( $? == 0 )) && (( retval == 0 ))
            then
                # VG has been deactivated, remove the tag.
                vg_tag deltag $I $SG_HOST_NAME
            fi
        fi
    done
}

function sg_reconfigure_start_volume_group
{
    declare -a VGS
    count=0
    sg_log 5 "sg_reconfigure_start_volume_group"

    for I in ${SG_VG[*]}
    do
        found=0
        for J in ${PREV_SG_VG[*]}
        do
            if [[ $I = $J ]]
            then
                sg_log 5 "Found VG $I"
                found=1
                break;
            fi
        done
        if ((found == 0 ))
        then
            if ! sg_check_if_reconfig_has_failed
            then
                sg_log 0 "New VG $I"
            fi
            VGS[$count]=$I
            ((count=count+1))
        fi
    done

    if (( ${#VGS[@]} > 0 ))
    then
        if sg_check_if_reconfig_has_failed
        then
            for I in ${VGS[@]}
            do
                sg_log 999 "WARNING: Not attempting to activate Volume Group "$I
            done
            to_exit=1
        else
            sg_activate_volume_group
        fi
    fi
}

function sg_reconfigure_stop_volume_group
{
    declare -a VGS
    count=0
    sg_log 5 "sg_reconfigure_stop_volume_group"

    for I in ${PREV_SG_VG[*]}
    do
        found=0
        for J in ${SG_VG[*]}
        do
            if [[ $I = $J ]]
            then
                sg_log 5 "Found VG $I"
                found=1
                break
            fi
        done
        if ((found == 0))
        then
            if ! sg_check_if_reconfig_has_failed
            then
                sg_log 0 "VG removed $I"
            fi
            VGS[$count]=$I
            ((count=count+1))
        fi
    done

    if (( ${#VGS[@]} > 0 ))
    then
        if sg_check_if_reconfig_has_failed
        then
            for I in ${VGS[@]}
            do
                sg_log 999 "WARNING: Not attempting to deactivate Volume Group "$I
            done
            to_exit=1
        else
            sg_deactivate_volume_group 
        fi
    fi
}

function main_sg_activate_vxvm_disk_group
{
    declare -a VXVM_DGS
    typeset -i R=0

    if (( ${#SG_VXVM_DG[@]} > 0 ))
    then
        while (($R < ${#SG_VXVM_DG[@]}))
        do
          VXVM_DGS[$R]=${SG_VXVM_DG[$R]}
          ((R=R+1))
        done

        if sg_check_if_reconfig_has_failed
        then
            for I in ${SG_VXVM_DG[@]}
            do
                sg_log 999 "WARNING: Not attempting to activate Disk Group "$I
            done
            to_exit=1
        else
            sg_activate_vxvm_disk_group
        fi

    fi
}

function main_sg_deactivate_vxvm_disk_group
{
    declare -a VXVM_DGS
    typeset -i R=0

    if (( ${#SG_VXVM_DG[@]} > 0 ))
    then
        while (($R < ${#SG_VXVM_DG[@]}))
        do
          VXVM_DGS[$R]=${SG_VXVM_DG[$R]}
          ((R=R+1))
        done

        sg_deactivate_vxvm_disk_group

    fi
}

function main_sg_activate_volume_group
{
    declare -a VGS
    typeset -i R=0

    if (( ${#SG_VG[@]} > 0 ))
    then 
        while (( $R < ${#SG_VG[@]} ))
        do
            VGS[$R]=${SG_VG[$R]}
            ((R=R+1))
        done
        if sg_check_if_reconfig_has_failed
        then
            for I in ${SG_VG[@]}
            do
                sg_log 999 "WARNING: Not attempting to activate Volume Group "$I
            done
            to_exit=1
        else
            sg_activate_volume_group
        fi
    fi
}

function main_sg_deactivate_volume_group
{
    declare -a VGS
    typeset -i R=0

    while (( $R < ${#SG_VG[@]} ))
    do
        VGS[$R]=${SG_VG[$R]}
        ((R=R+1))
    done
    if (( ${#VGS[@]} > 0 ))
    then
        sg_deactivate_volume_group
    fi
}

################
# main routine
################

#
# Module script must be specified with three required entry points:
# start, stop, and validate.
#
# The variable to_exit indicates the success or failure of the entry 
# point.
#

typeset -i to_exit=0
typeset -i is_vxvm_validate=0

if (( ${#SG_VG[*]} > 0 )); then
    typeset LVM_VER=$( lvm version > /dev/null 2>&1 ; \
        if (($? == 127));then echo 1; else echo 2;fi)
fi

case ${1} in
     start)
        main_sg_activate_vxvm_disk_group       
        main_sg_activate_volume_group
        ;;
     stop)        
        main_sg_deactivate_volume_group
        main_sg_deactivate_vxvm_disk_group
        ;;

     validate)
        if sg_check_if_storage_module_skip; then
            sg_log 5 "INFO: Package configured with VMFS storage. Validation of volume group will"\
                     "be done on node which has lowest node id or on the node where package"\
                     "is running."
            to_exit=0
        else
            sg_validate_volume_group
        fi
        ;;
    
    reconfigure_start)
        sg_reconfigure_start_disk_group
        sg_reconfigure_start_volume_group
        ;;

    reconfigure_stop)
        sg_reconfigure_stop_volume_group
        sg_reconfigure_stop_disk_group
        ;;

     *)
           sg_log 0 "INFO: Unknown operation - $1"
           ;;
esac

#
# Exit to indicate success/failure to the master_control_script.sh
#
exit $to_exit
