#!/usr/bin/perl -w
##################################################################
# Copyright (c) 2016 Hewlett Packard Enterprise Development LP
#
# (C) Copyright 2014-2015 Hewlett-Packard Development Company, L.P.
# @(#) Serviceguard cluster simulation functions
# @(#) Product Name : HP Serviceguard
# @(#) Product Version : A.12.10.00
# @(#) Patch Name :     
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HP Serviceguard.
#
# Changing this file may lead to issues in Simulator behaviour
#
###################################################################
 
use Config;
#use strict;
use Getopt::Long;
use warnings;
use CSIMSGCluster;
use Data::Dumper;
no warnings;
require 'StatusUtils.pl';

my @file_out = undef;
my $SGCONF;
my $clusterObj;

# subroutnines

sub getSGCONF{

    my $sgconffile = "/etc/cmcluster.conf";
    if (-f $sgconffile) {
        @file_out = qx(cat $sgconffile);
    } else {
        exit(1);	                 
    }

    my @sgconf_out = grep(/^SGCONF/, @file_out);
    my @split_sgconf_name = split(/=/,$sgconf_out[0]);
    chomp($split_sgconf_name[1]);
    $SGCONF = $split_sgconf_name[1];

    return $SGCONF;
}

sub getCDBVer {
    my ($sessionName)= @_;
    my $SGCONF=getSGCONF();
    my @v;
    my $sessionDir="$SGCONF/simulator". "/".$sessionName;
    my $fileName="$SGCONF/simulator" . "/" . $sessionName . "/" . $sessionName . "_cmviewcl_line";
    if (-e "$fileName"){
        open SRCASCII, "< $fileName"; 
        while ( <SRCASCII>){
            if ( $_=~/configuration_data_version/){
                @v=split("=",$_);
                return $v[1];
            }
        }
    }
    return -1;
}

# subroutine to delete the cluster configuration
sub deleteConfCluster{
    my ($cluster, $force)=@_;
    my $SGCONF=getSGCONF();
    if ( -d "$SGCONF/simulator/$cluster"){
        my $viewfile = "$SGCONF/simulator/$cluster/$cluster"."_cmviewcl_line";
        if ( -e $viewfile) {
            read_ClusterObj($cluster);
            if ($clusterObj->getStatus() eq "up") {
                print "Checking current status\n";
                print "cmdeleteconf: Cluster ".$cluster." has to be halted before being deleted.\n";
                return -1;
            }
        }
        unless($force) {
            print "Delete cluster $clusterName ([y]/n)";
            while (<STDIN>) {
                if ($_ =~ /^\s*$/){
                    $res='y';
                    last;
                }
                $res=$_;
                last;
            }
            if ($res=~/y/i) {
                `rm -r "$SGCONF/simulator/$cluster" 2>/dev/null` ;
                return 0; 
            }
        }
        else {
            `rm -r "$SGCONF/simulator/$cluster" 2>/dev/null` ;
            return 0; 
        }
    }
    else {
        print "Cluster $cluster does not exist.\n";
        return -1;
    }
    if ($? != 0){
        print "Unable to delete the cluster $cluster.\n";
        return -1;
    }
}

# subroutine to delete the package configuration
sub deleteConfPackage{
    my ($sessionName,$package,$force)=@_;
    my $index = 0;
    my $found = 0;
    
    read_ClusterObj($sessionName);
    
    foreach my $pkg (@{$clusterObj->{packages}}){
        if ($pkg->{name} eq $package) {
            if ($pkg->getStatus() eq "up") {
                print "Unable to delete package $package while it is running \n";
                return -1;
            }
            $found = 1;
            last;
        }
        $index += 1;
    }
    if ($found) {
        unless ($force) {
            print "Modify the package configuration ([y]/n)";
            while (<STDIN>) {
                if ($_ =~ /^\s*$/){
                    $res='y';
                    last;
                }
                $res=$_;
                last;
            }
         }
         if ($res=~/y/i || $force ) {
                splice(@{$clusterObj->{packages}},$index,1);
                write_ClusterObj($sessionName);
                return 0;
         }
         else{
                # Option 'n' was entered 
                return -1; 
         }
    } 
    else {
        print "Package $package does not exist. \n";
        return -1;
    }
}




# subroutine to validate a session

sub validateSession{
    my ($session)=@_;
    my $SGCONF = getSGCONF();
    my @sessionNames = `ls "$SGCONF/simulator" 2>/dev/null`;
    foreach my $s (@sessionNames){
        chomp ($s);
        if (($s eq $session)and ($s ne "")){
           return 0;
        }
    }
    return -1;
}

sub cm_view_cl {
    my ($sessionName, $cmd )= @_;
    my $SGCONF=getSGCONF();
    my $sessionDir="$SGCONF/simulator". "/".$sessionName;
    unless (-d "$sessionDir"){
       print "cmviewcl: Cannot view the cluster configuration: No such file or directory.\nEither this node is not configured in a cluster, user doesn't have \naccess to view the cluster configuration, or there is some obstacle\nto viewing the configuration. For a list of possible causes, \nsee the Serviceguard manual for cmviewcl.\n";
       return -1;
    }
    my $fileName="$SGCONF/simulator". "/".$sessionName."/".$sessionName."_cmviewcl_line";
    unless(open SRCASCII, "< $fileName") {
        print "cmviewcl: Cannot view the cluster configuration: No such file or directory.\nEither this node is not configured in a cluster, user doesn't have \naccess to view the cluster configuration, or there is some obstacle\nto viewing the configuration. For a list of possible causes,\nsee the Serviceguard manual for cmviewcl.\n";
        return -1;
    }
    if ($cmd=~/line/){
        while ( <SRCASCII>){
            print $_;
        }
    }
    else {
        my $clusterobj = &statusParser($fileName);
        my $cw = 0;
        $cw = length($clusterobj->getName());
        $cw += 7;
        my $hd1 = sprintf("%%-%d",$cw);
        my $hd = sprintf("%ss",$hd1);
        
        printf "\n$hd%-12s","CLUSTER","STATUS";
        printf "\n$hd%-12s",$clusterobj->getName(),$clusterobj->getStatus();

        my $numofnodes = @{$clusterobj->{nodes}};
        $cw = 0;
        foreach my $node (@{$clusterobj->{nodes}}) {
            if ( length($node->getName()) > $cw ) {
                    $cw = length($node->getName());
            } 
        }
        $cw += 7;
        $hd1 = sprintf("%%-%d",$cw);
        $nhd = sprintf("%ss",$hd1);
		printf "\n\n  $nhd%-12s%-12s","NODE","STATUS","STATE";
		
		my $iter = 0;
		foreach my $node (@{$clusterobj->{nodes}}) {
			$iter += 1;
			printf "\n  $nhd%-12s%-12s", $node->getName(),$node->getStatus(),$node->getState();
			my @pkgOnNode = $clusterobj->getFailOverPackageOnNode($node->getName());
			my $cw = 0;
            foreach my $pkg (@pkgOnNode) {
                if ( length($pkg->getName()) > $cw ) {
                        $cw = length($pkg->getName());
                }
            }
            $cw += 7;
            
            if (@pkgOnNode) {
                print "\n";
                my $hd1 = sprintf("%%-%d",$cw);
                my $hd = sprintf("%ss",$hd1);
                printf "\n    $hd%-16s%-17s%-11s%-11s","PACKAGE","STATUS","STATE","AUTO_RUN","NODE";
                
                foreach my $pkg (@pkgOnNode) {
                    printf "\n    $hd%-16s%-17s%-11s%-11s",$pkg->getName(),$pkg->getStatus(),$pkg->getState(),$pkg->getAutoRun(),$pkg->getOwner();
                }
                if ($iter lt $numofnodes) {
                        printf "\n\n  $nhd%-12s%-12s","NODE","STATUS","STATE";
                }
            }
        }
		print "\n";
		my @mnpkgs = $clusterobj->getMultiNodePackages();
		if (@mnpkgs) { 
			print "\nMULTI_NODE_PACKAGES\n";
		}
        $cw = 0;
        foreach my $pkg (@mnpkgs) {
            if ( length($pkg->getName()) > $cw ) {
                    $cw = length($pkg->getName());
            }
        }
        $cw += 7;
        my $numofmnps = @mnpkgs;
        $hd1 = sprintf("%%-%d",$cw);
        $hd = sprintf("%ss",$hd1);
				
        if ($numofmnps) {
            printf "\n  $hd%-16s%-17s%-11s%-11s","PACKAGE", "STATUS" ,"STATE" ,"AUTO_RUN" ,"SYSTEM";
        }
		foreach my $mnpObj (@mnpkgs) {
                my $nodeConfig = @{$mnpObj->{nodes}};
				my $runningNode = 0;
				foreach my $mypnode (@{$mnpObj->{nodes}}) {
					if ($mypnode->{state} eq "running") {
						$runningNode += 1;
					}
				}
                if ($runningNode && $runningNode != $nodeConfig) {
					my $pkg_status = $mnpObj->getStatus()."(".$runningNode."/".$nodeConfig.")";
					printf "\n  $hd%-16s%-17s%-11s%-11s",$mnpObj->getName(),$pkg_status,$mnpObj->getState(),$mnpObj->getAutoRun(),"no";
				}
				else {
					printf "\n  $hd%-16s%-17s%-11s%-11s",$mnpObj->getName(),$mnpObj->getStatus(),$mnpObj->getState(),$mnpObj->getAutoRun(),"no";
				}
		}
		print "\n";
		my @unownedPkgs = $clusterobj->getUnownedPackages();
        $cw = 0;
        foreach my $pkg (@unownedPkgs) {
            if ( length($pkg->getName()) > $cw ) {
                    $cw = length($pkg->getName());
            }
        }
        $cw += 7;
		if (@unownedPkgs) {
			print "\nUNOWNED_PACKAGES\n";
            my $hd1 = sprintf("%%-%d",$cw);
            my $hd = sprintf("%ss",$hd1);
			printf "\n    $hd%-16s%-17s%-11s%-11s","PACKAGE", "STATUS" ,"STATE" ,"AUTO_RUN" ,"NODE";
			foreach my $pkg (@unownedPkgs) {
                printf "\n    $hd%-16s%-17s%-11s%-11s",$pkg->getName(),$pkg->getStatus(),$pkg->getState(),$pkg->getAutoRun(),$pkg->getOwner();
			}
			print "\n";
		}
	}
		
	close SRCASCII;
	return 0;
}

sub cm_run_cl {

    my $cluster = @_;
    my $cmeval_output_file = "";
    $cluster = shift;
    #Call status parser to fill in data structure
    read_ClusterObj($cluster);
    if($clusterObj->getStatus() eq 'up') {
        foreach my $node (@{$clusterObj->{nodes}}) {
            print "cmruncl: Cluster is already running on \"$node->{name}\".\n";
        }
        return -1;
    }
    $clusterObj->setStatus("up");
    $clusterObj->setState("stable");
    $clusterObj->setIncarnation(1);
    $clusterObj->setSummary("ok");
    foreach my $node (@{$clusterObj->{nodes}}){
        $node->setStatus("up");
        $node->setState("running");
        $node->setInitialIncarnation(1);
        $node->setSummary("ok");
        foreach my $interface (@{$node->{interfaces}}) {
            $interface->setStatus("up");
        }
        foreach my $subnet (@{$node->{subnets}}) {
            $subnet->setStatus("up");
        }
    }
    if ($clusterObj->{qs}) {
        my $qsip=$clusterObj->{qs}->{ipaddress};
        foreach my $qsnode (@{$qsip->{nodes}}) {
            $qsnode->setStatus("up");
            $qsnode->setState("running");
        }
    }
    else {
        foreach my $lockobj (@{$clusterObj->{lockLun}}) {
            foreach my $node (@{$clusterObj->{nodes}}){
                if ($lockobj->getLockNodeName() eq $node->{name}) {
                    $lockobj->setLockStatus("up");
                }
            }
        }
    }

    my $packages = @{$clusterObj->{packages}};
    if ($packages) {
        foreach my $pkgs (@{$clusterObj->{packages}}) {
            foreach my $pNode (@{$pkgs->{nodes}}) {
                $pNode->setAvailable("yes");
                if($pkgs->getType() eq "failover") {                
                    $pNode->setState("blocked");
                }
            }
            # Change Package Monitor Subnet Status to up
            foreach my $subnet (@{$pkgs->{subnets}}) {
                foreach my $my_subnet_node (@{$subnet->{node}}) {
                    $my_subnet_node->setMonitorSubnetNodeStatus("up");
                }
            }
        }
    }
    # Use cmeval's intelligence to set data structure attributes for "cmruncl" command
    $cmeval_output_file = cmeval();
    parse_cmeval_output("cmruncl", $cmeval_output_file);
    write_ClusterObj($cluster);	
    return 0;
}

sub cm_halt_cl {
    my ($cluster,$force) = @_;
    my $cmd_status = 1;

    read_ClusterObj($cluster);
    if($clusterObj->getStatus() eq 'down') {
        print "The cluster currently does not appear to be active.\n";
        return -1;
    }
    unless($force) {
        foreach my $pkg (@{$clusterObj->{packages}}) {
            if($pkg->getState() eq 'running' && $pkg->getStatus() eq 'up') { 
                $cmd_status = 0;
                print "Package ",$pkg->getName()," is still running on ";	
                print $pkg->getOwner()," ";
                print "\n";
            }
        }
        unless($cmd_status){
            print "Use the -f option to forcefully halt the cluster/node including halting packages.\n";
            return -1;
        }
    }
    $clusterObj->setStatus("down");
    $clusterObj->setState("stable");
    $clusterObj->setSummary("critical");

    print "Disabling all packages from starting on nodes to be halted.\n";
    print "Warning:  Do not modify or enable packages until the halt operation is completed.\n";
    foreach my $pkg (@{$clusterObj->{packages}}) {
        change_pkg_status($clusterObj,$pkg,"down","");
        my $pkg_name = $pkg->getName();
        print "Halting package $pkg_name.\nSuccessfully halted package $pkg_name.\n";
        $pkg->setAutoRun($pkg->getConfigParam("initial_autorun"));
        foreach my $pNode (@{$pkg->{nodes}}){
            $pNode->setState("unknown");
            $pNode->setStatus("down");
        }
        # Change Package Monitor Subnet Status to unknown
        foreach my $subnet (@{$pkg->{subnets}}) {
            foreach my $my_subnet_node (@{$subnet->{node}}) {
                $my_subnet_node->setMonitorSubnetNodeStatus("unknown");
            }
        }
    }
    foreach my $node (@{$clusterObj->{nodes}}){
        change_node_status($clusterObj,$node,"down","unknown");
    }
    write_ClusterObj($cluster);
    return 0;	   
}
sub cm_run_pkg {
        
    my ($cluster,$nodeProvided,$ref_pkgs_array,$ref_nodes_array) = @_;
    my @mypkgs = @{$ref_pkgs_array};
    my @mynodes = @{$ref_nodes_array};
    my $node = $mynodes[0];
    chomp($node);
    
    #Call status parser to fill in data structure
    read_ClusterObj($cluster);
    if($clusterObj->{status} eq 'down')
    {
        print "Cluster is down...\nUnable to complete the package command\n";	
        return -1;
    }
    if($node) {
        my $nodeObj = $clusterObj->getNode($node);
        if ($nodeObj->getStatus() ne "up") {
            print "Node $node is currently not up to perform the requested operation\n";
            return -1;
        }
    }
    my $pkgFound = 0;
    
	foreach my $pkg (@mypkgs) {
        $pkgFound = 0;
        foreach my $mypkg (@{$clusterObj->{packages}}) {
			my $pkgName = $mypkg->getName();
            if ($pkgName eq $pkg){
				$pkgFound=1;
                $mypkg->setAutoRun("enabled");
				if($mypkg->getType() eq "failover") {
                    if ($mypkg->getStatus() eq 'up') {
                        my $node_name = $mypkg->getOwner();
                        print "Package $pkgName is already running on node $node_name\n";
                        return 1;
                    }
                    if ($node) {
                        foreach my $tmpnode (@{$mypkg->{nodes}}) {
                            $tmpnode->setAvailable("no");
                        }
                        my $mypNode = $mypkg->getNode($node); 
                        if ($mypNode->getSwitching() eq "enabled") {
                            $mypNode->setAvailable("yes");
                        }
                        else {
                            print "Unable to run package " . $pkgName ." on node ". $mypNode->getName() .". Node is not eligible\n";
                            return -1;
                        }
                    }
                }
                else {
                    if (@mynodes && $nodeProvided == 1) {
                        foreach my $mypNode (@{$mypkg->{nodes}}) {
                            my $nname = $mypNode->getName();
                            chomp($nname);
                            foreach my $node (@mynodes) {
                                if( $nname eq $node) {
                                    if ($mypNode->getStatus() eq "up") {
                                        print "Package $pkgName is already running on all requested nodes\n";
                                        return 0;
                                    }
                                    if ($mypNode->getSwitching() eq "enabled") {
                                        $mypNode->setAvailable("yes");
                                    }
                                    else {
                                        $mypNode->setAvailable("no");
                                        print "Unable to run package " . $pkgName ." on node ". $nname .". Node is not eligible\n";
                                    }
                                }
                            }
                        }
                    }
                    elsif (@mynodes && $nodeProvided == 0) {
                        foreach my $mypNode (@{$mypkg->{nodes}}) {
                            if ($mypNode->getSwitching() eq "enabled") {
                                $mypNode->setAvailable("yes");
                            }
                            else {
                                $mypNode->setAvailable("no");
                                print "Unable to run package " . $pkgName ." on node ". $mypNode->getName() .". Node is not eligible\n";
                            }
                        }	
                    }
				}
       	       				
            }
		}
		unless ($pkgFound) {
			print "cmrunpkg: Package $pkg not found\n";
			return -1;
		}
	}
    
	# Use cmeval's intelligence to set data structure attributes for "cmrunpkg" command
	my $cmeval_output_file = cmeval();
	parse_cmeval_output("cmrunpkg", $cmeval_output_file);
	write_ClusterObj($cluster);
	return 0;
}

sub cm_halt_pkg {
	 
	my ($cluster,$ref_nodes_array, $ref_pkgs_array) = @_;
	my $cmd_status = 1;
    my $fopdone = 0;
	my @mypkgs = @{$ref_pkgs_array};
    my @mynodes = @{$ref_nodes_array};
    
    my $node = $mynodes[0];
    #chomp(@mypkgs);
	read_ClusterObj($cluster);
    my $pkgFound = 0;
    my $nodeFound = 0; 
	foreach my $pkg (@mypkgs) {
		$pkgFound = 0;
        foreach my $mypkg (@{$clusterObj->{packages}}) {
			my $pkgName = $mypkg->getName();
			if ($pkgName eq $pkg){
				$pkgFound = 1;
				if($mypkg->getStatus() eq 'down') {
					print "Package $pkgName is already halted.\n";
					return 0;
				}
				
                if ($mypkg->getType() eq "failover") {
                    $mypkg->setAutoRun("disabled");
                    if ($node) {
                        chomp($node);
                        if ($node ne $mypkg->getOwner()) {
                            print "cmhaltpkg: Package " . $mypkg->getName() . " is running on ". $mypkg->getOwner() ."\n";
                            return -1;
                        }
                    }
                    $fopdone = 1;
                    foreach my $pNode (@{$mypkg->{nodes}}){
                        $pNode->setAvailable("no");
                    }
                }
                else {
                    if (@mynodes) {
                        foreach my $node (@mynodes) {
                            foreach my $pNode (@{$mypkg->{nodes}}){
                                if ($node eq $pNode->getName()) {
                                    $pNode->setAvailable("no");
                                }
                            }
                        }
                    }
                    else {
                        foreach my $pNode (@{$mypkg->{nodes}}){
                            $pNode->setAvailable("no");
                        }
                    }
                }
            }
		}
		unless ($pkgFound)  {
		   print "cmhaltpkg: Package $pkg not found\n";
		   return -1;
		}
	}
	# Use cmeval's intelligence to set data structure attributes for "cmhaltpkg" command
	
    my $cmeval_output_file = cmeval();
    parse_cmeval_output("cmhaltpkg", $cmeval_output_file);
    foreach my $pkg (@mypkgs) {        
        foreach my $mypkg (@{$clusterObj->{packages}}) {
            if ($mypkg->getName() eq $pkg && $mypkg->getType() eq "multi_node") {
                $node = $mynodes[0];
                unless ($node) {
                    foreach my $pNode (@{$mypkg->{nodes}}) {                    
                        $pNode->setState("halted");
                    }
                }
            }
        }
    }
	write_ClusterObj($cluster);	
    print "One or more packages or package instances have been halted.\n";
    if ($fopdone) {
        print "The failover packages have AUTO_RUN disabled and no new instance can start automatically.\n";
        print "To allow automatic start, enable AUTO_RUN \nvia cmmodpkg -e <package_name>\n";
    }
    return 0;	
}
sub cm_mod_pkg {
    my ($cluster,$node,$mode,@mypkgs) = @_;
    chomp(@mypkgs);
	#Call status parser to fill in data structure
	read_ClusterObj($cluster);
	if($clusterObj->{status} eq 'down')
	{
		print "Cluster is down...\nUnable to complete the package command\n";	
		return -1;
	}
	if ($mode eq "disable") {
        my $pkgFound = 0;
		foreach my $pkg (@mypkgs) {
			$pkgFound = 0;
			foreach my $mypkg (@{$clusterObj->{packages}}) {
				my $pkgName = $mypkg->getName();
				if ($pkgName eq $pkg){
					$pkgFound = 1;
					unless ($node) {
                        $mypkg->setAutoRun("disabled");
					}
					else {
						foreach my $mypnode (@{$mypkg->{nodes}}) {
							if ($node eq $mypnode->getName()) {
								$mypnode->setSwitching("disabled");
                                $mypnode->setAvailable("no");
                            }
						}
					}
				}
			}
			unless ($pkgFound) {
				print "cmrunpkg: Package $pkg not found\n";
				return -1;
			}
		}
		write_ClusterObj($cluster);
        return 0;
	}
    my $nodedown=0; 
    my $pkgFound = 0;
	foreach my $pkg (@mypkgs) {
		$pkgFound = 0;
		foreach my $mypkg (@{$clusterObj->{packages}}) {
			my $pkgName = $mypkg->getName();
			if ($pkgName eq $pkg){
				$pkgFound = 1;
				if ( $mypkg->getAutoRun() eq "disabled") {
				    unless ($node) {
                        $mypkg->setAutoRun("enabled");
						foreach my $mypnode (@{$mypkg->{nodes}}) {
                            my $nodeName = $mypnode->getName();
                            if ($mypkg->getType() eq "failover") {
                                my $nodeObj = $clusterObj->getNode($nodeName);
                                if ($nodeObj->getStatus() eq "up") {
                                    $mypnode->setAvailable("yes"); 
                                }
                                else {
                                    $mypnode->setAvailable("no"); 
                                }
                            }
						}
					}
					else {
                        my $mypnode = $mypkg->getNode($node);
						$mypnode->setSwitching("enabled");
						if ($mypkg->getType() eq "failover") {
                            if ($mypnode->getStatus() eq "up"){
                                $mypnode->setAvailable("yes");
                            }
						}
					}
				}
				else {
					unless ($node) {
						foreach my $mypnode (@{$mypkg->{nodes}}) {
							if ($mypnode->getSwitching() eq "disabled") {
								$mypkg->setAutoRun("enabled");
							}
							else { 
								if ($mypkg->getType() eq "failover") {
                                    if ($mypnode->getStatus() eq "up"){
                                        $mypnode->setAvailable("yes");
                                    }
								}
							}
						}
					}
					else {
                        my $mypnode = $mypkg->getNode($node);
						$mypnode->setSwitching("enabled");
						if ($mypkg->getType() eq "failover") {
                            if ($mypnode->getStatus() eq "up"){
                                $mypnode->setAvailable("yes");
                            }
						}
					}
				}	
            }
		}
		unless ($pkgFound) {
			print "cmmodpkg: Package $pkg not found\n";
			return -1;
		}
	}
	
	# Use cmeval's intelligence to set data structure attributes for "cmmodpkg" command
		
	my $cmeval_output_file = cmeval();
	parse_cmeval_output("cmmodpkg", $cmeval_output_file);
	write_ClusterObj($cluster);
	return 0;
}
sub cm_run_node {
	 
    my ($cluster,@nodes) = @_;
    my $cmd_status = 1;
    my $found = 0;

    read_ClusterObj($cluster);
    if($clusterObj->{status} eq 'down')
    {
        print "Cluster is down...\nUnable to complete the node command\n";	
        return -1;
    }
    foreach $node (@nodes) {
        $found = 0;
        chomp($node);
        foreach my $mynode (@{$clusterObj->{nodes}}) {
            if ($mynode->{name} eq $node) {
                $found = 1;
                if ($mynode->getStatus()eq 'up') {
                    print "cmrunnode: Cluster is already running on \"$mynode->{name}\".\n";
                    return -1;
                }
            }
        }
        unless ($found) {
            print "cmrunnode: Node $node is not a configured node name.\n";
            return -1;
        }
        foreach my $mypkg (@{$clusterObj->{packages}}) {
            if ($mypkg->getAutoRun() eq "enabled") {
                foreach my $pNode (@{$mypkg->{nodes}}){
                    my $nodeName = $pNode->getName();
                    if ($nodeName eq $node) {
                        $pNode->setStatus("up");
                        $pNode->setAvailable("yes");
                    }
                }
            }
        }
    } 
    # Use cmeval's intelligence to set data structure attributes for "cmhaltpkg" command

    my $cmeval_output_file = cmeval();
    parse_cmeval_output("cmrunnode", $cmeval_output_file);
    $clusterObj->setStatus("up");

    foreach $node (@nodes) {
        my $mynode = $clusterObj->getNode($node);
        change_node_status($clusterObj,$mynode,"up");
        foreach my $mypkg (@{$clusterObj->{packages}}) {
            # Change Package Monitor Subnet Status to up            
            foreach my $subnet (@{$mypkg->{monitored_subnet}}) {
                foreach my $my_subnet_node (@{$subnet->{node}}) {
                    if ($node eq $my_subnet_node->{name}) {
                        $my_subnet_node->setMonitorSubnetNodeStatus("up");
                    }
                }
            }
            if ($mypkg->getType() eq 'failover') {
                foreach my $pNode (@{$mypkg->{nodes}}){
                    if ($pNode->getName() eq $node) {
                        $pNode->setState("blocked");
                    }
                }
            }
        }
    }
    write_ClusterObj($cluster);	
    return 0;	
}

sub cm_halt_node {

    my ($cluster,$force,@nodes) = @_;
    my $nodes = 0;
    my $found = 0;
    my $cmd_status = 0;

    read_ClusterObj($cluster);
    if($clusterObj->getStatus() eq 'down') {
        print "The cluster currently does not appear to be active.\n";
        return -1;
    }
    foreach $nodeName (@nodes) {
        $found = 0;
        chomp($nodeName);
        foreach my $mynode (@{$clusterObj->{nodes}}) {
            if ($mynode->{name} eq $nodeName) {
                $found = 1;
                if ($mynode->getStatus() eq 'down') {
                    return 0;
                }
            }
        }
        unless ($found) {
            print "cmhaltnode: Node $nodeName is not a configured node name.\n";
            return -1;
        }
        unless($force) {
            foreach my $pkg (@{$clusterObj->{packages}}) {
                if($pkg->getState() eq 'running' && $pkg->getStatus() eq 'up') {
                    if($pkg->getType() eq 'failover') {
                        if($pkg->getOwner() eq $nodeName) {
                            print "Package ",$pkg->getName()," is still running on ";
                            print "$nodeName .";
                            print "\n";
                            $cmd_status = 1;
                            last; 
                        }
                    }
                    else {
                        foreach $node (@{$pkg->{nodes}}) {
                            if ($node->getStatus() eq 'up') {
                                print "Package ",$pkg->getName()," is still running on ";
                                print "$nodeName .";
                                print "\n";
                                $cmd_status = 1;
                                last;
                            }
                        }
                    }
                }
            }
            if($cmd_status){
                print "Use the -f option to forcefully halt the cluster/node including halting packages.\n";
                return -1;
            }
        }
        else {
            print "Disabling all packages from starting on nodes to be halted.\n";
            print "Warning:  Do not modify or enable packages until the halt operation is completed.\n";
        }
        # Generate input file for cmeval by modifying below attributes as:
        # node:node1|status=down
        # package:pkg1|node:node1|available=no
        # package:pkg2|node:node1|available=no
        my $node = $clusterObj->getNode($nodeName);
        $node->setStatus("down");
        $node->setState("halted");
        foreach my $pkg (@{$clusterObj->{packages}}) {
            foreach my $pNode (@{$pkg->{nodes}}){
                if ($node->getName() eq $pNode->getName()) {
                    $pNode->setAvailable("no");
                }			
            }
        }
    }

    my $cmeval_output_file = cmeval();
    parse_cmeval_output("cmhaltnode", $cmeval_output_file);
    foreach $node (@nodes) {
        my $mynode = $clusterObj->getNode($node);
        change_node_status($clusterObj,$mynode,"down","halted");
        foreach my $mypkg (@{$clusterObj->{packages}}) {
            # Change Package Monitor Subnet Status to unknown        
            foreach my $subnet (@{$mypkg->{monitored_subnet}}) {
                foreach my $my_subnet_node (@{$subnet->{node}}) {
                    if ($node eq $my_subnet_node->{name}) {
                        $my_subnet_node->setMonitorSubnetNodeStatus("unknown");
                    }
                }
            }
            foreach my $pNode (@{$mypkg->{nodes}}){
                my $nodeName = $pNode->getName();
                if ($nodeName eq $node) {
                    $pNode->setState("unknown");
                }
            }
        }
        
      
    }
    foreach my $node (@{$clusterObj->{nodes}}) {
        if ($node->getStatus() eq 'down') {
            $nodes++;
        }   
    }
    my $node_count = @{$clusterObj->{nodes}};
    if ($nodes eq @{$clusterObj->{nodes}}) {
        $clusterObj->setStatus("down");
    }
    write_ClusterObj($cluster);
    return 0;
}

sub simulate_node_failure {
	my ($cluster,$nodeName) = @_;
	read_ClusterObj($cluster);    
	my $node = $clusterObj->getNode($nodeName);
    unless($node){
		print "\nfail: Node  " . $node . " is not a configured node name";
		return -1;
	}
    
    print "Do you want to fail node $nodeName? ([y]/n)";
    while (<STDIN>) {
        if ($_ =~ /^\s*$/){
            $res='y';
            last;
        }
        $res=$_;
        last;
    }
    if ($res=~/y/i) {
        #go ahead
    }
    else {
        return 0;
    }
    
	if ($node) {
		$node->setStatus("down");
		$node->setState("failed");
		foreach my $pkg (@{$clusterObj->{packages}}) {
			foreach my $pNode (@{$pkg->{nodes}}){
				if ($node->getName() eq $pNode->getName()) {
					$pNode->setAvailable("no");
				}			
			}
		}
	}
	
	
	my $cmeval_output_file = cmeval();
	parse_cmeval_output("nodefailure", $cmeval_output_file);
	write_ClusterObj($cluster);	
    print "Failed node $nodeName \n";
    return 0;
	
}
sub simulate_node_recover {
	my ($cluster,$nodeName) = @_;
	read_ClusterObj($cluster);
    print "\nRecovering Node  " . $nodeName . " ... \n";
	my $node = $clusterObj->getNode($nodeName);
	if ($node) {
		foreach my $pkg (@{$clusterObj->{packages}}) {
			foreach my $pNode (@{$pkg->{nodes}}){
				if ($node->getName() eq $pNode->getName()) {
					$pNode->setAvailable("yes");
				}			
			}
		}
	}
	else{
		print "\nnrecover: Node  " . $node . " is not a configured node name";
		return -1;
	}	
	
	my $cmeval_output_file = cmeval();
	parse_cmeval_output("noderecover", $cmeval_output_file);
	change_node_status($clusterObj,$node,"up");
	write_ClusterObj($cluster);	
    print "recover: node $nodeName is recovered from failure\n";
    return 0;
}
sub simulate_package_failure {
	my ($cluster,$package) = @_;
    read_ClusterObj($cluster);
    
	#foreach my $pkg (@packages) {
    my $pkg = $package;
    chomp($pkg);
    my $pkgObj = $clusterObj->getPackage($pkg);
    unless($pkgObj) {
        print "fail: Package " . $package . " is not a configured package name\n";
        return -1;  
    }
    
    if ($pkgObj->getStatus() eq "down") {
        print "\nPackage $pkg is down\n\n";
        return -1;
    }
   	print "Do you want to fail package $package? ([y]/n)";
    while (<STDIN>) {
        if ($_ =~ /^\s*$/){
            $res='y';
            last;
        }
        $res=$_;
        last;
    }
    if ($res=~/y/i) {
        #go ahead
    }
    else {
        return 0;
    }

		$pkgObj->setStatus("down");
		$pkgObj->setState("failed");
		$pkgObj->setSummary("critical");
		$pkgObj->setAutoRun("disabled");
		my $lastowner = undef; 
		if ($pkgObj->getType() eq "failover" ) {
            
			$lastowner = $pkgObj->getOwner();
		}
		
		my $lastownerstatus = undef;
		$pkgObj->setOwner("unowned");
		foreach my $pNode (@{$pkgObj->{nodes}}) {
			$pNode->setAvailable("no");	
			my $node = $clusterObj->getNode($pNode->getName());
			if ($node->getName() eq $lastowner) { 
				$lastownerstatus = $node->getStatus(); 
				if ($pkgObj->{failfast} eq "enabled"){
					$node->setStatus("down");
					$node->setState("failed");
					#mark node as unavailable for all packages
					foreach my $mypkg (@{$clusterObj->{packages}}){
						foreach my $mypNode (@{$mypkg->{nodes}}) {
							if ($mypNode->getName() eq $node->getName()){
								$mypNode->setAvailable("no");
							}
						}
					}
				}
			}
		}
		set_last_failed_node($cluster,$pkgObj->getName(),$lastowner,$lastownerstatus);
	#}
	
	my $cmeval_output_file = cmeval();
	parse_cmeval_output("pkgfailure", $cmeval_output_file);
	write_ClusterObj($cluster);	
    print "Failed package $package \n";
    return 0;
}
sub simulate_package_recover {
	my ($cluster,$packageName) = @_;
	read_ClusterObj($cluster);
    print "Recovering $packageName ...\n";
    my $pkg = $packageName;
	#foreach my $pkg (@packages) {
	    chomp($pkg);
        $pkgObj = $clusterObj->getPackage($pkg);
        unless($pkgObj) {
            print "\nrecover: Package " . $pkg . " is not a configured package name";
            return -1;  
        }
        if ($pkgObj->getType() eq "multi_node" ) {
            foreach my $pNode (@{$pkgObj->{nodes}}) {
				$pNode->setAvailable("yes");
                change_pkg_status($clusterObj,$pkgObj,"down","");
			} 
        }
        elsif ($pkgObj->getType() eq "failover" && $pkgObj->getStatus() eq "down") {
            $line = get_last_failed_node($cluster,$pkg);
            if ($line) {
                my @data = split(",",$line);
                my $nodeName = $data[0];
                $node = $clusterObj->getNode($nodeName);
                my $lastownerstatus = $node->getStatus(); 
                #$pkgObj->setAutoRun("enabled");
                foreach my $pNode (@{$pkgObj->{nodes}}) {
                    if($pNode->getName() eq $nodeName) {
                        $pNode->setAvailable("yes");
                        change_pkg_status($clusterObj,$pkgObj,"down","");
                    }				
                }	
            }
            else {
                print "\nrecover: Details are unavailble where the package $pkg was running before failure.\n"
            }
        }
		
	#}
	#my $cmeval_output_file = cmeval();
	#parse_cmeval_output("pkgrecover", $cmeval_output_file);
    write_ClusterObj($cluster);	
    print "Recoverd $packageName\n";
    return 0;
}

sub simulate_interface_failure {
    my ($cluster,$interfaceName,$nodeName) = @_;
    my $found = 0; 
    
    read_ClusterObj($cluster);
    my $node = $clusterObj->getNode($nodeName);
	if ($node) {
        my $interface = $node->getInterface($interfaceName);
        if ($interface) {
            if ($interface->getStatus() eq "down") {
                print "\nfail: interface ". $interface . "is already down on node " . $nodeName . " \n";
                return -1;
            }
            print "Do you want to fail interface $interfaceName on the node $nodeName? ([y]/n)";
            while (<STDIN>) {
                if ($_ =~ /^\s*$/){
                    $res='y';
                    last;
                }
                $res=$_;
                last;
            }
            if ($res=~/y/i) {
                #go ahead
            }
            else {
                return -1;
            }
            $interface->setStatus("down");
            $interface->setDisabled("true");
            foreach my $ipadd (@{$interface->{ipaddress}}) {
                if ($ipadd->getHeartbeat() eq "true") {
                    my $hbCount = $node->countAvailableHBs();
                    unless ($hbCount) {
                        $found=1;
                        $node->setStatus("down");
                        $node->setState("failed");
                        foreach my $pkg (@{$clusterObj->{packages}}) {
                            foreach my $pNode (@{$pkg->{nodes}}){
                                if ($node->getName() eq $pNode->getName()) {
                                    $pNode->setAvailable("no");
                                }			
                            }
                        }
                    }
                }
            }
        }
        else{
            print "\nfail: interface  " . $interfaceName . " is not a configured on node " . $nodeName . "\n";
            return -1;
        }
        
	}
	else{
		print "\nfail: Node  " . $node . " is not a configured node name";
		return -1;
	}
    print "fail: interface ". $interface . " is now down on node " . $nodeName . " \n";
    if ($found) {
        my $cmeval_output_file = cmeval();
        print "fail: interface ". $interface . " is a heartbeat interface and thus node " . $nodeName . " is going down... \n";
        parse_cmeval_output("nodefailure", $cmeval_output_file);
    }
    
	write_ClusterObj($cluster);	
    return 0;
    
}

sub simulate_interface_recover {
    my ($cluster,$interfaceName,$nodeName) = @_;
    my $found = 0; 
    print "Recovering $interfaceName on $nodeName ...\n";
    read_ClusterObj($cluster);
    my $node = $clusterObj->getNode($nodeName);
	if ($node) {
        my $interface = $node->getInterface($interfaceName);
        if ($interface) {
            if ($interface->getStatus() eq "up") {
                print "\nrecover: interface ". $interfaceName . "is already up on node " . $nodeName . " \n";
                return -1;
            }
            $interface->setStatus("up");
            $interface->setDisabled("false");            
         }
    }
    
    write_ClusterObj($cluster);	
    print "recover: interface ". $interface . " is now up on node " . $nodeName . " \n";
    return 0;
    
}
sub read_ClusterObj {
	my $cluster = @_;
	$cluster = shift;
	my $SGCONF = getSGCONF();
	my $filename = "$SGCONF/simulator" . "/" . $cluster . "/" . $cluster . "_cmviewcl_line";
	# Call status parser to fill in data structure
	$clusterObj = &statusParser($filename);
}

sub write_ClusterObj {
	
	my $cluster = @_;
    $cluster = shift;
    my $SGCONF = getSGCONF();
	#my $filename="$SGCONF/simulator" . "/" . $clusterObj->{name} . "/" . $clusterObj->{name} . "_cmviewcl_line";
	my $filename = "$SGCONF/simulator" . "/" . $cluster . "/" . $cluster . "_cmviewcl_line";
	# Call status dumper to write into cmviewcl line output
    &statusDumper($clusterObj,$filename);
}
sub cmeval {
	my $SGCONF=getSGCONF();
	my $cmeval_input_file = "$SGCONF/simulator" . "/" . $clusterObj->{name} . "/cmeval.input";
	my $cmeval_output_file = "$SGCONF/simulator" . "/" . $clusterObj->{name} . "/cmeval.output";
	statusDumper($clusterObj,$cmeval_input_file);
	`cmeval -v -s -o $cmeval_output_file $cmeval_input_file`;
	return $cmeval_output_file;
}

sub parse_cmeval_output {
	
	my ($cmd, $cmeval_output_file) = @_;				
    	
	unless(open FH, $cmeval_output_file) {
		print "cmeval.output $!";
		return -1;
	}
	my $line;
	#my $package_node = 0;
	
	while (<FH>){
		$line = $_;
		my $pkg;
		my $pNode;
		my @package;
		my @pkg_node;
		my $pkg_status_down = 0;
		my $pkg_status_starting = 0;

		if ($line =~ /^package:/i) {
			my @attribute = split(/\|/, $line);
			foreach my $attrib (@attribute) {
				if($attrib =~ /^package:/i) {
					@package = split(":",$attrib);
					# Get Package Object for package $package[1]
					$pkg = $clusterObj->getPackage($package[1]);
				}
				if($attrib =~ /^node:/i) {
					@pkg_node = split(":",$attrib);
					# Get Package-Node Object for $pkg_node[1]
					$pNode = $pkg->getNode($pkg_node[1]);
				}
				if($attrib =~ /^action=/i) {
			    	chomp($attrib);
				    my @action = split("=",$attrib);
				    #if($action[1] eq 'starting') {
				    #    $package_node = 1;
				    #}
                    if ($action[1] eq "down" || $action[1] eq "failing") {
						$pkg_status_down = 1;
					}
                    if($action[1] eq "starting") {
						$pkg_status_starting = 1;
                    }
				}
			}	
		}
		if($cmd =~ "cmruncl") {
		    $clusterObj->setStatus("up");
			$clusterObj->setState("stable");
			$clusterObj->setIncarnation(1);
			$clusterObj->setSummary("ok");
			foreach my $node (@{$clusterObj->{nodes}}){
				change_node_status($clusterObj,$node,"up");
			}
			change_pkg_status($clusterObj,$pkg,"up",$pkg_node[1]);
                
		}
		elsif ($cmd =~ "cmrunpkg" ) {
            if($pkg_status_starting) {
			    change_pkg_status($clusterObj,$pkg,"up",$pkg_node[1]);
                if ($pkg->getType() eq "failover") {
                    $pkg->setAutoRun("disabled");
                }
            }
        }
        elsif ($cmd =~ "cmmodpkg") {
		    if($pkg_status_starting && $pkg->getAutoRun() eq "enabled" && $pkg->getType() eq "failover") {
			    change_pkg_status($clusterObj,$pkg,"up",$pkg_node[1]);
            }
		}
		elsif ($cmd =~ "cmhaltpkg") {
		    if ($pkg_status_down) {
				change_pkg_status($clusterObj,$pkg,"down",$pkg_node[1]);
                
		    }
		}
		elsif ($cmd =~ "cmrunnode") {
			if ($pkg_status_starting) {
				change_pkg_status($clusterObj,$pkg,"up",$pkg_node[1]);
			}
		}
		elsif($cmd =~ "cmhaltnode") {
			if ($pkg_status_down) {
				change_pkg_status($clusterObj,$pkg,"down",$pkg_node[1]);
				foreach my $node (@{$clusterObj->{nodes}}) {
				    if ($node->getName() eq $$pkg_node[1]) {
						change_node_status($clusterObj,$node,"down","halted");
					}
				}
				unless ($pkg->getType() eq "failover"){
					my $allPnodeDown = 0;
					foreach my $mypnode (@{$pkg->{nodes}}) {
						if ($mypnode->getStatus() eq "down") { $allPnodeDown += 1 ;}
					}
					my $numofnodes = @{$pkg->{nodes}};
					if ($allPnodeDown eq $numofnodes ) {
						$pkg->setStatus("down");
						$pkg->setState("halted");
					}
				}
			}
            if($pkg_status_starting && $pkg->getAutoRun() eq "enabled") {
                change_pkg_status($clusterObj,$pkg,"up",$pkg_node[1]);
			}
		}
		elsif($cmd =~ "nodefailure" || $cmd =~ "pkgfailure") {
			if ($pkg_status_down) {
			    change_pkg_status($clusterObj,$pkg,"fail",$pkg_node[1]);
			    if ($cmd =~  "nodefailure") {
                    foreach my $node (@{$clusterObj->{nodes}}) {
                        if($node->getName() eq $pkg_node[1]) {
                            change_node_status($clusterObj,$node,"fail");
                            
                        }
                    }
                }
				unless ($pkg->getType() eq "failover"){
					my $allPnodeDown = 0;
					foreach my $mypnode (@{$pkg->{nodes}}) {
						if ($mypnode->getStatus() eq "down") { $allPnodeDown += 1 ; }
					}
					my $numofnodes = @{$pkg->{nodes}};
					if ($allPnodeDown eq $numofnodes) {
						$pkg->setStatus("down");
						unless ($cmd =~ "pkgfailure") {$pkg->setState("halted");}
					}
				}
                #if ($cmd =~ "pkgfailure") { print "Failed package " . $pkg->getName() ."\n";}
                
			}
			if ($pkg_status_starting && $pkg->getAutoRun() eq "enabled") {
                my $tpNode = $pkg->getNode($pkg_node[1]) ;
                if ($tpNode->getSwitching() eq "enabled") {
                    change_pkg_status($clusterObj,$pkg,"up",$pkg_node[1]);
                }
                else {
                    print "Unable to run package " . $pkg->getName() ." on node ". $pkg_node[1] .". Node is not eligible\n";
                }
				
            }
            else {
                change_pkg_status($clusterObj,$pkg,"fail",$pkg_node[1]);
            }
		}
		elsif ($cmd =~ "noderecover") {
            if($pkg_status_starting && $pkg->getAutoRun() eq "enabled") {	
				change_pkg_status($clusterObj,$pkg,"up",$pkg_node[1]);                
				#print "Running package $package[1] on node $pkg_node[1]\n";
				#print "Successfully started package $package[1] on node $pkg_node[1]\n";
            }
        }
        elsif ($cmd =~ "pkgrecover" ) {
			if($pkg_status_starting) {	
				change_pkg_status($clusterObj,$pkg,"down",$pkg_node[1]);
                if ($cmd =~ "pkgfailure") { print "Recovered package " . $pkg->getName() ."\n";}
				#print "Running package $package[1] on node $pkg_node[1]\n";
				#print "Successfully started package $package[1] on node $pkg_node[1]\n";
            }
		}
		#elsif($cmd =~ "pkgrecover" ) {
		#	if($pkg_status_starting && $pkg->getAutoRun() eq "enabled") {	
		#		$pkg->setStatus("up");
		#		$pkg->setState("running");
		#		$pkg->setOwner($pkg_node[1]);
		#		$pkg->setHighlyAvailable("yes");
		#		$pkg->setSummary("ok");
		#	}
		#}
	}
	close FH;
}


sub set_node {

    my ($cluster, $node) = @_;
    my $node_exist = 0;
    my $SGCONF = getSGCONF();
    my $filename = "$SGCONF/simulator" . "/" . $cluster . "/" . $cluster . "_active_node";
    read_ClusterObj($cluster);
    foreach my $mynode (@{$clusterObj->{nodes}}) {
        if ($mynode->{name} eq $node) {
            `echo "$node" > $filename`;
            $node_exist = 1;
            return 0;
        }
    }
    unless($node_exist) {
        return -1;
    }
}

sub get_node {

   my ($cluster) = @_;
   my $SGCONF = getSGCONF();
   my $filename = "$SGCONF/simulator" . "/" . $cluster . "/" . $cluster . "_active_node";
   my $FH;
   unless (open $FH, "< $filename") {
       return undef;
   }
   my $node_name = <$FH>;
   close $FH;
   chomp($node_name);
   unless ($node_name) {
       return undef;
   }
   return $node_name;
}

sub set_last_failed_node {

    my ($cluster, $pkg, $node, $status, $reset) = @_;
    
    my $SGCONF = getSGCONF();
    my $filename = $SGCONF . "/simulator/" . $cluster . "/" . $cluster . "_" . $pkg . "_last_failed_node";
    if ($reset) {
        `rm -f $filename`;
        return 1;
    }
    `echo "$node,$status" > $filename`;
    
}

sub get_last_failed_node {

   my ($cluster,$pkg) = @_;
   my $SGCONF = getSGCONF();
   my $filename = $SGCONF . "/simulator/" . $cluster . "/" . $cluster . "_" . $pkg . "_last_failed_node";
   
   my $FH;
   unless (open $FH, "< $filename") {
       return undef;
   }
   my $node = <$FH>;
   close $FH;
   chomp($node);
   unless ($node) {
       return undef;
   }
   
   return $node;
}

#Subroutine to get a key with the highest value from a hash
sub get_key_with_highest_value_from_hash {
    my $hash   = @_;
    $hash = shift;
    my ($key, @keys) = keys   %$hash;
    my ($big, @vals) = values %$hash;

    for (0 .. $#keys) {
        if ($vals[$_] > $big) {
            $big = $vals[$_];
            $key = $keys[$_];
        }
    }
    return $key;
}
#Subroutine to get a key with the highest value from a hash
sub get_key_with_highest_value_from_hash_v6 {
    my $hash   = @_;
    $hash = shift;
    my ($key, @keys) = keys   %$hash;
    my ($big, @vals) = values %$hash;

    for (0 .. $#keys) {
        #print "big $big   key $key value #value\n";
        if ($vals[$_] > $big) {
            $big = $vals[$_];
            $key = $keys[$_];
        }
    }
    return $key;
}

sub validate {

}

sub generate_id {
	my $range = 10000;
	my $random_number = int(rand($range));
	return $random_number;
}


sub change_node_status {
	my ($clusterObj,$mynode,$status,$state) = @_;
	if ($status eq "up") {
		#my $mynode = $clusterObj->getNode($node);
		$mynode->setStatus("up");
		$mynode->setState("running");
		$mynode->setInitialIncarnation($clusterObj->getIncarnation());
		$mynode->setSummary("ok");
		foreach my $interface (@{$mynode->{interfaces}}) {
			$interface->setStatus("up");
		}
		foreach my $subnet (@{$mynode->{subnets}}) {
			$subnet->setStatus("up");
		}
		if ($clusterObj->{qs}) {
			my $qsip=$clusterObj->{qs}->{ipaddress};
			foreach my $qsnode (@{$qsip->{nodes}}) {
				if($qsnode->getName() eq $mynode->getName()){
					$qsnode->setStatus("up");
					$qsnode->setState("running");
				}
			}
		}
        else {
            foreach my $lockobj (@{$clusterObj->{lockLun}}) {
                if ($lockobj->getLockNodeName() eq $mynode->{name}) {
                    $lockobj->setLockStatus("up");
                }
            }
        }
	}
	elsif ($status eq "down") {
		#my $mynode = $clusterObj->getNode($node);
		$mynode->setStatus("down");
		$mynode->setState($state);
		$mynode->setInitialIncarnation($clusterObj->getIncarnation());
		$mynode->setSummary("critical");
		foreach my $interface (@{$mynode->{interfaces}}) {
			$interface->setStatus("unknown");
		}
		foreach my $subnet (@{$mynode->{subnets}}) {
			$subnet->setStatus("unknown");
		}
		if ($clusterObj->{qs}) {
			my $qsip=$clusterObj->{qs}->{ipaddress};
			foreach my $qsnode (@{$qsip->{nodes}}) {
				if($qsnode->getName() eq $mynode->getName()){
					$qsnode->setStatus("unknown");
					$qsnode->setState("unknown");
				}
			}
		}
        else {
            foreach my $lockobj (@{$clusterObj->{lockLun}}) {
                if ($lockobj->getLockNodeName() eq $mynode->getName()) {
                    $lockobj->setLockStatus("unknown");
                }
            }
        }
	}
	elsif ($status eq "fail") {
		#my $mynode = $clusterObj->getNode($node);
		$mynode->setStatus("down");
		$mynode->setState("failed");
		$mynode->setInitialIncarnation($clusterObj->getIncarnation());
		$mynode->setSummary("critical");
		foreach my $interface (@{$mynode->{interfaces}}) {
			$interface->setStatus("unknown");
		}
		foreach my $subnet (@{$mynode->{subnets}}) {
			$subnet->setStatus("unknown");
		}
		if ($clusterObj->{qs}) {
			my $qsip=$clusterObj->{qs}->{ipaddress};
			foreach my $qsnode (@{$qsip->{nodes}}) {
				if($qsnode->getName() eq $mynode->getName()){
					$qsnode->setStatus("unknown");
					$qsnode->setState("unknown");
				}
			}
		}
	}
	
}

sub change_pkg_status {
	my ($clusterObj,$pkg,$status,$node_name) = @_;
	if ($status eq "down") {
		unless ($pkg->getType() eq "multi_node") {
			$pkg->setStatus("down");
			#$pkg->setAutoRun("disabled");
			$pkg->setState("halted");
			$pkg->setHighlyAvailable("no");
			$pkg->setSummary("degraded");
			$pkg->setHalted("yes");
			$pkg->setOwner("unowned");
			unless($node_name) {
				foreach my $pNode (@{$pkg->{nodes}}) {
					$pNode->setStatus("down");
                    if ($pNode->getState() eq 'running'){
                        $pNode->setState("halted");
                    }
					#$pNode->setSwitching("disabled");
					$pNode->setLastHaltTime(time());
				}
			}
			else{
				$pNode = $pkg->getNode($node_name);
				$pNode->setStatus("down");
                if ($pNode->getState() eq 'running'){
                    $pNode->setState("halted");
                }
				#$pNode->setSwitching("disabled");
				$pNode->setLastHaltTime(time());
			}
		}
		else {
		    unless($node_name) {
                foreach my $pNode (@{$pkg->{nodes}}) {
					$pNode->setStatus("down");
                    $pNode->setState("halted");
                    #$pNode->setSwitching("disabled");
					$pNode->setLastHaltTime(time());
				}
			}
			else{
				$pNode = $pkg->getNode($node_name);
				$pNode->setStatus("down");
                $pNode->setState("halted");
				#$pNode->setSwitching("disabled");
				$pNode->setLastHaltTime(time());
			}
			my $numpNode = @{$pkg->{nodes}};
			my $numDownpNode = 0;
			foreach my $mypNode (@{$pkg->{nodes}}) {
				if ($mypNode->getStatus() eq "down") {
					$numDownpNode += 1 ;
				}
			}
			if ($numDownpNode == $numpNode){
				$pkg->setStatus("down");
                $pkg->setState("halted");
                $pkg->setHighlyAvailable("no");
				$pkg->setSummary("degraded");
				$pkg->setHalted("yes");
				$pkg->setOwner("unowned");
			}
		}
        
	}
	elsif ($status eq "up") {
        print "Running package ".$pkg->getName()." on node $node_name\n";
		$pkg->setStatus("up");
        $pkg->setState("running");
        $pkg->setHighlyAvailable("yes");
        $pkg->setSummary("ok");
        if ($pkg->getType() eq "failover") {
            $pkg->setOwner($node_name);
            foreach my $mypnode (@{$pkg->{nodes}}) {
                if($mypnode->{name} eq $node_name) {
                    $mypnode->setStatus("up");
                    $mypnode->setState("running");
                    $mypnode->setSwitching("enabled");
                    $mypnode->setLastHaltFailed("no");
                    next;
                }
            }
        }
        else {
            foreach my $mypnode (@{$pkg->{nodes}}) {
                if($mypnode->{name} eq $node_name)  {
                    if ($mypnode->getSwitching() eq "enabled") {
                        $mypnode->setStatus("up");
                        $mypnode->setState("running");
                        $mypnode->setLastHaltFailed("no");
                        next;
                    } else {
                        print "Unable to run package " . $pkgName ." on node ". $mypNode->getName() .". Node is not eligible\n";
                    }
                }
                
            }
		}
	    print "Successfully started package ".$pkg->getName()." on $node_name\n";
	}
	elsif ($status eq "fail") {
		unless ($pkg->getType() eq "multi_node") {
			$pkg->setStatus("down");
			$pkg->setState("failed");
			$pkg->setHighlyAvailable("no");
			$pkg->setSummary("degraded");
			$pkg->setHalted("yes");
			$pkg->setOwner("unowned");
			unless($node_name) {
				foreach my $pNode (@{$pkg->{nodes}}) {
					$pNode->setStatus("down");
					$pNode->setState("failed");
					$pNode->setLastHaltTime(time());
				}
			}
			else{
				$pNode = $pkg->getNode($node_name);
				$pNode->setStatus("down");
				$pNode->setState("failed");
				$pNode->setLastHaltTime(time());
			}
		}
		else {
		    unless($node_name) {
				foreach my $pNode (@{$pkg->{nodes}}) {
					$pNode->setStatus("down");
					$pNode->setState("failed");
					$pNode->setLastHaltTime(time());
				}
			}
			else{
				$pNode = $pkg->getNode($node_name);
				$pNode->setStatus("down");
				$pNode->setState("failed");
				$pNode->setLastHaltTime(time());
			}
			my $numpNode = @{$pkg->{nodes}};
			my $numDownpNode = 0;
			foreach my $mypNode (@{$pkg->{nodes}}) {
				if ($mypNode->getStatus() eq "down") {
					$numDownpNode += 1 ;
				}
			}
			if ($numDownpNode == $numpNode){
				$pkg->setStatus("down");
				$pkg->setState("failed");
				$pkg->setHighlyAvailable("no");
				$pkg->setSummary("degraded");
				$pkg->setHalted("yes");
				$pkg->setOwner("unowned");
			}
		}
	}
	
	
}


1;




