##################################################################
# (C) Copyright 2010-2015 Hewlett-Packard Development Company, L.P.
# @(#) utilities for ServiceGuard Easy Deployment
# @(#) Product Name : HP Serviceguard
# @(#) Product Version : A.12.10.00
# @(#) Patch Name : 
###################################################################

=pod

=head1 NAME

EDPkgTools - utilities for ServiceGuard Easy Deployment of Packages.

=cut

package EDPkgTools;

use strict;
use Sys::Syslog;
use File::Path;
use Carp;
use EDTools;

use Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw (create_default_pkg_ascii_file set_pkg_param_first set_pkg_param_next set_group_attributes
                   add_module_to_pkg get_pkg_param setup_nodes CaptureCommand is_sg_site_configured 
                   sg_discover_node_names get_sgsbin_path);


sub create_default_pkg_ascii_file {

    my ($pkgAsciiFile, $pkgType, $pkgname) = @_;
    my $cmmakepkg_cmd ;
    die "Missing package ascii file path and name" if (!$pkgAsciiFile);
    if(!$pkgType) {
        $pkgType = "failover"; 
    }

    if (!defined ($pkgname) ) {
        $pkgname = "sgeasy_cfs_tmp_pkg";
    }
    
    if( $pkgType eq "multi_node") {
        $cmmakepkg_cmd = "/usr/sbin/cmmakepkg -m sg/multi_node_all -n $pkgname";
    }                
    elsif ( $pkgType eq "multi_node_cfs") {
       $cmmakepkg_cmd = "/usr/sbin/cmmakepkg -m sg/cfs -n $pkgname";
    }         
    else {
        $cmmakepkg_cmd = "/usr/sbin/cmmakepkg -m sg/all -n $pkgname";
    } 
    $cmmakepkg_cmd = $cmmakepkg_cmd . " > $pkgAsciiFile"; 
    run_system_proc("cmmakepkg", $cmmakepkg_cmd);
}

sub set_pkg_param_first {

    my ($pkgAsciiFile, $param_name, $param_value ) = @_;
    my $edToolsObj = new EDTools("PkgEasyDeployment:");
    die "Missing package parameter or value " if (!$pkgAsciiFile || ! $param_name || ! $param_value);
     
    my $new = $pkgAsciiFile . ".tmp"; 
    open SRCPKGASC, "< $pkgAsciiFile" or 
                   die "Unable to open package ascii file"; 
    open DESTPKGASC, "> $new" or 
                    die "Unable to create new package ascii file";
    my $done = 0;
    my $found = 0;
    while( <SRCPKGASC> ) {
        my $line = $_;
        if( $line =~ /^#$param_name\b/ || $line =~ /^$param_name\b/ ) {
            $found = 1;
            chomp($line);
            $line =~ s/^#//g;
            $line = $param_name. "\t\t" . $param_value ; 
            if ( !$done ) {
            $done = 1;
            print DESTPKGASC $line;
            print DESTPKGASC "\n";
            next;
            }
        }
        print DESTPKGASC;
        next;
    }
    close SRCPKGASC;
    close DESTPKGASC;
    rename ($new, $pkgAsciiFile) or die "Unable to rename the package configuration file $pkgAsciiFile";
    if ( !$found) {
        $edToolsObj->screen_and_syslog("WARNING: No such package parameter: $param_name found\n");
    }
}

sub set_pkg_param_next {
    my ($pkgAsciiFile,$param_name, $param_value ) = @_;
    die "Missing package parameter or value " if (!$pkgAsciiFile || ! $param_name || ! $param_value);
    
    my $new = $pkgAsciiFile . ".tmp"; 
    open SRCPKGASC, "< $pkgAsciiFile" or
                   die "Unable to open package ascii file";
    open DESTPKGASC, "> $new" or
                    die "Unable to create new package ascii file";
    my $done =0;
    my $find =0;
    my @all_lines = <SRCPKGASC>;
    my $i =0;
    my $string;
    my $val=0;
    my $found = 0;
    my $index = 0;
    my @array;
    my $edToolsObj = new EDTools("PkgEasyDeployment:");
    
    foreach my $line (@all_lines) {
        chomp($line);
        if( $line =~ /^#$param_name\b/ || $line =~ /^$param_name\b/ ) { 
            $index = $val unless ($index);
            $line =~ s/^#//g;
            $line =~ s/\s+$//;
            if ($line eq $param_name) {
                $string = $param_name. "\t\t" . $param_value; 
                $array[$i] = $string;
            } else {
                $array[$i++] = $line;
                $found = 1;
            }
        }
        $val++;
    }
    if ($found) {
        $string = $param_name. "\t\t" . $param_value;
        $array[$i] = $string;
    }

    my $k=0;
    while( $k < $index ){
        print DESTPKGASC $all_lines[$k];
        print DESTPKGASC "\n"; 
        $k++;
    } 
    $k = 0;
    while($k <= $i) {
        print DESTPKGASC $array[$k];
        print DESTPKGASC "\n";
        $k++;
    }
    if($found) {
        $k = ($index + $i);
    }
    else {
        $k = $index + $i +1;
    }

    while( $k < scalar(@all_lines) ) {
        print DESTPKGASC $all_lines[$k];
        print DESTPKGASC "\n";
        $k++;
    }
    close SRCPKGASC;
    close DESTPKGASC;
    rename ($new, $pkgAsciiFile) or die "Unable to rename the package configuration file $pkgAsciiFile";

    if ( !$found) {
        $edToolsObj->screen_and_syslog("WARNING: No such package parameter: $param_name found\n");
    }
}

sub get_pkg_param {

    my ($pkgAsciiFile,$param_name) = @_;
    my $found = 0;
   
    die "Missing package parameter or value " if (! $param_name || !$pkgAsciiFile);
    
    my $edToolsObj = new EDTools("PkgEasyDeployment:");
    
    open SRCPKGASC, "< $pkgAsciiFile" or
                   die "Unable to open package ascii file";
    my $line;
    my @LINES;

    while( <SRCPKGASC> ) {
        if( $_ =~ /^$param_name\b/i) {
            $line = $_;
            chomp($line);
            $line =~ s/^#//g;
            push(@LINES, $line);
            $found = 1;
        }

    }

    close SRCPKGASC;


    my @RETURN;

    if ($found) {
        my @values;
        foreach (@LINES) {
            $line = $_;
            chomp($line);
            $line =~ s/\s+/ /g;
            my @values = split(' ', $line);
            push(@RETURN, $values[1]);
        }
    } else {
        $edToolsObj->screen_and_syslog("Warning: package paramter $param_name not found");
    }

    return @RETURN;
}

sub run_system_proc {

    my ($cmdname, $cmd) = @_;
    my $edToolsObj = new EDTools("PkgEasyDeployment:");
    if ($cmdname =~ /^cm/) {
    }
    open(CMD, "$cmd 2>&1 |") or die "Failed to start $cmdname";
    my @msgs;
    while (<CMD>) {
        push(@msgs, $_);
    }

    close(CMD);

    my $saved_errno = $!;
    my $saved_exit_status = ($? >> 8);
    if (($? >> 8) != 0) {
        foreach (@msgs) {
           chomp;
           $edToolsObj->log_from_subcmd($_, $cmdname . ":");
        }
        #die "$saved_errno ? "error closing command pipe: $saved_errno, stopping"";
    }
}

sub set_group_attributes {

    my ($pkgAsciiFile, $param_values ) = @_;
    die "Missing package parameter or value " if (!$param_values || !$pkgAsciiFile);
   
    open SRCPKGASC, ">> $pkgAsciiFile" or die "Unable to open package ascii file";
  
    my @params = split (',', $param_values);

    foreach my $val (@params) {
        chomp($val);
        $val =~ s/\:/\t\t/g;
        print SRCPKGASC $val;
        print SRCPKGASC "\n";
    }
    close (SRCPKGASC);
}

sub add_module_to_pkg {

    my ($pkgAsciiFile, $module_name, $module_version) = @_;
    my $cmmakepkg_cmd;
    
    die "Missing module name" if (!$pkgAsciiFile ||  !$module_name);
    my $edToolsObj = new EDTools("PkgEasyDeployment:");
    my $module_to_add = $module_name;

    if( $module_version) {
        $module_to_add = $module_to_add . ".$module_version";
    }
    my $new = $pkgAsciiFile . ".tmp";

    $cmmakepkg_cmd = "/usr/sbin/cmmakepkg -m $module_to_add -i" . $pkgAsciiFile ;
    $cmmakepkg_cmd = $cmmakepkg_cmd . " > $new";

    run_system_proc("cmmakepkg",$cmmakepkg_cmd);
    rename ( $new, $pkgAsciiFile) or die "Unable to rename the package configuration file $pkgAsciiFile";
}

sub setup_nodes {

    my ($nodes, $pkg_file_name) = @_;
    my $first = 0;
    foreach my $node ( @$nodes ) {
        if ( !$first ) {
            set_pkg_param_first($pkg_file_name, "node_name", $node);
            $first = 1;
        }
        else {
            set_pkg_param_next($pkg_file_name, "node_name", $node);
        }
    }
}

sub CaptureCommand {
    my ($myCmd) = @_;
    my @lines = ();
    my @output = ();
    open(CMD, "$myCmd 2>&1 |") or return ($?>>8 , @output);
    while (<CMD>) {
        push(@output, $_);
        last if eof(CMD);
    }
    close(CMD);
    my $exitcode = ($? >> 8);
    return($exitcode, @output);
}

sub is_sg_site_configured {
    my $path = get_sgsbin_path();
    chomp($path);
    my ($ret, @out) = CaptureCommand("$path/cmviewcl -f line -s config -l cluster");
    die "ERROR : Unable to execute cmviewcl" if ($ret);
    foreach (@out) {
        if (m/^site:/i)
        {
            return 1;
        }
    }
    return 0;
}

sub sg_discover_node_names {
    my $path = get_sgsbin_path();
    chomp($path);
    my @nodes;
    my ($ret, @out) = CaptureCommand("$path/cmviewcl -f line -s config -l node");
    die "ERROR : Unable to execute cmviewcl command" if ($ret);
    foreach (@out) {
        my $val=$_;
        if ($val =~ m/\|name=/)
        {
            my @cols=split("=",$val);
            chomp($cols[1]);
            push (@nodes, $cols[1]);
        }
    }
   return @nodes;
}

sub get_sgsbin_path {
    my $file = "/etc/cmcluster.conf";
    open SRCPKGASC, "< $file" or
                die "Unable to open the file $file : $!";
    my @lines = <SRCPKGASC>;
    my @pth = grep(/^SGSBIN/, @lines);
    close SRCPKGASC;
    die "Unable to find SBSBIN in $file" if(!@pth); 
    chomp($pth[0]);
    my @val = split("=", $pth[0]);
    return ($val[1]);   
}

1;
