##############################################################################
# (C) Copyright 2010-2015 Hewlett-Packard Enterprise Development Company, L.P.
# @(#) utilities for Serviceguard Easy Deployment
# @(#) Product Name    : HP Serviceguard
# @(#) Product Version : A.12.10.00
# @(#) Patch Name      : 
##############################################################################
=pod

=head1 NAME

EDTools - utilities for ServiceGuard Easy Deployment

=cut

package EDTools;

use strict;
use Sys::Syslog;
use File::Path;
use Carp;

sub new {
    my $pkg = shift;
    my $self = {};
    my $sgeasyLogDir = undef;
    $self->{syslogCmd} = shift;
    die "Missing command tag for syslog" if ! $self->{syslogCmd};
    $self->{logDir} = get_sgeasy_dir_path();
    $self->{logfile} = $self->{logDir}."/easy_deployment.log";
    # open a syslog session for logging
     openlog($self->{syslogCmd}, "ndelay,pid", "user");

    bless $self, $pkg;

    return $self;
}

=pod

=item

=cut

sub screen_and_log {
    my ($self, $msg, $cmd_tag, $preview) = @_;
    return unless defined($msg);
    $cmd_tag = "" if (! defined($cmd_tag));
    $preview = 0 if (! defined($preview));
    my $timestamp = localtime;
    print STDOUT $msg, "\n";
    if ($preview) {
        open LOGFILE, ">> /dev/null";
    }
    else {
        mkpath([$self->{logDir}], 0, 0700) unless (-d $self->{logDir});
        open LOGFILE, ">> $self->{logfile}" or die $!;
    }
    print LOGFILE "$timestamp $cmd_tag $msg", "\n";
}

=pod

=item

=cut

sub log_from_subcmd {
    my ($self, $msg, $cmd_tag) = @_;
    return unless defined($msg);
    $cmd_tag = "" if (! defined($cmd_tag));
    my $timestamp = localtime;
    open LOGFILE, ">> $self->{logfile}" or die $!;
    print LOGFILE "$timestamp $cmd_tag $msg", "\n";
    print STDOUT "--\t", $msg, "\n";
}

=pod

=item

=cut

sub screen_and_syslog {
    my ($self, $msg) = @_;
    return unless defined($msg);
    print STDOUT $msg, "\n";
    syslog("info", $msg);
}

=pod

=item

=cut

sub only_syslog {
    my ($self, $msg) = @_;
    return unless defined($msg);
    syslog("info", $msg);
}

=pod

=item

=cut

sub test_cmexec {
    my ($self, $node_name) = @_;
    die "Missing node name to cmexec" if ! $node_name;
    my $CMEXEC_CMD = get_sgbin_path()."/cmexec";
    `$CMEXEC_CMD $node_name "echo hi" > /dev/null 2>&1`;
    return $?;
}

=pod

=item

=cut

sub getDistroInfo {
    my $distroName;
    my $distroVersion;
    my $releaseFileContent;
    my @file_out;
    my @distro_name;
    my @split_distro_name;    
    my @split_again = ();
    my @ver_name_out;
    if (-f "/etc/redhat-release") {
        $releaseFileContent = `cat /etc/redhat-release`;
        $distroName = "RHEL";
        $releaseFileContent =~  /(\d+\.?\d+)/ ;
        $distroVersion = "$1";
        @split_again = split(/\./, $distroVersion);
        $distroVersion = $split_again[0];
    } elsif (-f "/etc/SuSE-release") {
        $releaseFileContent = `cat "/etc/SuSE-release"`;
        $distroName = "SLES";
        $releaseFileContent =~  /(\d+\.?\d+)/ ;
        $distroVersion = "$1";
        @split_again = split(/\./, $distroVersion);
        $distroVersion = $split_again[0];
    } elsif (-f "/etc/os-release") {
        @file_out = `cat "/etc/os-release"`;
        @distro_name = grep(/^ID/, @file_out);
        @split_distro_name = split(/=/,$distro_name[0]);
        chomp($split_distro_name[1]);
        $split_distro_name[1] =~ s/"//g;

        if ($split_distro_name[1] eq "sles") {
            $distroName = "SLES";
        } elsif ($split_distro_name[1] eq "rhel") {
            $distroName = "RHEL";
        }

        @ver_name_out = grep(/^PRETTY_NAME/, @file_out);
        chomp($ver_name_out[0]);
        $ver_name_out[0] =~  /(\d+\.?\d+)/ ;
        $distroVersion = "$1";
        @split_again = split(/\./, $distroVersion);;
        $distroVersion = $split_again[0];        
    } else {
        $distroName = "UNKNOWN";
        $distroVersion = 0;
    }
    return ($distroName, $distroVersion);
}

=pod

=item

=cut

sub get_sgconf_path {
    my $file = "/etc/cmcluster.conf";
    open SRCPKGASC, "< $file" or
                die "Unable to open the file $file : $!";
    my @lines = <SRCPKGASC>;
    my @pth = grep(/^SGCONF/, @lines);
    close SRCPKGASC;
    die "Unable to find SGCONF in $file" if(!@pth);
    chomp($pth[0]);
    my @val = split("=", $pth[0]);
    return ($val[1]);
}

=pod

=item

=cut

sub get_sgrun_path {
    my $file = "/etc/cmcluster.conf";
    open SRCPKGASC, "< $file" or
                die "Unable to open the file $file : $!";
    my @lines = <SRCPKGASC>;
    my @pth = grep(/^SGRUN/, @lines);
    close SRCPKGASC;
    die "Unable to find SGRUN in $file" if(!@pth);
    chomp($pth[0]);
    my @val = split("=", $pth[0]);
    return ($val[1]);
}

=pod

=item

=cut

sub get_sgeasy_dir_path {
    my ($distroName, $distroVer) = getDistroInfo();
    my $sgconf = get_sgconf_path();
    my $sgeasyLogDir = undef;
    if ($distroName eq "RHEL") {
        $sgeasyLogDir = $sgconf."/sgeasy";
    } elsif ($distroName eq "SLES") {
        $sgeasyLogDir = $sgconf."/sgeasy";
    } else {
        $sgeasyLogDir = "/var/adm/cmcluster/sgeasy/";
    }
    return ($sgeasyLogDir);
}

=pod

=item

=cut

sub get_sgbin_path {
    my $file = "/etc/cmcluster.conf";
    open SRCPKGASC, "< $file" or
                die "Unable to open the file $file : $!";
    my @lines = <SRCPKGASC>;
    my @pth = grep(/^SGSBIN/, @lines);
    close SRCPKGASC;
    die "Unable to find SBSBIN in $file" if(!@pth);
    chomp($pth[0]);
    my @val = split("=", $pth[0]);
    return ($val[1]);
}

=pod

=item

=cut

sub get_sglib_path {
    my $file = "/etc/cmcluster.conf";
    open SRCPKGASC, "< $file" or
                die "Unable to open the file $file : $!";
    my @lines = <SRCPKGASC>;
    my @pth = grep(/^SGLIB/, @lines);
    close SRCPKGASC;
    die "Unable to find SGLIB in $file" if(!@pth);
    chomp($pth[0]);
    my @val = split("=", $pth[0]);
    return ($val[1]);
}

1;
