####################################################################
# (C) Copyright 2007 Hewlett-Packard Development Company, L.P.
# @(#) Serviceguard MGNetwork.pm 
# @(#) Product Name                : HP Serviceguard
# @(#) Product Version             : A.12.10.00
# @(#) Patch Name                  : 
####################################################################

package MGPackage;
use Carp;

# network related subroutines for packages

$MGNetwork::TYPE = 0;
$MGInterconnectSubnet::TYPE = 1;
$MGMonitoredSubnet::TYPE = 1;


=pod

=item addSubnet <MGNetwork>

=cut

sub addSubnet {
    my ($self, $pSubnet) = @_;
    push(@{$self->{subnets}}, $pSubnet);
}

=pod

=item getSubnets

Returns a list of MGNetwork.

=cut

sub getSubnets {
    my ($self) = @_;
    return @{$self->{subnets}};
}

=pod

=item getSubnet <name>

=cut

sub getSubnet {
    my ($self, $name) = @_;
    foreach my $pSubnet (@{$self->{subnets}}) {
        if ($pSubnet->getName() eq $name) {
            return $pSubnet;
        }
    }
    return undef;
}

use Carp;

=pod

=item getIpAddresses

=cut
sub getIpAddresses {
    my ($self) = @_;
    my @results = ();
    foreach my $subnet ($self->getIpSubnets()) {
            push(@result, $subnet->getIpAddresses());
    }
    return @results;
}

=pod

=item getIpAddress <name>

=cut
sub getIpAddress {
    my ($self, $name) = @_;
    die "missing ip address" if ! $name;
    foreach my $ip (@{$self->{ipAddresses}}) {
        if ($ip eq $name) {
            return $ip;
        }
    }
    return undef;
}

=pod

=item addIpAddress <name>

=cut
sub addIpAddress {
    my ($self, $name) = @_;
    die "missing ip address" if ! $name;
    push(@{$self->{ipAddresses}}, $name);
}


# subnet object


package MGNetwork;

use strict;
use Carp;

=pod

=item stripIPv6Prefix

Returns an IPv6 address (Host IP or Subnet) without the prefix bits.

=cut
sub stripIPv6Prefix {
    my ($self, $ip) = @_;
    # IP will be something like fec0:0:0:1::/64
    if ($ip =~ /(\S+)\/(\d+)$/) {
        return $1;
    }
    # If they didn't pass in something with prefix bits, return their parameter
    # unchanged.
    return $ip;
}

=pod

=item isIPv6Address

Returns 1 if  IPv6 address (Host IP or Subnet) else 0

=cut
sub isIPv6Address {
    my ($self, $ip) = @_;
    # IP will be something like fec0:0:0:1::/64
    if ($ip =~ /\S+\:\S*$/) {
        return 1;
    }
    return 0;
}


sub new {
    my $pkg = shift;
    my $self = {};
    $self->{name} = shift;
    die "missing subnet name" unless $self->{name};
    $self->{isInterconnectSubnet} = $MGNetwork::TYPE;
    $self->{isMonitoredSubnet} = $MGNetwork::TYPE;
    $self->{monitoredSubnetAccess} = "";
    $self->{ipaddresses} = [];
    bless $self, $pkg;
    if ($self->isIPv6Address($self->{name})) {
        $self->{name}=$self->stripIPv6Prefix($self->{name});
    }
    return $self;
}

=pod

=item getName

=cut
sub getName {
    my ($self) = @_;
    return $self->{name};
}


=pod

=item setInterconnectFlag

=cut
sub setInterconnectFlag {
    my ($self) = @_;
    $self->{isInterconnectSubnet} = $MGinterconnectSubnet::TYPE;
}


=pod

=item isInterconnect

=cut
sub isInterconnect {
    my ($self) = @_;
    return $self->{isInterconnectSubnet};
}

=pod

=item setMonitoredFlag

=cut
sub setMonitoredFlag {
    my ($self) = @_;
    $self->{isMonitoredSubnet} = $MGMonitoredSubnet::TYPE;
}

=pod

=item isMonitored

=cut
sub isMonitored {
    my ($self) = @_;
    return $self->{isMonitoredSubnet};
}

=pod

=item getIpAddress <name>

=cut
sub getIpAddress {

    my ($self, $name) = @_;
    die "missing ip address" if ! $name;
    foreach my $ip (@{$self->{ipaddresses}}) {
        if ($ip eq $name) {
            return $ip;
        }
    }
}

=pod

=item getIpAddresses

=cut

sub getIpAddresses {
    my ($self) = @_;
    return @{$self->{ipaddresses}};
}
=pod

=item setIpAddress

=cut

sub setIpAddress {
    my ($self,$ipaddress) = @_;
    push(@{$self->{ipaddresses}},$ipaddress);
}

=pod

=item deleteIpAddress 

remove from ipaddresses array

=cut

sub deleteIpAddress {
    my ($self, $ip) = @_;
    die "no ip to delete" unless $ip;
    my @tmp = grep {$_ ne $ip} @{$self->{ipaddresses}};
    $self->{ipaddresses} = \@tmp;   
}

=pod

=item setMonitoredSubnetAccess

Internal use only

=cut
sub setMonitoredSubnetAccess {
    my ($self, $value) = @_;
    die "bad monitored_subnet_access" if ! $value;
    $self->{monitoredSubnetAccess} = $value;
}

=pod

=item getMonitoredSubnetAccess

=cut
sub getMonitoredSubnetAccess {
    my ($self) = @_;
    return $self->{monitoredSubnetAccess};
}


1; # for perl include
