####################################################################
# (C) Copyright 2007 Hewlett-Packard Development Company, L.P.
# @(#) Serviceguard  MGPackage.pm
# @(#) Product Name                : HP Serviceguard
# @(#) Product Version             : A.12.10.00
# @(#) Patch Name                  : 
####################################################################

=pod

=head1 NAME

MGPackage - a class for Serviceguard packages

=cut

package MGPackage;

use strict;
use Carp;
use MGService;
use MGFileSystem;
use MGPackageDependency;
use MGPackageWeight;
use MGResource;
use MGUser;
use MGNetwork;
use MGPackageControl;
use MGPackageConfig;
use MGPackageWrite;
use POSIX qw(uname);

=pod

=item new

Create a package object

=cut

sub new 
{
    my ($self,$name,$myConf) = @_;
    $self = {};
    $self->{name} = $name;
    die "missing name" if ! $self->{name};
    $self->{myConf} = $myConf;
    $self->{fileDir} = $ENV{'SB_ROOT'} . "/tmp/cmcluster/" . $self->{name} 
         if (defined($ENV{'SB_ROOT'})); 

    $self->{policies} = {};
    $self->{nodes} = [];
    $self->{services} = []; # plus control script data
    $self->{successorHaltTimeout} = 0;
    $self->{configParams} = [];
    $self->{controlParams} = [];
    $self->{pevParams} = [];
    $self->{users} = [];
    $self->{dependencies} = [];
    $self->{weights} = [];
    $self->{volumeGroups} = [];
    $self->{cvmDiskGroups} = [];
    $self->{vxvmDiskGroups} = [];
    $self->{fileSystems} = [];
    $self->{type} = '';
    $self->{cfsType} = '';
    $self->{subnets} = [];

    bless $self;
    return $self;
}

=pod

=item  setPevParam

=cut

sub setPevParam {
    my ($self, $name, $value) = @_;

    $name = lc($name);
    my $found = 0;
    my $last =  $#{$self->{pevParams}};
    for my $i (0 .. $last) {
        if (exists $self->{pevParams}[$i]{$name}) {
            $self->{pevParams}[$i] = {$name => $value};
            $found = 1;
            last;
        }
    }
    if (!$found) {
        $self->{pevParams}[$last+1] = {$name => $value};
    }
}

=pod

=item  setControlParam

=cut

sub setControlParam {
    my ($self, $name, $value) = @_;

    $name = lc($name);
    my $found = 0;
    my $last =  $#{$self->{controlParams}};
    for my $i (0 .. $last) {
        if (exists $self->{controlParams}[$i]{$name}) {
            $self->{controlParams}[$i] = {$name => $value};
            $found = 1;
            last;
        }
    }
    if (!$found) {
        $self->{controlParams}[$last+1] = {$name => $value};
    }
}


=pod

=item  setConfigParam

=cut

sub setConfigParam {
    my ($self, $name, $value) = @_;

    $name = lc($name);
    my $found = 0;
    my $last =  $#{$self->{configParams}};
    for my $i (0 .. $last) {
        if (exists $self->{configParams}[$i]{$name}) {
            $self->{configParams}[$i] = {$name => $value};
            $found = 1;
            last;
        }
    }
    if (!$found) {
        $self->{configParams}[$last+1] = {$name => $value};
    }
}

sub getConf {
    my ($self) = @_;
    return $self->{myConf};
}

=pod

=item  getConfigParam

=cut

sub getConfigParam {
    my ($self, $name) = @_;
    # lowercase everyname
    $name = lc($name);

    my $value = undef;
    my @configParams =  @{$self->{configParams}};
    my $last =  $#configParams;
    for my $i (0 .. $last) {
        if (exists $configParams[$i]{$name}) {
            $value =  $configParams[$i]{$name};
            last;
        }
    }
    return $value;
}

=pod

=item getVGs 

=cut

sub getVGs {
    my ($self) = @_;
    return $self->getArrayValues("volumeGroups");
}

=pod

=item getVxVMDG 

=cut

sub getVxVMDG {
    my ($self) = @_;
    return $self->getArrayValues("vxvmDiskGroups");
}

=pod

=item getCVMDG 

=cut

sub getCVMDG {
    my ($self) = @_;
    return $self->getArrayValues("cvmDiskGroups");
}

=pod

=item getArrayValues 

=cut

sub getArrayValues {
    my ($self, $arrayName) = @_;
    return @{$self->{$arrayName}};
}

=pod

=item getIpSubnet 

=cut

sub getIpSubnet {
    my ($self, $name) = @_;
    die "missing subnet name" unless $name;
    my @match = grep {$_->getName() eq $name} @{$self->{ip_subnets}};
    die "duplicate subnet found" if ((scalar @match) > 1);
    return $match[0] if (defined $match[0]);
    return undef;
}

=pod

=item getIpSubnets 

=cut

sub getIpSubnets {
    my ($self) = @_;
    return() if (!defined(@{$self->{ip_subnets}}));
    return (@{$self->{ip_subnets}});
}

sub getSGConfValue {
    my($var) = @_;

    my $sgConfFile = "/etc/cmcluster.conf";
    open (MYCONFFILE,$sgConfFile) or
          die "Failed to open $sgConfFile";

     my (@lines) = <MYCONFFILE>;
     foreach my $line (@lines) {
             next if ($line =~ /^#/); # skip comments
             next if ($line =~/^\s*$/); # skip blank lines
             my ($name, $value) = $line =~/^s*(\w+)=(\S.*)$/;
             if ($name eq $var) {
                 return($value);
             }
     }
     return("");
}


1; # for perl include
