####################################################################
# (C) Copyright 2007 Hewlett-Packard Development Company, L.P.
# @(#) Serviceguard MGPackageConfig.pm 
# @(#) Product Name                : HP Serviceguard
# @(#) Product Version             : A.12.10.00
# @(#) Patch Name                  : 
####################################################################

package MGPackage;

use strict;
use Carp;

=pod
=head1 NAME

MGPackageConfig - a class for Serviceguard packages

=cut


=pod

=item parseConfig 

=cut


sub parseConfig 
{
    my ($self, @lines) = @_;

    my $ncnt=0;
    my $lastService = undef;
    my $lastSubnet = undef;
    my $lastResource = undef;
    my $lastDependency = undef;
    my $lastWeight = undef;
    my $lastUser = undef;

    foreach my $line (@lines) {
        next if ($line =~ /^\#/); # skip comments
        next if ($line =~ /^\s*$/); # skip blank lines

        my ($name, $value) = $line =~ /^\s*(\w+)\s+(\S.*)$/;

        next if (! defined($value)); # skip lines w/o value

        if ($name =~ /PACKAGE_NAME/i) {
            $self->{$name} = $value;
        }
        elsif ($name =~ /NODE_NAME/i) {
            push(@{$self->{nodes}},$value);
        }
        elsif ($name =~ /SERVICE_NAME/i) {
            if (defined($lastService)) {
                push(@{$self->{services}},$lastService);
            }
            $lastService = MGService->new($value);

        }
        elsif ($name =~ /SERVICE_FAIL_FAST_ENABLED/i) {
            $lastService->setFailfastEnabled($value);
        }
        elsif ($name =~ /SERVICE_HALT_TIMEOUT/i) {
            $lastService->setHaltTimeout($value);
        }
        elsif ($name =~ /^SUBNET/i || $name eq "MONITORED_SUBNET") {
            if (defined($lastSubnet)) {
                push(@{$self->{subnets}},$lastSubnet);
            }
            $lastSubnet = MGNetwork->new($value);
            $lastSubnet->setMonitoredFlag();
        }
	elsif ($name =~ /MONITORED_SUBNET_ACCESS/i) {
	    $lastSubnet->setMonitoredSubnetAccess($value);
	}
        elsif ($name =~ /CLUSTER_INTERCONNECT_SUBNET/i) {
            if (defined($lastSubnet)) {
                push(@{$self->{subnets}},$lastSubnet);
            }
            $lastSubnet = MGNetwork->new($value);
	    $lastSubnet->setInterconnectFlag();	    
	    push(@{$self->{subnets}},$lastSubnet);
        }
        elsif ($name =~ /RESOURCE_NAME/i) {
            if ( defined($lastResource)) {
                push(@{$self->{resources}},$lastResource);
            }
            $lastResource = MGResource->new($value);
        }
        elsif ($name =~ /RESOURCE_START/i) {
            $lastResource->setStart($value);
        }
        elsif ($name =~ /RESOURCE_POLLING_INTERVAL/i) {
            $lastResource->setPollingInterval($value);
        }
        elsif ($name =~ /RESOURCE_UP_VALUE/i) {
            $lastResource->addCriteria(MGCriteria->new($value));
        }
        elsif ($name =~ /STORAGE_GROUP/i) {
            push(@{$self->{storageGroups}}, $value);
        }
        elsif ($name eq "RUN_SCRIPT") {
            $self->{controlPath} = $value;
        }
        elsif ($name =~ /USER_NAME/i) {
            $value =~ s/\s*$//;
            $lastUser = MGUser->new([split(/\s+/, $value)]);
        }
        elsif ($name =~ /USER_HOST/i) {
            $lastUser->setHost($value);
        }
        elsif ($name =~ /USER_ROLE/i) {
            $lastUser->setRole($value);
            push(@{$self->{users}}, $lastUser);
            undef $lastUser;
        }
        elsif ($name =~ /WEIGHT_NAME/i) {
            $lastWeight = $self->getWeight($value);
            if (! defined($lastWeight)) {
                $lastWeight = MGPackageWeight->new($self, $value);
                $self->addWeight($lastWeight);
            }
        }
        elsif ($name =~ /WEIGHT_VALUE/i) {
            $lastWeight->setValue($value);
        }
        elsif ($name =~ /DEPENDENCY_NAME/i) {
            $lastDependency = $self->getDependency($value);
            if (! defined($lastDependency)) {
                $lastDependency = MGPackageDependency->new($self, $value);
                $self->addDependency($lastDependency);
            }
        }
        elsif ($name =~ /DEPENDENCY_CONDITION/i) {
            $value =~ /^\s*(\S+)\s+\=\s+(\S+)\s*$/;
            $lastDependency->setDependee($1);
            $lastDependency->setStatus($2);
        }
        elsif ($name =~ /DEPENDENCY_LOCATION/i) {
               $lastDependency->setLocation($value);
        } else {
               $self->setConfigParam($name,$value);
        }
   } # end foreach

   # save the last ones
   if ( defined($lastService)) {
        push(@{$self->{services}},$lastService);
   }
   if ( defined($lastResource)) {
        push(@{$self->{resources}},$lastResource);
   }
   if ( defined($lastSubnet)) {
        push(@{$self->{subnets}},$lastSubnet);
   }
}

=pod

=item readConfig 

=cut

sub readConfig
{
   my ($self, $name) = @_;
   my @output = ();

   my $cmd = 'cmviewcl -p '.$name . '> /dev/null';
   # check package is configured
   system($cmd) == 0 or return($? >> 8);

   $cmd = 'cmgetconf -v 0 -p '.$name.'|';
   open(CMD,$cmd) or return ($? >>8);
   while (<CMD>) {
          push(@output,$_);
           last if eof(CMD);
   }
   close(CMD);
   $self->parseConfig(@output);
   return(0);

}

1; # for perl include
