####################################################################
# (C) Copyright 2007-2013 Hewlett-Packard Development Company, L.P.
# @(#) Serviceguard cmmigratepkg
# @(#) Product Name                : HP Serviceguard
# @(#) Product Version             : A.12.10.00
# @(#) Patch Name                  : 
####################################################################

=pod

=head1 NAME

MGPackageWrite - a class for Serviceguard packages

=cut

package MGPackage;

use strict;
use Carp;
use MGService;
use MGFileSystem;
use MGPackageDependency;
use MGPackageWeight;
use MGResource;
use MGUser;
use MGNetwork;
use POSIX qw(uname);

=pod

=item toControl 

=cut

sub toConfig()
{
    my ($self,$external,$noServices) = @_;
    my $ptype = lc($self->getConfigParam("package_type")); 
    die "Missing Package Type.\nPackage migration failed.\n" 
         if (!defined $ptype);


    my $out = "# Package generated by Migration Program \n";

    $out .= $self->formatPkgName();
    $out .= $self->formatModules($ptype);
    $out .= $self->formatConfigParams();
    $out .= $self->formatNodes();
    $out .= $self->formatOperationSeq();
    $out .= $self->formatPevParams();
    $out .= $self->formatServices($noServices);
    $out .= $self->formatInterconnectSubnet();
    $out .= $self->formatSubnet();
    $out .= $self->formatResources();
    $out .= $self->formatDependencies();
    $out .= $self->formatWeights();
    $out .= $self->formatStorage();
    $out .= $self->formatUsers();
    $out .= $self->formatControlParams();
    $out .= $self->formatExternal($external) if ( defined ($external));

    return $out;
}

sub formatExternal
{
     my ($self,$external) = @_;
     my $out = "";
      
     $out .= "external_script\t\t\t" . $external . "\n" 
              if (defined ($self->{start_script})); 
     return $out;
}

=pod

=item formatUser 

=cut

sub formatUser
{
     my ($self) = @_;
     my $out = "";
     foreach my $user (@{$self->{users}}) {
             $out .= $user->getAscii();
     }
     return $out;
}

=pod

=item formatDependencies 

=cut

sub formatDependencies
{
     my ($self) = @_;
     my $out = "";
     foreach my $dependency (@{$self->{dependencies}}) {
        $out .=  "dependency_name\t\t\t" .
                 $dependency->getName() . "\n";
        $out .= "dependency_condition\t\t\t" .
                 $dependency->getCondition() . "\n";
        $out .= "dependency_location\t\t\t" .
                 $dependency->getLocation() . "\n";
     }
     return $out;
}
=pod

=item formatWeights 

=cut

sub formatWeights
{
     my ($self) = @_;
     my $out = "";
     foreach my $weight (@{$self->{weights}}) {
        $out .=  "weight_name\t\t\t" .
                 $weight->getName() . "\n";
        $out .= "weight_value\t\t\t" .
                 $weight->getValue() . "\n";
     }
     return $out;
}

=pod

=item formatPkgName 

=cut

sub formatPkgName
{
     my ($self) = @_;
     return "package_name\t\t\t"  . $self->{name} . "\n";
}

=pod

=item formatModules 

=cut

sub formatModules
{
     my ($self,$ptype) = @_;
     my $out = "";

     $out.=<<MOD;

module_name                     sg/basic
module_version                  1
module_name                     sg/$ptype
module_version                  1
module_name                     sg/priority
module_version                  1
module_name                     sg/dependency
module_version                  1
module_name                     sg/weight
module_version                  1
module_name                     sg/monitor_subnet
module_version                  1
module_name                     sg/service
module_version                  1
module_name                     sg/volume_group
module_version                  1
module_name                     sg/filesystem
module_version                  1
module_name                     sg/pev
module_version                  1
module_name                     sg/external_pre
module_version                  1
module_name                     sg/external
module_version                  1
module_name                     sg/acp
module_version                  1
MOD

     if (defined ($self->{resources})) {
         $out .= "module_name\t\t\tsg/resource\n";
         $out .= "module_version\t\t\t1\n";
     }

     # check if failover & linux add in persistent reservation mod
     my $os_type = (uname) [0];
     if (($ptype eq "failover") && ($os_type eq "Linux")) {
         $out .= "module_name\t\t\tsg/pr_cntl\n";
         $out .= "module_version\t\t\t2\n";
     }

     # Include the sg/package_ip module only if package type is failover
     if ($ptype eq "failover") {
        $out .= "module_name\t\t\tsg/package_ip\n";
        $out .= "module_version\t\t\t1\n";
     }

     $out .= "\n";

     return $out;
}

=pod

=item formatConfigParams 

=cut

sub formatConfigParams
{
     my ($self) = @_;
     my $out = "";

    # generic parameters
    my @configParams = @{$self->{configParams}};
    for my $i (0 .. $#configParams) {
        my ($name, $value ) = each (%{$configParams[$i]});
        if (defined($name) and defined($value)) {
            if (!($name =~ /run_script$/i) && !($name =~ /halt_script$/i)) {
                $out .= $name . "\t" . $value . "\n";
            }
        }
    }
    return $out;
}

=pod

=item formatNodes 

=cut

sub formatNodes
{
     my ($self) = @_;
     my $out = "";

     foreach my $node (@{$self->{nodes}}) {
            $out .= "node_name\t\t\t" . $node . "\n";
     }
     return $out;
}

=pod

=item formatOperationSeq 

=cut

sub formatOperationSeq
{
     my ($self) = @_;
     my $out = "";

     my $ptype = lc($self->getConfigParam("package_type"));
     my $os_type = (uname) [0];
     if (($ptype eq "failover") && ($os_type eq "Linux")) {
         $out .= "operation_sequence\t\t\t\$SGCONF/scripts/sg/pr_cntl.sh\n";
     }

     $out.=<<OPER1;

operation_sequence                      \$SGCONF/scripts/sg/external_pre.sh
operation_sequence                      \$SGCONF/scripts/sg/volume_group.sh
operation_sequence                      \$SGCONF/scripts/sg/filesystem.sh

OPER1

     if ($ptype eq "failover") {
        $out .= "operation_sequence\t\t\t\$SGCONF/scripts/sg/package_ip.sh\n";
     }

     $out.=<<OPER2;

operation_sequence                      \$SGCONF/scripts/sg/external.sh
operation_sequence                      \$SGCONF/scripts/sg/service.sh

OPER2

     if (defined ($self->{resources})) {
         $out .= "operation_sequence\t\t\t\$SGCONF/scripts/sg/resource.sh\n";
     }
     return $out;

}

=pod

=item formatService 

=cut

sub formatService
{
    my ($self,$service) = @_;
    my $out = "";

    $out .= "service_name\t\t\t" .  $service->getName() . "\n";
    $out .= "service_cmd\t\t\t" . $service->getCommand() . "\n";

    my $restart =  $service->getMaxRestarts();
    $out .= "service_restart\t\t\t" . $restart . "\n"  if $restart;

    my $failFast = $service->getFailfastEnabled();
    $out .= "service_fail_fast_enabled\t\t\t" .
        $failFast . "\n" if $failFast;

    my $haltTimeout = $service->getHaltTimeout();
    $out .=  "service_halt_timeout\t\t\t" .
        $haltTimeout . "\n" if $haltTimeout;
    return $out;
}


=pod

=item formatServices 

=cut

sub formatServices
{
     my ($self,$noServices) = @_;
     my $out = "";
     # services
     foreach my $service (@{$self->{services}}) {
        $out .=  $service->displayService($noServices) . "\n";
     }
     return $out;
}
=pod

=item formatResources 

=cut


sub formatResources
{
     my ($self) = @_;
     my $out = "";
     return $out if (!defined ($self->{resources}));
     foreach my $resource (@{$self->{resources}}) {
        $out .=  "resource_name\t\t\t" .  $resource->{name} . "\n";
        $out .=  "resource_polling_interval\t" .
                  $resource->getPollingInterval() . "\n";
        $out .=  "resource_start\t\t\t" .  $resource->getStart() . "\n";
        foreach my $criteria ($resource->getCriterias()) {
            $out .= "resource_up_value\t\t\t" .
            $criteria->getName() . "\n";
        }

     }
     return $out;
}

=pod

=item formatInterconnectSubnet 

=cut

sub formatInterconnectSubnet
{
     my ($self) = @_;
     my $output = "";
     foreach my $subnet (@{$self->{interconnectsubnets}}) {
        next if !$subnet->isInterconnect();
        $output .= "cluster_interconnect_subnet\t\t\t" . $subnet->getName() . "\n";
     }
     return $output;
}
     
=pod

=item formatSubnet 

=cut

sub formatSubnet
{
     my ($self) = @_;
     my $output = "";

     foreach my $subnet ($self->getSubnets()) {
        next if $subnet->isInterconnect();
        next if !$subnet->isMonitored();
        $output .= "monitored_subnet\t" . $subnet->getName() . "\n";
	if ($subnet->getMonitoredSubnetAccess()) {
	    $output .= "monitored_subnet_access\t" . 
	      $subnet->getMonitoredSubnetAccess() . "\n";
	  }
     }

     foreach my $ipsubnet ($self->getSubnets()) {
        if ( (@{$ipsubnet->{ipaddresses}}) != 0 ) {
             $output .= "ip_subnet\t" .  $ipsubnet->getName() . "\n";
             foreach my $ip (@{$ipsubnet->{ipaddresses}}) {
                     $output .= "ip_address\t" . $ip . "\n";
             }
        }
     }
     $output .= "\n";
     return $output;
}



=pod

=item formatStorage 

=cut

sub formatStorage 
{
    # print all VG, CVM, VXVM vgs out
    my ($self) = @_;
    my $output = "";
    # access the internal arrays directly
    my @vgs = $self->getVGs();
    my @vxvms = $self->getVxVMDG();
    my @cvms = $self->getCVMDG();

    foreach my $vg (@vgs) {
        $output = $output . "vg\t\t\t" . $vg . "\n";
    }

    foreach my $dg (@vxvms) {
        $output = $output . "vxvm_dg\t\t\t" . $dg . "\n";
    }

    foreach my $dg (@cvms) {
        $output = $output . "cvm_dg\t\t\t" . $dg . "\n";
    }

    my $default = '';
    foreach my $fs ($self->getFileSystems()) {
        my $device =  $fs->getDevice() ;
        my $fsName =  $fs->getName() ;

        my $fsType = '"'. ($fs->getType() || $default) . '"';
        my $mountOpt = '"'. ($fs->getMountOptions() || $default) . '"' ;
        my $umountOpt = '"' . ($fs->getUmountOptions() || $default) . '"' ;
        my $fsckOpt =  '"' . ($fs->getFsckOptions() || $default) . '"';

        $output = $output. "fs_name\t\t\t" . $device . "\n";
        $output = $output. "fs_directory\t\t\t" . $fsName. "\n";
        $output = $output. "fs_type\t\t\t" . $fsType . "\n";
        $output = $output. "fs_mount_opt\t\t\t" . $mountOpt . "\n";
        $output = $output. "fs_umount_opt\t\t\t" . $umountOpt . "\n";
        $output = $output. "fs_fsck_opt\t\t\t" . $fsckOpt . "\n\n";
    }

    return $output;
}
=pod

=item formatUsers 

=cut

sub formatUsers {
    my ($self) = @_;
    my $out = "";
    foreach my $user (@{$self->{users}}) {
        $out .= $user->getAscii();
    }
    return $out;
}

=pod

=item formatPevParams 

=cut

sub formatPevParams
{
    my ($self) = @_;
    my $out = "";
    my $prefix = "";

    $prefix = "PEV_";
    my @pevParams = @{$self->{pevParams}};
    return $out if (@pevParams == 0);

    print "Add the prefix PEV_ to the following parameters in the " . 
          "external script:\n";

    for my $i (0 .. $#pevParams) {
        my ($name, $value ) = each (%{$pevParams[$i]});
        if (defined($name) and defined($value)) {
                $out .= $prefix . $name . "\t\t\t" . $value . "\n";
                print  $name . "\n";
        }
    }
    $out .= "\n";
    return $out;
}

=pod

=item formatControlParams 

=cut

sub formatControlParams
{
    my ($self) = @_;
    my $out = "";

    # generic parameters
    my @controlParams = @{$self->{controlParams}};
    for my $i (0 .. $#controlParams) {
        my ($name, $value ) = each (%{$controlParams[$i]});
        if (defined($name) and defined($value)) {
                $out .= $name . "\t\t\t" . $value . "\n";
        }
    }
    return $out;
}

=pod

=item writePackage 

=cut

sub writePackage
{

   my ($self,$fileName,$external,$noServices) = @_;
   my $out = undef;

   my $dir_path = $fileName;
   $dir_path =~ s/(.*)\/.*$/$1/;

   if ($dir_path ne $fileName) {
     system("mkdir -p $dir_path");
   } 

   open(NEW_PACKAGE_FILE,">$fileName") or
        die "Failed to open $fileName.\nPackage migration failed\n";
   print NEW_PACKAGE_FILE $self->toConfig($external,$noServices);
   close(NEW_PACKAGE_FILE);
   chmod(0755, $fileName);


}

=pod

=item toExternal 

=cut

sub toExternal()
{
    my ($self) = @_;

   my $sgConf = getSGConfValue("SGCONF");
   my $template = "$sgConf/examples/external_script.template";
   open(NEW_EXTERNAL_FILE,$template) or
        confess "Failed to open $template. Can not create External Script.";

    my (@lines) = <NEW_EXTERNAL_FILE>;
    my $out = "# External Script generated by Migration Program \n";
    # read template and add start/stop functions
    my $cnt = 0;
    while ($cnt < @lines) {
        $out .= $lines[$cnt];
        # insert users function
        if ($lines[$cnt] =~ /^sg_source_pkg_env \$*/) {
            $out .= "\n# Migrated Functions from " . $self->{name} . 
                    " package control script\n\n";
            foreach my $userfunc (@{$self->{users_functions}}) {
                    $out .= $userfunc;
            }
        }
        if ($lines[$cnt] =~ /# ADD your package start steps here/) {
            foreach my $stuff (@{$self->{start_script}}) {
                    $out .= $stuff;
            }
        }

        if ($lines[$cnt] =~ /# ADD your package halt steps here/) {
              foreach my $stopstuff (@{$self->{stop_script}}) {
                    $out .= $stopstuff;
              }
        }
        $cnt++;
    }
    return $out;
}

=pod

=item writeExternal 

=cut

sub writeExternal
{
   my ($self,$fileName) = @_;
   my $out = undef;

   open(NEW_PACKAGE_FILE,">$fileName") or
        die "Failed to open $fileName. Can not create package's External 
             script.\nPackage migration failed.\n";
   print NEW_PACKAGE_FILE $self->toExternal();
   close(NEW_PACKAGE_FILE);

   # set the correct permissions for the external script
   chmod(0744, $fileName);

}

=pod

=item runCommand

=cut

sub runCommand 
{
    my ($myCmd) = @_;
    my @lines = ();
    my @output = ();
    open(CMD, "$myCmd 2>&1 |") or die "Failed to open $myCmd : $!";
    while (<CMD>) {
        push(@output, $_);
        last if eof(CMD);
    }
    close(CMD);
    my $exitcode = ($? >> 8);
    return($exitcode, @output);
}
=pod

=item checkTkit

=cut

sub checkTkit
{
    my ($self,$pkgName, $fileName) = @_;
    my $sg_path = getSGConfValue("SGSBIN");
    my $tkitmigratepkg= $sg_path . "/tkitmigratepkg";
    if ( -f $tkitmigratepkg && -x $tkitmigratepkg) {
        my $cmd = "$tkitmigratepkg $pkgName $fileName";
        my ($ret, @out) = runCommand($cmd);
        if( $ret != 0 ) {
            print "@out";
            print "Toolkit package $pkgName migration failed.";
            exit(1);
        }
        print "@out";
    } else {
        print "Failed to find $tkitmigratepkg or ";
        print "it does not have executable permission.\n";
        die "Package Migration failed.\n";
    }
}

1; # for perl include
