####################################################################
# (C) Copyright 2007 Hewlett-Packard Development Company, L.P.
# @(#) Serviceguard MGResource.pm 
# @(#) Product Name                : HP Serviceguard
# @(#) Product Version             : A.12.10.00
# @(#) Patch Name                  : 
####################################################################

package MGPackage;

# EMS related subroutines for packages

=pod

=item getResources

Returns a list of MGResource.

=cut
sub getResources {
    my ($self) = @_;
    return @{$self->{resources}};
}

=pod

=item getResource <name>

Returns an MGResource.

=cut
sub getResource {
    my ($self, $name) = @_;
    foreach my $resource (@{$self->{resources}}) {
        if ($resource->getName() eq $name) {
            return $resource;
        }
    }
    return undef;
}

=pod

=item getDeferredResourceNames

=cut
sub getDeferredResourceNames {
    my ($self) = @_;
    my @tmp = ();
    foreach my $resource (@{$self->{resources}}) {
        my $resStr=lc($resource->getStart());
        if ($resStr eq "deferred") {
            push(@tmp , $resource->getName());
        }
    }
    return @tmp;
}

=pod

=head1 NAME

MGResource - class for Serviceguard package resources

=cut
package MGResource;

=pod

=item new <name>

=cut
sub new {
    my $pkg = shift;
    my $self = {};
    $self->{name} = shift;
    $self->{start} = undef;
    $self->{pollingInterval} = undef;
    $self->{criterias} = []; # spelled funny on purpose
    bless $self, $pkg;
    return $self;
}

=pod

=item getName

=cut
sub getName {
    my ($self) = @_;
    return $self->{name};
}

=pod

=item setStart <value>

=cut
sub setStart {
    my ($self, $value) = @_;
    $self->{start} = $value;
}

=pod

=item getStart

=cut
sub getStart {
    my ($self) = @_;
    return $self->{start};
}

=pod

=item setPollingInterval <value>

=cut
sub setPollingInterval {
    my ($self, $value) = @_;
    $self->{pollingInterval} = $value;
}

=pod

=item getPollingInterval

=cut
sub getPollingInterval {
    my ($self) = @_;
    return $self->{pollingInterval};
}

=pod

=item addCriteria <MGCriteria>

=cut
sub addCriteria {
    my ($self, $criteria) = @_;
    push(@{$self->{criterias}}, $criteria);
}

=pod

=item getCriterias

Returns a list of MGCriterias.

=cut
sub getCriterias {
    my ($self) = @_;
    return @{$self->{criterias}};
}

=pod

=item deleteCriterias <name> ...

=cut
sub deleteCriterias {
    my ($self, @names) = @_;
    foreach my $name (@names) {
        my @tmp = grep {$_ ne $name} @{$self->{criterias}};
        $self->{criterias} = [@tmp];
    }
}

=pod

=item deleteAllCriterias

=cut
sub deleteAllCriterias {
    my ($self) = @_;
    $self->{criterias} = [];
}


=pod

=head1 NAME

MGCriteria - a class for Serviceguard package resource criteria

=cut
package MGCriteria;

=pod

=item new <name>

=cut
sub new {
    my $pkg = shift;
    my $self = {};
    $self->{name} = shift;
    bless $self, $pkg;
    return $self;
}

=pod

=item getName

=cut
sub getName {
    my ($self) = @_;
    return $self->{name};
}


1; # for perl include
