####################################################################
# (C) Copyright 2007 Hewlett-Packard Development Company, L.P.
# @(#) Serviceguard MGService.pm 
# @(#) Product Name                : HP Serviceguard
# @(#) Product Version             : A.12.10.00
# @(#) Patch Name                  : 
####################################################################
# service related subroutines for packages
=pod

=head1 NAME

MGService - a class for Serviceguard services

=cut

package MGPackage;

=pod

=item getServices

=cut
sub getServices {
    my ($self) = @_;
    return @{$self->{services}};
}

=pod

=item getService

=cut
sub getService {
    my ($self, $name) = @_;
    foreach my $service (@{$self->{services}}) {
        if ($service->getName() eq $name) {
            return $service;
        }
    }
    return undef;
}

=pod

=head1 NAME

MGService - class for a Serviceguard package service

=cut
package MGService;

use strict;
use Carp;
use Scalar::Util qw(weaken);

=pod

=item new

Internal use only

=cut
sub new {
    my ($pkg,$value) = @_;
    my $self = {};
    $self->{name} = $value;
    die "missing name" if ! $self->{name};
    # initialize 
    $self->{maxRestarts} = "";
    $self->{command} = "";
    $self->{failfastEnabled} = "";
    $self->{haltTimeout} = "";
    bless $self, $pkg;
    return $self;
}

=pod

=item getName

=cut
sub getName {
    my ($self) = @_;
    return $self->{name};
}

=pod

=item setMaxRestarts

Internal use only

=cut
sub setMaxRestarts {
    my ($self, $value) = @_;
    $self->{maxRestarts} = $value;
}

=pod

=item getMaxRestarts

=cut
sub getMaxRestarts {
    my ($self) = @_;
    return $self->{maxRestarts};
}

=pod

=item setFailfastEnabled <value>

=cut
sub setFailfastEnabled {
    my ($self, $value) = @_;
    $self->{failfastEnabled} = $value;
}

=pod

=item getFailfastEnabled

=cut
sub getFailfastEnabled {
    my ($self) = @_;
    return $self->{failfastEnabled};
}

=pod

=item setHaltTimeout <value>

=cut
sub setHaltTimeout {
    my ($self, $value) = @_;
    $self->{haltTimeout} = $value;
}

=pod

=item getHaltTimeout

=cut
sub getHaltTimeout {
    my ($self) = @_;
    return $self->{haltTimeout};
}


=pod

=item setCommand

Internal use only

=cut
sub setCommand {
    my ($self, $value) = @_;
    die "bad command" if ! $value;
    $self->{command} = $value;
}

=pod

=item getCommand

=cut
sub getCommand {
    my ($self) = @_;
    die "bad service: " . $self->{name} if ! $self->{command};
    return $self->{command};
}

=pod

=item displayService

=cut
# return string for configfile
sub displayService
{

    my ($self,$noService) = @_;
    my $out = "";
    my $cmt = "";

    # if -s no service option then comment out services
    if ( defined $noService) {
         $cmt = "# ";
         print "Warning: all services are commented out of the" .
               " package configuration file\n";
    } 
    $out .= $cmt . "service_name\t" .  $self->getName() . "\n";
    $out .= $cmt . "service_cmd\t\"" . $self->getCommand() . "\"\n";

    my $restart =  $self->getMaxRestarts();
    $out .= $cmt . "service_restart\t" . $restart . "\n"  if $restart;

    my $failFast = $self->getFailfastEnabled();
    $out .= $cmt . "service_fail_fast_enabled\t" .
        $failFast . "\n" if $failFast;

    my $haltTimeout = $self->getHaltTimeout();
    $out .=  $cmt . "service_halt_timeout\t" .
        $haltTimeout . "\n" if $haltTimeout;

    return $out;
}

1; # for perl include
