#!/usr/bin/perl -w
##################################################################
# (C) Copyright 2014 Hewlett-Packard Development Company, L.P.
# @(#) Serviceguard cluster simulation status parser 
# @(#) Product Name : HP Serviceguard
# @(#) Product Version : A.12.10.00
# @(#) Patch Name :     
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HP Serviceguard.
#
# Changing this file may lead to issues in Simulator behaviour
#
###################################################################

use CSIMSGCluster;
use CSIMSGClusterNodes;
use CSIMSGClusterPackage ;
use Data::Dumper;

our $clusterobj = CSIMSGCluster->new();
our $node_cap_warn_flag = 0;
our $def_weight_warn_flag = 0;
our $pkg_smn_warn_flag = 0;
our $pkg_weight_warn_flag = 0;
our $pkg_dep_warn_flag = 0;
our $pkg_gr_warn_flag = 0;

#$fname="cmviewline.input";
#init($fname);
#$outfname="cmviewline.out";
#printCluster($clusterobj);
#dumpClusterConfiguration($clusterobj,$outfname);

sub resetClusterObj {
    $clusterobj = undef; 
    $clusterobj = CSIMSGCluster->new();
}

sub statusParser(){
	my ($filename) = @_;
	&resetClusterObj();
	#open FH, "$filename" or die "$filename $!";
	open FH, $filename;
	my @lines;
	while ($x=<FH>){
		push(@lines, $x);
	}
    
	close FH;
    parseView(@lines);
    return $clusterobj;	
}

sub parseView() {    
    my  (@lines) = @_;
	
    foreach my $line (@lines) {
    	my ($name, $value) = split(/=/, $line, 2);
        if ($name =~ /^package:/) {
            chomp($value);
            parsePackageLine($name, $value);
            unless ($clusterobj) {
                return;
            }
            next;
        }
        if ($name =~ /^node:/) {
            chomp($value);
            parseNodeLine($name, $value);
            unless ($clusterobj) {
                return;
            }
            next;
        }
        chomp($value);
        parseClusterLine($name, $value);
        unless ($clusterobj) {
            return;
        }
    }
    
    #printCluster();
    #dumpClusterConfiguration($clusterobj);
    
}

sub printCluster {
	my $clusterobj=shift;
	 
	print "\n\nCluster	: ",$clusterobj->getName();
	print "\nStatus	: ", $clusterobj->getStatus();

	print "-----------\n";

	foreach $nodes (@{$clusterobj->{nodes}}) {
		print "\nNode	: ", $nodes->getName();
		print "\nStatus	: ",$nodes->getStatus();	
	}
	print "-----------\n";
	
	foreach $pkg (@{$clusterobj->{packages}}) {
		print "\nPackage Node	:",$pkg->getName();
		print "\nPackage Style	:",$pkg->getStyle();
		print "\nPackage Status	:",$pkg->getStatus();
		print "\nPackage Type	:",$pkg->getType();
	}

    
	print "\n-----------\n";
	print "-----------\n";
	print "-----------\n";
	print Dumper(\$clusterobj);
	print "-----------\n";
	print "-----------\n";
	print "-----------\n";
}

sub parseClusterLine {
	my ($name, $value) = @_;

	my @nameParts = split(/\|/, $name);
	
	if ($nameParts[0] eq 'name') {
	    $clusterobj->setName($value);
	    return;	    
	}
	if ($nameParts[0] eq 'id') {
	    $clusterobj->setId($value);
	    return;	    
	}
	if ($nameParts[0] eq 'max_reformation_duration') {
	    $clusterobj->setMaxReformationDuration($value);
	    return;	    
	}
	if ($nameParts[0] eq 'cfs_online_timeout_changes') {
	    $clusterobj->setCfsOnlineTimeoutChanges($value);
	    return;	    
	}
	if ($nameParts[0] eq 'io_timeout_extension') {
	    $clusterobj->setIOTimeoutExtension($value);
	    return;
	}
	if ($nameParts[0] eq 'configured_io_timeout_extension') {
	    $clusterobj->setConfiguredIOTimeoutExtension($value);
	    return;
	}
	if ($nameParts[0] eq 'hostname_address_family') {
	    $clusterobj->setHostnameResolution($value);
	    return;
	}
	if ($nameParts[0] eq 'status') {
	    $clusterobj->setStatus($value);
	    return;
	}
	if ($nameParts[0] eq 'state') {
	    $clusterobj->setState($value);
	    return;
	}
	if ($nameParts[0] eq 'member_timeout') {
	    $clusterobj->setMemberTimeout($value);
	    return;
	}
	if ($nameParts[0] eq 'network_polling_interval') {
	    $clusterobj->setConfigParam('network_polling_interval', $value);
	    return;
	}
	if ($nameParts[0] eq 'network_failure_detection') {
	    my $configValue = uc($value);
	    $clusterobj->setConfigParam('network_failure_detection', $configValue);
	    return;	    
	}
	if ($nameParts[0] eq 'network_auto_failback') {
	    my $configValue = uc($value);
	    $clusterobj->setConfigParam('network_auto_failback', $configValue);
	    return;
	}
	if ($nameParts[0] eq 'auto_start_timeout') {
	    $clusterobj->setAutoStartTimeout($value);
	    return;
	}
	if ($nameParts[0] eq 'max_configured_packages') {
	    $clusterobj->setConfigParam('max_configured_packages', $value);
	    return;
	}
	if ($nameParts[0] eq 'cluster_pr_mode') {
	    $clusterobj->setClusterPR($value);
	    return;	    
	}	
	if ($nameParts[0] eq 'incarnation') {
	    $clusterobj->setIncarnation($value);
	    return;
	}
	if ($nameParts[0] eq 'configuration_data_version') {
	    $clusterobj->setConfigurationDataVersion($value);
	    return;
	}
	if ($nameParts[0] eq 'coordinator') {
	    $clusterobj->setConfigParam('coordinator', $value);
	    return;
	}
	if ($nameParts[0] eq 'comment') {
	    $clusterobj->{'comment'}=$value;
	    return;
	}
	if ($nameParts[0] eq 'cluster_formation_time') {
	    $clusterobj->setConfigParam('cluster_formation_time',$value);
	    return;
	}
	if ($nameParts[0] eq 'summary') {
	    $clusterobj->setSummary($value);
	    return;
	}
	if ($nameParts[0] eq 'heartbeat_interval') {
	    $clusterobj->setConfigParam('heartbeat_interval', $value);
	    return;
	}
	if ($nameParts[0] eq 'node_timeout') {
	    $clusterobj->setConfigParam('node_timeout',$value);
	    return;
	}
    if ($nameParts[0] eq 'cluster_lock') {
        my $lockobj = undef;
        if( $nameParts[1] =~ /^node:(.+)/ ) {
            $lockobj = $clusterobj->getLockLun($1);
            unless ($lockobj) {
                $lockobj = SGLockLun->new() ;
                $clusterobj->addLockLun($lockobj);
            }
            $lockobj->setLockNodeName($1);
        }
        if( $nameParts[2] eq 'lun' ) {
            $lockobj->setLockLun($value);
            
        }
        if( $nameParts[2] eq 'status' ) {
            $lockobj->setLockStatus($value);
           
        }
        if( $nameParts[2] eq 'interface_type' ) {
            $lockobj->setLockInterfaceType($value);
            
        }
        return;
    }
	if ($nameParts[0] =~ /^quorum_server:(.+)/) {
		my $quorumobj = $clusterobj->getQS();
		
        my $quorumServerName = $value;
        
        if ($nameParts[1] eq 'name') {
        	unless ($quorumobj) {
        		$quorumobj = SGQuorumServer->new();
        	}
        	$quorumobj->setName($quorumServerName);
        	$clusterobj->setQS($quorumobj);
        }
        if ($nameParts[1] eq 'polling_interval') {
        	$quorumobj->setPollingInterval($value);
        }
        if ($nameParts[1]  =~ /^ip_address:(.+)/) {
        	my $qsip = $quorumobj->getIpAddress();
        	if ($nameParts[2] eq 'name') {
        		my $ipaddress = $value;
        		unless ($qsip) {
        			$qsip = SGQuorumInterface->new();
        		}
        		$qsip->setIpAddress($ipaddress);
        		$quorumobj->setIpAddress($qsip);
        		
        	}
        	if ($nameParts[2]  =~ /^node:(.+)/) {
        		my $nodeName = unconvertName($1);
        		my $qsNode = $qsip->getNode($nodeName);
        		unless ($qsNode) {
        			$qsNode = SGQuorumNode->new();
        			$qsNode->setName($value);
        			$qsip->addNode($qsNode);	
        	
        		}
        		if ($nameParts[3] eq 'name') {
                    return;
            	}
				if ($nameParts[3] eq 'status') {
                    $qsNode->setStatus($value);
                    return;
               	}
                if ($nameParts[3] eq 'state') {
                    $qsNode->setState($value);
                    return;
                }
        	}
        
        }
	}
        if ($nameParts[0] =~ /^weight_default:/) {
            unless ($def_weight_warn_flag) {
                print "WARNING: Package weights are not supported on the simulated cluster. The related functionality might differ from real cluster.\n";
                $def_weight_warn_flag = 1;
            }
            return;
        }
        if ($nameParts[0] =~ /^site:/) {
            print "Metrocluster site configuration is not supported on the simulated cluster.\n";
            $clusterobj = undef;
            return;
        }
        if ($nameParts[0] eq 'load_balancing') {
            if ($value eq 'on') {
                print "WARNING: Load balancing is not supported on the simulated cluster. The related functionality might differ from real cluster.\n";
                return;
            }
        }
}

sub parseNodeLine{
	my ($name, $value) = @_;
	my @nameParts = split(/\|/, $name);
		
	if ($nameParts[0] =~ /^node:(.+)/) {
		if ($nameParts[1] eq 'name') {
			#print ("1",$value);
			$node = $clusterobj->getNode($value);
			
			unless ($node) {
                $node = CSIMSGClusterNodes->new();
                $node->{'name'} = $value;
		        $clusterobj->addNode($node);
		        return;
		    }
	    }
	    
	    if ($nameParts[1] eq 'id') {
            $node->setId($value);
            return;
        }
        if ($nameParts[1] eq 'node_pr_key') {
            $node->setPrKey($value);
            
            return;
        }
        if ($nameParts[1] eq 'siteid') {
            $node->setSiteId($value);
            my $siteName = $self->getSiteName($value);
            $node->setSiteName($siteName);
            my $site = $self->getSite(id=>$value);
            $site->addNode($node->getName()); 
            return;
        }
        if ($nameParts[1] eq 'status') {
            $node->setStatus($value);
            return;
        }
        if ($nameParts[1] eq 'state') {
            $node->setState($value);
            return;
        }
        if ($nameParts[1] eq 'sg_version') {
            $node->setSGVersion($value);
            return;
        }
        if ($nameParts[1] eq 'sg_patch') {
            $node->setSGPatch($value);
            return;
        }
        if ($nameParts[1] eq 'os_status') {
            $node->setOsStatus($value);
            return;
        }
        if ($nameParts[1] eq 'summary') {
            $node->setSummary($value);
            return;
        }
        if ($nameParts[1] eq 'os_release') {
            $node->setOSRelease($value);
            return;
        }
        if ($nameParts[1] eq 'os_distribution') {
            $node->setOSDistribution($value);
            return;
        }
        if ($nameParts[1] eq 'os_distribution_release') {
            $node->setOSDistributionRelease($value);
            return;
        }
        if ($nameParts[1] eq 'os_distribution_description') {
            $node->setOSDistributionDescription($value);
            return;
        }
        if ($nameParts[1] eq 'sgerac_installed') {
        	#ignore
        	return;
        } 
        if ($nameParts[1] eq 'cpu_architecture') {
            $node->setCpuArch($value);
            return;
        }
        if ($nameParts[1] =~ /^subnet:(.+)/) {
	        my $subnetName = unconvertName($1);
    	    my $subnet = $node->getSubnet($subnetName);

        	unless ($subnet) {
            	$subnet = SGNodeSubnet->new();
            	$subnet->setName($subnetName);
            	$node->addSubnet($subnet);
        	}

        	if ($nameParts[2] eq 'name') {
        		return; 
        	}
        	if ($nameParts[2] eq 'route_id') {
    	        $subnet->setRouteId($value);
        	    return;
        	}
        	if ($nameParts[2] eq 'ip_monitor') {
    	        $subnet->setIpMonitor($value);
        	    return;
        	}
	        if ($nameParts[2] eq 'status') {
	            $subnet->setStatus($value);
	            return;
	        }
	        if ($nameParts[2] eq 'polling_target') {
	            $subnet->setPollingTarget($value);
	            return;
	        }
	    }
    	if ($nameParts[1] =~ /^capacity:(.+)/) {
                unless ($node_cap_warn_flag) {
                    print "WARNING: Node capacities are not supported on the simulated cluster. The related functionality might differ from real cluster.\n";
                    $node_cap_warn_flag = 1;
                }

	        my $capacityName = $1;
	        my $capacity = $node->getCapacity($capacityName);
	        
	        unless ($capacity) {
	            $capacity = SGNodeCapacity->new();
	            $capacity->setName($capacityName);
	            $node->addCapacity($capacity);
	        }
	        
	        if ($nameParts[2] eq 'name') {
	            # ignore
	            return;
	        }
	        if ($nameParts[2] eq 'limit') {
	            $capacity->setValue($value);
	            return;
	        }
	        if ($nameParts[2] eq 'used') {
	            $capacity->setUsed($value);
	            return;
	        }
	        if ($nameParts[2] eq 'percent_used') {
	            $capacity->setPercent($value);
	            return;
	        }
	    }
	    if ($nameParts[1] =~ /^interface:(.+)/) {
	        my $interfaceName = unconvertName($1); 
	        my $interface = $node->getInterface($interfaceName);
	        unless ($interface) {
	            $interface = SGInterface->new();
	            $interface->setName($interfaceName);
	            $node->addInterface($interface);
	        }
	        if ($nameParts[2] eq 'name') {
	            # ignore
	            return;
	        }
	        if ($nameParts[2] eq 'status') {
	            $interface->setStatus($value);
	            return;
	        }
	        if ($nameParts[2] eq 'id') {
	            $interface->setId($value);
	            return;
	        }
	        if ($nameParts[2] eq 'disabled') {
	            $interface->setState($value);
	            $interface->setDisabled($value);
	            return;
	        }
			if ($nameParts[2] eq 'mac_address') {
	            $interface->setMacAddress($value);
	            return;
	        }
	        if ($nameParts[2] eq 'ppa') {
	            $interface->setPpa($value);
	            return;
	        }
	        if ($nameParts[2] eq 'bridged_net') {
	            $interface->setBridgeNet($value);
	            return;
	        }
	        if ($nameParts[2] eq 'mac_type') {
	            $interface->setMacType($value);
	            return;
	        }
	        if ($nameParts[2] eq 'type') {
	            $interface->setType($value);
	            return;
	        }
	        if ($nameParts[2] eq 'failure_type') {
	            $interface->setFailureType($value);
	            return;
	        }
	        if ($nameParts[2] eq 'hardware_path') {
	            $interface->setPath($value);
	            return;
	        }
	        if ($nameParts[2] eq 'nmid') {
	            $interface->setNmid($value);
	            return;
	        }
	        if ($nameParts[2] eq 'local_switch_peer') {
	            $interface->setLocalSwitchPeer($value);
	            return;
	        }
	        if ($nameParts[2] =~ /^ip_address:(.+)/) {
	            my $ipName = $1;
	            my $ipadd = $interface->getIpAddress($ipName);
	            unless ($ipadd) {
	            	$ipadd = SGInterfaceIPAddress->new();
	            	$ipadd->setName($ipName);
	            	$interface->addIpAddress($ipadd);
	            }
	            
	            if ($nameParts[3] eq 'name') {
	            	#ignore;
	            	return;
	            }
	            if ($nameParts[3] eq 'netmask') {
	            	$ipadd->setNetMask($value);
	            	return;
	            }
	            if ($nameParts[3] eq 'subnet') {
	            	$ipadd->setSubnet($value);
	            	return;
	            }
	            if ($nameParts[3] eq 'route_id') {
	                $ipadd->setRouteId($value);
	                return;
	            }
	            if ($nameParts[3] eq 'heartbeat') {
	                $ipadd->setHeartbeat($value);
	                return;
	            }
	        }
	       
	    }
	    if ($nameParts[1] eq 'initial_incarnation') {
	        $node->setInitialIncarnation($value);
	        return;
	    }
	    if ($nameParts[1] eq 'boot_timestamp') {
	        $node->setBootTime($value);
	        return;
	    }
	}
      
	return;
}
sub parsePackageLine{
	my ($name, $value) = @_;
    my @nameParts = split(/\|/, $name);
    
    if ($nameParts[1] eq 'name') {
	    $pkg = $clusterobj->getPackage($value);
	    unless ($pkg){
	     	$pkg = CSIMSGClusterPackage->new();
	      	$pkg->setName($value);
	       	$clusterobj->addPackage($pkg);
	    } 
    }
    if ($nameParts[1] eq 'style') {
      	$pkg->setStyle($value);
        return;
    }
	if ($nameParts[1] eq 'pkg_pr_mode') {
		$pkg->setPrMode($value);
		return;
	}
	if ($nameParts[1] eq 'id') {
		$pkg->setId($value);
		return;
	}
	if ($nameParts[1] eq 'status') {
		$pkg->setStatus($value);
		return;
	}
	if ($nameParts[1] eq 'type') {
                if(($value eq 'system_multi_node') && ($pkg_smn_warn_flag == 0)) {
                    print "WARNING: System multi-node packages are not supported on the simulated cluster. The related functionality might differ from real cluster.\n";
                    $pkg_smn_warn_flag = 1;
                }
		$pkg->setType($value);
		return;
	}
	if ($nameParts[1] eq 'state') {
		$pkg->setState($value);
		return;
	}
	if ($nameParts[1] eq 'owner') {
		$pkg->setOwner($value);
		return;
	}
	if ($nameParts[1] eq 'package_halted') {
		$pkg->setHalted($value);
		return;
	}
	if ($nameParts[1] eq 'autorun') {
		$pkg->setAutoRun($value);
		return;
	}
	if ($nameParts[1] eq 'initial_autorun') {
		$pkg->setConfigParam("initial_autorun", $value);
		return;
	}
	if ($nameParts[1] eq 'failfast') {
		$pkg->setConfigParam("failfast", $value);
		return;
	}
	if ($nameParts[1] eq 'run_script') {
		$pkg->setConfigParam('run_script',$value);
		return;
	}
	if ($nameParts[1] eq 'run_script_timeout') {
		$pkg->setConfigParam("run_script_timeout", $value);
		return;
	}
	if ($nameParts[1] eq 'halt_script') {
		$pkg->setConfigParam("halt_script", $value);
		return;
	}
	if ($nameParts[1] eq 'halt_script_timeout') {
		$pkg->setConfigParam("halt_script_timeout", $value);
		return;
	}
	if ($nameParts[1] eq 'script_log_file') {
		$pkg->setConfigParam("script_log_file", $value);
		return;
	}
	if ($nameParts[1] eq 'successor_halt_timeout') {
		$pkg->setConfigParam("successor_halt_timeout", $value);
		return;
	}
	if ($nameParts[1] eq 'priority') {
		$pkg->setConfigParam("priority", $value);
		return;
	}
	if ($nameParts[1] eq 'all_cluster_nodes') {
		$pkg->setConfigParam("NODE_NAME", "*");
		$pkg->setConfigParam("all_cluster_nodes", $value);
		return;
	}
	if ($nameParts[1] eq 'failover_policy') {
		$pkg->setFailoverPolicy($value);
		return;
	}
	if ($nameParts[1] eq 'failback_policy') {
		$pkg->setFailbackPolicy($value);
		return;
	}
	if ($nameParts[1] eq 'log_level') {
		 # ignore
		return;
	}
	if ($nameParts[1] eq 'local_lan_failover_allowed') {
		my $configValue = "NO";
		$configValue = "YES" if $value eq "enabled";
		$pkg->setConfigParam("local_lan_failover_allowed", $configValue);
		return;
	}
    if ($nameParts[1] =~ /^subnet:(.+)/) {
        my $monitor_subnet = $1;
        my $monitorSubnetObj = $pkg->getMonitorSubnet($monitor_subnet);
        unless ($monitorSubnetObj) {
            $monitorSubnetObj = SGPackageMonitorSubnet->new();
            $monitorSubnetObj->setMonitorSubnet($monitor_subnet);
            $pkg->addSubnet($monitorSubnetObj);
        }
        if ($nameParts[2] eq 'monitored_subnet_access') {
            $monitorSubnetObj->setMonitorSubnetAccess($value);
            return;
        }
        if ($nameParts[2] =~ /^node:(.+)/) {
            my $monitor_subnet_node = $1;
            my $monitorSubnetNodeObj = $monitorSubnetObj->getMonitorSubnetNode($monitor_subnet_node);
            unless ($monitorSubnetNodeObj) {
                $monitorSubnetNodeObj = SGPackageMonitorSubnetNode->new();
                $monitorSubnetNodeObj->setMonitorSubnetNode($monitor_subnet_node);
                $monitorSubnetObj->addMonitorSubnetNode($monitorSubnetNodeObj);
            }   
            if ($nameParts[3] eq 'status') {
                $monitorSubnetNodeObj->setMonitorSubnetNodeStatus($value);
            }          
            return;
        }
    }
	if ($nameParts[1] =~ /^ip_subnet:(.+)/) {
        my $subnet = $1;
        my $subnetObj = $pkg->getIpSubnet($subnet);
        unless ($subnetObj) {
            $subnetObj = SGPackageIP->new();
            $subnetObj->setIpSubnet($subnet);
            $pkg->addIp($subnetObj);
        }
		if ($nameParts[2] eq 'ip_address') {
            my $ipObj = SGPackageIPName->new();
            $ipObj->setPackageIPName($value);
            $subnetObj->addIpAddress($ipObj);
            return;
        }
        if ($nameParts[2] eq 'ip_subnet_node') {
            $subnetObj->setIpSubnetNode($value);
            return;
        }
	}
	if ($nameParts[1] eq 'highly_available') {
		$pkg->setHighlyAvailable($value);
		return;
	}
	if ($nameParts[1] eq 'summary') {
		$pkg->setSummary($value);
		return;
	}
	if ($nameParts[1] =~ /^weight:(.+)/) {
                unless ($pkg_weight_warn_flag) {
                    print "WARNING: Package weights are not supported on the simulated cluster. The related functionality might differ from real cluster.\n";
                    $pkg_weight_warn_flag = 1;
                }

		my $weightName = $1; 
		my $weight = $pkg->getWeight($weightName);
		unless ($weight) {
			$weight = SGWeight->new();
			$weight->setName($weightName);
			$pkg->addWeight($weight);
		}
		if ($nameParts[2] eq 'name') {
			#ignore;
			return;
		}
		if ($nameParts[2] eq 'value') {
			$weight->setValue($value);
			return;
		}
	}
	
	if ($nameParts[1] =~ /^node:(.+)/) {
		my $nodeName = unconvertName($1);
		my $pNode = $pkg->getNode($nodeName);
		unless ($pNode) {
			$pNode = SGPackageNode->new();
			$pNode->setName($nodeName);
			$pkg->addNode($pNode);
		}
		if ($nameParts[2] eq 'name') {
			# ignore
			return;
		}
		if ($nameParts[2] eq 'status') {
			$pNode->setStatus($value);
			return;
		}
		if ($nameParts[2] eq 'state') {
			$pNode->setState($value);
			return;
		}
		if ($nameParts[2] eq 'switching') {
			$pNode->setSwitching($value);
			return;
		}
		if ($nameParts[2] eq 'type') {
			$pNode->setType($value);
			return;
		}
		if ($nameParts[2] eq 'order') {
			$pNode->setOrder($value);
			return;
		}
		if ($nameParts[2] eq 'last_run_time') {
			$pNode->setLastRunTime($value);
			return;
		}
		if ($nameParts[2] eq 'last_halt_time') {
			$pNode->setLastHaltTime($value);
			return;
		}
		if ($nameParts[2] eq 'available') {
			$pNode->setAvailable($value);
			return;
		}
		if ($nameParts[2] eq 'last_halt_failed') {
			$pNode->setLastHaltFailed($value);
			return;
		}
	}
		
	if ($nameParts[1] =~ /^module_name:(.+)/) {
		my $moduleName = $1;
		my $pModule = $pkg->getModule($moduleName);
		unless ($pModule) {
			$pModule = SGPackageModule->new();
			$pkg->addModule($pModule);
		}
		if ($nameParts[2] eq 'module_name') {
			$pModule->setName($moduleName);
			return;
		}
		if ($nameParts[2] eq 'module_version') {
			$pModule->setVersion($value);
			return;
		}
		if ($nameParts[2] eq 'module_scripts') {
			$pModule->setScript($value);
			return;
		}
	}
	if ($nameParts[1] =~ /^operation_sequence:(.+)/) {
		if ($nameParts[2] eq 'operation_sequence') {
			$pkg->addOperationSeq($value);
			return;
		}
	}
	if (($nameParts[1] eq 'concurrent_vgchange_operations') 
		|| ($nameParts[1] eq 'vgchange_cmd') 
		|| ($nameParts[1] eq 'enable_threaded_vgchange') 
		|| ($nameParts[1] eq 'vxvm_dg_retry')
		|| ($nameParts[1] eq 'cvm_activation_cmd')
		|| ($nameParts[1] eq 'vxvol_cmd')
		|| ($nameParts[1] eq 'deactivation_retry_count')
		|| ($nameParts[1] eq 'kill_processes_accessing_raw_devices')
		|| ($nameParts[1] eq 'concurrent_fsck_operations')
		|| ($nameParts[1] eq 'concurrent_mount_and_umount_operations')
		|| ($nameParts[1] eq 'fs_mount_retry_count')
		|| ($nameParts[1] eq 'fs_umount_retry_count')
		 ) {
		$pkg->setConfigParam($nameParts[1], $value);
		return;
	}
    if ($nameParts[1] =~ /^vg:(.+)/) {
        if ($nameParts[2] eq 'vg') {
            $pkg->addVg($value);
            return;
        }
    }
    
	if ($nameParts[1] =~ /^filesystem:(.+)/) {
		my $fileSystem = $1;
		my $fs = $pkg->getFileSystem($fileSystem);
		unless ($fs) {
            $fs = SGPackageFileSystem->new();
			$fs->setName($fileSystem);
			$pkg->addFileSystem($fs);
		}
		if ($nameParts[2] eq 'fs_server') {
			$fs->setFsServer($value);
			return;
		}
		if ($nameParts[2] eq 'fs_type') {
			$fs->setFsType($value);
			return;
		}
		if ($nameParts[2] eq 'fs_directory') {
			$fs->setFsDirectory($value);
			return;
		}
		if ($nameParts[2] eq 'fs_mount_opt') {
			$fs->setFsMountOpt($value);
			return;
		}
		if ($nameParts[2] eq 'fs_umount_opt') {
			$fs->setFsUMountOpt($value);
			return;
		}
		if ($nameParts[2] eq 'fs_fsck_opt') {
			$fs->setFsFsckOpt($value);
			return;
		}
	}
	if ($nameParts[1] =~ /^external_script:(.+)/) {
		my $ext = unconvertName($1);
		$pkg->addExternalScript($ext);
	}
	if ($nameParts[1] =~ /^external_pre_script:(.+)/) {
        my $extpre = unconvertName($1);
        my $externalPreScriptObj = SGPackageExternalPreScript->new();
        $externalPreScriptObj->setExternalPreScript($extpre);
        $pkg->addExternalPreScript($externalPreScriptObj);
	}
	if ($nameParts[1] =~ /^service:(.+)/) {
		my $serviceName = $1;
		
		my $service = $pkg->getService($serviceName);
		unless ($service) {
			$service = SGPackageService->new();
			$service->setName($serviceName);
			$pkg->addService($service);
		}
		if ($nameParts[2] eq 'id') {
			$service->setServiceConfigParam("id", $value);
			return;
		}
		if ($nameParts[2] eq 'failfast') {
			$service->setServiceConfigParam("failfast", $value);
			return;
		}
		if ($nameParts[2] eq 'halt_timeout') {
			$service->setServiceConfigParam("halt_timeout", $value);
			return;
		}
		if ($nameParts[2] eq 'command') {
			$service->setServiceConfigParam("command", $value);
			return;
		}
		if ($nameParts[2] eq 'restart') {
			$service->setServiceConfigParam("restart", $value);
			return;
		}
		if ($nameParts[2] =~ /^node:(.+)/) {
			my $nodeName = unconvertName($1); 
			my $pNode = $pkg->getNode($nodeName);
			unless ($pNode) {
				$pNode = SGPackageNode->new($nodeName);
				$pkg->addNode($pNode);
			}
			my $serviceNode = $pNode->getServiceNode($nodeName);
			unless ($serviceNode) {
				$serviceNode = SGPackageServiceNode->new();
				$serviceNode->setServiceName($serviceName);
				$serviceNode->setName($nodeName);
				$pNode->addServiceNode($serviceNode);
			}
			if ($nameParts[3] eq 'status') {
				$serviceNode->setStatus($value);
				return;
			}
			if ($nameParts[3] eq 'pid') {
				$serviceNode->setPid($value);
				return;
			}
			if ($nameParts[3] eq 'restart_limit') {
				$serviceNode->setMaxRestarts($value);
				return;
			}
			if ($nameParts[3] eq 'restart_count') {
				$serviceNode->setRestarts($value);
				return;
			}
		}
	}
	if ($nameParts[1] =~ /^dependency:(.+)/) {
                unless ($pkg_dep_warn_flag) {
                    print "WARNING: Package dependency is not supported on the simulated cluster. The related functionality might differ from real cluster.\n";
                    $pkg_dep_warn_flag = 1;
                }

		$depName = unconvertName($1);
		my $dependency = $pkg->getDependency($depName);
		unless ($dependency) {
			$dependency = SGDependency->new();
			$dependency->setName($depName);
			$pkg->addDependency($dependency);
		}
		if ($nameParts[2] eq 'name') {
			#ignore;
			return;
		}
		if ($nameParts[2] eq 'package_id') {
			$dependency->setPackageId($value);
			return;
		}
		if ($nameParts[2] eq 'location') {
			$dependency->setLocation($value);
			return;
		}
		if ($nameParts[2] eq 'condition') {
			$dependency->setCondition($value);
			return;
		}
		if ($nameParts[2] =~ /^node:(.+)/) {
			my $nodeName = unconvertName($1);
			if ($nameParts[3] eq 'satisfied') {
				$dependency->addNode($nodeName, $value);
				return;
			}
		}
	}
        if ($nameParts[1] =~ /^generic_resource:(.+)/) {
                unless ($pkg_gr_warn_flag) {
                    print "WARNING: Generic resources are not supported on the simulated cluster. The related functionality might differ from real cluster.\n";
                    $pkg_gr_warn_flag = 1;
                }
                return;
        }       
        if ($nameParts[1] =~ /^sc_site:(.+)/) {
                print "Site controller packages are not supported on the simulated cluster. The related functionality might differ from real cluster.\n";
                $clusterobj = undef;
                return;
        }
	if ($nameParts[1] =~ /^package_description:(.+)/) {
		$pkg->setPackageDescription($value);
	}
	    
	return;
}

# internal use only
# unconvertName() replaces any '|' and '#' name characters with '.' and ':'
# See config/config_line.c convert_name() where we do the opposite.
sub unconvertName {
    my $name = shift;
    #confess "missing name" unless $name;
    my $newName = $name;
    $newName =~ s/\|/./g;
    $newName =~ s/#/:/g;
    return $newName;
}

# The statusDumper function will dump the output in cmviewcl -v -f line in a file inside the csim dir.
sub statusDumper {
    
    my ($clusterobj,$filename) = @_;
    my $clusterName= $clusterobj->getName() ;
    my $ID= $clusterobj->getId() ;
    #my $filename= "cmviewline.out";
    open FH, ">$filename" or die $!;
    
    print FH "name=".$clusterobj->getName()."\n";
    print FH "id=".$clusterobj->getId()."\n";
    print FH "status=".$clusterobj->getStatus()."\n";
    print FH "state=".$clusterobj->getState()."\n";
    print FH "summary=".$clusterobj->getSummary()."\n";
    print FH "coordinator=".$clusterobj->{coordinator}."\n";
    print FH "incarnation=".$clusterobj->{incarnation}."\n";
    print FH "cluster_formation_time=".$clusterobj->{cluster_formation_time}."\n";
    print FH "configuration_data_version=".$clusterobj->getConfigurationDataVersion()."\n";
    print FH "member_timeout=".$clusterobj->getMemberTimeout()."\n";
    print FH "io_timeout_extension=".$clusterobj->getIOTimeoutExtension()."\n";
    print FH "configured_io_timeout_extension=".$clusterobj->getConfiguredIOTimeoutExtension()."\n";
    print FH "auto_start_timeout=".$clusterobj->{auto_start_timeout}."\n";
    print FH "network_polling_interval=".$clusterobj->{network_polling_interval}."\n";
    print FH "hostname_address_family=".$clusterobj->{hostnameResolution}."\n";
    print FH "max_reformation_duration=".$clusterobj->{max_reformation_duration}."\n";
    print FH "max_configured_packages=".$clusterobj->{max_configured_packages}."\n";
    print FH "cluster_pr_mode=".$clusterobj->getClusterPR()."\n";
     
    
    foreach my $mynode (@{$clusterobj->{nodes}}){
        print FH "node:$mynode->{name}"."|name=$mynode->{name}"."\n";
        print FH "node:$mynode->{name}"."|status=$mynode->{status}"."\n";
        print FH "node:$mynode->{name}"."|state=$mynode->{state}"."\n";
        print FH "node:$mynode->{name}"."|initial_incarnation=$mynode->{initialIncarnation}"."\n";
        print FH "node:$mynode->{name}"."|os_status=$mynode->{os_status}"."\n";
        print FH "node:$mynode->{name}"."|summary=$mynode->{summary}"."\n";
        print FH "node:$mynode->{name}"."|id=$mynode->{id}"."\n";
        print FH "node:$mynode->{name}"."|node_pr_key=$mynode->{node_pr_key}"."\n";
        print FH "node:$mynode->{name}"."|sg_version=$mynode->{sgVersion}"."\n";
        print FH "node:$mynode->{name}"."|os_release=$mynode->{osRelease}"."\n";
        print FH "node:$mynode->{name}"."|os_distribution=$mynode->{os_distribution}"."\n";
        print FH "node:$mynode->{name}"."|os_distribution_release=$mynode->{os_distribution_release}"."\n";
        print FH "node:$mynode->{name}"."|os_distribution_description=$mynode->{os_distribution_description}"."\n";
        print FH "node:$mynode->{name}"."|cpu_architecture=$mynode->{cpu_architecture}"."\n";
        print FH "node:$mynode->{name}"."|boot_timestamp=$mynode->{bootTime}"."\n";        
        foreach my $myinterface (@{$mynode->{interfaces}}) {
            print FH "node:$mynode->{name}"."|interface:$myinterface->{name}"."|name=$myinterface->{name}\n";
            print FH "node:$mynode->{name}"."|interface:$myinterface->{name}"."|id=$myinterface->{id}\n";
            print FH "node:$mynode->{name}"."|interface:$myinterface->{name}"."|status=$myinterface->{status}\n";
            print FH "node:$mynode->{name}"."|interface:$myinterface->{name}"."|disabled=$myinterface->{disabled}\n";
            print FH "node:$mynode->{name}"."|interface:$myinterface->{name}"."|mac_address=$myinterface->{mac_address}\n";
            print FH "node:$mynode->{name}"."|interface:$myinterface->{name}"."|ppa=$myinterface->{ppa}\n";
            print FH "node:$mynode->{name}"."|interface:$myinterface->{name}"."|bridged_net=$myinterface->{bridgeNet}\n";
            print FH "node:$mynode->{name}"."|interface:$myinterface->{name}"."|mac_type=$myinterface->{mac_type}\n";
            print FH "node:$mynode->{name}"."|interface:$myinterface->{name}"."|type=$myinterface->{type}\n";
            foreach my $myipadd (@{$myinterface->{ipaddress}}){
                print FH "node:$mynode->{name}"."|interface:$myinterface->{name}"."|ip_address:$myipadd->{name}"."|name=$myipadd->{name}"."\n";
                print FH "node:$mynode->{name}"."|interface:$myinterface->{name}"."|ip_address:$myipadd->{name}"."|netmask=$myipadd->{netmask}"."\n";
                print FH "node:$mynode->{name}"."|interface:$myinterface->{name}"."|ip_address:$myipadd->{name}"."|subnet=$myipadd->{subnet}"."\n";
                if ($myipadd->{route_id}) {
                    print FH "node:$mynode->{name}"."|interface:$myinterface->{name}"."|ip_address:$myipadd->{name}"."|route_id=$myipadd->{route_id}"."\n";
                }
                if ($myipadd->{heartbeat}) {
                    print FH "node:$mynode->{name}"."|interface:$myinterface->{name}"."|ip_address:$myipadd->{name}"."|heartbeat=$myipadd->{heartbeat}"."\n";
                }	
            }
        }   
        foreach my $mysubnet (@{$mynode->{subnets}})  {
            print FH "node:$mynode->{name}"."|subnet:$mysubnet->{name}"."|name=$mysubnet->{name}\n";
            if ($mysubnet->{route_id}) {
                print FH "node:$mynode->{name}"."|subnet:$mysubnet->{name}"."|route_id=$mysubnet->{route_id}\n";
            }
            print FH "node:$mynode->{name}"."|subnet:$mysubnet->{name}"."|status=$mysubnet->{status}\n";
            print FH "node:$mynode->{name}"."|subnet:$mysubnet->{name}"."|ip_monitor=$mysubnet->{ip_monitor}\n";
            if ($mysubnet->{polling_target}) {
                print FH "node:$mynode->{name}"."|subnet:$mysubnet->{name}"."|polling_target=$mysubnet->{polling_target}\n";
            }
        }
        foreach my $mycap (@{$mynode->{capacities}}) {
            print FH "node:$mynode->{name}"."|capacity:$mycap->{capacityName}"."|name=$mycap->{capacityName}\n";
            print FH "node:$mynode->{name}"."|capacity:$mycap->{capacityName}"."|limit=$mycap->{capacityValue}\n";		
            print FH "node:$mynode->{name}"."|capacity:$mycap->{capacityName}"."|used=$mycap->{capacityUsed}\n";	
            print FH "node:$mynode->{name}"."|capacity:$mycap->{capacityName}"."|percent_used=$mycap->{capacityPercent}\n";
        }
    }
    
    foreach my $mypkg (@{$clusterobj->{packages}}) {
        print FH "package:",$mypkg->{name},"|name=",$mypkg->{name},"\n";
        print FH "package:",$mypkg->{name},"|style=",$mypkg->{style},"\n";
        print FH "package:",$mypkg->{name},"|type=",$mypkg->{type},"\n";
        print FH "package:",$mypkg->{name},"|status=",$mypkg->{status},"\n";
        print FH "package:",$mypkg->{name},"|state=",$mypkg->{state},"\n";
        print FH "package:",$mypkg->{name},"|highly_available=",$mypkg->{highlyAvailable},"\n";
        print FH "package:",$mypkg->{name},"|summary=",$mypkg->{summary},"\n";
        print FH "package:",$mypkg->{name},"|autorun=",$mypkg->{autoRun},"\n";
        if ($mypkg->{package_halted}) {
            print FH "package:",$mypkg->{name},"|package_halted=",$mypkg->{package_halted},"\n";
        }
        if ( $mypkg->{type} eq "failover") {
            print FH "package:",$mypkg->{name},"|owner=",$mypkg->{owner},"\n";
        }
        print FH "package:",$mypkg->{name},"|id=",$mypkg->{id},"\n";
        print FH "package:",$mypkg->{name},"|all_cluster_nodes=",$mypkg->{all_cluster_nodes},"\n";
        print FH "package:",$mypkg->{name},"|initial_autorun=",$mypkg->{initial_autorun},"\n";
        if ( $mypkg->{type} eq "failover") {
            print FH "package:",$mypkg->{name},"|failover_policy=",$mypkg->{failover_policy},"\n";
            print FH "package:",$mypkg->{name},"|failback_policy=",$mypkg->{failback_policy},"\n";
        }
        print FH "package:",$mypkg->{name},"|failfast=",$mypkg->{failfast},"\n";
        print FH "package:",$mypkg->{name},"|run_script_timeout=",$mypkg->{run_script_timeout},"\n";
        print FH "package:",$mypkg->{name},"|halt_script_timeout=",$mypkg->{halt_script_timeout},"\n";
        print FH "package:",$mypkg->{name},"|priority=",$mypkg->{priority},"\n";
        print FH "package:",$mypkg->{name},"|successor_halt_timeout=",$mypkg->{successor_halt_timeout},"\n";
        print FH "package:",$mypkg->{name},"|script_log_file=",$mypkg->{script_log_file},"\n";
		
        foreach my $weight (@{$mypkg->{weights}}) {
            print FH "package:",$mypkg->{name},"|weight:",$weight->{name},"|name=",$weight->{name},"\n";
            print FH "package:",$mypkg->{name},"|weight:",$weight->{name},"|value=",$weight->{value},"\n";
        }
        foreach my $mypnode (@{$mypkg->{nodes}}) {
            print FH "package:",$mypkg->{name},"|node:",$mypnode->{name},"|","name=",$mypnode->{name},"\n";
            print FH "package:",$mypkg->{name},"|node:",$mypnode->{name},"|","status=",$mypnode->{status},"\n";
            unless ($mypkg->{type} eq "system_multi_node") {
                print FH "package:",$mypkg->{name},"|node:",$mypnode->{name},"|","state=",$mypnode->{state},"\n";
            }
            print FH "package:",$mypkg->{name},"|node:",$mypnode->{name},"|","switching=",$mypnode->{switching},"\n";
            print FH "package:",$mypkg->{name},"|node:",$mypnode->{name},"|","last_run_time=",$mypnode->{last_run_time},"\n";
            print FH "package:",$mypkg->{name},"|node:",$mypnode->{name},"|","last_halt_time=",$mypnode->{last_halt_time},"\n";
            print FH "package:",$mypkg->{name},"|node:",$mypnode->{name},"|","available=",$mypnode->{available},"\n";
            print FH "package:",$mypkg->{name},"|node:",$mypnode->{name},"|","last_halt_failed=",$mypnode->{last_halt_failed},"\n";
            if ( $mypkg->{type} eq "failover") {
                print FH "package:",$mypkg->{name},"|node:",$mypnode->{name},"|","type=",$mypnode->{type},"\n";
                print FH "package:",$mypkg->{name},"|node:",$mypnode->{name},"|","order=",$mypnode->{order},"\n"; 
            }
        }
        foreach my $my_monitor_subnet (@{$mypkg->{monitored_subnet}}) {
            print FH "package:",$mypkg->{name},"|subnet:",$my_monitor_subnet->{name},"|","name=",$my_monitor_subnet->{name},"\n";
            print FH "package:",$mypkg->{name},"|subnet:",$my_monitor_subnet->{name},"|","monitored_subnet_access=",$my_monitor_subnet->{monitored_subnet_access},"\n";
            foreach my $my_subnet_node (@{$my_monitor_subnet->{node}}) {
                print FH "package:",$mypkg->{name},"|subnet:",$my_monitor_subnet->{name},"|","node:",$my_subnet_node->{name},"|status=",$my_subnet_node->{status},"\n";
            }
        }
        foreach my $service (@{$mypkg->{services}}) {
            print FH "package:",$mypkg->{name},"|service:",$service->{name},"|name=",$service->{name},"\n";
            print FH "package:",$mypkg->{name},"|service:",$service->{name},"|id=",$service->{id},"\n";
            print FH "package:",$mypkg->{name},"|service:",$service->{name},"|failfast=",$service->{failfast},"\n";
            print FH "package:",$mypkg->{name},"|service:",$service->{name},"|halt_timeout=",$service->{halt_timeout},"\n";
            print FH "package:",$mypkg->{name},"|service:",$service->{name},"|command=",$service->{command},"\n";
            print FH "package:",$mypkg->{name},"|service:",$service->{name},"|restart=",$service->{restart},"\n";
            foreach my $mypnode (@{$mypkg->{nodes}}) {
                foreach my $myserNode (@{$mypnode->{services}}) {
                    if($service->{name} eq $myserNode->{service_name}){
                        print FH "package:",$mypkg->{name},"|service:",$service->{name},"|node:",$myserNode->{name},"|status=".$myserNode->{status},"\n";
                        print FH "package:",$mypkg->{name},"|service:",$service->{name},"|node:",$myserNode->{name},"|restart_limit=".$myserNode->{restart_limit},"\n";
                        print FH "package:",$mypkg->{name},"|service:",$service->{name},"|node:",$myserNode->{name},"|restart_count=".$myserNode->{restart_count},"\n";
                    }
                }
            }
        }
        foreach my $mydep (@{$mypkg->{dependencies}}) {
            print FH "package:",$mypkg->{name},"|dependency:",$mydep->{name},"|name=",$mydep->{name},"\n";
            print FH "package:",$mypkg->{name},"|dependency:",$mydep->{name},"|package_id=",$mydep->{package_id},"\n";
            print FH "package:",$mypkg->{name},"|dependency:",$mydep->{name},"|location=",$mydep->{location},"\n";
            print FH "package:",$mypkg->{name},"|dependency:",$mydep->{name},"|condition=",$mydep->{condition},"\n";
            foreach my $depnode (@{$mydep->{node}}) {
                print FH "package:",$mypkg->{name},"|dependency:",$mydep->{name},"|node:",$depnode->{name},"|satisfied=",$depnode->{satisfied},"\n";
            }
        }
        foreach my $ip (@{$mypkg->{ip}}) {
            foreach my $pkg_ip (@{$ip->{ip_address}}) { # for ipv6 addresses
                my @ip_addr = split("/",$pkg_ip->{name});
                print FH "package:",$mypkg->{name},"|ip_address:",$ip_addr[0],"|name=",$ip_addr[0],"\n";
            }
        }
        $mypkg->{package_description} =~ s/^"(.*)"$/$1/;
        print FH "package:",$mypkg->{name},"|package_description=\"",$mypkg->{package_description},"\"\n";
        foreach my $mymodule (@{$mypkg->{module}}) {
            print FH "package:",$mypkg->{name},"|module_name:",$mymodule->{name},"|module_name=",$mymodule->{name},"\n";
            print FH "package:",$mypkg->{name},"|module_name:",$mymodule->{name},"|module_version=",$mymodule->{module_version},"\n";
            if ($mymodule->{module_script}) {
                print FH "package:",$mypkg->{name},"|module_name:",$mymodule->{name},"|module_scripts=",$mymodule->{module_script},"\n";
            }
        }
        foreach my $myopseq (@{$mypkg->{operation_sequence}}) {
            print FH "package:",$mypkg->{name},"|operation_sequence:",$myopseq,"|operation_sequence=",$myopseq,"\n";
        }
        #print FH "package:",$mypkg->{name},"|concurrent_vgchange_operations=",$mypkg->{concurrent_vgchange_operations},"\n";    
        #print FH "package:",$mypkg->{name},"|enable_threaded_vgchange=",$mypkg->{enable_threaded_vgchange},"\n";
        
        foreach my $ip (@{$mypkg->{ip}}) {
            print FH "package:",$mypkg->{name},"|ip_subnet:",$ip->{ip_subnet},"|ip_subnet=",$ip->{ip_subnet},"\n";
            print FH "package:",$mypkg->{name},"|ip_subnet:",$ip->{ip_subnet},"|ip_subnet=",$ip->{ip_subnet},"\n";
            foreach my $pkg_ip (@{$ip->{ip_address}}) {
                print FH "package:",$mypkg->{name},"|ip_subnet:",$ip->{ip_subnet},"|ip_address=",$pkg_ip->{name},"\n";
            }
        }
        print FH "package:",$mypkg->{name},"|vgchange_cmd=",$mypkg->{vgchange_cmd},"\n";
        
        foreach my $vg (@{$mypkg->{vgs}}) {
            print FH "package:",$mypkg->{name},"|vg:",$vg,"|vg=",$vg,"\n";
        }
        
        #print FH "package:",$mypkg->{name},"|cvm_activation_cmd=",$mypkg->{cvm_activation_cmd},"\n";
        #print FH "package:",$mypkg->{name},"|vxvol_cmd=",$mypkg->{vxvol_cmd},"\n";
        #print FH "package:",$mypkg->{name},"|vxvm_dg_retry=",$mypkg->{vxvm_dg_retry},"\n";
        #print FH "package:",$mypkg->{name},"|deactivation_retry_count=",$mypkg->{deactivation_retry_count},"\n";
        #print FH "package:",$mypkg->{name},"|kill_processes_accessing_raw_devices=",$mypkg->{kill_processes_accessing_raw_devices},"\n";
        print FH "package:",$mypkg->{name},"|concurrent_fsck_operations=",$mypkg->{concurrent_fsck_operations},"\n";
        print FH "package:",$mypkg->{name},"|concurrent_mount_and_umount_operations=",$mypkg->{concurrent_mount_and_umount_operations },"\n";
        print FH "package:",$mypkg->{name},"|fs_mount_retry_count=",$mypkg->{fs_mount_retry_count},"\n";
        print FH "package:",$mypkg->{name},"|fs_umount_retry_count=",$mypkg->{fs_umount_retry_count},"\n";
		
		
        foreach my $extpre (@{$mypkg->{external_pre_script}}) {
            print FH "package:",$mypkg->{name},"|external_pre_script:",$extpre->{name},"|external_pre_script=",$extpre->{name},"\n";
        }
        foreach my $ext (@{$mypkg->{external_script}}) {
        print FH "package:",$mypkg->{name},"|external_script:",$ext,"|external_script=",$ext,"\n";
        }

        print FH "package:",$mypkg->{name},"|pkg_pr_mode=",$mypkg->{prMode},"\n";

        foreach my $myfs (@{$mypkg->{file_system}}) {
            # Remove double quotes
            $myfs->{fs_name} =~ s/^"(.*)"$/$1/;
            $myfs->{fs_server} =~ s/^"(.*)"$/$1/;
            $myfs->{fs_directory} =~ s/^"(.*)"$/$1/;
            $myfs->{fs_type} =~ s/^"(.*)"$/$1/;
            $myfs->{fs_mount_opt} =~ s/^"(.*)"$/$1/;
            $myfs->{fs_umount_opt} =~ s/^"(.*)"$/$1/;
            $myfs->{fs_fsck_opt} =~ s/^"(.*)"$/$1/;
            print FH "package:",$mypkg->{name},"|filesystem:",$myfs->{fs_name},"|filesystem=",$myfs->{fs_name},"\n";
            print FH "package:",$mypkg->{name},"|filesystem:",$myfs->{fs_name},"|fs_name=",$myfs->{fs_name},"\n";
            print FH "package:",$mypkg->{name},"|filesystem:",$myfs->{fs_name},"|fs_server=\"",$myfs->{fs_server},"\"\n";
            print FH "package:",$mypkg->{name},"|filesystem:",$myfs->{fs_name},"|fs_directory=",$myfs->{fs_directory},"\n";
            print FH "package:",$mypkg->{name},"|filesystem:",$myfs->{fs_name},"|fs_type=\"",$myfs->{fs_type},"\"\n";
            print FH "package:",$mypkg->{name},"|filesystem:",$myfs->{fs_name},"|fs_mount_opt=\"",$myfs->{fs_mount_opt},"\"\n";
            print FH "package:",$mypkg->{name},"|filesystem:",$myfs->{fs_name},"|fs_umount_opt=\"",$myfs->{fs_umount_opt},"\"\n";
            print FH "package:",$mypkg->{name},"|filesystem:",$myfs->{fs_name},"|fs_fsck_opt=\"",$myfs->{fs_fsck_opt},"\"\n";
        }
    }
    if ($clusterobj->{lockLun}) {
        foreach my $lockobj (@{$clusterobj->{lockLun}}) {
            print FH "cluster_lock|node:"."$lockobj->{node_name}"."|lun="."$lockobj->{lun}\n";
            print FH "cluster_lock|node:"."$lockobj->{node_name}"."|status="."$lockobj->{status}\n";
            print FH "cluster_lock|node:"."$lockobj->{node_name}"."|interface_type="."$lockobj->{interface_type}\n";
        }
    
    }
    if ($clusterobj->{qs}) {
        print FH "quorum_server:"."$clusterobj->{qs}->{name}"."|name="."$clusterobj->{qs}->{name}\n";
        print FH "quorum_server:"."$clusterobj->{qs}->{name}"."|polling_interval="."$clusterobj->{qs}->{polling_interval}\n";
        my $qsip=$clusterobj->{qs}->{ipaddress};
    
        foreach my $qsnode (@{$qsip->{nodes}}) {
    	    print FH "quorum_server:"."$clusterobj->{qs}->{name}"."|node:"."$qsnode->{name}"."|status="."$qsnode->{status}\n";
    	    print FH "quorum_server:"."$clusterobj->{qs}->{name}"."|node:"."$qsnode->{name}"."|state="."$qsnode->{state}\n";
        }
        print FH "quorum_server:"."$clusterobj->{qs}->{name}"."|ip_address:"."$qsip->{ip_address}"."|name="."$qsip->{ip_address}\n";
        foreach my $qsnode (@{$qsip->{nodes}}) {
    	    print FH "quorum_server:"."$clusterobj->{qs}->{name}"."|ip_address:"."$qsip->{ip_address}"."|node:"."$qsnode->{name}"."|name="."$qsnode->{name}\n";
    	    print FH "quorum_server:"."$clusterobj->{qs}->{name}"."|ip_address:"."$qsip->{ip_address}"."|node:"."$qsnode->{name}"."|status="."$qsnode->{status}\n";
    	    print FH "quorum_server:"."$clusterobj->{qs}->{name}"."|ip_address:"."$qsip->{ip_address}"."|node:"."$qsnode->{name}"."|state="."$qsnode->{state}\n";
        }
    }
    print FH "authorized_host:ANY_SERVICEGUARD_NODE|name=ANY_SERVICEGUARD_NODE\n";   
    print FH "authorized_host:ANY_SERVICEGUARD_NODE|user:ANY_USER|name=ANY_USER\n";
    print FH "authorized_host:ANY_SERVICEGUARD_NODE|user:ANY_USER|role=monitor\n";
    print FH "product_module:security|name=security\n";
    print FH "product_module:security|version=A.3.0.0\n";
    print FH "product_module:PACKAGE_MANAGER_MODULE_VERSION|name=PACKAGE_MANAGER_MODULE_VERSION\n";
    print FH "product_module:PACKAGE_MANAGER_MODULE_VERSION|version=A.02.00\n";
    print FH "product_module:CM_RCOMM_MODULE_VERSION|name=CM_RCOMM_MODULE_VERSION\n";
    print FH "product_module:CM_RCOMM_MODULE_VERSION|version=A.02.00\n";

    close FH;
    
}



1;
