#!/usr/bin/perl -w
##################################################################
# (C) Copyright 2014 Hewlett-Packard Development Company, L.P.
# @(#) Serviceguard cluster simulation command parser
# @(#) Product Name : HP Serviceguard
# @(#) Product Version : A.12.10.00
# @(#) Patch Name :     
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HP Serviceguard.
#
# Changing this file may lead to issues in Simulator behaviour
#
###################################################################

use Config;
#use strict;
use Getopt::Long;
use warnings;
use CSIMSGCluster;
use CSIMFunctions;
use CSIMSGClusterNodes;
use  CSIMSGClusterPackage;
use Data::Dumper;
require 'StatusUtils.pl';

# Global hashes to keep cluster and package attributes
my %clusterAttributes=();
my $clusterObj=CSIMSGCluster->new();
my $packageObj=CSIMSGClusterPackage->new();
my @clusterAttributeArrays="";
my %packageAttributes=();
my @packageAttributeArrays="";
my $sessionName="";
my $max_size=39;
my $CMVIEWCL="";
my $pkgName="";
my $clusterName="";
my $noOfNodesInCluster=0;
my $noOfSubnet=0;
my $filename = "";
my $SGCONF;


#subroutine to do some validation of mandatory attributes of cluster and package. It takes type (can be cluster or package) of object as input.

sub validateMandatoryAttribute {
    my ($type)=@_;
    unless ($type) {
        print "Missing type";
        return -1;
    }
    if($type eq "cluster") {    
        my $cluster_name = $clusterObj->getName();    
        if ($cluster_name eq "") {
            die "No value specified for CLUSTER_NAME at line.\nError found in Cluster Ascii File."; 
        }
        
        # Found that in normal SG cluster we dont validate follwoing scenario, so commented it out , need to finalize this.
        #       if ($clusterAttributes{'NODE_NAME'} eq ""){
        #           die "Undefined variable NODE_NAME.\n There must at least one node in the cluster.\n cmapplyconf: Error found in cluster Ascii File."; 
        #       }

        #    if ($noOfNodesInCluster == 2){
        #        if (($clusterAttributes{'CLUSTER_LOCK_LUN'} =~/^\s*$/) and ($clusterAttributeArrays[2] =~/^QS_HOST$/) and
        #          (($clusterAttributes{'FIRST_CLUSTER_LOCK_VG'} =~/^\s*$/) or ($clusterAttributes{'FIRST_CLUSTER_LOCK_PV'} =~/^\s*$/))){
        #            die "No Cluster Lock has been configured.\nPlease specify the Cluster Lock device in the cluster ascii file. \ncmapplyconf: Error found in cluster file\n";
        #       }
        # }
        #    if ($noOfNodesInCluster > 4){
        #        if (($clusterAttributes{'CLUSTER_LOCK_LUN'} !~/^\s*$/)or
        #           (($clusterAttributes{'FIRST_CLUSTER_LOCK_VG'} !~/^\s*$/) or 
        #            ($clusterAttributes{'FIRST_CLUSTER_LOCK_PV'} !~/^\s*$/))){
        #            die "You can configure only Quorum Server as Cluster Lock if your cluster has more than 4 nodes. \n";
        #        }
        #   }
    }

    #if ($type eq "package"){
    #   if ($packageAttributes{'PACKAGE_NAME'} eq ""){
    #      die "No value specified for PACKAGE_NAME at line.\nError found in Package Ascii File."; 
    # 	}
    #}        
     
}

#subrountine to  read the cluster/package ascii files and read the attributes and populate it to internal data storage.

sub parse_and_populate_attributes {
	
    my ($asciiFile,$type)=@_;
    my $line;		      # To process individual line of ascii file.
    my $nodeObj;					 
    my $interfaceObj;				 
    my $subnetObj;    
    my $ipObj;
    my $quorumObj;
    my $ip_routeId = 0; 
    my $subnet_routeId = 0;
    my @subnet_list;
    my $service_id = 0;

    $filename =  $SGCONF . "/simulator/" . $sessionName . "/" . $sessionName . "_cmviewcl_line";

    unless (open SRCASCII, "< $asciiFile") {
        print "Unable to open Ascii File $asciiFile \n";
        return -1;
    }

    if ($type eq 'cluster') {
        my $nodeId = 0;
        my $interfaceId = 0;
        my $bridge_net = 0;
        my $no_of_bytes_matchd = 0;
        my %ip_subnet_association = ();
        my %ipv6_subnet_association = ();
        my $clusterId=time();

        $clusterObj->setId($clusterId);

        while (<SRCASCII>) {
            $line=$_;
            if ($line =~ /^#/ || $line =~ /^\n/) {
                next;
            }
            $line=~ s/\s+/ /g;
            $line=~ s/^\s*//g;
            chomp($line);
            
            if ($line =~ /^CLUSTER_NAME/i) {
                my @cluster = split(" ",$line);
                $clusterObj->setName($cluster[1]);
            }
            elsif ($line =~ /^HOSTNAME_ADDRESS_FAMILY/i) {
                my @host_addrs_family = split(" ",$line);
                $clusterObj->setHostnameResolution(lc($host_addrs_family[1]));
            }
            elsif ($line =~ /^NODE_NAME/i) {
                my @nodeValue = split(" ",$line);
                $nodeObj = CSIMSGClusterNodes->new();
                $nodeObj->{'name'} = $nodeValue[1];

                my $boot_timestamp = time(); #Set bootTime for a node
                $nodeObj->setBootTime($boot_timestamp);
        
                #Set node Id
                $nodeObj->setId(++$nodeId);
                $nodeObj->constructNodePrKey($clusterId, $nodeId);
                
                #Initialize interface id for new node
                $interfaceId = 0;
                $bridge_net = 0;
                $ip_routeId = 0; 
        
                #Add node to cluster object
                $clusterObj->addNode($nodeObj);	
                $noOfNodesInCluster++;
                
                if ($quorumObj) {
                    #Add node name to quorum object
                    my $qsipObj = $quorumObj->getIpAddress();
                    my $qsnodeObj = SGQuorumNode->new();
                    $qsnodeObj->setName($nodeValue[1]);
                    $qsipObj->addNode($qsnodeObj);
                }
            }
            elsif ($line =~ /^NETWORK_INTERFACE/i) {
                my @interface = split(" ",$line);
                $interfaceObj = SGInterface->new();
                $interfaceObj->{name}= $interface[1];
                $interfaceObj->setId(++$interfaceId);
                $interfaceObj->setBridgeNet(++$bridge_net);
                $nodeObj->addInterface($interfaceObj);
            }
            elsif ($line =~ /^STATIONARY_IP/i || $line =~ /^HEARTBEAT/i) {
                my @ip = split(" ",$line);
                $ipObj = SGInterfaceIPAddress->new();
                $ipObj->setName($ip[1]);
                if($ip[0] =~ 'HEARTBEAT') {
                    $ipObj->setHeartbeat("true");
                }
                $ipObj->setRouteId(++$ip_routeId);
                $interfaceObj->addIpAddress($ipObj);	
            }
            elsif ($line =~ /^CLUSTER_LOCK_LUN/i) {
                my @lun = split(" ",$line);
                my $lockobj = $clusterObj->getLockLun($lun[1]);
                unless ($lockobj) {
                    $lockobj = SGLockLun->new() ;
                    $clusterObj->addLockLun($lockobj);
                }
                $lockobj->setLockLun($lun[1]);
                $lockobj->setLockNodeName($nodeObj->{name});
            }
            elsif ($line =~ /^SUBNET/i) {
                my @subnet = split(" ",$line);
                #my @byte = split(/\./,$subnet[1]);
                
                $subnetObj = SGNodeSubnet->new();
                $subnetObj->setName($subnet[1]);
                $subnetObj->setRouteId(++$subnet_routeId);
                push(@subnet_list,$subnetObj);
            }
            elsif ($line =~ /^IP_MONITOR/i) {
                my @monitor = split(" ",$line);
                $subnetObj->setIpMonitor(lc($monitor[1]));
            }
            elsif ($line =~ /^POLLING_TARGET/i) {
                my @polling_target = split(" ",$line);
                $subnetObj->setPollingTarget($polling_target[1]);
            }
            elsif ($line =~ /^CFS_ONLINE_TIMEOUT_CHANGES/i) {
                my @timeout = split(" ",$line);
                $clusterObj->setCfsOnlineTimeoutChanges($timeout[1]);	    		    
            }
            elsif ($line =~ /^MEMBER_TIMEOUT/i) {
                my @timeout = split(" ",$line);
                $clusterObj->setMemberTimeout($timeout[1]);	
            }
            elsif ($line =~ /^AUTO_START_TIMEOUT/i) {
                my @timeout = split(" ",$line);
                $clusterObj->setAutoStartTimeout($timeout[1]);
            }
            elsif ($line =~ /^CONFIGURED_IO_TIMEOUT_EXTENSION/i) {
                my @timeoutext = split(" ",$line);
                $clusterObj->setConfiguredIOTimeoutExtension($timeoutext[1]);
            }			
            elsif ($line =~ /^NETWORK_POLLING_INTERVAL/i) {
                my @interval = split(" ",$line);
                $clusterObj->setConfigParam('network_polling_interval',$interval[1]);
            }
            elsif ($line =~ /^NETWORK_FAILURE_DETECTION/i) {
                my @detection = split(" ",$line);
                $clusterObj->setConfigParam('network_failure_detection', $detection[1]);
            }
            elsif ($line =~ /^NETWORK_AUTO_FAILBACK/i) {
                my @failback = split(" ",$line);
                $clusterObj->setConfigParam('network_auto_failback', $failback[1]);
            }
            elsif ($line =~ /^MAX_CONFIGURED_PACKAGES/i) {
                my @packages = split(" ",$line);
                $clusterObj->setConfigParam('max_configured_packages', $packages[1]);
                
            }
            elsif ($line =~ /^CLUSTER_PR_MODE/i) {
                my @pr = split(" ",$line);
                $clusterObj->setClusterPR($pr[1]);
            }
            elsif ($line =~ /^HEARTBEAT_INTERVAL/i) {
                my @hb = split(" ",$line);
                $clusterObj->setConfigParam('heartbeat_interval', $hb[1]);	
            }
            elsif ($line =~ /^NODE_TIMEOUT/i) {
                my @timeout = split(" ",$line);
                $clusterObj->setConfigParam('node_timeout',$timeout[1]);	
            }							 
            elsif ($line =~ /^QS_HOST/i) {
                my @qs = split(" ",$line);
                $quorumObj = SGQuorumServer->new();
                $quorumObj->setName($qs[1]);
                my $qsipObj = SGQuorumInterface->new();
                $quorumObj->setIpAddress($qsipObj);
                $clusterObj->setQS($quorumObj);
            }
            elsif ($line =~ /^QS_POLLING_INTERVAL/i)	{
                my @qs_polling = split(" ",$line);
                $quorumObj->setPollingInterval($qs_polling[1]);
            }
            elsif ($line =~ /^QS_TIMEOUT_EXTENSION/i) {
                next;
            }
            elsif ($line =~ /^QS_ADDR/i) {
                #my @qs_ip_addr = split(" ",$line);
                #my $qsipObj = SGQuorumInterface->new();
                #$qsipObj->setIpAddress($qs_ip_addr[1]);
                #$quorumObj->setIpAddress($qsipObj);
            }
            elsif ($line =~ /^LOAD_BALANCING/i) {
                my @lb = split(" ",$line);
                if(lc($lb[1]) eq 'on') {
                    print "WARNING: Load balancing is not supported on the simulated cluster. The related functionality might differ from real cluster.\n";
                }
            }
            elsif (($line =~ /^CAPACITY_NAME/i) || ($line =~ /^CAPACITY_VALUE/i)) { 
                unless ($node_cap_warn_flag) {
                    print "WARNING: Node capacities are not supported on the simulated cluster. The related functionality might differ from real cluster.\n";
                    $node_cap_warn_flag = 1;
                }
            }
            elsif (($line =~ /^WEIGHT_NAME/i) || ($line =~ /^WEIGHT_DEFAULT/i)) {
                unless ($def_weight_warn_flag) {
                    print "WARNING: Package weights are not supported on the simulated cluster. The related functionality might differ from real cluster.\n";
                    $def_weight_warn_flag = 1;
                }
            }
            elsif (($line =~ /^SITE_NAME/i) || ($line =~ /^SITE/i)) {
                my @param = split(" ",$line);
                print "Metrocluster site configuration is not supported on the simulated cluster.\n";
                return -1;
            }
        }         
         
        my $name=$clusterObj->getName();
        my $cdb_ver=getCDBVer($name);
        chomp($cdb_ver);
        $cdb_ver=$cdb_ver + 2;
        
        $clusterObj->setConfigurationDataVersion("$cdb_ver");    
        
        my $timeout=getIOTimeout();
        $clusterObj->setIOTimeoutExtension($timeout); 
        #print Dumper $clusterObj;	
        #Following code is written for IP <=> SUBNET association 
        #Logic is to compare each byte of IP with each subnet byte
        #and associate the subnet whose maximum bytes matches.

        foreach my $mynode (@{$clusterObj->{nodes}}) {
            foreach my $myinterface (@{$mynode->{interfaces}}) {
                foreach my $myipadd (@{$myinterface->{ipaddress}}) {
                    for(my $i = 0; $i <  @subnet_list; $i++) {
                        my $isIPv6 = $myinterface->isIPv6Address($myipadd->{name});
                        my $isIPv6Subnet = $myinterface->isIPv6Address($subnet_list[$i]->{name});
                        if (!$isIPv6 && !$isIPv6Subnet) {
                            my @ip_byte = split(/\./,$myipadd->{name});
                            my @subnet_byte = split(/\./,$subnet_list[$i]->{name});
                            for(my $j = 0,my $k = 0; $j < @ip_byte; $j++, $k++) {
                                if ($ip_byte[$j] eq $subnet_byte[$k]) {
                                    $no_of_bytes_matchd++; #count for max bytes matched
                                }
                                else {
                                    next;
                                }
                            }
                            #Add subnet <=> 'no_of_bytes_matchd' (key value pair) to hash ip_subnet_association
                            $ip_subnet_association{$subnet_list[$i]->{name}} = $no_of_bytes_matchd;
                            $no_of_bytes_matchd = 0; 
                            my $subnet = get_key_with_highest_value_from_hash(\%ip_subnet_association);
                            $myipadd->setSubnet($subnet);
                        }
                    }
                    
                }
                foreach my $myipadd (@{$myinterface->{ipaddress}}) {
                    my $isIPv6 = $myinterface->isIPv6Address($myipadd->{name});
                    if ($isIPv6) {
                        for(my $i = 0; $i <  @subnet_list; $i++) {
                            my $isIPv6Subnet = $myinterface->isIPv6Address($subnet_list[$i]->{name});
                            if ($isIPv6Subnet) {
                                $no_of_bytes_matchd =0;
                                my @ip_byte = split(/\:/,$myipadd->{name});
                                my @subnet_byte = split(/\:/,$subnet_list[$i]->{name});
                                for(my $j = 0; $j < @subnet_byte; $j++) {
                                    if ($ip_byte[$j] eq $subnet_byte[$j]) {
                                        $no_of_bytes_matchd++; #count for max bytes matched
                                    }
                                }
                            }
                            #Add subnet <=> 'no_of_bytes_matchd' (key value pair) to hash ip_subnet_association
                            $ipv6_subnet_association{$subnet_list[$i]->{name}} = $no_of_bytes_matchd;
                            $no_of_bytes_matchd = 0; 
                            #For each IP get the subnet whose max bytes matched
                        }
                        my $subnet = get_key_with_highest_value_from_hash_v6(\%ipv6_subnet_association);
                        $myipadd->setSubnet($subnet);
                        
                        # for IPv6 ipaddress we will have ffff:ffff:ffff:ffff:: as default
                        # as this is being a limitation in simulated environment. 
                        $myipadd->setNetMask("ffff:ffff:ffff:ffff::");
                    }
                }
            }
        }
        foreach my $sub (@subnet_list) {
            foreach my $mynode (@{$clusterObj->{nodes}}) {
                $mynode->addSubnet($sub);
            }
        }
        #print Dumper $clusterObj;	
    }
    elsif ($type eq 'package') {

        my $pkgNodeObj;
        my $moduleObj;
        my $subnetObj;
        my $monitorSubnetObj;
        my $externalPreScriptObj;
        my $fsObj;
        my $weightObj;
        my $serviceObj;
        my $nodeOrder = 0;
        my $dependencyObj;
        my $dependency_name;
        my $line_no = 0;
        my $flag = 0;
        
        $clusterObj = &statusParser($filename);

        while (<SRCASCII>) {
            $line=$_;
            if ($line =~ /^#/ || $line =~ /^\n/) {
                $line_no++;
                next;
            }
            $line=~ s/\s+/ /g;
            $line=~ s/^\s*//g;
            chomp($line);
             
            if ($line =~ /^PACKAGE_NAME/i) {
                my @package = split(" ",$line);
                $packageObj->setName($package[1]); 
                $packageObj->setId(generate_id());            	
            }
            elsif ($line =~ /^MODULE_NAME/i) {
                my @module = split(" ",$line);
                $moduleObj = SGPackageModule->new();
                $moduleObj->setName($module[1]);
                $packageObj->addModule($moduleObj);
            }
            elsif ($line =~ /^MODULE_VERSION/i) {
                my @version = split(" ",$line);
                $moduleObj->setVersion($version[1]);
            }
            elsif ($line =~ /^MODULE_SCRIPTS/i) {
                my @script = split(" ",$line);
                $moduleObj->setScript($script[1]);
            }
            elsif ($line =~ /^PACKAGE_TYPE/i) {
                my @type = split(" ",$line);
                if ((lc($type[1]) eq 'system_multi_node') && ($pkg_smn_warn_flag == 0)) {
                    print "WARNING: System multi-node packages are not supported on the simulated cluster. The related functionality might differ from real cluster.\n";
                    $pkg_smn_warn_flag = 1;
                }
                $packageObj->setType($type[1]);
            }
            elsif ($line =~ /^MONITORED_SUBNET\b/i) {
                my @monitired_subnet = split(" ",$line);
                $monitorSubnetObj = SGPackageMonitorSubnet->new();
                $monitorSubnetObj->setMonitorSubnet($monitired_subnet[1]);
                $packageObj->addSubnet($monitorSubnetObj);    
                
                # Add node and it's subnet status of package 
                foreach my $subnetNode (@{$packageObj->{nodes}})
                {  
                    my $monitorSubnetNodeObj = SGPackageMonitorSubnetNode->new();
                    $monitorSubnetNodeObj->setMonitorSubnetNode($subnetNode->{name}); 
                    foreach my $mynode (@{$clusterObj->{nodes}}) {
                        if ($mynode->{name} eq $subnetNode->{name}) {
                            $monitorSubnetNodeObj->setMonitorSubnetNodeStatus($mynode->{status});
                            last;
                        }
                    }
 
                    $monitorSubnetObj->addMonitorSubnetNode($monitorSubnetNodeObj);
                }
            }
            elsif ($line =~ /^MONITORED_SUBNET_ACCESS\b/i) {
                my @monitor_subnet_access = split(" ",$line);
               
                $monitorSubnetObj->setMonitorSubnetAccess($monitor_subnet_access[1]);
            }
            elsif ($line =~ /^IP_SUBNET/i) {
                my @subnet = split(" ",$line);
                $subnetObj = SGPackageIP->new(); 
                $subnetObj->setIpSubnet($subnet[1]);
                $packageObj->addIp($subnetObj);
            }
            elsif ($line =~ /^IP_ADDRESS/i) {
                my @ip = split(" ",$line);
                my $ipObj = SGPackageIPName->new();
                $ipObj->setPackageIPName($ip[1]);
                $subnetObj->addIpAddress($ipObj);
            }
            elsif ($line =~ /^IP_SUBNET_NODE/i) {
                my @subnetNode = split(" ",$line);
                $subnetObj->setIpSubnetNode($subnetNode[1]);
            }
            elsif ($line =~ /^PACKAGE_DESCRIPTION/i) {
                my @description = split(" ",$line);
                shift @description;
                $packageObj->setPackageDescription("@description"); 
            }
            elsif (($line =~ /^GENERIC_RESOURCE_NAME/i) || ($line =~ /^GENERIC_RESOURCE_EVALUATION_TYPE/i) ||
                   ($line =~ /^GENERIC_RESOURCE_UP_CRITERIA/i)) {
                unless ($pkg_gr_warn_flag) {
                    print "WARNING: Generic resources are not supported on the simulated cluster. The related functionality might differ from real cluster.\n";
                    $pkg_gr_warn_flag = 1;
                }
            }
            elsif (($line =~ /^SC_SITE/i) || ($line =~ /^SC_SITE_MONITOR_INTERVAL/i)) {
                print "Site controller packages are not supported on the simulated cluster.\n";
                return -1;
            }
            elsif ($line =~ /^NODE_NAME/i) {
                my @node = split(" ",$line);
                if ($node[1] eq '*') {
                    my @nodes;
                    parseNodeNameStar(\@nodes);
                    foreach my $node_name (@nodes) {
                        $pkgNodeObj = SGPackageNode->new();
                        $pkgNodeObj->setName($node_name);
                        my $nodeObj = $clusterObj->getNode($node_name);
                        if ($flag == 1) {
                            if ($packageObj->getType() eq 'failover') {
                                if ($nodeObj->getStatus() eq "up") {
                                    $pkgNodeObj->setState("blocked");
                                }
                            }
                            else {
                                if ($nodeObj->getStatus() eq "up") {
                                    $pkgNodeObj->setState("halted");
                                }  
                            }
                        }
                        if ($flag == 0) {
                            if ($packageObj->getType() eq 'failover') {
                                $pkgNodeObj->setType("Primary");
                                if ($nodeObj->getStatus() eq "up") {
                                    $pkgNodeObj->setState("blocked");                               
                                }
                            }
                            else {
                                if ($nodeObj->getStatus() eq "up") {
                                    $pkgNodeObj->setState("halted");
                                }
                            }
                        $flag=1;
                        }
                        $pkgNodeObj->setOrder(++$nodeOrder);
                        $packageObj->addNode($pkgNodeObj);
                    }
                }
                else {
                    $pkgNodeObj = SGPackageNode->new();
                    $pkgNodeObj->setName($node[1]);
                     my $nodeObj = $clusterObj->getNode($node[1]);
                    if ($flag == 1) {
                        if ($packageObj->getType() eq 'failover') {
                            if ($nodeObj->getStatus() eq "up") {
                                $pkgNodeObj->setState("blocked");
                            }
                        }
                        else {
                            if ($nodeObj->getStatus() eq "up") {
                                $pkgNodeObj->setState("halted");
                            }  
                        }
                    }
                    if ($flag == 0) {
                        if ($packageObj->getType() eq 'failover') {
                            $pkgNodeObj->setType("Primary");
                            if ($nodeObj->getStatus() eq "up") {
                                $pkgNodeObj->setState("blocked");                               
                            }
                        }
                        else {
                            if ($nodeObj->getStatus() eq "up") {
                                $pkgNodeObj->setState("halted");
                            }
                        }
                        $flag=1;
                    }
                    $pkgNodeObj->setOrder(++$nodeOrder);
                    $packageObj->addNode($pkgNodeObj);
                    
                }
            }
            elsif ($line =~ /^AUTO_RUN/i) {
                my @auto_run = split(" ",$line);
                if($auto_run[1] eq 'yes') {
                    $packageObj->setConfigParam("initial_autorun", "enabled");
                    if ($clusterObj->getStatus() eq "up" && $packageObj->getType() eq 'failover') {
                        $packageObj->setAutoRun("disabled");
                    }
                    else {
                        $packageObj->setAutoRun("enabled");
                    }
                }
                else {
                    $packageObj->setConfigParam("initial_autorun", "disabled");
                    $packageObj->setAutoRun("disabled");
                }
            }
            elsif ($line =~ /^VGCHANGE_CMD/i) {
                my @cmd = split(" ",$line);
                my $vg_chng_cmd = join " ", @cmd[1..$#cmd];
                $packageObj->setConfigParam("vgchange_cmd", $vg_chng_cmd);
            }
            elsif ($line =~ /^VG/i) {
                my @vgs = split(" ",$line);
                $packageObj->addVg($vgs[1]);
                # Set pkg_pr_mode
                $packageObj->{prMode} = "pr_enabled";
            }
            elsif ($line =~ /^CONCURRENT_FSCK_OPERATIONS/i) {
                my @fsck_op = split(" ",$line);
                $packageObj->setConfigParam("concurrent_fsck_operations", $fsck_op[1]);
            }
            elsif ($line =~ /^CONCURRENT_MOUNT_AND_UMOUNT_OPERATIONS/i) {
                my @mount_op = split(" ",$line);
                $packageObj->setConfigParam("concurrent_mount_and_umount_operations", $mount_op[1]);
            }
            elsif ($line =~ /^FS_MOUNT_RETRY_COUNT/i) {
                my @count = split(" ",$line);
                $packageObj->setConfigParam("fs_mount_retry_count", $count[1]);
            }
            elsif ($line =~ /^FS_UMOUNT_RETRY_COUNT/i) {
                my @count = split(" ",$line);
                $packageObj->setConfigParam("fs_umount_retry_count", $count[1]);
            }
            elsif ($line =~ /^NODE_FAIL_FAST_ENABLED/i) {
                my @count = split(" ",$line);
				if($count[1] eq "yes") { $packageObj->setConfigParam("failfast", "enabled");}
				else { $packageObj->setConfigParam("failfast", "disabled"); }
            }
            elsif ($line =~ /^RUN_SCRIPT_TIMEOUT/i) {
                my @timeout = split(" ",$line);
                $packageObj->setConfigParam("run_script_timeout", $timeout[1]);
            }
            elsif ($line =~ /^HALT_SCRIPT_TIMEOUT/i) {
                my @timeout = split(" ",$line);
                $packageObj->setConfigParam("halt_script_timeout", $timeout[1]);
            }
            elsif ($line =~ /^SUCCESSOR_HALT_TIMEOUT/i) {
                my @timeout = split(" ",$line);
                $packageObj->setConfigParam("successor_halt_timeout", $timeout[1]);
            }
            elsif ($line =~ /^SCRIPT_LOG_FILE/i) {
                my @log_file = split(" ",$line);
                $packageObj->setConfigParam("script_log_file", $log_file[1]);
            }
            elsif ($line =~ /^OPERATION_SEQUENCE/i) {
                my @seq = split(" ",$line);
                $packageObj->addOperationSeq($seq[1]);
                foreach my $module (@{$packageObj->{module}}) {
                    my $tmp = ($module->{name}) . ".sh";
                    my @tmp_module = split("\/",$tmp);
                    if ($line =~ /$tmp_module[1]/i) {
                        my @module_script = split(" ",$line);
                        $module->setScript($module_script[1]);
                    }
                }
            }
            elsif($line =~ /^FAILOVER_POLICY/i) {
                my @policy = split(" ",$line);
                $packageObj->setFailoverPolicy($policy[1]);
            }
            elsif($line =~ /^FAILBACK_POLICY/i) {
                my @policy = split(" ",$line);
                $packageObj->setFailbackPolicy($policy[1]);
            }
            elsif ($line =~ /^PRIORITY/i) {
                my @priority = split(" ",$line);
                $packageObj->setConfigParam("priority", $priority[1]);
            }
             elsif($line =~ /^DEPENDENCY_NAME/i) {
                unless ($pkg_dep_warn_flag) {
                    print "WARNING: Package dependency is not supported on the simulated cluster. The related functionality might differ from real cluster.\n";
                    $pkg_dep_warn_flag = 1;
                }
                my @dependency = split(" ",$line);
                $dependencyObj = SGDependency->new();
                $dependencyObj->setName($dependency[1]);
                $dependency_name = $dependency[1];
                $packageObj->addDependency($dependencyObj);
            }
            elsif ($line =~ /^DEPENDENCY_CONDITION/i) {
                unless ($pkg_dep_warn_flag) {
                    print "WARNING: Package dependency is not supported on the simulated cluster. The related functionality might differ from real cluster.\n";
                    $pkg_dep_warn_flag = 1;
                }
                my $dependency_pkg_exist = 0;
                my @dependency_cond = split(" ",$line);
                my $cluster_Obj = &statusParser($filename);
                
                # Package can't be depend on itself.
                # Check to verify whether applied package is dependent on itself. 
                if ($packageObj->{name} eq $dependency_cond[1]) {
                    print "A package may not depend on itself." .
                          "The dependency $dependency_name in package $packageObj->{name} has specified a dependency on itself.\n";
                    return -1;
                }

                # Check if dependency package is already configured.
                # If package array has element, the variable $#{$cluster_Obj->{packages}} will return value >= 0. 
                if ($#{$cluster_Obj->{packages}} >= 0) {
                    foreach my $mypkg (@{$cluster_Obj->{packages}}) {
                        if ($mypkg->{name} eq $dependency_cond[1]) {
                            $dependency_pkg_exist = 1;
                            # Get package id of dependent package
                            $dependencyObj->setPackageId($mypkg->{id});
                            
                            # If "dependency_condition" is "up" means that this package requires the package identified
                            # by "package_name" to be up (status reported by cmviewcl is "up").
                            # If  "up" is specified, the dependency rules are as follows:
                            # * A multi-node package can depend only on another multi-
                            # node or system multi-node package.
                            # * A failover package whose failover_policy is
                            # min_package_node can depend only on a multi-node or
                            # system multi-node package.
                            # * A failover package whose failover_policy is
                            # configured_node can depend on a multi-node or system
                            # multi-node package, or another failover package whose
                            # failover_policy is configured_node.

                            if ($dependency_cond[3] eq 'up') {
                                if ($packageObj->getType() eq 'multi_node' || $packageObj->getType() eq 'system_multi_node') {
                                    if ($mypkg->getType() ne 'multi_node' && $mypkg->getType() ne 'system_multi_node') {
                                        print "A non-failover package $packageObj->{name} may not depend on a failover package $mypkg->{name}\n";
                                        return -1;
                                    }
                                }
                                elsif ($packageObj->getType() eq 'failover' && $packageObj->getFailoverPolicy() eq 'min_package_node') {
                                    if ($mypkg->getType() ne 'multi_node' && $mypkg->getType() ne 'system_multi_node' ) {
                                        print "The package $packageObj->{name} has failover policy as MIN_PACKAGE_NODE.\n" .
                                            "This package must not depend on a failover package $mypkg->{name}.\n";
                                        return -1;
                                    }
                                    #if ($packageObj->{location} eq 'any_node' || $packageObj->{location} eq 'different_node') {
                                    #     print "Invalid dependency defined for package $packageObj->{name} on package $mypkg->{name}.\n" .
                                    #         "Exclusionary dependencies and dependencies with a DEPENDENCY_LOCATION of ANY_NODE or DIFFERENT_NODE \n" .
                                    #         "are only allowed between FAILOVER packages with a CONFIGURED_NODE failover policy.\n";
                                    #     return -1;
                                    # }
                                }
                                elsif ($packageObj->getType() eq 'failover' && $packageObj->getFailoverPolicy() eq 'configured_node') {
                                    if ($mypkg->getType() ne 'multi_node' && $mypkg->getType() ne 'system_multi_node' && $mypkg->getType() eq 'failover' 
                                        && $mypkg->{failover_policy} ne 'configured_node' ) {
                                            print "The package $packageObj->{name} must not depend on package $mypkg->{name} because,\n" .
                                                "package $mypkg->{name} has MIN_PACKAGE_NODE failover policy.\n";
                                            return -1;
                                    }
                                }
                            }
                            elsif ($dependency_cond[3] eq 'down') {
                                # If the "down" value is specified, the exclusion dependency must be
                                # mutual; that is, if pkgA depends on pkgB to be down, pkgB must
                                # also depend on pkgA to be down.
                                print "Package $packageObj->{name} has defined an exclusionary dependency on package $mypkg->{name}, but $mypkg->{name} has not\n" . 
                                    "defined an exclusionary dependency on $packageObj->{name}. Exclusionary dependencies must be mutual.\n";
                                return -1;

                            }
                        }
                    } 
                }
                # Check if dependency package does not exist and package array is empty (-1)
                if (($#{$cluster_Obj->{packages}} < 0) || !($dependency_pkg_exist)) {
                     print "Dependency $dependency_name in package $packageObj->{name} specifies a non-existent package: $dependency_cond[1]\n";
                     print "Package $packageObj->{name} has a dependency on a package that is is not configured in the cluster.\n";
                     return -1;
                }
                # Add dependency nodes
                foreach my $mypnode (@{$packageObj->{nodes}}) {
                    $dependencyObj->addNode($mypnode->{name},"no");
                }
                my $dependency = join " ", @dependency_cond[1..$#dependency_cond];
                $dependencyObj->setCondition($dependency);
            }
            elsif ($line =~ /^DEPENDENCY_LOCATION/i) {
                unless ($pkg_dep_warn_flag) {
                    print "WARNING: Package dependency is not supported on the simulated cluster. The related functionality might differ from real cluster.\n";
                    $pkg_dep_warn_flag = 1;
                }
                my @dependency_location = split(" ",$line);
                $dependencyObj->setLocation($dependency_location[1]);
            }
            elsif ($line =~ /^EXTERNAL_SCRIPT/i) {
                my @script = split(" ",$line);
                $packageObj->addExternalScript($script[1]);
            }
            elsif ($line =~ /^EXTERNAL_PRE_SCRIPT/i) {
                my @script = split(" ",$line);
                $externalPreScriptObj = SGPackageExternalPreScript->new();
                $externalPreScriptObj->setExternalPreScript($script[1]);
                $packageObj->addExternalPreScript($externalPreScriptObj);
            }
            elsif ($line =~ /^FS_NAME/i) {
                my @fs_name = split(" ",$line);
                $fsObj=SGPackageFileSystem->new();
                $fsObj->setName($fs_name[1]);
                $packageObj->addFileSystem($fsObj);
            }
            elsif ($line =~ /^FS_SERVER/i) {
                my @fs_server = split(" ",$line);
                $fsObj->setFsServer($fs_server[1]);
            }
            elsif ($line =~ /^FS_DIRECTORY/i) {
                my @fs_directory = split(" ",$line);
                $fsObj->setFsDirectory($fs_directory[1]);
            }
            elsif ($line =~ /^FS_TYPE/i) {
                my @fs_type = split(" ",$line);
                $fsObj->setFsType($fs_type[1]);
            }
            elsif ($line =~ /^FS_MOUNT_OPT/i) {
                my @fs_mount_opt = split(" ",$line);
                $fsObj->setFsMountOpt($fs_mount_opt[1]);
            }
            elsif ($line =~ /^FS_UMOUNT_OPT/i) {
                my @fs_umount_opt = split(" ",$line);
                $fsObj->setFsUMountOpt($fs_umount_opt[1]);
            }
            elsif ($line =~ /^FS_FSCK_OPT/i) {
                my @fs_fsck_opt = split(" ",$line);
                $fsObj->setFsFsckOpt($fs_fsck_opt[1]);
            }         
            elsif ($line =~ /^WEIGHT_NAME/i) {
                unless ($pkg_weight_warn_flag) {
                    print "WARNING: Package weights are not supported on the simulated cluster. The related functionality might differ from real cluster.\n";
                    $pkg_weight_warn_flag = 1;
                }
                my @weight = split(" ",$line);
                $weightObj = SGWeight->new();
                $weightObj->setName($weight[1]);
                $packageObj->addWeight($weightObj);
            }
            elsif ($line =~ /^WEIGHT_VALUE/i) {
                unless ($pkg_weight_warn_flag) {
                    print "WARNING: Package weights are not supported on the simulated cluster. The related functionality might differ from real cluster.\n";
                    $pkg_weight_warn_flag = 1;
                }
                my @weight = split(" ",$line);
                $weightObj->setValue($weight[1]);
            }
            elsif ($line =~ /^SERVICE_NAME/i) {
                my @service = split(" ",$line);
                $serviceObj=SGPackageService->new();
                $serviceObj->setName($service[1]);
                $serviceObj->setServiceConfigParam("id", $service_id);
                $service_id+=1;
                $packageObj->addService($serviceObj);
                foreach my $pNode (@{$packageObj->{nodes}}) {
                    my $serviceNode = SGPackageServiceNode->new();
                    $serviceNode->setServiceName($serviceObj->getName());
                    $serviceNode->setName($pNode->getName());
                    $serviceNode->setStatus("down");
                    $serviceNode->setMaxRestarts("unknown");
                    $serviceNode->setRestarts(0);
                    $pNode->addServiceNode($serviceNode);
                }
            }       
            elsif ($line =~ /^SERVICE_CMD/i) {
                my @cmd = split(" ",$line);
                my $cmd_arg = @cmd;
                my $cmds = "";
                foreach my $co (@cmd) {
                    if ($co eq 'service_cmd') {
                        next;
                    }
                    $cmds = $cmds." ".$co;
                }
                $cmds =~ s/^\s+//;    		 	
                $serviceObj->setServiceConfigParam("command", $cmds);
            }
            elsif ($line =~ /^SERVICE_RESTART/i) {
                my @restart = split(" ",$line);
                $serviceObj->setServiceConfigParam("restart", $restart[1]);
            }
            elsif ($line =~ /^SERVICE_FAIL_FAST_ENABLED/i) {
                my @service = split(" ",$line);
                my $value = $service[1];
                if (lc($value) eq 'no') {
                    $serviceObj->setServiceConfigParam("failfast", "disabled");
                }
                else {
                    $serviceObj->setServiceConfigParam("failfast", "enabled");
                }
            }
            elsif ($line =~ /^SERVICE_HALT_TIMEOUT/i) {
                my @service = split(" ",$line);
                $serviceObj->setServiceConfigParam("halt_timeout", $service[1]);
            }

        }
        $clusterObj = &statusParser($filename);
        $clusterObj->addPackage($packageObj);
        
    }
    close SRCASCII;
    return 0;
}

# subroutine to expand the "*" specified as node_name . 
sub parseNodeNameStar {
    my ($nodes) = @_;
    my $clusterObj = &statusParser($filename);
    foreach my $mynode (@{$clusterObj->{nodes}}) {
        push(@$nodes, $mynode->{name});
    }
}
    

#subroutine to  load attribtutes , it will call the parse_and_populate_attributes
sub loadAttributes {
    my ($asciiFile,$type)=@_;
    
    if (parse_and_populate_attributes($asciiFile,$type)) {
        return -1;
    }
    
     
    no warnings;
       
}    

# subroutine to validate parameter value by taking parameter names and values.
sub validateParam {
    my ($param_name,$param_value)=@_;
    #die "Missing Parameter Name or Parameter Value" if (!$param_name || (!$param_value and ($param_value !=0))) ;

    if (($param_name =~/CLUSTER_NAME/i) or ($param_name =~/PACKAGE_NAME/i)) {
        my $len=length($param_value);
        if ($len >= $max_size) {
            print "Error: $param_name can not exceed " . $max_size -1 . "characters \n";
        }
        if ($param_value eq "") {
            print "Error: $param_name can not be blank. \n";
        }
    }
    if ($param_name =~/MAX_CONFIGURED_PACKAGES/i ) {
        if (($param_value=~/[^0-9]/) or ($param_value > 300) or ($param_value < 0)) {
            print "Invalid value of $param_name. It can not be greater than 300 or less than 0 \n";
        } 
    }
    if ($param_name=~/failover_policy/i) {
        if (( $param_value !~/configured_node\b/i) and ($param_value !~/min_package_node\b/i)and 
            ($param_value !~/site_preferred_manual\b/i) and ($param_value !~/site_preferred\b/i)) {
            print "Failover Policy should be either configured_node , site_preferred_manual ,\n site_preferred or min_package_node";           
        }
    }
    if ($param_name=~/failback_policy/i) {
        if (( $param_value !~/manual\b/i) and ($param_value !~/automatic\b/i)) {
            print " Failback Policy should be either manual or automatic.";  
        }
    }
    if ($param_name=~/priority/i) {
        if (( $param_value !~/no_priority\b/i) and ($param_value < 1 or $param_value > 3000)) {
            print " Priority should be either no_priority or in range of 1 to 3000 ";  
        }
    } 
    if ($param_name=~/package_type/i) {
        if (( $param_value !~/failover\b/i) and ($param_value !~/multinode\b/i)and ($param_value !~/system_multinode\b/i)) {
            print " Package type should be either failover or multinode or system_multinode.";  
        }
    } 
}
#subroutine to validate package services
sub validateServices {
    my ($serStr)=@_; 
    die "Missing depStr" if (!$serStr);
    my @ser_vals=split(':',$serStr);
    my $serName=""; 
    my $serCmd=""; 
    foreach my $vals (@ser_vals) {
        if ($vals =~/SERVICE_NAME/i) {
            next;
        }
        my @sub_vl=split('\!',$vals);
        $serName= shift(@sub_vl);
        foreach my $vl (@sub_vl) {
            if (($vl =~/none/i) or ($vl=~/unlimited/i) or ($vl=~/yes/i) or ($vl=~/no/i) or ($vl=~/^\d+/)) {
                next;
            }
            my @v=split(" ",$vl);
            # Removing leading '"' if present
            $v[0]=~s/^"//g;
            # checking if service command is executable or not \n";
            if (! ( -x $v[0])) {
                print "Service Command $v[0] specified for Service $serName is not executable.\nUnable to verify $pkgName.\n";
            }
        }
    }
}

# subroutine to validate package dependencies. 
sub validateDependencies {
    my ($depStr)=@_;
    my @names; 
    my $SAME_NODE=0;
    my $ANY_NODE=0;
    my $DIFFERENT_NODE=0;
    my $ALL_NODE=0;
    my $UP=0;
    my $DOWN=0;
    my $depPkgName="";
    my $depName="";
    my @dep_vals=split(':',$depStr);
    my $res = 'n';

    unless ($depStr) {
        print "Missing depStr";
        return -1;
    }

    foreach my $vals (@dep_vals) {
        if ($vals =~/DEPENDENCY_NAME/i) {
            next;        
        }
        my @vl=split('\!',$vals);
        $depName=$vl[0];
        foreach my $v (@vl) {
            if ( $v =~/same_node/i) {
                 $SAME_NODE=1;
            }
            if ( $v =~/any_node/i) {
                 $ANY_NODE=1;
            }
            if ( $v =~/different_node/i) {
                 $DIFFERENT_NODE=1;
            }
            if ( $v =~/all_node/i) {
                 $ALL_NODE=1;
            }
            if ( $v =~/up/i) {
                 my @tmp=split("=",$v);
                 $depPkgName=$tmp[0];
                 $UP=1;
            }
            if ( $v =~/down/i) {
                 my @tmp=split("=",$v);
                 $depPkgName=$tmp[0];
                 $DOWN=1;
            }
            #my $CMVIEWCL_CMD= $CMVIEWCL . " -v -f line";
            #my $res=`$CMVIEWCL_CMD`;
            if ($res!~/package:$depPkgName|name=$depPkgName\b/i) {
                print "Dependency $depName in package $pkgName specifies a non-existent package: $depPkgName.\nUnable to verify Package $pkgName.\n";
                return -1;
            }
        }
        if (( $DOWN == 1) and (($DIFFERENT_NODE ==1) or ($ANY_NODE ==1))) { 
            print "Dependency Location can not be DIFFERENT_NODE or ANY_NODE when Dependency Condition is specified as down\n";
            return -1;
        }
        if (($UP==1) and ($ALL_NODE==1)) {
            print "Dependency Location can not be ALL_NODE when Dependency Condition is specified as up\n";
            return -1;
        }
    }
}

sub parse_Configuration {
    
    my ($type, $fileName, $session)=@_;
    my $res = 'n';
    my $doask = 0 ;
    $sessionName = $session;
    $SGCONF = getSGCONF();
     
    unless ($type) {
       print "Please specify type of configuration\n";
       return -1;
    }
    if (($type ne 'cluster') && ($type ne 'package')){
       print "Type should be only cluster or package\n";
       return -1;
    }
    unless ($fileName) {
       print "Please specify Ascii File name along with path if it is not present in the current dir\n";
       return -1;
    }   

    if (loadAttributes($fileName,$type)) {
        return -1;
    }
    validateMandatoryAttribute($type);
    if (-d "$SGCONF/simulator/$sessionName") {
        
        $doask = 1;
    }
    if ($type=~/cluster/i) {
        `echo "MAX_CONFIGURED_PACKAGES set to $clusterObj->{max_configured_packages}" 1>&2`;
        `echo "MAX_CONFIGURED_PACKAGES set to $clusterObj->{max_configured_packages}" 1>&2`;
        foreach my $node (@{$clusterObj->{nodes}}) {
            print "Adding node " . $node->getName() . " to cluster " . $sessionName ."\n";
        }    
    }
    else {
        #validateDependencies($packageAttributeArrays[0]);
        #validateServices($packageAttributeArrays[1]);
        $doask = 1;
    }
    if ($doask) {
        `echo "Modify the $type configuration([y]/n)?" 1>&2`;
        while (<STDIN>) {
            if ($_ =~ /^\s*$/){
                $res='y';
                last;
            }
            $res=$_;
            last;
        }
    } else { $res ='y'; }
     
    if (($res=~/y/i)) {
        if ($type eq "cluster") {
            my $sessionDir = "$SGCONF/simulator" . "/" . $sessionName;
            mkdir $sessionDir;
        }
        statusDumper($clusterObj, $filename);
        $ret = set_node($clusterObj->{name},$clusterObj->{nodes}[0]->{name});
    }
    else {
        print "cmapplyconf: Unable to apply the configuration\n";
        return -1;
    }    
    return 0; 
}

sub getIOTimeout {
    my @vl;
    $vl[12]=70000000;
    return $vl[12];
}

1;


