####################################################################
# (C) Copyright 2007-2013 Hewlett-Packard Development Company, L.P.
# @(#) Serviceguard src_file.sh
# @(#) Product Name                : HP Serviceguard
# @(#) Product Version             : A.12.10.00
# @(#) Patch Name                  : 
####################################################################
# This script takes the name of the flat_file sources it in 
# and writes out sorted key=value pairs 
# Intentions:
# 1. evaluate flat file first for values that in itself has variable embedded
# 2. sort out out-of-order arrays
# Assumptions: 
# 1. the variables of the flat file should be a subset of the module attributes
#    if not, cmmakepkg will catch that and report error back
# 2. no parent/child relationship in the flat file 
#    i.e. no array grouping like this
#       service_name[0]=name
#       service_cmd[0]=cmd
# 

# input is the name of the flat file only
if (($# == 0 )) || [[ ! -f $1 ]]; then
    echo "Usage: ${0##*/} flat_file"
    exit 1
fi

ff=$1

# read in the flat file and remove blank lines an comments and only assignments
data=$(awk < $ff  '
    /^#/                        { next }
    /^[[:space:]]*#/            { next }
    /^[[:space:]]*$/            { next }
    /^[[:space:]]*[^= ][^= ]*=/ { print; next }' | sed -e 's/\\\$/\\\\\\$/')

# evaluation input first - 
# some toolkit conf has variables within a value
# evaluate first so those variables gets evaluted and in the specified 
# order in the flat file
data_v2=$data
eval "$data"

# save variable name in a list to be sorted
typeset names="";
typeset I;
typeset quotes=""

saved_IFS="$IFS"
# define new line as internal field separator
IFS='
'
for I in $data
do
    name=${I%%=*}
    names="${names} $name"    
done

for I in $data_v2
do
    name=${I%%=*}
    value=${I##*=}
    # check for "" in the value
    value_v2="${value#\"}"
    value_v2="${value_v2%\"}"
    if [[ ${value_v2} != ${value} ]]
    then
        quotes="${quotes} $name"
    fi
done

IFS="$saved_IFS"

# sort the variable name to group array together and save the output
typeset sorted_names=$(for N in $names; do echo $N; done | sort)

# output the variables according to the sorted list
typeset -i cnt=0
typeset array_name=" "
typeset indices=" "

function print_array
{

    sorted_indices=$(for N in $indices; do echo $N; done | sort -u -n)
    for index in $sorted_indices
    do
        eval value="\${$array_name[$index]}"
        echo $quotes | grep -q -w "$array_name\[$index\]"
        if (( $? != 0 )); then
            echo "$array_name[$index]=$value"
        else
            echo "$array_name[$index]=\"$value\""
        fi
    done
    # reset
    array_name=""
    indices=""
}

for I in $sorted_names
do
    # if I is array
    if [[ ${I%\[*} != ${I} ]]; then

        # if new array name is the same as old one, print old array first
        if [[ $array_name != ${I%%\[*} ]]; then
            print_array
        fi

       # save array name and index to be sorted later
        array_name=${I%%\[*}
        index=${I##*\[}
        index=${index%\]}
        indices="${indices} $index"

    else
    # if I is not array

        print_array
        eval value="\${$I}"
        echo $quotes | grep -q -w "$I"
        if (( $? != 0 )); then
            echo "$I=$value"
        else
            echo "$I=\"$value\""
        fi
    fi
done

print_array

