/*@ Copyright [2014] Hewlett-Packard Development Company, L.P.
 *
 * This program is free software; you can redistribute it and/or modify it under the terms 
 * of version 2 of the GNU General Public License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, write to:
 *   Free Software Foundation, Inc.
 *   51 Franklin Street, Fifth Floor
 *   Boston, MA 02110-1301, USA.
 *
 * 
 * Changes : 
 *         This file has been modified on February 07, 2014 to fulfill certain product requirements.
 *         
 *         The changes done are as follows :
 *                  1. Removed header files : "getopt.h" "unistd.h" and "config.h"
 *                  2. Removed the functions "handle_options" and "usage" as these were not 
 *                     required for receiving and processing inputs.
 *                  3. Removed the function "linkloop" and accommodate its functionality in "main"
 *                     function only.
 *                  4. Added, removed, renamed a few variables to conform to our internal variable
 *                     naming conventions.
 *                  5. Added/Removed few statements/comments.
 *
 */

/* linkloop: Loopback testing for layer-2 connectivity
 *           via 802.2LLC TEST message.
 *
 * Tested against HPUX-11.0
 *
 * Written by Oron Peled <oron@actcom.co.il>
 *
 * Some code snippets adapted from spak
 * (http://www.xenos.net/software/spak/)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <net/if.h>	
#include <arpa/inet.h>
#include "linkloop.h"

#define	TEST_CMD	0xE3	/* From 802.2LLC */

size_t  pack_len = 1500; 
char	*program = NULL;

int	timeout = 1;
int	retries = 2;
int	debug = 0;

void sig_alarm(int signo) {
     /*fprintf(stderr, "TIMEOUT\n");*/
}

void set_timeout() {
   struct sigaction sa;

   sa.sa_handler = sig_alarm;
   sa.sa_flags = 0;
   sigfillset(&sa.sa_mask);	
   sigdelset(&sa.sa_mask, SIGINT);	
   sigdelset(&sa.sa_mask, SIGQUIT);
   sigdelset(&sa.sa_mask, SIGTERM);
   if(sigaction(SIGALRM, &sa, NULL) < 0) {
   	perror("sigaction");
   	exit(1);
   }
   if(alarm(timeout) < 0) {
   	perror("alarm");
   	exit(1);
   }
}

int main(int argc, char *argv[]) {
   int sock_id, i, ret;
   int verbose=0;
   char hostname[128];
   u_int8_t source_mac[IFHWADDRLEN];
   u_int8_t destination_mac[IFHWADDRLEN];
   struct llc_packet send_pack;
   struct llc_packet recv_pack;
   char eth[1024];

   program = argv[0];

   if(argc <= 1) {
   	fprintf(stderr, "Usage: %s <eth0> <target_mac>\n", program);
   	return 1;
   }
   if(!parse_address(destination_mac, argv[2])) {
   	fprintf(stderr, "%s: bad DST address - %s\n", program, argv[2]);
   	exit(1);
   }
   if ((argc > 3) && argv[3][1]=='v'){
      verbose=1;
      printf ("verbose\n");
   }

   if((sock_id = socket(AF_INET, SOCK_PACKET, htons(ETH_P_802_2))) == -1) {
   	perror("socket");
   	return 1;
   }
   strncpy(eth,argv[1],sizeof(eth));
   get_hwaddr(sock_id, eth, source_mac);

   mk_test_packet(&send_pack, source_mac, destination_mac, pack_len,0);
   if(debug)
   	dump_packet(&send_pack);
   ret=0;

   for(i = 0; i < retries; i++) {
      int received=0;
      set_timeout();
      send_packet(sock_id, argv[1], &send_pack);
      if (ret){
         perror("send");
         return 1;
      }
      gethostname(hostname,128);
      while (1){
         if(recv_packet(sock_id, &recv_pack)) {
            if((recv_pack.llc.ctrl & TEST_CMD) != TEST_CMD) {
               continue;
               fprintf(stderr, "got unexpected packet\n");
               dump_packet(&recv_pack);
               printf("NOCMD:%s %s\n",hostname,eth);
               exit(1);
            }
            printf("%s %s\n",hostname,eth);
            return 0;
         } else {
            break;
         }
      }
   }
   return 0;
}
