/*@ Copyright [2014] Hewlett-Packard Development Company, L.P.
 *
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of version 2 of the GNU General Public License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program;
 * if not, write to:
 *   Free Software Foundation, Inc.
 *   51 Franklin Street, Fifth Floor
 *   Boston, MA 02110-1301, USA.
 *
 *
 * Changes :
 *         This file has been modified on February 07, 2014 to fulfill certain product requirements.
 *
 *         The changes done are as follows :
 *                  1. Removed header files : "math.h" "sys/ioctl.h" "netinet/if_ether.h" 
 *                     "sys/select.h" and "config.h"
 *                  2. Added "bind" function call.
 *                  3. Changes are made to listen on only one interface at a time. To do this,
 *                     removed "for" loop at the time of getting h/w address of interface and 
 *                     sending response back to client. 
 *                  4. Added, removed, renamed a few variables to conform to our internal variable
 *                     naming conventions.
 *                  5. Added/Removed few statements/comments.
 */

/*
 * Written by Oron Peled <oron@actcom.co.il>
 *
 * Some code snippets adapted from spak
 * (http://www.xenos.net/software/spak/)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <net/if.h>
#include "linkloop.h"

static char	*program = NULL;

u_int8_t	source_mac[IFHWADDRLEN];
u_int8_t	destination_mac[IFHWADDRLEN];

int main(int argc, char *argv[]) {
   int sock_id, ret, pack_len;
   struct llc_packet send_pack;
   struct llc_packet recv_pack;
   struct sockaddr myaddr;
   struct ifreq ifr;
   int verbose=0;
   char eth[1024];
   strcpy (eth, argv[1]);

   program = argv[0];
   if ((argc > 2) && strstr(argv[2],"v"))
       verbose=1;

   if ((sock_id = socket(AF_INET, SOCK_PACKET, htons(ETH_P_802_2))) == -1) {
	perror("socket");
	return 1;
   }

   memset(&myaddr, '\0', sizeof(myaddr));
   myaddr.sa_family = AF_INET;
   strcpy(myaddr.sa_data, eth);

   ret = bind(sock_id, &myaddr, sizeof(struct sockaddr));

   get_hwaddr(sock_id, argv[1], source_mac);

   do {
      pack_len = recv_packet(sock_id, &recv_pack);
      if (!pack_len) {
         printf("timeout\n");
         continue;
      }
      if (verbose)
         printf("packet received on %s\n",eth);
         /*dump_packet(&recv_pack);*/
         memcpy(destination_mac, recv_pack.eth_hdr.ether_shost, IFHWADDRLEN);
         mk_test_packet(&send_pack, source_mac, destination_mac, pack_len, 1);
         send_packet(sock_id, argv[1], &send_pack);
         if (verbose)
            printf("packet sent from %s to %s\n",eth, destination_mac);
   } while(1);
   return 0;
}
